package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.intf.CosService;
import com.bocloud.cos.params.CosParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 运维服务
 *
 * @author weiwei
 */
@RestController
@RequestMapping("/v1/cos")
@Api(value = "运维服务管理", tags = "运维服务管理")
public class CosController {

    @Autowired
    private CosService cosService;

    /**
     * 列表
     *
     * @param pager
     * @param condition
     * @param context
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取运维服务列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cosService.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter(),
                pager.getSimple(), context);
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "删除运维服务", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cosService.remove(id, context);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "查找运维服务详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return cosService.detail(id);
    }

    /**
     * 执行运维服务
     *
     * @param params
     * @param context
     * @return
     */
    @PutMapping
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "执行运维服务", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult execute(@RequestBody CosParams object, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cosService.execute(object, context);
    }

}
