package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.Document;
import com.bocloud.cos.intf.DocumentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 使用文档控制器
 *
 * @author tyl
 * @update fanzx
 * @since 2021/3/8 16:26
 */
@RestController
@RequestMapping("/v1/documents")
@Api(value = "使用文档", tags = "使用文档")
public class DocumentController {
    private final DocumentService documentService;

    public DocumentController(DocumentService documentService) {
        this.documentService = documentService;
    }

    @ApiOperation(value = "创建使用文档", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public BocloudResult create(@RequestBody Document document, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.create(document, context);
    }

    @ApiOperation(value = "获取使用文档列表树", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult list(Pager pager,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.list(pager.getParams(), context);
    }

    @ApiOperation(value = "模糊搜索查询文档树", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/search")
    public BocloudResult search(@RequestParam(value = Common.NAME) String name,
                                @RequestParam(value = "category") String category,
                                @RequestParam(value = "status") String status,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.search(name, category, status, context);
    }

    @ApiOperation(value = "修改文档信息", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public BocloudResult modify(@PathVariable(value = Common.ID) Long id,
                                @RequestBody Document document,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.modify(id, document, context);
    }

    @ApiOperation(value = "删除使用文档", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.remove(id, context);
    }

    @ApiOperation(value = "获取使用文档详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return documentService.detail(id);
    }

    @PostMapping("/pictures")
    @ApiOperation(value = "删除使用文档图片", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new BocloudResult(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        multipartFile.getContentType();
        // 判断文件名
        if (originalFilename == null) {
            return new BocloudResult(false, "文件名不能为空");
        }
        if (!(originalFilename.endsWith(".png") || originalFilename.endsWith(".PNG") || originalFilename.endsWith(".jpg") || originalFilename.endsWith(".JPG"))) {
            return new BocloudResult(false, "文件格式错误，只支持PNG或JPG格式的图片");
        }
        return documentService.upload(multipartFile);
    }

    @DeleteMapping("/pictures")
    @ApiOperation(value = "删除使用文档图片", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult upload(@RequestBody List<String> paths) {
        return documentService.delete(paths);
    }

    /**
     * 管理端控制自服务展示
     *
     * @param document
     * @param context
     * @return
     */
    @ApiOperation(value = "控制自服务展示文档", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping
    public BocloudResult serviceDisplay(@RequestBody Document document, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.serviceDisplay(document.getDisplay(), context);
    }

    /**
     * 管理端控制自服务展示
     *
     * @param document
     * @param context
     * @return
     */
    @ApiOperation(value = "控制自服务展示文档", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/display")
    public BocloudResult getDisplay(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.getDisplay(context);
    }
}
