package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.intf.ExternalService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 对外展示大屏控制器
 *
 * @author tyl
 * @since 2020/12/23 15:10
 */
@RestController
@RequestMapping("/v1/external")
@Api(value = "云资源大屏", tags = "云资源大屏")
public class ExternalScreenController {
    private final ExternalService externalService;

    public ExternalScreenController(ExternalService externalService) {
        this.externalService = externalService;
    }

    @ApiOperation(value = "资源总览", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/resource/overview")
    public BocloudResult resource(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.resourceOverview(dcId, context);
    }

    @ApiOperation(value = "云主机申请趋势", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/vms/trend")
    public BocloudResult vmsTrend(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.vmsTrend(dcId, context);
    }

    @ApiOperation(value = "云资源申请趋势", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/resource/trend")
    public BocloudResult resourceTrend(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.resourceTrend(dcId, context);
    }

    @ApiOperation(value = "租户云资源统计", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/tenants/resource")
    public BocloudResult resourceCount(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.tenantResourceCount(dcId, context);
    }

    @ApiOperation(value = "项目云资源统计", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/projects/resource")
    public BocloudResult storageCount(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.projectResourceCount(dcId, context);
    }

    @ApiOperation(value = "业务云资源统计", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/business/resource")
    public BocloudResult systemResource(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.businessResourceCount(dcId, context);
    }

}
