package com.bocloud.cos.booter.controller;


import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.Payment;
import com.bocloud.cos.intf.PaymentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 支付记录
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@RestController
@RequestMapping("/v1/payments")
@Api(value = "支付记录", tags = "支付记录")
public class PaymentController {

    private final PaymentService paymentService;

    public PaymentController(PaymentService paymentService) {
        this.paymentService = paymentService;
    }

    /**
     * 获取支付记录列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取支付记录列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Payment>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return paymentService.list(pager, context);
    }

    /**
     * 添加支付记录
     *
     * @param payment
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加支付记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody Payment payment,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return paymentService.create(payment, context);
    }

    /**
     * 修改支付记录
     *
     * @param id
     * @param payment
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改支付记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody Payment payment, @Value(Common.REQ_CONTEXT) RequestContext context) {
        payment.setId(id);
        return paymentService.modify(payment, context);
    }

    /**
     * 删除支付记录
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除支付记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return paymentService.remove(id, context);
    }

    /**
     * 查看支付记录详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看支付记录详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Payment> detail(@PathVariable(value = Common.ID) Long id) {
        return paymentService.detail(id);
    }
}
