package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.PortalPanel;
import com.bocloud.cos.intf.PortalPanelService;
import com.bocloud.cos.model.PortalPanelBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 首页配置controller
 *
 * @author lyy
 * @version 2.0
 * @time 2017年12月27日
 */
@RestController
@RequestMapping("/v1/portal/panels")
@Api(value = "主页配置管理", tags = "主页配置管理")
public class PortalPanelController {

    @Autowired
    private PortalPanelService portalPanelService;

    /**
     * 查询配置列表展示
     *
     * @param serverName
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "主页配置列表展示", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult list(@RequestParam(value = "serverName", required = false) String serverName,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != serverName) {
            return portalPanelService.list(serverName, context);
        } else {
            return new BocloudResult(false, "参数格式不正确");
        }
    }

    /**
     * 更新首页配置
     *
     * @param portalPanelBean
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "保存主页配置", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping
    public BocloudResult save(@RequestBody PortalPanelBean portalPanelBean,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != portalPanelBean) {
            if (!PortalPanelService.CSC.equals(portalPanelBean.getServerName()) && !PortalPanelService.CMC.equals(portalPanelBean.getServerName())) {
                return new BocloudResult(false, "参数格式不正确");
            }
            if (portalPanelBean.getConfig() == null) {
                return new BocloudResult(false, "参数格式不正确");
            }
            List<PortalPanel> portalPanels = JSONObject.parseArray(portalPanelBean.getConfig(), PortalPanel.class);
            return portalPanelService.save(portalPanels, portalPanelBean.getServerName(), context.getTarget(), context.getTenant(), null);
        } else {
            return new BocloudResult(false, "参数格式不正确");
        }
    }

    @ApiOperation(tags = {"CMC"}, value = "恢复默认设置", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/reset")
    public BocloudResult reset(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return portalPanelService.reset(PortalPanelService.CMC, context);
    }

}
