package com.bocloud.cos.booter.controller;


import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.intf.ServiceBillService;
import com.bocloud.cos.intf.ServiceOrderService;
import com.bocloud.cos.model.ChargeResultTrend;
import com.bocloud.cos.model.LineChartResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * cmc首页概览数据controller
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月3日
 */
@RestController
@RequestMapping("v1/portal")
@Api(value = "CMC首页概览数据", tags = "CMC首页概览数据")
public class PortalStatsController {

    private final ServiceOrderService serviceOrderService;
    private final ServiceBillService serviceBillService;

    public PortalStatsController(ServiceOrderService serviceOrderService, ServiceBillService serviceBillService) {
        this.serviceOrderService = serviceOrderService;
        this.serviceBillService = serviceBillService;
    }

    /**
     * 获取云主机服务使用趋势
     *
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "获取云主机服务使用趋势", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/platform/statisticsOfComputer")
    public BocloudResult<ChargeResultTrend> statisticsOfComputer() {
        return serviceOrderService.vmTrend();
    }

    /**
     * 租户消费总额概览
     *
     * @param code    查询类型
     * @param range   查询范围
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "租户消费总额概览", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/charge/stats")
    public BocloudResult<LineChartResult> statsScp(@RequestParam(value = Common.CODE) String code, @RequestParam(value = "range") String range, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.stats(code, range, context);
    }

}
