package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.cos.entity.Recycled;
import com.bocloud.cos.intf.RecycledService;
import com.bocloud.cos.model.ResourceRecycledParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 回收站
 */
@Api(value = "回收站", tags = "回收站")
@RestController
@RequestMapping("/v1/recycles")
public class RecycledController {
    @Autowired
    private RecycledService recycledService;

    @ApiOperation(tags = {"CMC", "CSC"}, value = "加入回收站", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public BocloudResult create(@RequestParam(value = Common.PARAMS) String params,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            Recycled recycled = JSONObject.parseObject(object.toJSONString(), Recycled.class);
            return recycledService.create(recycled, context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询回收站列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return recycledService.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter(), context);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除回收站记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return recycledService.remove(id, context);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "根据订单删除回收站记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/sn")
    public BocloudResult removeBySn(@RequestParam String sn, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (!StringUtils.isEmpty(sn)) {
            return recycledService.remove(sn, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "回收站还原")
    @PatchMapping("/{id}/restore")
    public BocloudResult restore(@PathVariable(value = Common.ID) Long id,
                                 @RequestBody ResourceRecycledParam resourceRecycledParam,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(JSON.toJSONString(resourceRecycledParam));
        if (null != object) {
            Long resourceId = object.getLong("resourceId");
            String type = object.getString("type");
            return recycledService.restore(resourceId, id, type, context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 清空回收站
     *
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除回收站记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping()
    @Deprecated
    public BocloudResult clear(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return recycledService.clear(context);
    }

}
