package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.Icon;
import com.bocloud.cos.intf.IconService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.List;

/**
 * 图标controller层
 *
 * @author lyy
 * @version 2.0
 * @time 2017年12月13日
 */
@Api(value = "图标", tags = "图标")
@RestController
@RequestMapping("/v1/icons")
public class ResourceIconController {
    private Logger logger = LoggerFactory.getLogger(ResourceIconController.class);
    @Autowired
    private IconService iconService;

    /**
     * 删除图标
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除模型图标", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return iconService.remove(id);
    }

    /**
     * 列表展示
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询模型图标", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult list(Pager pager) {
        return iconService.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter(),
                pager.getSimple());
    }

    /**
     * 图标上传
     *
     * @param request
     * @param response
     * @param context
     * @return
     * @throws IOException
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "上传图标", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public BocloudResult upload(MultipartFile[] files, HttpServletRequest request, HttpServletResponse response,
                                @Value(Common.REQ_CONTEXT) RequestContext context) throws IOException {
        try {
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
            List<MultipartFile> fileList = multipartRequest.getFiles("files");
            if (fileList == null || fileList.isEmpty()) {
                return new BocloudResult(false, "文件错误");
            }
            MultipartFile item = fileList.get(0);
            Icon icon = new Icon();
            icon.setName(URLDecoder.decode(item.getName(), "UTF-8"));
            byte[] data = IOUtils.toByteArray(item.getInputStream());
            icon.setIcon("data:image/png;base64," + Base64.encodeBase64String(data));
            icon.setReserved(false);
            icon.setCategory(request.getParameter("category"));
            return iconService.create(icon);
        } catch (Exception e) {
            logger.error("文件上传出错", e);
        }
        return new BocloudResult(false, "文件错误");
    }
}
