package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.intf.ResourceScreenService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 云资源大屏控制层
 *
 * @author tyl
 * @since 2020/11/30 14:14
 */
@RestController
@RequestMapping("/v1/cloud/resource")
@Api(value = "资源大屏", tags = "资源大屏")
public class ResourceScreenController {
    private final ResourceScreenService resourceScreenService;

    public ResourceScreenController(ResourceScreenService resourceScreenService) {
        this.resourceScreenService = resourceScreenService;
    }

    @GetMapping("/overview")
    @ApiOperation(value = "资源总体情况", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult totalOverview(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.totalOverview(context);
    }

    @GetMapping("/used")
    @ApiOperation(value = "资源使用情况", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult usedOverview(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.usedOverview(context);
    }

    @GetMapping
    @ApiOperation(value = "平台情况", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult platformOverview(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.platformOverview(context);
    }

    @GetMapping("/types")
    @ApiOperation(value = "获取所有平台类型", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult vendorTypes(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.vendorTypes(context);
    }

    @GetMapping("/count")
    @ApiOperation(value = "云资源统计", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult count(@RequestParam(value = "type", required = false) String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.count(type, context);
    }

    @GetMapping("/status")
    @ApiOperation(value = "云主机状态统计", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult status(@RequestParam(value = "type", required = false) String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.status(type, context);
    }

    @GetMapping("/apply")
    @ApiOperation(value = "云主机申请趋势", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult resourceUsed(@RequestParam(value = "type", required = false) String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.resourceApply(type, context);

    }

    @GetMapping("/trend")
    @ApiOperation(value = "云资源使用趋势", response = BocloudResult.class, tags = {"CMC"}, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult pubCloudResourceTrend(@RequestParam(value = "type", required = false) String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.cloudResourceTrend(type, context);
    }
}
