package com.bocloud.cos.booter.controller;


import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.cos.intf.ServiceCatalogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

/**
 * 服务目录
 *
 * @author Administrator
 */
@RestController
@RequestMapping("/v1/services/catalogs")
@Api(value = "服务目录管理", tags = "服务目录管理")
public class ServiceCatalogController {

    private final ServiceCatalogService serviceCatalogService;

    public ServiceCatalogController(ServiceCatalogService serviceCatalogService) {
        this.serviceCatalogService = serviceCatalogService;
    }

    /**
     * 获取服务目录
     *
     * @param pager
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取服务目录列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<GridBean<ServiceCatalog>> list(Pager pager) {
        pager.setRows(100);
        return serviceCatalogService.list(pager);
    }

    /**
     * 添加服务目录
     *
     * @param catalog
     * @param context
     * @return
     */
    @PostMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加服务目录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result create(@RequestBody ServiceCatalog catalog,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        catalog.setCreatorId(context.getTarget());
        catalog.setMenderId(context.getTarget());
        return serviceCatalogService.create(catalog);
    }

    /**
     * 移除服务目录
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除服务目录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceCatalogService.remove(id, context.getTarget());
    }

    /**
     * 查询服务目录详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取服务目录详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<ServiceCatalog> detail(@PathVariable Long id) {
        return serviceCatalogService.detail(id);
    }

    /**
     * 查询租户下的服务
     *
     * @param tenantId
     * @return
     */
    @GetMapping("/tenant/catalogs")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户下的服务列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(@RequestParam(value = "tenantId") Long tenantId) {
        return serviceCatalogService.list(tenantId);
    }

    /**
     * 查询指定服务目录下已发布服务
     *
     * @param code
     * @return
     */
    @GetMapping("/{code}/services")
    @ApiOperation(tags = {"CSC"}, value = "查询指定服务目录下已发布服务", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<List<CloudService>> list(@Value(Common.REQ_CONTEXT) RequestContext context,
                                                  @PathVariable(value = "code") String code,
                                                  @RequestParam(value = "tenantId", required = false) Long tenantId) {
        tenantId = Optional.ofNullable(tenantId).orElse(context.getTenant());
        List<CloudService> cloudServices = serviceCatalogService.listPublishedServicesByCatalog(code, tenantId);
        return new BocloudResult<>(true, cloudServices, "查询成功");
    }
}
