package com.bocloud.cos.booter.controller;


import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.ServiceCategory;
import com.bocloud.cos.intf.ServiceCategoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 服务类型controller
 *
 * @author Administrator
 */
@RestController
@RequestMapping("/v1/services/categories")
@Api(value = "服务类型管理", tags = "服务类型管理")
public class ServiceCategoryController {

    private final ServiceCategoryService serviceCategoryService;

    public ServiceCategoryController(ServiceCategoryService serviceCategoryService) {
        this.serviceCategoryService = serviceCategoryService;
    }

    /**
     * 获取服务类型
     *
     * @param pager
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取服务类型列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<GridBean<ServiceCategory>> list(Pager pager) {
        return serviceCategoryService.list(pager);
    }


    /**
     * 获取服务产品类型树
     *
     * @return
     */
    @GetMapping("/tree")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取服务产品类型树", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<JSONArray> tree() {
        return serviceCategoryService.tree();
    }


    /**
     * 获取服务类型列表
     *
     * @param pager
     * @return
     */
    @GetMapping("/condition")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取服务产品列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<List<ServiceCategory>> listByCondition(Pager pager) {
        return serviceCategoryService.list(pager.getParams());
    }

    /**
     * 添加服务类型
     *
     * @param serviceCategory
     * @param context
     * @return
     */
    @PostMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加服务类型", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result create(@RequestBody ServiceCategory serviceCategory,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceCategoryService.create(serviceCategory, context);

    }

    /**
     * 服务编辑
     *
     * @param id
     * @param serviceCategory
     * @param context
     * @return
     */
    @PutMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "编辑服务", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody ServiceCategory serviceCategory,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceCategoryService.modify(id, serviceCategory, context);
    }

    /**
     * 移除服务类型
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除服务类型", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceCategoryService.remove(id, context.getTarget());
    }

    /**
     * 查询服务目录详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取服务类型详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<ServiceCategory> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceCategoryService.detail(id);
    }

}
