package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.ShoppingCart;
import com.bocloud.cos.intf.ShoppingCartService;
import com.bocloud.cos.model.ApplicationParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 购物车控制器
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2019/2/18
 */
@RestController
@RequestMapping("/v1/shopping/cart")
@Api(value = "购物车管理", tags = "购物车管理")
public class ShoppingCartController {

    private final ShoppingCartService shoppingCartService;

    public ShoppingCartController(ShoppingCartService shoppingCartService) {
        this.shoppingCartService = shoppingCartService;
    }

    /**
     * 查询购物车清单列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询购物车清单列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<ShoppingCart>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.list(pager, context);
    }

    /**
     * 删除购物车清单
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除购物车清单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.remove(id, context);
    }

    /**
     * 查询购物车详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询购物车详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<ShoppingCart> detail(@PathVariable(value = Common.ID) Long id) {
        return shoppingCartService.detail(id);
    }

    /**
     * 清空购物车
     *
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "清空购物车", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping
    public Result empty(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.empty(context);
    }


    /**
     * 更新购物车
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "更新购物车", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result empty(@PathVariable Long id, @RequestBody ApplicationParam applicationParam, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.update(id, applicationParam, context);
    }


    /**
     * 添加到购物车
     *
     * @param applicationParam
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加到购物车", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result addToShoppingCar(@RequestBody ApplicationParam applicationParam, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.create(applicationParam, context);
    }

    /**
     * 批量申请
     *
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "购物车批量申请", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/apply")
    public Result apply(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.batchApply(context);
    }
}
