package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.SkuSpec;
import com.bocloud.cos.intf.SkuSpecService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/v1/skus/specs")
@Api(value = "产品规格管理", tags = "产品规格管理")
public class SkuSpecController {

    private final SkuSpecService skuSpecService;

    public SkuSpecController(SkuSpecService skuSpecService) {
        this.skuSpecService = skuSpecService;
    }

    /**
     * 获取产品规格列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取产品规格列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<GridBean<SkuSpec>> list(Pager pager) {
        return skuSpecService.list(pager);
    }

    /**
     * 查看产品规格详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看产品规格详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<SkuSpec> detail(@PathVariable(value = Common.ID) Long id) {
        return skuSpecService.detail(id);
    }

    /**
     * 查看产品规格列表
     *
     * @param code
     * @param categoryId
     * @return
     */
    @GetMapping("/listbycondition")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看产品规格列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult listSpecs(@RequestParam(value = Common.CODE, required = false) String code,
                                   @RequestParam(value = "categoryId", required = false) Long categoryId) {
        return skuSpecService.listSpecs(code, categoryId);
    }

    /**
     * 查看产品规格列表
     *
     * @param code
     * @param serverId
     * @param type
     * @return
     */
    @GetMapping("/listbycode")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看产品规格列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<List<SkuSpec>> listSpecsByCode(@RequestParam(value = Common.CODE, required = false) String code,
                                                        @RequestParam(value = "serverId", required = false) Long serverId,
                                                        @RequestParam(value = Common.TYPE, required = false) String type,
                                                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return skuSpecService.listSpecsByCode(serverId, code, type, context);
    }
}
