package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.AccountRecord;
import com.bocloud.cos.entity.TenantAccount;
import com.bocloud.cos.intf.TenantAccountService;
import com.bocloud.cos.params.AccountActionParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 租户授信账户 controller
 *
 * @author tyl
 * @time 2020/7/15 周三 19:51
 */
@RestController
@RequestMapping("/v1/tenants")
@Api(value = "授信账号管理", tags = "授信账号管理")
public class TenantAccountController {
    private final TenantAccountService tenantAccountService;

    public TenantAccountController(TenantAccountService tenantAccountService) {
        this.tenantAccountService = tenantAccountService;
    }

    @PostMapping("/account")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "租户授信账号", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result assignAccount(@RequestBody TenantAccount tenantAccount, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.assignAccount(tenantAccount, requestContext);
    }

    @DeleteMapping("/{id}/account")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除租户授信账号", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result deleteAccount(@PathVariable(Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.deleteAccount(id, requestContext);
    }

    @PutMapping("/account")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "租户授信账号操作", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result accountAction(@RequestBody AccountActionParams params, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.accountAction(params.getTenantId(), params.getBillId(), params.getAction(), params.getMoney(), requestContext);
    }

    @GetMapping("/{id}/account")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户授信详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<TenantAccount> accountDetail(@PathVariable(Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.accountDetail(id, requestContext);
    }

    @GetMapping("/account/records")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询授信账号记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<List<AccountRecord>> accountRecord(@RequestParam(value = "tenantId") Long tenantId,
                                                            @RequestParam(value = "action") String action,
                                                            @RequestParam(value = "startTime") String startTime,
                                                            @RequestParam(value = "endTime") String endTime,
                                                            @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.accountRecord(tenantId, action, startTime, endTime, requestContext);
    }
}
