package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.VendorSyncRecord;
import com.bocloud.cos.intf.VendorSyncRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 云平台同步记录
 *
 * @author zhangdi
 * @since Oct 29,2019
 */

@RestController
@RequestMapping("/v1/vendors/records")
@Api(value = "云平台同步管理", tags = "云平台同步管理")
public class VendorSyncRecordController {

    private final VendorSyncRecordService vendorSyncRecordService;

    public VendorSyncRecordController(VendorSyncRecordService vendorSyncRecordService) {
        this.vendorSyncRecordService = vendorSyncRecordService;
    }

    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取云平台同步记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<GridBean<VendorSyncRecord>> list(Pager pager) {
        return vendorSyncRecordService.list(pager);
    }

    /**
     * 删除
     *
     * @param id
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除云平台同步记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        return vendorSyncRecordService.remove(id);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取云平台同步记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<VendorSyncRecord> detail(@PathVariable(value = Common.ID) Long id) {
        return vendorSyncRecordService.detail(id);
    }

    /**
     * 根据云平台ID删除
     *
     * @param vendorId
     * @return
     */
    @DeleteMapping("/vendors/{vendorId}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "根据云平台ID删除平台同步记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result removeByVendor(@PathVariable(value = Common.VENDORID) Long vendorId) {
        return vendorSyncRecordService.removeByVendor(vendorId);
    }

    /**
     * 根据时间段删除
     *
     * @param startTime
     * @param endTime
     * @param vendorId
     * @return
     */
    @DeleteMapping("/time")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "根据时间段删除平台同步记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result removeByTime(@RequestParam(value = "startTime", required = false) String startTime, @RequestParam(value = "endTime", required = false) String endTime, @RequestParam(value = "vendorId", required = false) Long vendorId) {
        return vendorSyncRecordService.removeByTime(startTime, endTime, vendorId);
    }

}
