package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.InteractiveRecord;
import com.bocloud.cos.entity.WorkOrder;
import com.bocloud.cos.intf.WorkOrderService;
import com.bocloud.cos.model.StatsResult;
import com.bocloud.cos.params.WorkOrderParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author admin
 */
@RestController
@RequestMapping("/v1/workorders")
@Api(value = "问题工单", tags = "问题工单")
public class WorkOrderController {

    private final WorkOrderService workOrderService;

    public WorkOrderController(WorkOrderService workOrderService) {
        this.workOrderService = workOrderService;
    }

    /**
     * 查询问题工单列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取问题工单列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<WorkOrder>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return workOrderService.list(pager, context);
    }


    /**
     * 查询工单概况
     *
     * @param tenantId
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取问题工单列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/order-pie")
    public BocloudResult<List<StatsResult>> list(@RequestParam(value = "tenantId") Long tenantId) {
        return workOrderService.orderPie(tenantId);
    }

    /**
     * 查询问题工单详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询问题工单详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<WorkOrder> detail(@PathVariable(value = Common.ID) Long id) {
        return workOrderService.detail(id);
    }

    /**
     * 删除工单
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除工单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return workOrderService.remove(id, context);
    }

    /**
     * 添加工单
     *
     * @param params
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加工单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody WorkOrderParams params,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return workOrderService.create(params, context);
    }

    /**
     * 修改工单
     *
     * @param id
     * @param params
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改工单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody WorkOrderParams params, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return workOrderService.modify(id, params, context);
    }

    /**
     * 创建沟通记录
     *
     * @param interactiveRecord
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "创建沟通记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/records")
    public Result createRecord(@PathVariable(value = Common.ID) Long id,
                               @RequestBody InteractiveRecord interactiveRecord, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return workOrderService.createRecord(id, interactiveRecord, context);
    }

    /**
     * 查询沟通记录
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询沟通记录", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/records")
    public BocloudResult<List<InteractiveRecord>> records(@PathVariable(value = Common.ID) Long id) {
        return workOrderService.records(id);
    }
}
