package com.bocloud.cos.booter.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.OperateResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.*;
import com.bocloud.cos.internal.SmsInternalService;
import com.bocloud.cos.internal.SmsSystemConfigService;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.intf.*;
import com.bocloud.cos.model.*;
import com.bocloud.cos.operate.*;
import com.bocloud.cos.params.CloneParams;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.service.ResourceEventPublisher;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;

/**
 * 申请监听器
 *
 * @author weiwei
 */

/**
 * @author weiwei
 */
@Component
public class FlowEventListener {

    Logger logger = LoggerFactory.getLogger(FlowEventListener.class);
    private final AmqpTemplate amqpTemplate;
    private final ResourceApplicationService resourceApplyService;
    private final CosService cosService;
    private final RosService rosService;
    private final SmsSystemConfigService configService;
    private final SmsInternalService socService;
    private final ServiceOrderRepository orderRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final OrderItemRepository orderItemRepository;
    private final OrderSnService orderSnService;
    private final SmsTenantService tenantService;
    private final ServiceCategoryRepository serviceCategoryRepository;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final ResourceQuotaService resourceQuotaService;
    private final ResourceEventPublisher resourceEventPublisher;
    private final LockFactory lockFactory;

    public FlowEventListener(AmqpTemplate amqpTemplate, ResourceApplicationService resourceApplyService,
                             CosService cosService, RosService rosService, SmsSystemConfigService configService,
                             SmsInternalService socService, ServiceOrderRepository orderRepository,
                             CloudServiceRepository cloudServiceRepository, ServiceSkuRepository serviceSkuRepository,
                             OrderItemRepository orderItemRepository, OrderSnService orderSnService, SmsTenantService tenantService,
                             ServiceCategoryRepository serviceCategoryRepository, ResourceQuotaRepository resourceQuotaRepository,
                             ResourceQuotaService resourceQuotaService, ServiceBillRepository billRepository,
                             BillItemRepository itemRepository, ResourceEventPublisher resourceEventPublisher, LockFactory lockFactory) {
        this.amqpTemplate = amqpTemplate;
        this.resourceApplyService = resourceApplyService;
        this.cosService = cosService;
        this.rosService = rosService;
        this.configService = configService;
        this.socService = socService;
        this.orderRepository = orderRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.serviceSkuRepository = serviceSkuRepository;
        this.orderItemRepository = orderItemRepository;
        this.orderSnService = orderSnService;
        this.tenantService = tenantService;
        this.serviceCategoryRepository = serviceCategoryRepository;
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.resourceQuotaService = resourceQuotaService;
        this.resourceEventPublisher = resourceEventPublisher;
        this.lockFactory = lockFactory;
    }

    @RabbitListener(queues = "bocloud.flow.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            if (!StringUtils.isNotEmpty(content)) {
                return;
            }
            logger.info("收到审核结果消息：{}", content);
            JSONObject object = JSONObject.parseObject(content, JSONObject.class);
            if (null == object || object.isEmpty()) {
                return;
            }
            String category = object.getString("ApplyCategory");
            // 如果消息是服务类则处理，不是则处理，消息返回mq
            if (ApplyCategory.ApplicationOperate.name().equalsIgnoreCase(category) || ApplyCategory.AlterationOperate
                    .name().equalsIgnoreCase(category) || ApplyCategory.ExtensionOperate.name().equalsIgnoreCase(category)
                    || ApplyCategory.UnsubscribeOperate.name().equalsIgnoreCase(category)
                    || ApplyCategory.ResourceComputerClone.name().equalsIgnoreCase(category)) {
                logger.info("收到服务相关审核结果消息:" + content);
                String status = object.getString("ApplyStatus");
                operate(category, object, status);
            } else {
                logger.info("收到其他结果消息，发到其他机器");
                logger.info("This message category [{}] should not process in this service, reject it!", category);
                Object headerObject = message.getMessageProperties().getHeaders().get("retrytimes");
                Long retry = null == headerObject ? 0L : (Long) headerObject;
                if (retry < 20) {
                    message.getMessageProperties().setHeader("retrytimes", retry + 1);
                    message.getMessageProperties().setDelay(1000);
                    amqpTemplate.convertAndSend("bocloud.flow.queue", message);
                } else {
                    logger.warn("This message has retry for 20 times drop it!");
                }
            }
        } catch (Exception e) {
            logger.error("operate message error:", e);
        }
    }

    /**
     * 处理任务类审批消息
     *
     * @param category
     * @param object
     * @param status
     */
    private void operate(String category, JSONObject object, String status) {
        if (BocloudStatus.Service.APPROVED.name().equals(status)) {// 审批通过
            switch (ApplyCategory.valueOf(category)) {
                case ApplicationOperate:
                    ApplicationParam param =
                            JSONObject.parseObject(object.getString("ApplyParams"), ApplicationParam.class);
                    ServiceOrder applyOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext applyContext =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    //处理审批后的订单逻辑
                    param.setContext(applyContext);
                    applyOrder.setApplyParam(JSONObject.toJSONString(param));
                    applyOrderHandle(applyOrder);
                    break;
                case AlterationOperate:
                    //变更审批后的参数
                    AlterationParam alteration =
                            JSONObject.parseObject(object.getString("ApplyParams"), AlterationParam.class);
                    ServiceOrder alterationOrder =
                            JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext context =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    //处理审批后的订单逻辑
                    AlterationParam oldParams =
                            JSONObject.parseObject(alterationOrder.getApplyParam(), AlterationParam.class);
                    alteration.setContext(context);
                    alteration.setExpiredTime(oldParams.getExpiredTime());
                    alterationOrderHandle(alterationOrder, alteration);
                    break;
                case ExtensionOperate:
                    ExtensionParam extensionParam =
                            JSONObject.parseObject(object.getString("ApplyParams"), ExtensionParam.class);
                    ServiceOrder extensionOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext extensionContext =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    //处理审批后的订单逻辑
                    extensionParam.setContext(extensionContext);
                    extensionOrderHandle(extensionOrder, extensionParam);
                    break;
                case UnsubscribeOperate:
                    CancelationParam cancelationParam =
                            JSONObject.parseObject(object.getString("ApplyParams"), CancelationParam.class);
                    ServiceOrder cancelationOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext cancelationContext =
                            JSONObject.parseObject(object.getString("ApplyContext"), RequestContext.class);
                    cancelationParam.setContext(cancelationContext);
                    unsubscribeOrderHandle(cancelationOrder, cancelationParam);
                    break;
                case ResourceComputerClone:
                    CloneParams config = JSONObject.parseObject(object.getString("ApplyParams"), CloneParams.class);
                    RequestContext cloneContext =
                            JSONObject.parseObject(object.getString("ApplyContext"), RequestContext.class);
                    Result cloneResult = resourceApplyService.clone(config, cloneContext);
                    if (cloneResult.isFailed()) {
                        NotifyMessage message =
                                new NotifyMessage(cloneContext, "服务实例【" + config.getInstanceName() + "】克隆失败通知",
                                        cloneResult.getMessage(), true, false);
                        amqpTemplate.convertAndSend("event.notice.queue", message);
                    }
                    break;
                default:
                    break;
            }
        } else if (BocloudStatus.Service.UNRATIFIED.name().equals(status)) {// 审批不通过
            Boolean isCancel = object.getBoolean("ApplyIsCancel");
            switch (ApplyCategory.valueOf(category)) {
                case ApplicationOperate:
                    ApplicationParam param =
                            JSONObject.parseObject(object.getString("ApplyParams"), ApplicationParam.class);
                    ServiceOrder order = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    order.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(order);
                    RequestContext context =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    List<OrderItem> items = orderItemRepository.listByOrderId(order.getId());
                    if (order.getServiceCode().startsWith("kvs.") || order.getServiceCode().startsWith("rds.") || order
                            .getServiceCode().startsWith("cms.")) {
                        //软件安装特殊处理 回调处理配额那边软件安装直回调一个消息处理
                        items.removeIf(orderItem -> !(orderItem.getServiceCode().startsWith("kvs.") || orderItem.getServiceCode()
                                .startsWith("rds.") || orderItem.getServiceCode().startsWith("cms.")));
                    }
                    /*List<QuotaBean> quotas = param.getQuotas();
                    for (OrderItem item : items) {
                        OrderMessage applyMessage = new OrderMessage(item.getSn(), OrderMessage.Operation.Application,
                                OrderItemStatus.OpenedFailure, null, "资源申请失败：" + ((isCancel != null && isCancel) ? "用户已取消" : "审批未通过"), null, null, null,
                                param.getConfigs().getString("category"), context);
                        applyMessage.setQuotas(quotas);
                        amqpTemplate.convertAndSend("resource.result.queue", applyMessage);
                    }*/

                    order.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(order);
                    String applyMsg = "订购失败：" + ((isCancel != null && isCancel) ? "用户已取消" : "审批未通过");
                    // 取消审批的订单
                    items.forEach(item -> {
                        item.setStatus(OrderItemStatus.OpenedFailure);
                        // item.setOrderId(null);
                        item.setErrorMsg(applyMsg);
                        orderItemRepository.update(item);
                    });

                    // 站内信
                    String subject = "服务实例【" + param.getConfigs().getString("name") + "】订购失败通知";
                    NotifyMessage orderMessage = new NotifyMessage(context, subject, applyMsg, true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", orderMessage);

                    // 弹窗通知
                    resourceEventPublisher.send(
                            new OperateResult(false, applyMsg, "resource." + param.getConfigs().getString("category").toLowerCase(), null,
                                    OperateResult.OperateCategory.ResourceEvent, context));

                    break;
                case AlterationOperate:
                    AlterationParam alteration =
                            JSONObject.parseObject(object.getString("ApplyParams"), AlterationParam.class);
                    ServiceOrder alterOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    alterOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(alterOrder);
                    RequestContext alterationContext =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    AlterationParam oldParams =
                            JSONObject.parseObject(alterOrder.getApplyParam(), AlterationParam.class);
                    alteration.setExpiredTime(oldParams.getExpiredTime());
                    String alterMsg = "变更失败：" + ((isCancel != null && isCancel) ? "用户已取消" : "审批未通过");
                    List<OrderItem> alterItems = orderItemRepository.listByOrderId(alterOrder.getId());
                    for (OrderItem alterItem : alterItems) {
                        alterItem.setStatus(OrderItemStatus.ChangeFailure);
                        alterItem.setErrorMsg(alterMsg);
                        orderItemRepository.update(alterItem);
                    }

                    // 站内信
                    String alterSubject = "服务实例【" + alteration.getConfigs().getString("name") + "】变更失败通知";
                    NotifyMessage alterMessage = new NotifyMessage(alterationContext, alterSubject, "资源申请失败：配额不足", true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", alterMessage);

                    // 弹窗通知
                    resourceEventPublisher.send(
                            new OperateResult(false, alterMsg, "resource." + alteration.getConfigs().getString("category").toLowerCase(), null,
                                    OperateResult.OperateCategory.ResourceEvent, alterationContext));
                    break;
                case ExtensionOperate:
                    ExtensionParam extension =
                            JSONObject.parseObject(object.getString("ApplyParams"), ExtensionParam.class);
                    ServiceOrder extensionOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext extensionContext =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    List<OrderItem> extensionItems = orderItemRepository.listByOrderId(extensionOrder.getId());
                    //批量更新订单项
                    extensionItems.forEach(orderItem -> {
                        orderItem.setStatus(OrderItemStatus.ExtensionFailure);
                        orderItem.setErrorMsg("延期失败：" + ((isCancel != null && isCancel) ? "用户已取消" : "审批未通过"));
                        orderItemRepository.update(orderItem);
                    });
                    extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(extensionOrder);
                    NotifyMessage message =
                            new NotifyMessage(extensionContext, "服务实例【" + extension.getInstanceName() + "】延期失败通知", (isCancel != null && isCancel) ? "用户已取消" : "审批未通过",
                                    true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", message);
                    break;
                case UnsubscribeOperate:
                    CancelationParam cancelationParam =
                            JSONObject.parseObject(object.getString("ApplyParams"), CancelationParam.class);
                    ServiceOrder cancelationOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext cancelationContext =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    List<OrderItem> cancelationItems = orderItemRepository.listByOrderId(cancelationOrder.getId());
                    //批量更新订单项
                    cancelationItems.forEach(orderItem -> {
                        orderItem.setStatus(OrderItemStatus.UnsubscribeFailure);
                        String errorMsg = "退订失败：审批未通过";
                        if (isCancel != null && isCancel) {
                            errorMsg = "退订失败：用户已取消";
                        }
                        orderItem.setErrorMsg(errorMsg);
                        orderItemRepository.update(orderItem);
                    });
                    cancelationOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(cancelationOrder);
                    NotifyMessage cancelationMessage =
                            new NotifyMessage(cancelationContext, "服务实例【" + cancelationParam.getInstanceName() + "】退订失败通知",
                                    (isCancel != null && isCancel) ? "用户已取消" : "审批未通过",
                                    true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", cancelationMessage);
                default:
                    break;
            }
        }
    }

    private void extensionOrderHandle(ServiceOrder extensionOrder, ExtensionParam extensionParam) {
        try {
            ExtensionParam param = JSONObject.parseObject(extensionOrder.getApplyParam(), ExtensionParam.class);
            //延期月份
            Integer extension = extensionParam.getExtension();
            DurationModel.Mode mode = DurationModel.Mode.Month;
            param.setExtension(extensionParam.getExtension());
            List<OrderItem> orderItems = orderItemRepository.listByOrderId(extensionOrder.getId());
            for (OrderItem orderItem : orderItems) {
                DurationModel duration = JSONObject.parseObject(orderItem.getDuration(), DurationModel.class);
                DurationModel durationModel = new DurationModel();
                mode = duration.getMode();
                durationModel.setMode(duration.getMode());
                durationModel.setAmount(extension);
                orderItem.setDuration(JSONObject.toJSONString(durationModel));
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(orderItem.getEffectiveTime());
                if (duration.getMode().equals(DurationModel.Mode.Month)) {
                    calendar.add(Calendar.MONTH, extension);
                } else {
                    calendar.add(Calendar.YEAR, extension);
                }
                orderItem.setExpiredTime(calendar.getTime());
                orderItem.setStatus(OrderItemStatus.Extensioning);
                orderItemRepository.update(orderItem);
            }
            if (mode.equals(DurationModel.Mode.Year)) {
                param.setExtension(extensionParam.getExtension() * 12);
            }
            extensionOrder.setApplyParam(JSONObject.toJSONString(param));
            extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            orderRepository.update(extensionOrder);
        } catch (Exception e) {
            logger.error("extension order handle error", e);
        }
    }

    private void alterationOrderHandle(ServiceOrder alterationOrder, AlterationParam params) {
        try {
            ResourceOperate alterationOperate = ResourceOperateFactory.getInstance(params.getService());
            //找之前待审批订单项 更新订单项
            List<OrderItem> auditItems = orderItemRepository.listAuditItems(alterationOrder.getId());
            auditItems.forEach(item -> {
                item.setStatus(OrderItemStatus.Canceld);
                item.setOrderId(null);
                orderItemRepository.update(item);
            });

            OrderItem oldItem = null;
            Map<String, Integer> oldQuota;
            ElementModel elementModelObj = params.getUpgrade().get(0);
            String serviceCode = elementModelObj.getServiceCode();
            // 1. 根据资源sn找到资源《订购》的订单
            ServiceOrder applyOrder = orderRepository.queryOrderBySnAndType(params.getInstance(), ChargeCommon.Type.apply.name());
            if (null == applyOrder) {
                throw new IllegalArgumentException("旧订单未找到");
            }
            // 2。根据serviceCode找到变更资源的sn
            List<OrderItem> orderItems = orderItemRepository.listByOrderId(applyOrder.getId(), serviceCode);
            if (null == orderItems || orderItems.isEmpty()) {
                throw new IllegalArgumentException("旧订单项未找到");
            }

            OrderItem applyItem = orderItems.get(0);
            List<ElementModel> applyElementModel = JSON.parseArray(applyItem.getElements(), ElementModel.class);
            Integer count = 1;
            if (null != applyElementModel && !applyElementModel.isEmpty()) {
                ElementModel elementModel = applyElementModel.get(0);
                count = elementModel.getCount();
            }
            // 3. 根据变更资源的sn找到最新有效的变更资源的订单项
            // 3.1 最近订单项
            oldItem = orderItemRepository.queryLastSuccessBySn(applyItem.getSn());
            if (null == oldItem) {
                throw new IllegalArgumentException("最近订单项未找到");
            }
            // 3.2 最近订单
            ServiceOrder oldOrder = orderRepository.query(oldItem.getOrderId());
            if (null == oldOrder) {
                throw new IllegalArgumentException("最近订单未找到");
            }
            // 4。生成旧配额
            oldQuota = OrderItemUtils.getItemQuota(Lists.newArrayList(oldItem));

            //生成新的订单项
            List<OrderItem> newAlterationItems = alterationOperate.buildAlterationItem(params.getService(), params.getInstance(),
                    params.getOperation(), params.getUpgrade(), params.getExpiredTime());
            Tenant tenant = tenantService.detail(alterationOrder.getTenantId());
            Assert.notNull(tenant, "租户信息不存在");
            // 订单项设置折扣
            for (OrderItem newAlterationItem : newAlterationItems) {
                List<ElementModel> elementModelList =
                        JSONArray.parseArray(newAlterationItem.getElements(), ElementModel.class);
                Integer finalCount = count;
                elementModelList.forEach(elementModel -> {
                    elementModel.setRatio(tenant.getDiscount());
                    elementModel.setCount(finalCount);
                });
                newAlterationItem.setElements(JSONObject.toJSONString(elementModelList));
                newAlterationItem.setFinalPrice(newAlterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                newAlterationItem.setOrderId(alterationOrder.getId());
            }

            //配额处理
            Map<String, Integer> alterationQuota = OrderItemUtils.getItemQuota(newAlterationItems);
            List<QuotaBean> quotaBeans = new ArrayList<>();
            // 变更操作 配额差值
            // 变更后订单项的配额

            logger.info("变更后订单项的配额: {}", JSON.toJSONString(alterationQuota));
            logger.info("变更前订单项得配额: {}", JSON.toJSONString(oldQuota));
            for (Map.Entry<String, Integer> entry : alterationQuota.entrySet()) {
                String quotaCode = entry.getKey();
                // 旧配额
                Integer number = Optional.ofNullable(oldQuota.get(quotaCode)).orElse(0);
                // 新配额和旧配额得差值
                quotaBeans.add(new QuotaBean(quotaCode, alterationQuota.get(quotaCode) - number));
                alterationQuota.put(quotaCode, alterationQuota.get(quotaCode) - number);
            }
            // 校验配额并更新配额
            logger.info("校验配额并且更新配额的量：" + JSON.toJSONString(alterationQuota));
            Result result = handleQuota(alterationQuota, tenant.getId());

            //操作产生的配额
            params.setQuotas(quotaBeans);
            if (result.isFailed()) {
                // 配额不足订单取消
                alterationOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                orderRepository.update(alterationOrder);
                RequestContext alterationContext = params.getContext();
                AlterationParam oldParams =
                        JSONObject.parseObject(alterationOrder.getApplyParam(), AlterationParam.class);
                params.setExpiredTime(oldParams.getExpiredTime());

                String msg = "变更失败：配额不足";
                List<OrderItem> alterItems = orderItemRepository.listByOrderId(alterationOrder.getId());
                for (OrderItem alterItem : alterItems) {
                    alterItem.setStatus(OrderItemStatus.ChangeFailure);
                    alterItem.setErrorMsg(msg);
                    orderItemRepository.update(alterItem);
                }
                // 通知用户
                String subject = "服务实例【" + params.getInstanceName() + "】变更失败通知";
                NotifyMessage message = new NotifyMessage(alterationContext, subject, msg, true, false);
                amqpTemplate.convertAndSend("event.notice.queue", message);
            }
            Assert.isTrue(result.isSuccess(), result.getMessage());
            switch (params.getOperation()) {
                case Upgrade:
                    if (alterationOperate instanceof PaasResourceOperate) {
                        for (OrderItem alterationItem : newAlterationItems) {
                            List<ElementModel> elementModels =
                                    JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                            if ("vmware.storage.disk".equals(alterationItem.getServiceCode())) {
                                ElementModel diskElement = elementModels.get(0);
                                List<SpecModel> diskSpecModel = diskElement.getSpecs();
                                Integer disk =
                                        diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                                .reduce(Integer::sum).orElse(0);
                                params.setDataDisk(disk);
                            } else {
                                ElementModel upgradeElement = elementModels.get(0);
                                List<SpecModel> specs = upgradeElement.getSpecs();
                                Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                                        .reduce(Integer::sum).orElse(0);
                                BigDecimal memory = specs.stream()
                                        .map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                        .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
//                                Integer disk = specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
//                                        .reduce(Integer::sum).orElse(0);
                                params.setUpgradeSkuId(upgradeElement.getSkuId());
                                params.setCpu(cpu);
                                params.setMemory(memory);
//                                params.setSystemDisk(disk);
                            }
                            alterationItem.setElements(JSONObject.toJSONString(elementModels));
                            alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                            alterationItem.setOrderId(alterationOrder.getId());
                        }
                    } else {
                        // 获取升级规格 设置升级后的cpu内存大小到变更参数里面
                        OrderItem newAlterationItem = newAlterationItems.get(0);
                        List<ElementModel> elementModels =
                                JSONArray.parseArray(newAlterationItem.getElements(), ElementModel.class);
                        ElementModel element = elementModels.get(0);
                        List<SpecModel> specs = element.getSpecs();
                        Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                                .reduce(Integer::sum).orElse(0);
                        BigDecimal memory =
                                specs.stream().map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                        .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                        params.setUpgradeSkuId(element.getSkuId());
                        params.setCpu(cpu);
                        params.setMemory(memory);
                        Integer systemDisk = specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                .reduce(Integer::sum).orElse(0);
                        if (!systemDisk.equals(0)) {
                            params.setSystemDisk(systemDisk);
                        }
                    }
                    break;
                case EditDisk:
                    OrderItem editItem = newAlterationItems.get(0);
                    List<ElementModel> editDiskElements =
                            JSONArray.parseArray(editItem.getElements(), ElementModel.class);
                    editDiskElements.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                    editItem.setElements(JSONObject.toJSONString(editDiskElements));
                    editItem.setFinalPrice(editItem.getBasicPrice().multiply(tenant.getDiscount()));
                    editItem.setOrderId(alterationOrder.getId());
                    break;
                case AddDisk:
                    int diskTotal = 0;
                    for (OrderItem alterationItem : newAlterationItems) {
                        List<ElementModel> diskElements =
                                JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                        ElementModel diskElement = diskElements.get(0);
                        List<SpecModel> diskSpecModel = diskElement.getSpecs();
                        Integer disk = diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                .reduce(Integer::sum).orElse(0);
                        diskTotal += disk;
                        // 订单项设置折扣
                        diskElements.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                        alterationItem.setElements(JSONObject.toJSONString(diskElements));
                        alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                        alterationItem.setOrderId(alterationOrder.getId());
                    }
                    params.setDataDisk(diskTotal);
                default:
                    break;
            }
            params.setCategory(ServiceResourceMapping.getResourceCategory(params.getService()).name());
            params.setOldStatus(oldItem.getStatus());
            // 保存新订单
            orderItemRepository.batchSave(newAlterationItems, newAlterationItems.size());
            params.setNewInstance(newAlterationItems.get(0).getSn());
            params.setOrderSn(alterationOrder.getSn());
            alterationOrder.setApplyParam(JSONObject.toJSONString(params));
            // 预付费生成未支付账单
            if (ChargeCommon.ChargeMode.PreCharge == alterationOrder.getChargeMode()) {
                alterationOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
                orderRepository.update(alterationOrder);
            } else {
                alterationOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
                orderRepository.update(alterationOrder);
                if (alterationOperate instanceof PaasResourceOperate) {
                    amqpTemplate.convertAndSend("paas.resource.modification.queue", params);
                } else {
                    amqpTemplate.convertAndSend("iaas.resource.modification.queue", params);
                }
            }
        } catch (Exception e) {
            logger.error("alteration order handle error", e);
        }
    }

    /**
     * 申请订单审批处理(生成新订单项，配额更新)
     *
     * @param order
     */
    private void applyOrderHandle(ServiceOrder order) {
        try {
            ApplicationParam param = JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class);
            ResourceOperate applyOperate = ResourceOperateFactory.getInstance(order.getServiceCode());
            CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
            //找待审批订单（有流程的话就有待审批订单，需要更新订单项）
            List<OrderItem> auditItems = orderItemRepository.listAuditItems(order.getId());
            if (!auditItems.isEmpty()) {
                //生成新订单项
                List<OrderItem> items = applyOperate.buildApplyItem(order.getServiceCode(), param.getEmption(), param.getElements());
                // 扣除配额
                Map<String, Integer> newQuota = OrderItemUtils.getItemQuota(items);
                Result result = handleQuota(newQuota, order.getTenantId());
                // 配额不足，取消订单，通知用户
                if (result.isFailed()) {
                    order.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(order);
                    RequestContext context = param.getContext();
                    String msg = "变更失败：配额不足";
                    // 取消审批的订单
                    auditItems.forEach(item -> {
                        item.setStatus(OrderItemStatus.OpenedFailure);
                        // item.setOrderId(null);
                        item.setErrorMsg(msg);
                        orderItemRepository.update(item);
                    });

                    String subject = "服务实例【" + service.getName() + "】订购失败通知";
                    NotifyMessage message = new NotifyMessage(context, subject, "资源申请失败：配额不足", true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", message);
                }
                // 配额校验失败，直接返回
                Assert.isTrue(result.isSuccess(), result.getMessage());
                Assert.isTrue(!items.isEmpty(), "生成订单内容错误");
                Tenant tenant = tenantService.detail(order.getTenantId());
                Assert.notNull(tenant, "租户信息不存在");
                if (!ListTool.isEmpty(items)) {
                    for (OrderItem item : items) {
                        item.setOrderId(order.getId());
                        List<ElementModel> elementModels = JSONArray.parseArray(item.getElements(), ElementModel.class);
                        if (elementModels.isEmpty()) {
                            ElementModel elementModel = new ElementModel();
                            elementModel.setRatio(tenant.getDiscount());
                            elementModels = Lists.newArrayList(elementModel);
                        } else {
                            elementModels.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                        }
                        item.setElements(JSONObject.toJSONString(elementModels));
                        item.setFinalPrice(item.getBasicPrice().multiply(tenant.getDiscount()));
                        orderItemRepository.save(item);
                    }
                    auditItems.forEach(item -> {
                        item.setStatus(OrderItemStatus.Canceld);
                        item.setOrderId(null);
                        orderItemRepository.update(item);
                    });

                }
                order.setItems(items);
            }
            ApplicationParam applicationParam = JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class);
            JSONObject configs = applicationParam.getConfigs();
            if (null == configs) {
                configs = new JSONObject();
            }
            Long skuId = null;
            ServiceSku sku = null;
            for (ElementModel element : applicationParam.getElements()) {
                if (service.getCode().equals(element.getServiceCode())) {
                    skuId = element.getSkuId();
                    sku = serviceSkuRepository.query(skuId);
                    break;
                }
            }
            ServiceCategory category = null;
            configs.put("skuId", skuId);
            if (null != sku) {
                configs.put("skuName", sku.getName());
                category = serviceCategoryRepository.query(sku.getCategoryId());
            }
            if (null != category) {
                configs.put("categoryId", category.getId());
                configs.put("categoryCode", category.getCode());
                configs.put("categoryProps", category.getProps());
            }
            configs.put("category", ServiceResourceMapping.getResourceCategory(order.getServiceCode()).name());
            if (StringUtils.isNotEmpty(service.getTemplateName())) {
                configs.put("templateName", service.getTemplateName());
            }
            applicationParam.setService(service.getCode());
            applicationParam.setConfigs(configs);
            if (ChargeCommon.ChargeMode.PostCharge == order.getChargeMode()) {
                order.setOrderStatus(ChargeCommon.OrderStatus.paid);
            } else {
                order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            }
            applicationParam.setOrder(order);
            //年计费下发到底层服务特殊处理
            EmptionModel emption = applicationParam.getEmption();
            DurationModel duration = emption.getDuration();
            DurationModel.Mode mode = duration.getMode();
            //下发到底层服务的时候按年计费时间做特殊处理
            if (mode == DurationModel.Mode.Year) {
                EmptionModel paramEmption = applicationParam.getEmption();
                DurationModel dura = paramEmption.getDuration();
                dura.setAmount(dura.getAmount() * 12);
                dura.setMode(DurationModel.Mode.Month);
                paramEmption.setDuration(dura);
                applicationParam.setEmption(paramEmption);
            }
            order.setApplyParam(JSONObject.toJSONString(applicationParam));
            //后付费
            if (ChargeCommon.ChargeMode.PostCharge == order.getChargeMode()) {
                order.setOrderStatus(ChargeCommon.OrderStatus.paid);
                orderRepository.update(order);
                if ("vmware.nsx.vpc".equalsIgnoreCase(service.getCode())) {
                    logger.info("后付费NSX-T Vpc申请参数结构：" + JSONObject.toJSONString(applicationParam));
                    amqpTemplate.convertAndSend("network.creation.queue", applicationParam);
                } else {
                    List<ApplicationParam> messageParams = Lists.newArrayList(applicationParam);
                    logger.info("后付费资源申请参数结构：" + JSONObject.toJSONString(messageParams));
                    if (applyOperate instanceof PaasResourceOperate) {
                        amqpTemplate.convertAndSend("paas.resource.creation.queue", messageParams);
                    } else if (applyOperate instanceof IaasResourceOperate) {
                        amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                    } else {
                        amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                    }
                }
            } else {
                order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
                logger.info("====> Update service_order param: {}", JSON.toJSONString(order));
                orderRepository.update(order);
            }
        } catch (Exception e) {
            logger.error("apply order handle error", e);
        }
    }

    /**
     * 退订处理
     *
     * @param cancelationOrder
     * @param cancelationParam
     */
    private void unsubscribeOrderHandle(ServiceOrder cancelationOrder, CancelationParam cancelationParam) {
        try {
            ResourceOperate cancelationOperate = ResourceOperateFactory.getInstance(cancelationOrder.getServiceCode());
            cancelationOrder.setApplyParam(JSONObject.toJSONString(cancelationParam));
            cancelationOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
            orderRepository.update(cancelationOrder);
            List<OrderItem> extensionItems = orderItemRepository.listByOrderId(cancelationOrder.getId());
            for (OrderItem orderItem : extensionItems) {
                orderItem.setStatus(OrderItemStatus.Unsubscribing);
                orderItemRepository.update(orderItem);
            }
            if (cancelationOperate instanceof PaasResourceOperate) {
                amqpTemplate.convertAndSend("paas.resource.deletion.queue", cancelationParam);
            } else {
                if ("vmware.nsx.vpc".equals(cancelationOrder.getServiceCode())) {
                    amqpTemplate.convertAndSend("network.resource.deletion.queue", cancelationParam);
                } else {
                    amqpTemplate.convertAndSend("iaas.resource.deletion.queue", cancelationParam);
                }
            }
        } catch (Exception e) {
            logger.error("cancelation order handle error", e);
            String subject = "服务实例【" + cancelationParam.getInstanceName() + "】退订失败通知";
            NotifyMessage message = new NotifyMessage(cancelationParam.getContext(), subject, e.getMessage(), true, false);
            amqpTemplate.convertAndSend("event.notice.queue", message);
        }
    }

    /**
     * 调用boc
     * http://10.10.5.23:30022/boclouddeployerapi/cmpCluster/v2.4/createCluster
     *
     * @param callback
     * @param result
     * @return
     */
    private Result callBoc(String callback, JSONObject params, String result, String message, RequestContext context) {
        String token = socService.getToken(context.getTarget(), context.getCatalog().name());
        logger.info("获取token：{}", token);
        HttpClient httpClient = new HttpClient(20 * 1000 * 1000, PostDataFormat.RAW);
        try {
            params.put("result", result);
            params.put("callback", callback);
            params.put("message", message);
            Map<String, Object> header = new HashMap<>();
            header.put("accept", "application/json");
            header.put("content-type", "application/json");
            if (StringUtils.isNotEmpty(token)) {
                header.put("token", token);
            }
            String url = configService.getByCode("k8sAddress") + "/boclouddeployerapi/cmpCluster/v2.4/createVM";
            logger.info("调用boc地址：{}", url);
            logger.info("调用boc参数：{}", params.toJSONString());
            return httpClient.post(header, MapTools.simpleMap("vmInfo", params), url);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return Result.FAILED("回调容器平台失败");
        } finally {
            httpClient.close();
        }
    }

    private Result callBoc(String callback, JSONObject params, String bocUrl, String result, String message,
                           RequestContext context) {
        HttpClient httpClient = new HttpClient(20 * 1000 * 1000, PostDataFormat.RAW);
        String token = socService.getToken(context.getTarget(), context.getCatalog().name());
        logger.info("获取token：{}", token);
        try {
            params.put("callback", callback);
            params.put("result", result);
            params.put("message", message);
            Map<String, Object> header = new HashMap<>();
            header.put("accept", "application/json");
            header.put("content-type", "application/json");
            if (StringUtils.isNotEmpty(token)) {
                header.put("token", token);
            }
            String url = configService.getByCode("k8sAddress") + bocUrl;
            logger.info("调用boc地址：{}", url);
            logger.info("调用boc参数：{}", params.toJSONString());
            return httpClient.post(header, params, url);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return Result.FAILED("回调容器平台失败");
        } finally {
            httpClient.close();
        }
    }

    private Result handleQuota(Map<String, Integer> quotaMap, Long tenantId) {
        Tenant tenant = tenantService.detail(tenantId);
        if (!tenant.getQuotaAble()) {
            return new Result(true, "租户未开启配额,不需要计算配额");
        }
        if (null == quotaMap || quotaMap.isEmpty()) {
            return new Result(true, "无配额信息需更新");
        }
        List<ResourceQuota> list = resourceQuotaRepository.listByTenantId(tenantId);
        Assert.notEmpty(list, "未查询到配额信息");
        Map<String, ResourceQuota> map = new HashMap<>(8);
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (Map.Entry<String, Integer> entry : quotaMap.entrySet()) {
            String code = entry.getKey();
            if (!"vmware.storage.disk".equals(code)) {
                String quotaName;
                if (QuotaCategory.CPU.getCode().equals(code)) {
                    quotaName = QuotaCategory.CPU.getName();
                } else if (QuotaCategory.MEM.getCode().equals(code)) {
                    quotaName = QuotaCategory.MEM.getName();
                } else if (QuotaCategory.DISK.getCode().equals(code)) {
                    quotaName = QuotaCategory.DISK.getName();
                } else {
                    CloudService cloudService = cloudServiceRepository.queryByCode(code);
                    quotaName = cloudService.getName();
                }
                if (!map.containsKey(entry.getKey())) {
                    return new Result(false, "未查询到" + quotaName + "配额信息");
                }
                ResourceQuota quota = map.get(entry.getKey());
                if (entry.getValue() + quota.getUsed() > quota.getQuota()) {
                    return new Result(false, "剩余" + quotaName + "配额不足");
                }
                quota.setQuotaActionValue(entry.getValue().longValue());
                quotas.add(quota);
            }
        }
        Result result = resourceQuotaService.quotaAction(quotas, ResourceQuota.Action.Order, tenantId);
        if (result.isFailed()) {
            return result;
        }
        return new Result(true, "更新配额信息成功");
    }

}
