package com.bocloud.cos.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.Result;
import com.bocloud.cos.intf.ResourceApplicationService;
import com.bocloud.cos.model.ApplicationParam;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 资源编排资源申请队列
 *
 * @author weiwei
 */

@Component
public class RosApplicationListener {
    private static final Logger logger = LoggerFactory.getLogger(RosApplicationListener.class);

    @Autowired
    private ResourceApplicationService resourceApplyService;
    @Autowired
    private AmqpTemplate amqpTemplate;

    /**
     * 将云平台同步记保存到数据库
     *
     * @param param
     */
    @RabbitListener(queues = "ros.application.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void saveRecord(ApplicationParam param) {
        if (null == param) {
            logger.warn("ros application param is null!");
            return;
        }
        JSONObject configs = param.getConfigs();
        configs.put("noFlow", true);
        Result result = resourceApplyService.process(param, param.getContext());
        if (result.isFailed()) {
            if (StringUtils.isNotEmpty(param.getCallback())) {
                amqpTemplate.convertAndSend(param.getCallback(), JSONObject.toJSON(result));
            }
        }
    }

}
