package com.bocloud.cos.booter.scheduler;

import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.enums.ChargeCommon.ChargeMode;
import com.bocloud.cos.enums.ChargeCommon.OrderStatus;
import com.bocloud.cos.intf.ServiceOrderService;
import com.bocloud.cos.repository.ServiceOrderRepository;
import com.bocloud.service.core.CurrentService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 支付定时任务，用来处理超过48小时未支付的订单
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月29日
 */
@Component
public class PaymentScheduler {

    private static Logger logger = LoggerFactory.getLogger(PaymentScheduler.class);
    @Autowired
    private ServiceOrderService orderService;
    @Autowired
    private ServiceOrderRepository orderRepository;
    @Autowired
    private CurrentService currentService;

    @Scheduled(cron = "0 0/10 * * * ?")
    protected void monitor() {
        if (!currentService.isLeader()) {
            return;
        }
        try {
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("orderStatus", OrderStatus.unpay.name()), Sign.EQ));
            List<ServiceOrder> upayOrders = orderRepository.list(1, Integer.MAX_VALUE, new HashMap<>(), params);
            for (ServiceOrder upayOrder : upayOrders) {
                if (ChargeMode.PostCharge == upayOrder.getChargeMode()) {
                    continue;
                }
                // 下单时间
                Date gmtCreate = upayOrder.getGmtCreate();
                if (null == gmtCreate) {
                    continue;
                }
                Date currentTime = new Date();
                // 未支付超过48小时自动取消支付
                if (currentTime.getTime() - gmtCreate.getTime() > 48 * 60 * 60 * 1000) {
                    Catalog catalog = Catalog.Tenant;
                    Long target = upayOrder.getTenantId();
                    if (null != upayOrder.getUserId()) {
                        catalog = Catalog.User;
                        target = upayOrder.getUserId();
                    }
                    orderService.cancel(upayOrder.getSn(),
                            new RequestContext(catalog, target, upayOrder.getTenantId(), null, null));
                }
            }
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
        }
    }
}
