package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 账户操作记录实体
 *
 * @author tyl
 * @time 2020/7/14 周二 19:35
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("account_record")
public class AccountRecord extends Generic {
    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 金额
     */
    @Column("money")
    private BigDecimal money;
    /**
     * 租户id
     */
    @Column("tenant_id")
    private Long tenantId;
    @IgnoreAll
    private String tenantName;
    /**
     * 账户操作
     */
    @Column("action")
    private TenantAccount.Action action;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 修改者id
     */
    @Column("mender_id")
    private Long menderId;

    /**
     * 账单明细id
     */
    @Column("bill_id")
    private Long billId;

    public AccountRecord(BigDecimal money, Long tenantId, TenantAccount.Action action, Date gmtCreate, Long menderId) {
        this.money = money;
        this.tenantId = tenantId;
        this.action = action;
        this.gmtCreate = gmtCreate;
        this.menderId = menderId;
    }

    public AccountRecord(BigDecimal money, Long tenantId, TenantAccount.Action action, Date gmtCreate, Long menderId, Long billId) {
        this.money = money;
        this.tenantId = tenantId;
        this.action = action;
        this.gmtCreate = gmtCreate;
        this.menderId = menderId;
        this.billId = billId;
    }
}
