package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 账单明细
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@NoArgsConstructor
@AllArgsConstructor
@Data
@EqualsAndHashCode(callSuper = true)
@Table("bill_item")
public class BillItem extends Generic {

    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;

    /**
     * 账单id
     */
    @Column("bill_id")
    private Long billId;

    /**
     * 订单明细id
     */
    @Column("item_id")
    private Long itemId;

    /**
     * 商品规格id
     */
    @Column("sku_id")
    private Long skuId;

    /**
     * 计费模式
     */
    @Column("charge_mode")
    private ChargeCommon.ChargeMode chargeMode;

    /**
     * 资源名称
     */
    @Column("resource")
    private String resource;

    /**
     * 账单金额
     */
    @Column("price")
    private BigDecimal price;

    /**
     * 账单开始时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("bill_start")
    private Date billStart;

    /**
     * 账单结束时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("bill_end")
    private Date billEnd;

    /**
     * 账单金额
     */
    @Column("money")
    private BigDecimal money;

    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;

    /**
     * 账单类别
     */
    @Column("catalog")
    private Catalog catalog;

    /**
     * 相关账单id
     */
    @Column("related_item_id")
    private Long relatedItemId;

    /**
     * 服务名称
     */
    @IgnoreAll
    private String serviceName;

    @Column("user_id")
    private Long userId;

    @Column("tenant_id")
    private Long tenantId;

    @Column("project_id")
    private Long projectId;

    @Column("business_id")
    private Long businessId;

    @Column("service_code")
    private String serviceCode;

    @Column("instance")
    private String instance;

    public enum Catalog {
        /**
         * 订购
         */
        standard,
        /**
         * 变更
         */
        alteration,
        /**
         * 延期
         */
        extension,
        /**
         * 退订
         */
        unsubscribe
    }
}
