package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.*;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 扣费项
 *
 * @author tyl
 * @since 2021/5/25 10:51
 */
@Data
@EqualsAndHashCode(callSuper = true)
@AllArgsConstructor
@NoArgsConstructor
@Table("charge_item")
public class ChargeItem extends Generic {

    /**
     * id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;

    /**
     * 子订单编号（和order_item的sn及资源的uuid对应）
     */
    @Column("item_sn")
    private String itemSn;


    /**
     * 产品规格id
     */
    @Column("item_id")
    private Long itemId;

    /**
     * 价格(单次付费的时候为总价 跑批扣费项的时候为每小时价格)
     */
    @Column("price")
    private BigDecimal price;

    /**
     * 计费类型
     *
     * @see Type
     */
    @Column("type")
    private Type type;

    @Column("is_deleted")
    private Boolean deleted;

    @Column("service_code")
    private String serviceCode;

    /**
     * 创建时间
     */
    @Setter
    @Getter
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;


    public enum Type {
        /**
         * 跑批循环计费（后付费的时候按小时计价 一天汇总一次账单）
         */
        loop,
        /**
         * 单次计费（包年包月的时候一次记录）
         */
        once,
    }
}
