package com.bocloud.cos.entity;

import com.bocloud.cos.model.ServiceFlowBean;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreAll;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.GenericEntity;
import com.bocloud.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

import java.math.BigDecimal;
import java.util.List;

/**
 * 服务
 *
 * @author Administrator
 */
@Table("cloud_service")
@Data
@EqualsAndHashCode(callSuper = true)
@AllArgsConstructor
@NoArgsConstructor
public class CloudService extends GenericEntity {

    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 产品编号
     */
    @Column("sn")
    private String sn;

    /**
     * 服务目录id
     */
    @Column("catalog_id")
    private Long catalogId;

    @IgnoreAll
    private String catalogName;
    /**
     * 服务编码
     */
    @Column("code")
    private String code;

    /**
     * 图标id
     */
    @Column("icon_id")
    private Long iconId;

    /**
     * 是否是公有云服务
     */
    @Column("is_public")
    private Boolean isPublic;

    /**
     * 模板名称
     */
    @Column("template_name")
    private String templateName;
    /**
     * 平台类型
     */
    @Column("vendor_type")
    private String vendorType;

    /**
     * 折扣
     */
    @Column("discount")
    private BigDecimal discount;

    /**
     * 是否计费
     */
    @Column("billable")
    private Boolean billable;

    @Column("bill_policy")
    private BillPolicy billPolicy;

    /**
     * 基础定价
     */
    @Column("basic_price")
    private String basicPrice;

    /**
     * 是否内置
     */
    @Column("is_reserved")
    private Boolean reserved;

    /**
     * 任务模板id
     */
    @Column("template_id")
    private Long templateId;

    /**
     * 单位,当没有单位时则表示不需要配额
     */
    @Column("unit")
    private String unit;

    @IgnoreAll
    private Long tenantId;
    @IgnoreAll
    private String icon;
    /**
     * 排序字段
     */
    @IgnoreAll
    private Integer priority;
    /**
     * 当前配额数量
     */
    @IgnoreAll
    private Long quota;
    /**
     * 最小配额数量
     */
    @IgnoreAll
    private Long minQuota;
    /**
     * 最大配额数量
     */
    @IgnoreAll
    private Long maxQuota;
    /**
     * 能否接触授权
     */
    @IgnoreAll
    private Boolean releasable;
    /**
     * 是否已经授权
     */
    @IgnoreAll
    private Boolean check;
    @IgnoreAll
    private Long slaId;
    @IgnoreAll
    private Boolean hasCategory;
    /**
     * 服务绑定的流程
     */
    @IgnoreAll
    private List<ServiceFlowBean> flowList;

    /**
     * 服务状态枚举
     */
    public enum Status {
        /**
         * 上架
         */
        PUBLISHED,
        /**
         * 下架
         */
        UNPUBLISHED,
    }

    /**
     * 计费策略枚举
     */
    public enum BillPolicy {
        /**
         * 灵活计费
         */
        agility,
        /**
         * 规格计费
         */
        spec
    }
}
