package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 问题沟通记录
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月8日
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("interactive_record")
public class InteractiveRecord extends Generic {

    // ID
    @PK(PKStrategy.AUTO)
    private Long id;
    // 评论
    @Column("content")
    private String content;
    // 工单ID
    @Column("order_id")
    private Long orderId;
    // 发送人
    @Column("user_id")
    private Long userId;
    // 租户ID
    @Column("tenant_id")
    private Long tenantId;
    // 账号类型
    @Column("catalog")
    private String catalog;
    // 发送时间
    @IgnoreUpdate
    @Column("gmt_create")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;
    // 附件
    @Column("file")
    private String file;
    @IgnoreAll
    private String userName;
    @IgnoreAll
    private String portrait;

    /**
     * @param content
     * @param orderId
     * @param userId
     * @param tenantId
     * @param gmtCreate
     */
    public InteractiveRecord(String content, Long orderId, Long userId, Long tenantId, Date gmtCreate, String catalog) {
        super();
        this.content = content;
        this.orderId = orderId;
        this.userId = userId;
        this.tenantId = tenantId;
        this.gmtCreate = gmtCreate;
        this.catalog = catalog;
    }
}
