package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 支付记录
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@NoArgsConstructor
@AllArgsConstructor
@Table("payment")
public class Payment extends Generic {

    /**
     * id
     */
    @Setter
    @Getter
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 账单id
     */
    @Setter
    @Getter
    @Column("bill_id")
    private Long billId;
    /**
     * 流水号
     */
    @Setter
    @Getter
    @Column("sn")
    private String sn;
    /**
     * 支付渠道:alipay,xxx
     */
    @Setter
    @Getter
    @Column("channel")
    private String channel;
    /**
     * 账户
     */
    @Setter
    @Getter
    @Column("account")
    private String account;
    /**
     * 支付金额
     */
    @Setter
    @Getter
    @Column("money")
    private BigDecimal money;
    /**
     * 支付状态
     */
    @Setter
    @Getter
    @Column("status")
    private String status;
    /**
     * 用户id
     */
    @Setter
    @Getter
    @Column("user_id")
    private Long userId;
    /**
     * 租户id
     */
    @Setter
    @Getter
    @Column("tenant_id")
    private Long tenantId;
    /**
     * 开始时间
     */
    @Setter
    @Getter
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 支付成功时间
     */
    @Setter
    @Getter
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 用户名称
     */
    @Setter
    @Getter
    @IgnoreAll
    private String userName;
    /**
     * 租户名称
     */
    @Setter
    @Getter
    @IgnoreAll
    private String tenantName;

}
