package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 账单
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月11日
 */
@NoArgsConstructor
@AllArgsConstructor
@Table("service_bill")
public class ServiceBill extends Generic {

    /**
     * id
     */
    @Setter
    @Getter
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 流水号
     */
    @Setter
    @Getter
    @Column("sn")
    private String sn;
    /**
     * 订单id
     */
    @Setter
    @Getter
    @Column("order_id")
    private Long orderId;
    /**
     * 状态
     */
    @Setter
    @Getter
    @Column("status")
    private ChargeCommon.BillStatus status;
    /**
     * 账单金额
     */
    @Setter
    @Getter
    @Column("money")
    private BigDecimal money;
    /**
     * 优惠金额
     */
    @Setter
    @Getter
    @Column("discount_money")
    private BigDecimal discountMoney;
    /**
     * 租户id
     */
    @Setter
    @Getter
    @Column("user_id")
    private Long userId;

    @Setter
    @Getter
    @IgnoreAll
    private String userName;
    /**
     * 项目id
     */
    @Setter
    @Getter
    @Column("project_id")
    private Long projectId;
    /**
     * 业务ID
     */
    @Setter
    @Getter
    @Column("business_id")
    private Long businessId;
    /**
     * 租户id
     */
    @Setter
    @Getter
    @Column("tenant_id")
    private Long tenantId;
    /**
     * 账期
     */
    @Setter
    @Getter
    @Column("month")
    private String month;
    /**
     * 创建时间
     */
    @Setter
    @Getter
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 预付费变更或者延期的参数
     */
    @Setter
    @Getter
    @Column("params")
    private String params;
    /**
     * 订单明细
     */
    @Setter
    @Getter
    @IgnoreAll
    private List<com.bocloud.cos.entity.BillItem> items;
    /**
     * 租户名称
     */
    @Setter
    @Getter
    @IgnoreAll
    private String tenantName;

    /**
     * 资源名称
     */
    @Setter
    @Getter
    @Column("resource_name")
    private String resourceName;
    /**
     * 资源类型
     */
    @Setter
    @Getter
    @IgnoreAll
    private String serviceName;

    /**
     * 业务类型
     */
    @Setter
    @Getter
    @IgnoreAll
    private String businessName;

    /**
     * 资源编码
     */
    @Setter
    @Getter
    @IgnoreAll
    private String serviceCode;
    /**
     * 平台名称
     */
    @Setter
    @Getter
    @IgnoreAll
    private String vendorName;
    @Setter
    @Getter
    @IgnoreAll
    private String projectName;
    /**
     * 平台类型
     */
    @Setter
    @Getter
    @IgnoreAll
    private String vendorType;
    /**
     * 平台id
     */
    @Setter
    @Getter
    @IgnoreAll
    private Long vendorId;
    /**
     * 计费类型
     */
    @Setter
    @Getter
    @IgnoreAll
    private String chargeMode;
    /**
     * 账单类型
     */
    @Setter
    @Getter
    @Column("bill_type")
    private String billType;
    /**
     * 订单号
     */
    @Setter
    @Getter
    @IgnoreAll
    private String orderSn;
    /**
     * 支付金额
     */
    @Setter
    @Getter
    @IgnoreAll
    private BigDecimal payMoney;

    /**
     * @param sn
     * @param orderId
     * @param status
     * @param money
     * @param tenantId
     * @param month
     * @param gmtCreate
     */
    public ServiceBill(String sn, Long orderId, ChargeCommon.BillStatus status, BigDecimal money, Long tenantId,
                       String month, Date gmtCreate) {
        super();
        this.sn = sn;
        this.orderId = orderId;
        this.status = status;
        this.money = money;
        this.tenantId = tenantId;
        this.month = month;
        this.gmtCreate = gmtCreate;
    }

}
