package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 订单
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@NoArgsConstructor
@AllArgsConstructor
@Data
@Table("service_order")
public class ServiceOrder extends Generic {

    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 订单编号
     */
    @Column("sn")
    private String sn;
    /**
     * 收费模式：PostCharge,PreCharge
     */
    @Column("charge_mode")
    private ChargeCommon.ChargeMode chargeMode;
    /**
     * 是否接受SLA
     */
    @Column("sla_agreed")
    private Boolean slaAgreed;
    /**
     * 资源状态:creating,ready,failed,exception
     */
    @Column("resource_status")
    private ChargeCommon.ResourceStatus resourceStatus;
    /**
     * 订单状态：unpay[未支付],paid【已支付】,canceld【已取消】
     */
    @Column("order_status")
    private ChargeCommon.OrderStatus orderStatus;

    /**
     * 订单类型
     */
    @Column("order_type")
    private ChargeCommon.Type type;
    /**
     * 用户ID
     */
    @Column("user_id")
    private Long userId;

    /**
     * 业务ID
     */
    @Column("business_id")
    private Long businessId;
    /**
     * 项目ID
     */
    @Column("project_id")
    private Long projectId;
    /**
     * 租户ID
     */
    @Column("tenant_id")
    private Long tenantId;
    /**
     * 优惠券ID
     */
    @Column("coupon_id")
    @Deprecated
    private Long couponId;
    /**
     * 是否删除
     */
    @Column("is_deleted")
    private Boolean deleted;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 修改时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 订单备注
     */
    @Column("remarks")
    private String remarks;
    /**
     * 异常信息：如资源创建异常，超时未支付等消息
     */
    @Column("message")
    private String message;
    /**
     * 申请参数
     */
    @Column("apply_param")
    private String applyParam;
    /**
     * 服务编码
     */
    @Column("service_code")
    private String serviceCode;
    /**
     * 关联的云平台id
     */
    @Column("vendor_id")
    private Long vendorId;
    /**
     * 是否自动续订
     */
    @Column("auto_renew")
    private Boolean autoRenew;
    /**
     * 平台名称
     */
    @IgnoreAll
    private String vendorName;

    /**
     * 用户名称
     */
    @IgnoreAll
    private String userName;
    /**
     * 租户名称
     */
    @IgnoreAll
    private String tenantName;
    /**
     * 优惠券名称
     */
    @IgnoreAll
    @Deprecated
    private String couponName;
    /**
     * 订单总金额
     */
    @IgnoreAll
    private BigDecimal money;
    /**
     * 订单明细
     */
    @IgnoreAll
    private List<OrderItem> items;
    /**
     * 服务名称
     */
    @IgnoreAll
    private String serviceName;
    @IgnoreAll
    private String projectName;
}
