package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 租户账户实体类
 *
 * @author tyl
 * @time 2020/7/14 周二 18:55
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("tenant_account")
public class TenantAccount extends Generic {
    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 租户id
     */
    @Column("tenant_id")
    private Long tenantId;
    /**
     * 租户名称
     */
    @IgnoreAll
    private String tenantName;
    /**
     * 账户余额
     */
    @Column("balance")
    private BigDecimal balance;
    /**
     * 账户总额度
     */
    @Column("account_total")
    private BigDecimal accountTotal;
    /**
     * 已分配额度
     */
    @Column("account_allocated")
    private BigDecimal accountAllocated;
    /**
     * 已用额度
     */
    @Column("account_used")
    private BigDecimal accountUsed;
    /**
     * 剩余额度
     */
    @IgnoreAll
    private BigDecimal balanceFree;
    /**
     * 最大额度（授信时需要）
     */
    @IgnoreAll
    private BigDecimal maxBalance;
    /**
     * 最小额度（设置时需要）
     */
    @IgnoreAll
    private BigDecimal minBalance;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 修改时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 能否透支
     */
    @Column("overdraft")
    private Boolean overdraft;
    /**
     * 修改者id
     */
    @Column("mender_id")
    private Long menderId;

    public TenantAccount(Long tenantId, BigDecimal balance, BigDecimal accountTotal, BigDecimal accountAllocated, BigDecimal accountUsed, Date gmtCreate, Date gmtModify, Long menderId, Boolean overdraft) {
        this.tenantId = tenantId;
        this.accountTotal = accountTotal;
        this.accountAllocated = accountAllocated;
        this.accountUsed = accountUsed;
        this.gmtCreate = gmtCreate;
        this.gmtModify = gmtModify;
        this.menderId = menderId;
        this.balance = balance;
        this.overdraft = overdraft;
    }

    public TenantAccount(Long tenantId, BigDecimal balance, BigDecimal accountTotal, BigDecimal accountAllocated, BigDecimal accountUsed, Boolean overdraft) {
        this.tenantId = tenantId;
        this.accountTotal = accountTotal;
        this.accountAllocated = accountAllocated;
        this.accountUsed = accountUsed;
        this.balance = balance;
        this.overdraft = overdraft;
    }

    public enum Action {
        /**
         * 充值
         */
        Recharge,
        /**
         * 增加 授信总额增加
         */
        Increase,
        /**
         * 削减 授信总额减少
         */
        Reduce,
        /**
         * 消费 使用额度增加
         */
        Consume,
        /**
         * 回滚 使用额度减少
         */
        Callback,
        /**
         * 分配 分配额度增加
         */
        Assign,
        /**
         * 回收 分配额度减少
         */
        Recycle,
    }
}
