package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreUpdate;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.GenericEntity;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 问题工单实体类
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月8日
 */
@Table("work_order")
public class WorkOrder extends GenericEntity {

    // ID
    @PK(PKStrategy.AUTO)
    private Long id;
    // 问题类别
    @Column("catalog")
    private String catalog;
    // 手机号码
    @Column("phone")
    private String phone;
    // 联系邮箱
    @Column("email")
    private String email;
    // 问题填报时间
    @Column("report_time")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date reportTime;
    // 租户ID
    @Column("tenant_id")
    private Long tenantId;
    // 故障内容
    @Column("content")
    private String content;
    // 服务星级
    @Column("star")
    private Integer star;
    // 子订单号
    @Column("order_item_sn")
    private String orderItemSn;
    // 资源ID
    @Column("instance_uuid")
    private String instanceUuid;
    // 资源类别
    @Column("resource_catalog")
    private String resourceCatalog;
    // 资源类型
    @Column("resource_category")
    private String resourceCategory;

    /**
     *
     */
    public WorkOrder() {
        super();
    }

    /**
     * @param catalog
     * @param phone
     * @param email
     * @param reportTime
     * @param tenantId
     * @param content
     * @param star
     * @param orderItemSn
     * @param instanceUuid
     * @param resourceCatalog
     * @param resourceCategory
     */
    public WorkOrder(String catalog, String phone, String email, Date reportTime, Long tenantId, String content,
                     Integer star, String orderItemSn, String instanceUuid, String resourceCatalog,
                     String resourceCategory) {
        super();
        this.catalog = catalog;
        this.phone = phone;
        this.email = email;
        this.reportTime = reportTime;
        this.tenantId = tenantId;
        this.content = content;
        this.star = star;
        this.orderItemSn = orderItemSn;
        this.instanceUuid = instanceUuid;
        this.resourceCatalog = resourceCatalog;
        this.resourceCategory = resourceCategory;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getCatalog() {
        return catalog;
    }

    public void setCatalog(String catalog) {
        this.catalog = catalog;
    }

    public String getPhone() {
        return phone;
    }

    public void setPhone(String phone) {
        this.phone = phone;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public Date getReportTime() {
        return reportTime;
    }

    public void setReportTime(Date reportTime) {
        this.reportTime = reportTime;
    }

    public Long getTenantId() {
        return tenantId;
    }

    public void setTenantId(Long tenantId) {
        this.tenantId = tenantId;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public Integer getStar() {
        return star;
    }

    public void setStar(Integer star) {
        this.star = star;
    }

    public String getOrderItemSn() {
        return orderItemSn;
    }

    public void setOrderItemSn(String orderItemSn) {
        this.orderItemSn = orderItemSn;
    }

    public String getInstanceUuid() {
        return instanceUuid;
    }

    public void setInstanceUuid(String instanceUuid) {
        this.instanceUuid = instanceUuid;
    }

    public String getResourceCatalog() {
        return resourceCatalog;
    }

    public void setResourceCatalog(String resourceCatalog) {
        this.resourceCatalog = resourceCatalog;
    }

    public String getResourceCategory() {
        return resourceCategory;
    }

    public void setResourceCategory(String resourceCategory) {
        this.resourceCategory = resourceCategory;
    }

}
