package com.bocloud.cos.internel.entity;

import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreAll;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.GenericEntity;
import com.bocloud.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

import java.util.List;

/**
 * @author zhangdi
 * @version 1.0
 * @since 2018年4月23日
 */
@Data
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
@Table("pool_group")
public class PoolGroup extends GenericEntity {

    // ID
    @PK(value = PKStrategy.AUTO)
    private Long id;
    // 父资源池ID(暂时无用)
    @Column("parent_id")
    private Long parentId;
    // 资源池类别(计算/网络/存储)
    @Column("category")
    private String category;
    // 绑定策略(共享/独占)
    @Column("binding_policy")
    private String bindingPolicy;
    // 所属云平台
    @Column("vendor_id")
    private Long vendorId;
    // 资源池绑定的策略id
    @Column("policy_id")
    private Long policyId;
    // 地域
    @Column("region")
    private String region;
    // 地域
    @Column("is_public")
    private Boolean isPublic;
    // 可用域
    @Column("availablitiy_zone")
    private String availablitiyZone;
    // 优先类型
    @Column("priority_type")
    private PriorityType priorityType;
    @IgnoreAll
    private String children;
    // 子节点
    @IgnoreAll
    private List<PoolGroup> childrenList;
    // 平台名称
    @IgnoreAll
    private String vendorName;
    // 平台类型
    @IgnoreAll
    private String vendorType;
    // 策略类型
    @IgnoreAll
    private String policyType;
    // 地域名称
    @IgnoreAll
    private String regionName;
    // 可用域名称
    @IgnoreAll
    private String azName;
    @IgnoreAll
    private Boolean hasNsx;

    /**
     * @param parentId
     * @param category
     * @param children
     * @param childrenList
     */
    public PoolGroup(Long parentId, String category, String children, List<PoolGroup> childrenList) {
        super();
        this.parentId = parentId;
        this.category = category;
        this.children = children;
        this.childrenList = childrenList;
    }

    public enum BindingPolicy {
        exclusive, // 独占
        share// 共享
    }

    public enum PriorityType {
        Cpu, // CPU
        Memory, // 内存
        Disk// 存储
    }
}
