package com.bocloud.cos.repository;

import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.AccountRecord;
import com.bocloud.cos.entity.Attachment;
import com.bocloud.cos.entity.TenantAccount;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 服务操作业务实现
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/3/4 18:07
 */
@Repository
public class AccountRecordRepository extends BasicGenericDao<AccountRecord, Long> {

    public AccountRecordRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 保存租户授信账户操作记录
     *
     * @param accountRecord
     * @return
     */
    public Boolean saveAccountRecord(AccountRecord accountRecord) {
        return this.baseSave(accountRecord);
    }


    /**
     * 根据租户id获取授信账号操作记录
     *
     * @param tenantId
     * @return
     */
    public List<AccountRecord> getRecords(Long tenantId) {
        String sql = "select a.*,t.name tenant_name from account_record a left join tenant t on a.tenant_id = t.id where a.tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return this.list(AccountRecord.class, sql, params);
    }


    /**
     * 根据条件获取授信账号操作记录
     *
     * @param tenantId
     * @return
     */
    public List<AccountRecord> getRecords(Long tenantId, TenantAccount.Action action, String start, String end) {
        StringBuilder builder = new StringBuilder(
                "select a.*,t.name tenant_name from account_record a left join tenant t on a.tenant_id = t.id where 1=1");
        HashMap<String, Object> params = new HashMap<>(8);
        if (null != tenantId && 0L != tenantId) {
            builder.append(" and a.tenant_id = :tenantId");
            params.put("tenantId", tenantId);
        }
        if (null != action) {
            builder.append(" and a.action = :action");
            params.put("action", action.name());
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        return this.list(AccountRecord.class,builder.toString(),params);
    }


}
