package com.bocloud.cos.repository;

import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.cos.internel.entity.BillVmLabel;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 服务操作业务实现
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/3/5 9:57
 */
@Repository
public class BillVmLabelRepository extends BasicGenericDao<BillVmLabel, Long> {


    public BillVmLabelRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * @param end
     * @param labelValue
     * @return java.util.List<com.bocloud.internel.entity.BillVmLabel>
     * @Author: lxy
     * @Description: 根据标签查询云主机账单金额
     * 5.6 已经弃用server_label，现表已经删除，如需统计需要进入资源表中查询
     * @Date 2021/5/29 16:06
     * @Param start
     * @throws:
     **/
    public List<BillVmLabel> listByLabelType(String start, String end, String labelValue) {
        Map<String, Object> params = new HashMap<>();

        StringBuilder builder =
                new StringBuilder("SELECT cs.name as vm_name,item.sn as sn,bill.gmt_create,label.type,bill.money as money");
        if (StringUtils.isNotEmpty(labelValue)) {
            builder.append(
                    ",JSON_UNQUOTE( JSON_EXTRACT( content, JSON_UNQUOTE( REPLACE ( JSON_SEARCH( content, 'all', '"
                            + labelValue + "', NULL, '$[*].key' ), 'key', 'value' ) ))) AS label_value ");
        }
        builder.append(
                " FROM server_label as label,cloud_server AS cs,order_item AS item,bill_item AS bill WHERE cs.id = label.target AND uuid=item.sn AND bill.item_id=item.id");
        if (StringUtils.isNotEmpty(labelValue)) {
            builder.append(" and JSON_CONTAINS( content -> '$[*].key', '\"" + labelValue + "\"', '$' ) ");
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and bill.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and bill.gmt_create <= :end");
            params.put("end", end);
        }
        return this.list(BillVmLabel.class,builder.toString(),params);
    }
}
