package com.bocloud.cos.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.Document;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 使用文档数据访问层
 *
 * @author tyl
 * @update fanzx
 * @since 2021/3/8 14:20
 */
@Repository
public class DocumentRepository extends JdbcGenericDao<Document, Long> {


    public DocumentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 查询父文档
     *
     * @return
     */
    public Document selectParent(String cagegory) {
        String sql = "select * from document where is_deleted = 0 and parent_id = 0 and category = :category";
        Map<String, Object> map = MapTools.simpleMap("category", cagegory);
        List<Document> list = this.list(Document.class, sql, map);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    /**
     * 查询列表
     *
     * @param params
     * @return
     * @throws BoCloudDBException
     */
    public List<Document> listAllCategory(List<Param> params) throws BoCloudDBException {
        String sql = "select * from document where is_deleted = 0";
        if (params != null) {
            sql += " and category = :category ";
        }
        Map<String, Object> param = QueryBuilder.getParam(params);
        if (params != null) {
            sql = QueryBuilder.buildRaw(sql, params, null, "");
            return this.list(Document.class,sql,param);
        } else {
            return this.list(Document.class,sql);
        }
    }

    /**
     * 查询文档是否存在于数据库中
     *
     * @param name
     * @param id
     * @param parentId
     * @return
     */
    public Document queryByName(String name, Long parentId, Long id) {
        String sql = "select * from document where is_deleted = 0 ";
        Map<String, Object> map = new HashMap<>();
        if (name != null) {
            sql += " and name = :name ";
            map.put("name", name);
        }
        if (parentId != null) {
            sql += " and parent_id = :parentId ";
            map.put("parentId", parentId);
        }
        if (id != null) {
            sql += " and id = :id ";
            map.put("id", id);
        }
        List<Document> list = this.list(Document.class, sql, map);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    /**
     * 删除操作文档
     *
     * @param id
     * @param menderId
     */
    public boolean update(Long id, Long menderId) {
        String sql = "update document set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId , status = :status where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        params.put("status", "UNPUBLISHED");
        return this.execute(sql, params) > 0;
    }

    /**
     * 查看文档详细信息
     *
     * @param id
     * @return
     * @throws BoCloudDBException
     */
    public Document detail(Long id) throws BoCloudDBException {
        String sql = "select * from document where is_deleted = 0 and id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<Document> list = this.list(Document.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean display(Boolean display, RequestContext context) {
        String sql = "update document set gmt_modify = :gmtModify, mender_id = :menderId,display = :display";
        Map<String, Object> params = MapTools.simpleMap("gmtModify", new Date());
        params.put("menderId", context.getTarget());
        params.put("display", display);
        return this.execute(sql, params) > 0;
    }

    public Document queryDisplay(RequestContext context) {
        String sql = "select * from document where is_deleted = 0";
        List<Document> list = this.list(Document.class, sql, null);
        return list.isEmpty() ? null : list.get(0);
    }
}