package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.Icon;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

@Service
public class IconRepository extends BasicGenericDao<Icon, Long> {

    public IconRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<Icon> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select * from icon where category = :category ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(Icon.class, sql, paramMap);
    }

    public Icon query(Long id) {
        String sql = "select * from icon where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Icon> icons = this.list(Icon.class, sql, params);
        return icons.isEmpty() ? null : icons.get(0);
    }

    public Boolean delete(Long id) {
        String sql = "delete from icon where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from icon where category = :category ";
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

}
