package com.bocloud.cos.repository;

import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.OrderSn;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 订单号DAO
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月17日
 */
@Repository
public class OrderSnRepository extends BasicGenericDao<OrderSn, Long> {

    public OrderSnRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public OrderSn query(Long catalogId, Long serviceId, Long categoryId) {
        String sql = "select a.* from order_sn a where a.catalog_id = :catalogId and a.service_id = :serviceId and a.category_id = :categoryId";
        Map<String, Object> params = MapTools.simpleMap("catalogId", catalogId);
        params.put("serviceId", serviceId);
        params.put("categoryId", categoryId);
        List<OrderSn> orderSns = this.list(OrderSn.class, sql, params);
        return ListTool.isEmpty(orderSns) ? null : orderSns.get(0);
    }

    public OrderSn query(Long skuId) {
        String sql = "select a.* from order_sn a where a.sku_id = :skuId";
        Map<String, Object> params = MapTools.simpleMap("skuId", skuId);
        List<OrderSn> orderSns = this.list(OrderSn.class, sql, params);
        return ListTool.isEmpty(orderSns) ? null : orderSns.get(0);
    }

    public Boolean modify(OrderSn orderSn) {
        return this.update(orderSn);
    }

    public void modify(Long id, Long currentNum) {
        String sql = "update order_sn set current_num = " + currentNum + " where id = " + id;
        this.execute(sql);
    }

    public Boolean remove(Long id) {
        String sql = "delete from order_sn where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        return this.execute(sql, paramMap) > 0;
    }

    public Boolean removeByCatalogId(Long catalogId) {
        String sql = "delete from order_sn where catalog_id = :catalogId";
        Map<String, Object> paramMap = MapTools.simpleMap("catalogId", catalogId);
        return this.execute(sql, paramMap) > 0;
    }

    public Boolean removeByServiceId(Long serviceId) {
        String sql = "delete from order_sn where service_id = :serviceId";
        Map<String, Object> paramMap = MapTools.simpleMap("serviceId", serviceId);
        return this.execute(sql, paramMap) > 0;
    }

    public Boolean removeByCategoryId(Long categoryId) {
        String sql = "delete from order_sn where category_id = :categoryId";
        Map<String, Object> paramMap = MapTools.simpleMap("categoryId", categoryId);
        return this.execute(sql, paramMap) > 0;
    }

    public Boolean removeBySkuId(Long skuId) {
        String sql = "delete from order_sn where sku_id = :skuId";
        Map<String, Object> paramMap = MapTools.simpleMap("skuId", skuId);
        return this.execute(sql, paramMap) > 0;
    }

    public OrderSn query(Long catalogId, Long serviceId, Long categoryId, Long skuId) {
        String sql = "select a.* from order_sn a where a.catalog_id = :catalogId and a.service_id = :serviceId and a.category_id = :categoryId and a.sku_id = :skuId";
        Map<String, Object> params = MapTools.simpleMap("catalogId", catalogId);
        params.put("serviceId", serviceId);
        params.put("categoryId", categoryId);
        params.put("skuId", skuId);
        List<OrderSn> orderSns = this.list(OrderSn.class, sql, params);
        if (ListTool.isEmpty(orderSns)) {
            return null;
        }
        return (OrderSn) orderSns.get(0);
    }

}
