package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.Payment;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 支付记录数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository
public class PaymentRepository extends BasicGenericDao<Payment, Long> {

    public PaymentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<Payment> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql = "select a.*,b.name user_name,c.name tenant_name from payment a left join users b on b.id = a.user_id left join tenant c on c.id = a.tenant_id where 1 = 1 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(Payment.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from payment a where 1 = 1";
        sql = QueryBuilder.build(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) {
        String sql = "delete from payment where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.execute(sql, params) > 0;
    }

    public List<Payment> listByBillId(Long billId) {
        String sql = "select a.*,b.name user_name,c.name tenant_name from payment a left join users b on b.id = a.user_id left join tenant c on c.id = a.tenant_id where a.bill_id = :billId";
        Map<String, Object> params = MapTools.simpleMap("billId", billId);
        return this.list(Payment.class, sql, params);
    }

    public Boolean removeByBillId(Long billId) {
        String sql = "delete from payment where bill_id = :billId";
        Map<String, Object> params = MapTools.simpleMap("billId", billId);
        return this.execute(sql, params) > 0;
    }

    public Payment query(Long id) {
        String sql = "select a.*,b.name user_name,c.name tenant_name from payment a left join users b on b.id = a.user_id left join tenant c on c.id = a.tenant_id where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Payment> list = this.list(Payment.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean existByCondition(String column, String value) {
        String sql = "select * from payment where " + column + " = :value";
        Map<String, Object> params = MapTools.simpleMap("value", value);
        List<Payment> list = this.list(Payment.class, sql, params);
        if (list.isEmpty()) {
            return false;
        }
        return true;
    }
}
