package com.bocloud.cos.repository;

import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.PortalPanel;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author lyy
 */
@Repository
public class PortalPanelRepository extends BasicGenericDao<PortalPanel, Long> {

    public PortalPanelRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<PortalPanel> listPortalPanel(String serverName) {
        String sql = "select * from portal_panel where server_name = :serverName and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        return this.list(PortalPanel.class, sql, params);
    }

    public List<PortalPanel> list(String serverName, Long tenantId, Long projectId) {
        String sql = "select * from portal_panel where server_name = :serverName and is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        if (null != tenantId && 0L != tenantId) {
            sql += " and tenant_id=:tenantId ";
            params.put("tenantId", tenantId);
        }
        if (null != projectId && 0L != projectId) {
            sql += " and project_id=:projectId ";
            params.put("projectId", projectId);
        }
        return this.list(PortalPanel.class, sql, params);
    }

    public Boolean remove(Long id) {
        String sql = "update portal_panel set is_deleted = 1 where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }

    public Boolean removeByServerName(String serverName, Long userId) {
        String sql = "update portal_panel set is_deleted = 1,mender_id=:userId where server_name = :serverName and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<PortalPanel> queryDefaultByServerName(String serverName) {
        String sql = "select * from portal_panel_default where server_name = :serverName";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        return this.list(PortalPanel.class, sql, params);
    }

    public List<PortalPanel> queryByServerName(String serverName) {
        String sql = "select * from portal_panel where server_name = :serverName";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        return this.list(PortalPanel.class, sql, params);
    }

    public List<PortalPanel> queryByServerName(String serverName, Long tenantId) {
        String sql = "select * from portal_panel where server_name = :serverName  and tenant_id=:tenantId";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        params.put("tenantId", tenantId);
        return this.list(PortalPanel.class, sql, params);
    }

}
