package com.bocloud.cos.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.cos.entity.QuotaHistory;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 服务操作业务实现
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/3/4 15:20
 */

@Repository
public class QuotaHistoryRepository extends BasicGenericDao<QuotaHistory, Long> {

    public QuotaHistoryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public boolean batchSaveQuotaHistory(List<QuotaHistory> quotaHistorys) {
        return this.batchSave(quotaHistorys,quotaHistorys.size());
    }

    public List<QuotaHistory> getQuotaUsedHistory(String start, String end, Long tenantId, String type) throws BoCloudDBException {
        StringBuilder builder = new StringBuilder("select a.* from quota_history a where   a.category = 'Meta' and  a.action in ('Order','Callback') ");
        Map<String, Object> params = new HashMap<>();
        if (null != tenantId && 0 != tenantId) {
            builder.append(" and a.target = :tenantId");
            params.put("tenantId", tenantId);
        }
        if (StringUtils.isNotEmpty(type)) {
            builder.append(" and a.code = :type");
            params.put("type", type);
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        return this.list(QuotaHistory.class,builder.toString(),params);
    }


    public List<QuotaHistory> getServiceQuotaHistory(String start, String end, Long tenantId) throws BoCloudDBException {
        StringBuilder builder = new StringBuilder("select a.* from quota_history a where a.category = 'Service' and  a.action in ('Order','Callback') ");
        Map<String, Object> params = new HashMap<>();
        if (null != tenantId && 0 != tenantId) {
            builder.append(" and a.target = :tenantId");
            params.put("tenantId", tenantId);
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        return this.list(QuotaHistory.class,builder.toString(),params);
    }


}
