package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.Recycled;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository
public class RecycledRepository extends BasicGenericDao<Recycled, Long> {
    public RecycledRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public int count(List<Param> params, List<Long> projectIds) {
        String sql = "select count(1) from recycled a where a.is_deleted = 0  ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<Recycled> list(int page, int rows, Map<String, String> sorter, List<Param> params,
                               List<Long> projectIds) {
        String sql = "select a.* from recycled a where a.is_deleted = 0";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(Recycled.class, sql, paramMap);
    }

    public Boolean remove(Long id, Long userId) {
        String sql = "update recycled set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Boolean remove(String sn, Long userId) {
        String sql = "update recycled set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where sn = :sn";
        Map<String, Object> params = MapTools.simpleMap("sn", sn);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Recycled query(Long id) {
        String sql = "select a.* from recycled a where a.is_deleted = 0 and a.id = :id  ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Recycled> list = this.list(Recycled.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public int count(Long userId, Long tenantId, List<Long> projectIds, List<Param> params) {
        String subquerySql = "select a.* from recycled a where a.is_deleted = 0  ";
        Map<String, Object> param = QueryBuilder.getParam(params);
        subquerySql = QueryBuilder.build(subquerySql, params, new HashMap<>(8), "a");
        String sql = "select distinct count(1) from (" + subquerySql + ") c  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or c.owner_id =:userId or c.mender_id = :userId ";
            param.put("userId", userId);
        }
        if (null != tenantId && 0L != tenantId) {
            sql += " or c.tenant_id = :tenantId";
            param.put("tenantId", tenantId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or c.project_id in (:projectIds) ";
            param.put("projectIds", projectIds);
        }
        sql += ")";
        return this.countQuery(sql, param).intValue();
    }

    public List<Recycled> list(int page, int rows, Long userId, Long tenantId, List<Long> projectIds,
                               List<Param> params, Map<String, String> sorter) {
        String subquerySql = "select a.* from recycled a where a.is_deleted = 0";
        subquerySql = QueryBuilder.build(subquerySql, params, new HashMap<>(8), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        String sql = "select distinct  d.*  from (" + subquerySql + ") d  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or d.owner_id =:userId or d.mender_id = :userId ";
            paramMap.put("userId", userId);
        }
        if (null != tenantId && 0L != tenantId) {
            sql += " or d.tenant_id = :tenantId";
            paramMap.put("tenantId", tenantId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or d.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql += " )";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, null, sorter), "d");
        return this.list(Recycled.class, sql, paramMap);
    }
}
