package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.Ros;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 资源编排服务数据访问
 *
 * @author weiwei
 * @version 1.0
 * @since 2020年3月11日
 */
@Repository
public class RosRepository extends JdbcGenericDao<Ros, Long> {

    public RosRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<Ros> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql = "select a.*,b.name category_name from ros a left join service_category b on a.category_id = b.id where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(Ros.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from ros a where a.is_deleted = 0";
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<Ros> list = this.list(Ros.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (Ros obj : list) {
            beans.add(new SimpleBean(obj.getId(), obj.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from ros a where a.is_deleted = 0";
        sql = QueryBuilder.build(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id, Long userId) {
        String sql = "update ros set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Ros query(Long id) {
        String sql = "select a.* from ros a where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Ros> list = this.list(Ros.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateStatus(Long id, String status) {
        String sql = "update ros set status = :status where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

}
