package com.bocloud.cos.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * 服务类别数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository
public class ServiceCatalogRepository extends JdbcGenericDao<ServiceCatalog, Long> {

    public ServiceCatalogRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ServiceCatalog> list(int page, int rows, Map<String, String> sorter, List<Param> params)
            throws BoCloudDBException {
        String sql = " select a.* from service_catalog a where a.is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(ServiceCatalog.class, sql, paramMap);
    }

    public ServiceCatalog getByCode(String code) throws BoCloudDBException {
        String sql = "select a.* from service_catalog a where a.is_deleted = 0 and a.code = :code limit 1";
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("code", code);
        List<ServiceCatalog> catalogs = this.list(ServiceCatalog.class, sql, paramMap);
        return ListTool.isEmpty(catalogs) ? null : catalogs.get(0);
    }

    public List<ServiceCatalog> listAll() throws BoCloudDBException {
        String sql = "select a.* from service_catalog  a where a.is_deleted = 0";
        return this.list(ServiceCatalog.class, sql);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select a.* from service_catalog a where a.is_deleted = 0";
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<ServiceCatalog> serviceCatalogs = this.list(ServiceCatalog.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (ServiceCatalog serviceCatalog : serviceCatalogs) {
            SimpleBean bean = new SimpleBean(serviceCatalog.getId(), serviceCatalog.getName());
            beans.add(bean);
        }
        return beans;
    }

    public int count(List<Param> params) throws BoCloudDBException {
        String sql = "select count(1) from service_catalog a where a.is_deleted = 0  ";
        sql = QueryBuilder.build(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id, Long userId) throws BoCloudDBException {
        String sql = "update service_catalog set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public ServiceCatalog query(Long id) throws BoCloudDBException {
        String sql = "select * from service_catalog a where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<ServiceCatalog> list = this.list(ServiceCatalog.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public Integer getMaxId() throws BoCloudDBException {
        String sql = "select max(a.id) from service_catalog a";
        Map<String, Object> paramMap = new HashMap<String, Object>();
        return this.countQuery(sql, paramMap).intValue();
    }

}
