package com.bocloud.cos.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ServiceFlowConfig;
import com.bocloud.cos.enums.ApplyCategory;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.List;
import java.util.Map;

/**
 * @author tyl
 * @time 2020/8/25 周二 10:54
 */
@Repository
public class ServiceFlowConfigRepository extends BasicGenericDao<ServiceFlowConfig, Long> {

    public ServiceFlowConfigRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public boolean deleteById(Long serviceId) throws BoCloudDBException {
        String sql = "delete from service_flow_config where  service_id = :serviceId";
        return this.execute(sql, MapTools.simpleMap("serviceId", serviceId)) >= 0;
    }

    public boolean deleteByIds(List<Long> serviceIds) throws BoCloudDBException {
        String sql = "delete from service_flow_config where  service_id in (:serviceIds)";
        return this.execute(sql, MapTools.simpleMap("serviceIds", serviceIds)) >= 0;
    }

    public List<ServiceFlowConfig> getServiceFlowConfig(Long serviceId) throws BoCloudDBException {
        String sql = "select a.* from service_flow_config a where a.service_id=:serviceId";
        return this.list(ServiceFlowConfig.class, sql, MapTools.simpleMap("serviceId", serviceId));
    }

    public Long getServiceFlowId(Long serviceId, ApplyCategory category) throws BoCloudDBException {
        String sql = "select a.* from service_flow_config a where a.service_id=:serviceId and a.category = :category ";
        Map<String, Object> params = MapTools.simpleMap("serviceId", serviceId);
        params.put("category", category.name());
        List<ServiceFlowConfig> result = this.list(ServiceFlowConfig.class, sql, params);
        Assert.isTrue(result.size() <= 1, "more than one service flow config in table");
        return result.isEmpty() ? null : result.get(0).getFlowId();
    }

    /**
     * 执行流程是否被服务绑定
     *
     * @param flowId
     * @return
     */
    public boolean exists(Long flowId) {
        String sql = "select count(*) from service_flow_config a where a.flow_id=:flowId";
        return super.countQuery(sql, MapTools.simpleMap("flowId", flowId)) > 0;
    }

}
