package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.Sign;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ServiceSla;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 服务协议数据访问
 *
 * @author weiwei
 * @version 1.0
 * @since 2020年3月11日
 */
@Repository
public class ServiceSlaRepository extends JdbcGenericDao<ServiceSla, Long> {

    public ServiceSlaRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ServiceSla> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        StringBuilder sql = new StringBuilder("select a.*,b.name service_name,c.id catalog_id,c.name catalog_name from service_sla a"
                + " left join cloud_service b on b.id = a.service_id "
                + " left join service_catalog c on c.id = b.catalog_id where a.is_deleted = 0");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        for (Param param : params) {
            Map<String, Object> paramParam = param.getParam();
            if (paramParam.containsKey("catalogId")) {
                sql.append(" and b.catalog_id = :catalogId");
                paramParam.remove("catalogId");
            }
        }
        sql = new StringBuilder(QueryBuilder.buildRaw(sql.toString(), new Pager(page, rows, params, sorter), "a"));
        return this.list(ServiceSla.class, sql.toString(), paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from service_sla a where a.is_deleted = 0";
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<ServiceSla> list = this.list(ServiceSla.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (ServiceSla obj : list) {
            beans.add(new SimpleBean(obj.getId(), obj.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) {
        StringBuilder sql = new StringBuilder("select count(1) from service_sla a left join cloud_service cs on a.service_id = cs.id where a.is_deleted = 0 ");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        Integer catalogId = null;
        for (Param param : params) {
            Map<String, Object> paramParam = param.getParam();
            if (paramParam.containsKey("catalogId")) {
                catalogId = (Integer) paramParam.get("catalogId");
                sql.append(" and cs.catalog_id = :catalogId");
                paramParam.remove("catalogId");
            }
        }
        sql = new StringBuilder(QueryBuilder.build(sql.toString(), params, null, "a"));
        if (catalogId != null) {
            params.add(new Param(MapTools.simpleMap("catalogId", catalogId), Sign.EQ));
        }
        return this.countQuery(sql.toString(), paramMap).intValue();
    }

    public Boolean remove(Long id, Long userId) {
        String sql = "update service_sla set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public ServiceSla query(Long id) {
        String sql = "select a.*,b.name service_name,c.id catalog_id,c.name catalog_name from service_sla a"
                + " left join cloud_service b on b.id = a.service_id"
                + " left join service_catalog c on c.id = b.catalog_id  where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<ServiceSla> list = this.list(ServiceSla.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean existByCondition(String column, String value) {
        String sql = "select * from service_sla where " + column + " = :value and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("value", value);
        List<ServiceSla> slas = this.list(ServiceSla.class, sql, params);
        return !ListTool.isEmpty(slas);
    }

    public List<ServiceSla> listByService(Long serviceId, String status) {
        String sql = "select a.* from service_sla a where a.is_deleted = 0 and a.service_id = :serviceId";
        Map<String, Object> params = MapTools.simpleMap("serviceId", serviceId);
        if (StringUtils.isNotEmpty(status)) {
            sql += " and a.status = :status";
            params.put("status", status);
        }
        return this.list(ServiceSla.class, sql, params);
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql = "update service_sla set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

}
