package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.SkuSpec;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * sku规格数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository
public class SkuSpecRepository extends BasicGenericDao<SkuSpec, Long> {

    public SkuSpecRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<SkuSpec> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select a.*,b.name sku_name,b.category_id,b.status sku_status from sku_spec a left join service_sku b on b.id = a.sku_id where 1 = 1 ";
        sql = QueryBuilder.build(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.list(SkuSpec.class, sql, param);
    }

    public boolean remove(Long id) {
        String sql = "delete from sku_spec where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from sku_spec where 1 = 1";
        sql = QueryBuilder.build(sql, params, null, "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public SkuSpec query(Long id) {
        String sql = "select a.*,b.name sku_name,b.category_id,b.status sku_status from sku_spec a left join service_sku b on b.id = a.sku_id where a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<SkuSpec> specs = this.list(SkuSpec.class, sql, paramMap);
        return ListTool.isEmpty(specs) ? null : specs.get(0);
    }

    public List<SkuSpec> listByCategory(String category, Long categoryId) {
        String sql = "select a.*,b.name sku_name,b.category_id,b.status sku_status from sku_spec a left join service_sku b on b.id = a.sku_id where a.spec_category = :category and b.status = 'Released'";
        Map<String, Object> paramMap = MapTools.simpleMap("category", category);
        if (null != categoryId && 0L != categoryId) {
            sql += " and b.category_id = :categoryId";
            paramMap.put("categoryId", categoryId);
        }
        return this.list(SkuSpec.class, sql, paramMap);
    }

    public List<SkuSpec> listByCondition(Long categoryId) {
        String sql = "select a.*,b.name sku_name,b.category_id,b.status sku_status from sku_spec a left join service_sku b on b.id = a.sku_id where b.status = 'Released' and b.category_id = :categoryId";
        Map<String, Object> paramMap = MapTools.simpleMap("categoryId", categoryId);
        return this.list(SkuSpec.class, sql, paramMap);
    }

    public SkuSpec listBySpecValue(String category, Long value) {
        String sql = " SELECT * FROM sku_spec spec WHERE spec.spec_category = :category and spec.spec_value = :value ";
        Map<String, Object> paramMap = MapTools.simpleMap("category", category);
        paramMap.put("value", value);
        List<SkuSpec> specs = this.list(SkuSpec.class, sql, paramMap);
        return ListTool.isEmpty(specs) ? null : specs.get(0);
    }

    public List<SkuSpec> listBySkuId(Long skuId) {
        String sql = "select a.*,b.name sku_name,b.category_id,b.status sku_status from sku_spec a left join service_sku b on b.id = a.sku_id where a.sku_id = :skuId";
        Map<String, Object> paramMap = MapTools.simpleMap("skuId", skuId);
        return this.list(SkuSpec.class, sql, paramMap);
    }

    public boolean removeBySkuId(Long skuId) {
        String sql = "delete from sku_spec where sku_id = :skuId";
        Map<String, Object> params = MapTools.simpleMap("skuId", skuId);
        return this.execute(sql, params) > 0;
    }

    public boolean removeByCategory(String category) {
        String sql = "delete from sku_spec where spec_category = :category";
        Map<String, Object> params = MapTools.simpleMap("category", category);
        return this.execute(sql, params) > 0;
    }

    public List<SkuSpec> listUpgradeSpec(String code) {
        String sql = " SELECT *,"
                + " ( SELECT spec.spec_value FROM sku_spec spec WHERE sku.id = spec.sku_id and spec.spec_category = sku.code and spec.spec_name = 'cpu') AS cpu,"
                + " ( SELECT spec.spec_value FROM sku_spec spec WHERE sku.id = spec.sku_id and spec.spec_category = sku.code and spec.spec_name = 'mem' ) AS mem"
                + " FROM service_sku sku where sku.code = :code and sku.status = 'Released' ";
        Map<String, Object> paramMap = MapTools.simpleMap("code", code);
        return this.list(SkuSpec.class, sql, paramMap);
    }

}
