package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.SlaInstance;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 服务协议实例数据访问
 *
 * @author weiwei
 * @version 1.0
 * @since 2020年3月31日
 */
@Repository
public class SlaInstanceRepository extends BasicGenericDao<SlaInstance, Long> {

    public SlaInstanceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<SlaInstance> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql = "select a.*,b.name service_name,c.id catalog_id,c.name catalog_name from sla_instance a"
                + " left join cloud_service b on b.id = a.service_id"
                + " left join service_catalog c on c.id = b.catalog_id where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(SlaInstance.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from sla_instance a where a.is_deleted = 0";
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<SlaInstance> list = this.list(SlaInstance.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (SlaInstance obj : list) {
            beans.add(new SimpleBean(obj.getId(), obj.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from sla_instance a where a.is_deleted = 0";
        sql = QueryBuilder.build(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id, Long userId) {
        String sql = "update sla_instance set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public SlaInstance query(Long id) {
        String sql = "select a.*,b.name service_name,c.id catalog_id,c.name catalog_name from sla_instance a"
                + " left join cloud_service b on b.id = a.service_id"
                + " left join service_catalog c on c.id = b.catalog_id  where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<SlaInstance> list = this.list(SlaInstance.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

}
