package com.bocloud.cos.repository;

import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.TenantAccount;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 租户授信账户数据操作类
 *
 * @author tyl
 * @time 2020/7/15 周三 10:04
 */
@Repository
public class TenantAccountRepository extends BasicGenericDao<TenantAccount, Long> {

    public TenantAccountRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 根据租户id删除授信账户
     *
     * @param tenantId
     * @return
     */
    public boolean delete(Long tenantId) {
        String sql = "delete from tenant_account where tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return execute(sql, params) > 0;
    }

    /**
     * 根据租户id获取授信账户信息
     *
     * @param tenantId
     * @return
     */
    public TenantAccount getAccount(Long tenantId) {
        String sql = "select a.*,t.name tenant_name from tenant_account a left join tenant t on a.tenant_id = t.id where a.tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        List<TenantAccount> tenantAccounts = this.list(TenantAccount.class, sql, params);
        Assert.isTrue(tenantAccounts.size() <= 1, "more than one tenant account in the table");
        return tenantAccounts.isEmpty() ? null : tenantAccounts.get(0);
    }



    /**
     * 删除租户授信账户操作记录
     *
     * @param id
     * @return
     */
    public boolean deleteAccount(Long id) {
        String sql = "delete from  account_record where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }


    public List<Tenant> getAllTenant() {
        String sql = "select * from tenant where is_deleted = 0";
        List<Map<String, Object>> list = this.list(sql);
        List<Tenant> tenantList  = new ArrayList<>();
        for (Map<String, Object> map : list) {
            Tenant tenant = new Tenant();
            tenant.setId((Long) map.get("id"));
            tenant.setName((String) map.get("name"));
            tenantList.add(tenant);
        }
        return tenantList;
    }
}
