package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.VendorSyncRecord;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class VendorSyncRecordRepository extends BasicGenericDao<VendorSyncRecord, Long> {

    public VendorSyncRecordRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<VendorSyncRecord> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select a.*,b.name as creator_name from vendor_sync_record a left JOIN users b on a.creator_id = b.id where 1 = 1 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(VendorSyncRecord.class, sql);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from vendor_sync_record where 1 = 1 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        return this.countQuery(sql, new HashMap<>()).intValue();
    }

    public boolean delete(Long id) {
        String sql = "delete from vendor_sync_record where id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        return this.execute(sql, paramMap) > 0;
    }

    public VendorSyncRecord query(Long id) {
        String sql = "select * from vendor_sync_record where id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<VendorSyncRecord> list = this.list(VendorSyncRecord.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public boolean deleteByVendor(Long vendorId) {
        String sql = "delete from vendor_sync_record where vendor_id = :vendorId ";
        Map<String, Object> paramMap = MapTools.simpleMap("vendorId", vendorId);
        return this.execute(sql, paramMap) > 0;
    }

    public VendorSyncRecord queryByVendor(Long vendorId) {
        String sql = "select * from vendor_sync_record where vendor_id = :vendorId ";
        Map<String, Object> paramMap = MapTools.simpleMap("vendorId", vendorId);
        List<VendorSyncRecord> list = this.list(VendorSyncRecord.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public boolean deleteByTime(String startTime, String endTime, Long vendorId) {
        String sql = "delete from vendor_sync_record where end_time between :startTime and :endTime and vendor_id = :vendorId ";
        Map<String, Object> paramMap = MapTools.simpleMap("startTime", startTime);
        paramMap.put("endTime", endTime);
        paramMap.put("vendorId", vendorId);
        return this.execute(sql, paramMap) > 0;
    }

}
