package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.WorkOrder;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 问题工单数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月13日
 */
@Repository
public class WorkOrderRepository extends BasicGenericDao<WorkOrder, Long> {

    public WorkOrderRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from work_order a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from work_order a where a.is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<WorkOrder> orders = this.list(WorkOrder.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (WorkOrder order : orders) {
            beans.add(new SimpleBean(order.getId(), order.getName()));
        }
        return beans;
    }

    public List<WorkOrder> list(Integer page, Integer rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from work_order a where a.is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(WorkOrder.class, sql, paramMap);
    }

    public boolean remove(Long id, Long menderId, Long tenantId) {
        String sql = "update work_order set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        params.put("tenantId", tenantId);
        return this.execute(sql, params) > 0;
    }

    public WorkOrder query(Long id) {
        String sql = "select a.* from work_order a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<WorkOrder> list = this.list(WorkOrder.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public WorkOrder detailByUuid(String uuid) {
        String sql = "select a.* from work_order a where a.is_deleted = 0 and a.instance_uuid = :uuid";
        Map<String, Object> paramMap = MapTools.simpleMap("uuid", uuid);
        List<WorkOrder> list = this.list(WorkOrder.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public WorkOrder detailByName(String name) {
        String sql = "select a.* from work_order a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        List<WorkOrder> list = this.list(WorkOrder.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<WorkOrder> listByTenant(Long tenantId) {
        String sql = "select a.* from work_order a where a.is_deleted = 0 and a.tenant_id = :tenantId";
        Map<String, Object> paramMap = MapTools.simpleMap("tenantId", tenantId);
        return this.list(WorkOrder.class, sql, paramMap);
    }
}
