package com.bocloud.cos.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.utils.DateUtils;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Hu Wentao
 * @Description: cms内部调用
 * @date 2020/11/25 17:15
 */
@Component
public class CmsInternalService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public CmsInternalService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }


    private final Map<String, Object> headers = new HashMap<>(4);
    private static final Service SERVICE = Service.create(BoCloudService.CMS);


    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

    public BocloudResult getDailyAlarm(String start, String end) {
        JSONObject object = new JSONObject();
        object.put("start", String.valueOf(DateUtils.stringToDate(start, DateUtils.STANDARD_DATETIME_FORMAT).getTime() / 1000));
        object.put("end", String.valueOf(DateUtils.stringToDate(end, DateUtils.STANDARD_DATETIME_FORMAT).getTime() / 1000));
        Map<String, Object> paramMap = MapTools.simpleMap(Common.PARAMS, object.toJSONString());
        paramMap.put(Common.ACTION, "barChart");
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/alarms/chart", BoCloudMethod.GET, headers,
                paramMap);
        return remoteService.invoke();
    }

    public BocloudResult getTop5(Long vendorId, String type, Integer limit) {
        JSONObject object = new JSONObject();
        object.put("vendorId", vendorId);
        object.put("type", type);
        object.put("limit", limit);
        Map<String, Object> paramMap = MapTools.simpleMap(Common.PARAMS, object.toJSONString());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tops", BoCloudMethod.GET, headers,
                paramMap);
        return remoteService.invoke();
    }
}
