package com.bocloud.cos.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.model.RequestContext;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * 服务操作业务实现
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/2/15 9:40
 */
@Slf4j
@Component
public class FmsFlowService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public FmsFlowService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }


    private final Map<String, Object> headers = new HashMap<>(4);
    private static final Service SERVICE = Service.create(BoCloudService.FMS);


    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

    public BocloudResult startInstanceById(Long id, RequestContext context, Long project, Map<String, Object> args){
        HashMap<String, Object> map = new HashMap<>();
        map.put("defId",id);
        map.put("context",JSONObject.toJSONString(context));
        map.put("project", null == project ? -1 : project);
        map.put("params",JSONObject.toJSONString(args));
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/instances", BoCloudMethod.POST, headers,
                map);
        BocloudResult invoke = remoteService.invoke();
        log.info("远程调用结果: {}",JSONObject.toJSONString(invoke));
        if (null == invoke || invoke.isFailed()){
            return new BocloudResult(false,"实例启动失败!");
        }
        return invoke;
    }

    public long countTodo(RequestContext context) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("context",JSONObject.toJSONString(context));
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/instances/count", BoCloudMethod.POST, headers,
                map);
        BocloudResult invoke = remoteService.invoke();
        log.info("远程调用结果: {}",JSONObject.toJSONString(invoke));
        if (invoke.isFailed()){
            log.error("远程调用开启实例失败，失败原因{}",invoke.getData());
            return -1;
        }
        return Long.parseLong(invoke.getData().toString());
    }
}
