package com.bocloud.cos.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.internel.entity.CloudServer;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * cmp服务内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月22日
 */
@Component
public class ImsServerService {

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;

    private static Service service = Service.create(BoCloudService.IMS);

    public BocloudResult vmsRestore(Long id, RequestContext context) {
        Map<String, Object> params = MapTools.simpleMap(Common.ACTION, "restore");
        params.put(Common.ACTION, "restore");
        Map<String, Object> headers = MapTools.simpleMap(Common.API_KEY, context.getApiKey());
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(service, "/v1/vms/" + id, BoCloudMethod.PATCH, headers, params);
        remoteService.setDataFormat(PostDataFormat.RAW);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        return result;
    }

    public BocloudResult serverDetail(String apiKey, Long serverId) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(Common.API_KEY, apiKey);
        JSONObject object = new JSONObject();
        object.put("serverId", serverId);
        object.put(Common.CONDITION, "basic");
        RemoteService remoteService = serviceFactory.build(service, "/v1/vms", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        return remoteService.invoke();
    }

    /**
     * 分页查询虚拟机列表
     *
     * @param token
     * @param pager
     * @return
     */
    public BocloudResult<GridBean<CloudServer>> list(String token, Pager pager) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(Common.TOKEN, token);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(service, "/v1/vms", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return result;
        }
        String content = JSONObject.toJSONString(result.getData());
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return new BocloudResult<>(true, gridBean, result.getMessage());
    }

}
