package com.bocloud.cos.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.internel.entity.Cms;
import com.bocloud.cos.internel.entity.Kvs;
import com.bocloud.cos.internel.entity.Rds;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * @author tyl
 * @since 2020/10/26 15:46
 */
@Component
public class PmsInternalService implements InitializingBean {
    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;
    private final Map<String, Object> headers = new HashMap<>(4);
    private static final Service SERVICE = Service.create(BoCloudService.PMS);

    @Autowired
    public PmsInternalService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    /**
     * 获取健值数据库列表
     *
     * @param vendorId
     * @return
     */
    public BocloudResult getKVSList(Long vendorId) {
        ArrayList<Param> params = new ArrayList<>();
        if (null != vendorId && 0L != vendorId) {
            params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/kvs", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取关系数据库列表
     *
     * @param vendorId
     * @return
     */
    public BocloudResult getRDSList(Long vendorId) {
        ArrayList<Param> params = new ArrayList<>();
        if (null != vendorId && 0L != vendorId) {
            params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/rds", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取中间件列表
     *
     * @param vendorId
     * @return
     */
    public BocloudResult getCMSList(Long vendorId) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("page", 1);
        paramMap.put("rows", 99999L);
        if (null != vendorId && 0L != vendorId) {
            JSONArray jsonArray = new JSONArray();
            JSONObject jsonObject = new JSONObject();
            JSONObject param = new JSONObject();
            param.put("vendorId", vendorId);
            jsonObject.put("param", param);
            jsonObject.put("sign", "EQ");
            jsonArray.add(jsonObject);
            paramMap.put(Common.PARAMS, jsonArray);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/cms", BoCloudMethod.GET, headers, paramMap);
        return remoteService.invoke();
    }

    public BocloudResult rdsRestore(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/rds/" + id + "/restore", BoCloudMethod.PATCH, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        return result;
    }

    public Rds rdsDetail(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/rds/" + id + "/basics", BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        String data = JSONObject.toJSONString(result.getData());
        return JSONObject.parseObject(data, Rds.class);
    }

    public BocloudResult cmsRestore(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/cms/" + id + "/restore", BoCloudMethod.PATCH, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        return result;
    }

    public Cms cmsDetail(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/cms/" + id + "/basics", BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        String data = JSONObject.toJSONString(result.getData());
        return JSONObject.parseObject(data, Cms.class);
    }

    public BocloudResult kvsRestore(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/kvs/" + id + "/restore", BoCloudMethod.PATCH, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        return result;
    }

    public Kvs kvsDetail(Long id) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/kvs/" + id + "/basics", BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        String data = JSONObject.toJSONString(result.getData());
        return JSONObject.parseObject(data, Kvs.class);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
