package com.bocloud.cos.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.internel.entity.*;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 服务授权
 *
 * @author Administrator
 */
@Component
@Slf4j
public class SmsInternalService implements InitializingBean {

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    private Map<String, Object> headers = new HashMap<>(4);
    private static Service service = Service.create(BoCloudService.SMS);

    public BocloudResult<AccountSecurity> secKey(String apiKey) {
        Map<String, Object> params = Maps.newHashMap();
        params.put("apiKey", apiKey);
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/user/key",
                BoCloudMethod.OTHERS, headers, params);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<AccountSecurity>(true, JSONObject.parseObject(data, AccountSecurity.class), "success");
    }

    public BocloudResult<List<Manager>> listByRoles(List<Long> roleIds) {
        Map<String, Object> params = Maps.newHashMap();
        params.put(Common.PARAMS, roleIds);
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/manager/listbyroles",
                BoCloudMethod.OTHERS, headers, params);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<List<Manager>>(true, JSONArray.parseArray(data, Manager.class), "success");
    }

    public BocloudResult<User> detail(Long id) {
//        String url = UrlTranslator.translate("/v1/managers", id);
        String url = "/v1/managers/" + id;
        RemoteService remoteService = serviceFactory.build(service, url, BoCloudMethod.LOAD, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<User>(true, JSONObject.parseObject(data, User.class), "success");
    }

    public BocloudResult<List<User>> listUsers(Long tenantId) {
        Map<String, Object> params = Maps.newHashMap();
        params.put(Common.TENANTID, tenantId);
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/user/listbytenant",
                BoCloudMethod.OTHERS, headers, params);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<List<User>>(true, JSONArray.parseArray(data, User.class), "success");
    }

    public BocloudResult<Tenant> get(Long id) {
        RemoteService remote = serviceFactory.build(service, "/v1/tenants/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, Tenant.class), "success");
    }

    public BocloudResult batchUpdate(List<Tenant> tenants) {
        RemoteService remote = serviceFactory.build(service, "/tenants/batchupdate", BoCloudMethod.POST, headers,
                MapTools.simpleMap(Common.PARAMS, JSONObject.toJSONString(tenants)));
        return remote.invoke();
    }

    public String getSystemConfig(String code) {
        Map<String, Object> paramMap = MapTools.simpleMap("code", code);
        RemoteService remote = serviceFactory.build(service, "/v1/sysconfigs", BoCloudMethod.GET, headers, paramMap);
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        Object data = result.getData();
        if (null == data) {
            return null;
        }
        return data.toString();
    }

    public BocloudResult<Tenant> getByCode(String groupCode) {
        RemoteService remote = serviceFactory.build(service, "/tenants/getbygroupcode", BoCloudMethod.GET, headers,
                MapTools.simpleMap("groupCode", groupCode));
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, Tenant.class), "success");
    }

    public String getServicePath(Long serviceId) throws Exception {
        RemoteService remote = serviceFactory.build(service, "/v1/permissions/info", BoCloudMethod.GET, headers,
                MapTools.simpleMap("serviceId", serviceId));
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        String data = JSONObject.toJSONString(result.getData());
        Permission permission = JSONObject.parseObject(data, Permission.class);
        return permission != null ? permission.getPath() : null;
    }

    public String getToken(Long targetId, String catalog) {
        Map<String, Object> paramMap = MapTools.simpleMap("targetId", targetId);
        paramMap.put("catalog", catalog);
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/v1/sso/certification",
                BoCloudMethod.GET, headers, paramMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        return String.valueOf(result.getData());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

    public List<Permission> getPermission(String name) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listByName");
        object.put("name", name);
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/v1/permissions/condition", BoCloudMethod.GET, headers, MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        return JSONArray.parseArray(JSONObject.toJSONString(result.getData()), Permission.class);
    }

    public BocloudResult updatePermission(Permission permission) {
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/v1/permissions/" + permission.getId(), BoCloudMethod.PUT, headers, permission);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    public Boolean getLicenseService(String serviceCode){
        JSONObject object = new JSONObject();
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/v1/vendors" , BoCloudMethod.GET, headers, object);
        BocloudResult<JSONObject> invoke = (BocloudResult<JSONObject>) remoteService.invoke();
        JSONObject data = (JSONObject) invoke.getData();
        boolean flag = false;
        if(!data.isEmpty()){
            JSONArray jsonArray = JSONArray.parseArray(data.getString("rows"));
            for (Object o : jsonArray) {
                JSONObject parse = (JSONObject) JSONObject.parse(o.toString());
                if(serviceCode.equals(parse.getString("type"))){
                    flag = true;
                }
            }
        }
        return flag;
    }

    public Boolean isInclude(List<JSONObject> dictionaryList,String code){
        for (JSONObject o : dictionaryList) {
            List<JSONObject> collect = o.getJSONArray("children").toJavaList(JSONObject.class).stream()
                    .filter(i -> (null != i.getString("value") && code.equals(i.getString("value"))))
                    .collect(Collectors.toList());
            if (collect.size() > 0){
                log.info("查询到绑定服务: {}",JSONObject.toJSONString(collect));
                return true;
            }
        }
        return false;
    }

    public Boolean getLicenseServiceCheck(String serviceCode){
        JSONObject object = new JSONObject();
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/v1/vendors" , BoCloudMethod.GET, headers, object);
        BocloudResult<List<JSONObject>> invoke = (BocloudResult<List<JSONObject>>) remoteService.invoke();
        JSONObject data = (JSONObject) invoke.getData();
        if (data.get("rows") != null) {
            JSONArray rows = data.getJSONArray("rows");
            for (Object row : rows) {
                JSONObject vendor = JSONObject.parseObject(row.toString());
                if (serviceCode.equals(vendor.getString("type"))) {
                    return true;
                }
            }
        }
        return false;
    }

}
