package com.bocloud.cos.intf;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.*;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.enums.ApplyCategory;
import com.bocloud.cos.model.BatchServiceFlowBean;
import com.bocloud.cos.model.ServiceFlowBean;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 云服务业务接口
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
public interface CloudServiceService {
    /**
     * 分页查询云服务
     *
     * @param pager
     * @return
     */
    BocloudResult<GridBean<CloudService>> list(Pager pager);

    /**
     * 删除云服务
     *
     * @param id
     * @param userId
     * @return
     */
    Result remove(Long id, Long userId);

    /**
     * 服务详情
     *
     * @param id
     * @return
     */
    BocloudResult<CloudService> detail(Long id);

    /**
     * 添加服务
     *
     * @param cloudService
     * @return
     */
    Result create(CloudService cloudService);

    /**
     * modify
     *
     * @param bean
     * @param id
     * @return
     */
    Result modify(CloudService bean, Long id);

    /**
     * 查询可授权服务范围
     *
     * @param id
     * @param event
     * @param quotaAble
     * @param requestContext
     * @return
     */
    BocloudResult<List<CloudService>> getAbleServices(Long id, String event, Boolean quotaAble,
                                                      RequestContext requestContext);

    /**
     * 修改服务状态
     *
     * @param id
     * @param status
     * @param userId
     * @return
     */
    Result updateStatus(Long id, String status, Long userId);

    @Transactional(rollbackFor = BoCloudDBException.class)
    Result updateTemplate(Long id, String templateName, RequestContext context);

    /**
     * 批量上架服务
     *
     * @param ids
     * @param target
     * @return
     */
    Result batchPublished(List<Long> ids, Long target);

    /**
     * 批量下架服务
     *
     * @param ids
     * @param target
     * @return
     */
    Result batchUnpublished(List<Long> ids, Long target);

    /**
     * 根据code获取cloudService
     *
     * @param codes
     * @return
     */
    BocloudResult<List<CloudService>> listByCode(List<String> codes);

    /**
     * 查询租户服务
     *
     * @param id
     * @param requestContext
     * @return
     */
    BocloudResult<List<CloudService>> getTenantService(Long id, RequestContext requestContext);

    /**
     * 根据condition方式查询服务
     *
     * @param condition
     * @return
     */
    BocloudResult condition(String condition, RequestContext context);

    /**
     * 查询租户授权服务的平台类型列表
     *
     * @param catalogId
     * @param tenant
     * @return
     */
    BocloudResult<List<String>> vendors(Long catalogId, Long tenant);

    /**
     * 获取租户指定平台下的服务
     *
     * @param vendor
     * @param catalogId
     * @param tenant
     * @return
     */
    BocloudResult<List<CloudService>> vendorService(String vendor, Long catalogId, Long tenant);

    /**
     * 服务绑定流程
     *
     * @param id               服务id
     * @param serviceFlowBeans 流程对象数组
     * @param context          请求对象
     * @return
     */
    Result configFlow(Long id, List<ServiceFlowBean> serviceFlowBeans, RequestContext context);

    /**
     * 根据服务id获取绑定的流程
     *
     * @param id       服务id
     * @param category 流程类型
     * @return
     */
    Long getFlowIdByServiceId(Long id, ApplyCategory category);

    /**
     * 服务批量绑定流程
     *
     * @param serviceFlowBeans
     * @param context
     * @return
     */
    Result batchConfigFlow(BatchServiceFlowBean serviceFlowBeans, RequestContext context);
}
