package com.bocloud.cos.intf;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.entity.Document;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author tyl
 * @since 2021/3/8 14:58
 */
public interface DocumentService {

    /**
     * 图片上传
     *
     * @param multipartFile
     * @return
     */
    BocloudResult upload(MultipartFile multipartFile);

    /**
     * 图片删除
     *
     * @param paths
     * @return
     */
    BocloudResult delete(List<String> paths);

    /**
     * 新增文档
     *
     * @param document
     * @param context
     * @return
     */
    BocloudResult create(Document document, RequestContext context);

    /**
     * 查询文档树列表信息
     *
     * @param pager
     * @param context
     * @return
     */
    BocloudResult list(List<Param> paramList, RequestContext context);

    /**
     * 修改文档信息
     *
     * @param id
     * @param document
     * @return
     */
    BocloudResult modify(Long id, Document document, RequestContext context);

    /**
     * 删除使用文档
     *
     * @param id
     * @param context
     * @return
     */
    BocloudResult remove(Long id, RequestContext context);

    /**
     * 展示文档信息(单条)
     *
     * @param id
     * @return
     */
    BocloudResult detail(Long id);

    /**
     * 模糊搜索查询树
     *
     * @param name
     * @param context
     * @return
     */
    BocloudResult search(String name, String category, String status, RequestContext context);

    /**
     * 管理端控制自服务展示
     *
     * @param display
     * @param context
     * @return
     */
    BocloudResult serviceDisplay(Boolean display, RequestContext context);

    /**
     * 获取文档显示状态
     *
     * @param context
     * @return
     */
    BocloudResult getDisplay(RequestContext context);
}
