package com.bocloud.cos.intf;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.cos.entity.ResourceQuota;

import java.util.List;

/**
 * 配额操作类
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年11月23日
 */
public interface ResourceQuotaService {

    /**
     * 删除租户所有配额
     *
     * @param target
     * @param requestContext
     * @return
     */
    Result delete(Long target, RequestContext requestContext);

    /**
     * 删除租户指定配额
     *
     * @param target         租户id
     * @param category       配额类型
     * @param code           配额编码
     * @param requestContext
     * @return
     */
    Result delete(Long target, String category, String code,
                  RequestContext requestContext);

    /**
     * 查询配额
     *
     * @param condition
     * @param code
     * @param target
     * @return
     * @throws Exception
     */
    BocloudResult getQuota(String condition, String code, Long target);

    /**
     * 配额
     *
     * @param tenantId
     * @param quotas
     * @param requestContext
     * @return
     */
    Result assign(Long tenantId, List<ResourceQuota> quotas, RequestContext requestContext);

    /**
     * 配额操作
     *
     * @param quotas   配额目标
     * @param action   操作类型
     * @param tenantId 租户ID
     * @return
     */
    Result quotaAction(List<ResourceQuota> quotas, ResourceQuota.Action action, Long tenantId);

    /**
     * 查询配额信息
     *
     * @param id
     * @param type
     * @param requestContext
     * @return
     */
    BocloudResult<List<ResourceQuota>> quotas(Long id, String type, RequestContext requestContext);

    /**
     * 检查是否存在服务配额使用
     *
     * @param id 租户id
     * @return
     */
    Result checkServiceQuota(Long id);

    /**
     * 获取元数据配额
     *
     * @param requestContext
     * @return
     */
    BocloudResult<List<ResourceQuota>> getMetaQuota(RequestContext requestContext);

    BocloudResult<List<ResourceQuota>> getTenantMetaQuota(Long id, RequestContext requestContext);
}
