package com.bocloud.cos.intf;

import com.bocloud.common.model.*;
import com.bocloud.cos.entity.BillItem;
import com.bocloud.cos.entity.ServiceBill;
import com.bocloud.cos.model.LineChartResult;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.util.List;

/**
 * 账单业务接口
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
public interface ServiceBillService {

    /**
     * 查询列表
     *
     * @param pager
     * @param context
     * @return
     */
    BocloudResult<GridBean<ServiceBill>> list(Pager pager, RequestContext context);

    /**
     * 导出账单列表
     *
     * @param paramList
     * @param requestContext
     * @return
     */
    XSSFWorkbook export(List<Param> paramList, RequestContext requestContext);

    /**
     * 字典查询服务账单
     *
     * @param condition
     * @param context
     * @return
     */
    BocloudResult listByCondition(String condition, RequestContext context);

    /**
     * 创建
     *
     * @param bill
     * @param context
     * @return
     */
    Result create(ServiceBill bill, RequestContext context);

    /**
     * 修改
     *
     * @param bill
     * @param context
     * @return
     */
    Result modify(ServiceBill bill, RequestContext context);

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    BocloudResult<ServiceBill> detail(Long id);

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    Result remove(Long id, RequestContext context);

    /**
     * 查询列表
     *
     * @param pager
     * @param context
     * @return
     */
    BocloudResult<GridBean<BillItem>> items(Pager pager, RequestContext context);

    /**
     * 创建
     *
     * @param item
     * @param context
     * @return
     */
    Result createItem(BillItem item, RequestContext context);

    /**
     * 修改
     *
     * @param item
     * @param context
     * @return
     */
    Result modifyItem(BillItem item, RequestContext context);

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    BocloudResult<BillItem> detailItem(Long id);

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    Result removeItem(Long id, RequestContext context);

    /**
     * 批量保存
     *
     * @param billItems
     * @return
     */
    Result batchSave(List<BillItem> billItems);

    /**
     * 支付
     *
     * @param sn
     * @param couponId
     * @param context
     * @return
     */
    Result pay(String sn, Long couponId, RequestContext context);

    /**
     * 取消支付
     *
     * @param sn
     * @param context
     * @return
     */
    Result cancel(String sn, RequestContext context);

    /**
     * 账单概览
     *
     * @param code    查询类型
     * @param range   概览范围
     * @param context
     * @return
     */
    BocloudResult<LineChartResult> stats(String code, String range, RequestContext context);

    /**
     * 账单概览
     *
     * @param code
     * @param start
     * @param end
     * @param city
     * @param managerId
     * @param tenantId
     * @return
     */
    BocloudResult dashboard(String code, String start, String end, String city, Long managerId, Long tenantId);

    /**
     * 放入回收站产生的费用
     *
     * @param orderItemSn
     * @return
     */
    Result recycleBill(String orderItemSn);

    /**
     * 账单报表
     *
     * @param code       编码
     * @param vendorType 平台类型
     * @param range      统计范围
     * @param tenantId   租户id
     * @param context
     * @return
     */
    BocloudResult report(String code, String vendorType, String range, Long tenantId, RequestContext context);

}
