package com.bocloud.cos.intf;

import com.bocloud.common.model.*;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.model.ChargeResultTrend;
import com.bocloud.cos.model.LineChartResult;
import com.bocloud.cos.model.ServiceOrderInfo;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.util.List;

/**
 * 订单业务接口
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
public interface ServiceOrderService {

    /**
     * 查询订购服务数量
     *
     * @param requestContext
     * @return
     */
    BocloudResult<List<ServiceOrderInfo>> orderCount(RequestContext requestContext, String token);

    /**
     * 查询数量
     *
     * @param params
     * @param context
     * @return
     */
    int count(List<Param> params, RequestContext context);

    /**
     * 查询列表
     *
     * @param pager
     * @param context
     * @return
     */
    BocloudResult<GridBean<ServiceOrder>> list(Pager pager, RequestContext context);

    /**
     * 导出订单
     *
     * @param paramList
     * @param requestContext
     * @return
     */
    XSSFWorkbook export(List<Param> paramList, RequestContext requestContext);

    /**
     * 云主机服务申请趋势
     *
     * @return
     */
    BocloudResult<ChargeResultTrend> vmTrend();

    /**
     * 查询订单top5
     *
     * @param start
     * @param end
     * @param tenant
     * @param zone
     * @return
     */
    BocloudResult<LineChartResult> top5(String start, String end, Boolean tenant, Boolean zone);

    /**
     * 创建
     *
     * @param order
     * @param context
     * @return
     */
    BocloudResult<ServiceOrder> create(ServiceOrder order, RequestContext context);

    /**
     * 修改
     *
     * @param order
     * @param context
     * @return
     */
    Result modify(ServiceOrder order, RequestContext context);

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    BocloudResult<ServiceOrder> detail(Long id);

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    Result remove(Long id, RequestContext context);

    /**
     * 查询列表
     *
     * @param pager
     * @param context
     * @return
     */
    BocloudResult<GridBean<OrderItem>> items(Pager pager, RequestContext context);

    /**
     * 创建
     *
     * @param item
     * @param context
     * @return
     */
    BocloudResult<OrderItem> createItem(OrderItem item, RequestContext context);

    /**
     * 修改
     *
     * @param item
     * @param context
     * @return
     */
    Result modifyItem(OrderItem item, RequestContext context);

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    BocloudResult<OrderItem> detailItem(Long id);

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    Result removeItem(Long id, RequestContext context);

    /**
     * 根据订单类型查询订单列表
     *
     * @param chargeMode
     * @return
     */
    BocloudResult<List<ServiceOrder>> orders(String chargeMode);

    /**
     * 更新订单,用于k8s申请回调
     *
     * @param order
     * @param context
     * @return
     */
    Result updateOrder(ServiceOrder order, RequestContext context);

    /**
     * 支付
     *
     * @param sn
     * @param couponId
     * @param context
     * @return
     */
    Result pay(String sn, Long couponId, RequestContext context);

    /**
     * 取消支付
     *
     * @param sn
     * @param context
     * @return
     */
    Result cancel(String sn, RequestContext context);

    /**
     * 产品统计
     *
     * @param code
     * @return
     */
    BocloudResult<LineChartResult> stats(String code);

    /**
     * 更新订单明细
     *
     * @param sn
     * @param status
     * @param context
     * @return
     */
    Result updateItem(String sn, String status, RequestContext context);

    /**
     * 删除租户相关订单信息
     *
     * @param tenantId
     * @param context
     * @return
     */
    Result removeTenantOrder(Long tenantId, RequestContext context);

    /**
     * 更新element
     *
     * @param sn
     * @param elements
     * @param context
     * @return
     */
    Result updateElements(String sn, String elements, RequestContext context);

    /**
     * 创建变更订单
     *
     * @param sn
     * @param skuId
     * @param context
     * @return
     */
    Result createAlterationItem(String sn, Long skuId, RequestContext context);

    /**
     * 重新分配
     *
     * @param id
     * @param tenantId
     * @param projectId
     * @param userId
     * @param context
     * @return
     */
    BocloudResult<ServiceOrder> redistribute(Long id, Long tenantId, Long projectId, Long userId,
                                             RequestContext context);

    /**
     * 订单字典查询
     *
     * @param condition
     * @param context
     * @return
     */
    BocloudResult listByCondition(String condition, RequestContext context);

    /**
     * 根据sn查询订单信息
     *
     * @param condition
     * @param sn
     * @return
     */
    BocloudResult<ServiceOrder> queryBySn(String condition, String sn);


    /**
     * 检查是否有正在进行中的订单
     *
     * @param sn
     * @return
     */
    String underwayCheck(String sn);
}
