package com.bocloud.cos.intf;

import com.bocloud.common.model.*;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.params.SkuParams;

import java.util.List;

/**
 * 商品SKU业务操作接口
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
public interface ServiceSkuService {

    /**
     * 查询商品SKU列表
     *
     * @param pager
     * @return
     */
    BocloudResult<GridBean<ServiceSku>> list(Pager pager);

    /**
     * 查询商品SKU详情
     *
     * @param id
     * @return
     */
    BocloudResult<ServiceSku> detail(Long id);

    /**
     * 创建商品SKU
     *
     * @param sku
     * @param context
     * @return
     */
    Result create(SkuParams sku, RequestContext context);

    /**
     * 修改商品SKU
     *
     * @param id
     * @param sku
     * @param context
     * @return
     */
    Result modify(Long id, SkuParams sku, RequestContext context);

    /**
     * 删除商品SKU
     *
     * @param id
     * @param managerId
     * @return
     */
    Result remove(Long id, Long managerId);

    /**
     * 通过规格查询
     *
     * @param spec
     * @param categoryId
     * @return
     */
    BocloudResult<ServiceSku> getBySpec(String spec, Long categoryId);

    /**
     * 批量更新sku状态
     *
     * @param ids
     * @param status
     * @param userId
     * @return
     */
    Result updateStatus(List<Long> ids, String status, Long userId);

    BocloudResult<List<ServiceSku>> listByTemplate(Long templateId);

    BocloudResult<List<ServiceSku>> list(List<Param> params);
}
