package com.bocloud.cos.intf;

import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceTenant;
import com.bocloud.cos.model.GrantBean;

import java.util.List;

/**
 * 服务授权
 *
 * @author Administrator
 */
public interface ServiceTenantService {

    /**
     * 租户权限详情
     *
     * @param id
     * @return
     */
    BocloudResult<List<ServiceTenant>> detail(Long id);

    /**
     * 服务批量授权与收权
     *
     * @param serviceIds     服务id
     * @param tenantIds      租户id集合
     * @param type           操作类型
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    Result grantBatch(List<Long> serviceIds, List<Long> tenantIds, String type, RequestContext requestContext) throws InternalServerException;

    /**
     * 单一租户同时授权多个服务
     *
     * @param tenantId
     * @param serviceBeans
     * @param requestContext
     * @return
     */
    Result grantService(Long tenantId, GrantBean serviceBeans, RequestContext requestContext);

    /**
     * 单一服务同时授权多个租户
     *
     * @param serviceId      服务id
     * @param tenantIds      租户id集合
     * @param requestContext
     * @return
     */
    Result grantTenant(Long serviceId, List<Long> tenantIds, RequestContext requestContext);

    /**
     * 根据租户删除租户服务关系表
     *
     * @param id
     * @param requestContext
     * @return
     */
    Result deleteByTenant(Long id, RequestContext requestContext);

    /**
     * 查询租户授权的服务列表
     *
     * @param code
     * @param suffix
     * @param context
     * @return
     */
    BocloudResult<List<CloudService>> listByTenant(String code, String suffix, RequestContext context);

}
