package com.bocloud.cos.intf;

import com.bocloud.common.model.*;
import com.bocloud.cos.entity.InteractiveRecord;
import com.bocloud.cos.entity.WorkOrder;
import com.bocloud.cos.model.StatsResult;
import com.bocloud.cos.params.WorkOrderParams;

import java.util.List;

/**
 * 问题工单业务接口
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月8日
 */
public interface WorkOrderService {

    /**
     * 创建工单
     *
     * @param order
     * @param context
     * @return
     */
    Result create(WorkOrderParams order, RequestContext context);

    /**
     * 删除工单
     *
     * @param id
     * @param context
     * @return
     */
    Result remove(Long id, RequestContext context);

    /**
     * 查询工单详情
     *
     * @param id
     * @return
     */
    BocloudResult<WorkOrder> detail(Long id);

    /**
     * 分页查询工单列表
     *
     * @param pager
     * @param context
     * @return
     */
    BocloudResult<GridBean<WorkOrder>> list(Pager pager, RequestContext context);

    /**
     * 查询租户工单概览
     *
     * @param tenantId
     * @return
     */
    BocloudResult<List<StatsResult>> orderPie(Long tenantId);

    /**
     * 创建问题沟通记录
     *
     * @param id
     * @param record
     * @param context
     * @return
     */
    Result createRecord(Long id, InteractiveRecord record, RequestContext context);

    /**
     * 查询工单问题沟通列表
     *
     * @param orderId
     * @return
     */
    BocloudResult<List<InteractiveRecord>> records(Long orderId);

    /**
     * 修改工单
     *
     * @param id
     * @param order
     * @param context
     * @return
     */
    Result modify(Long id, WorkOrderParams order, RequestContext context);

    /**
     * 工单数量
     *
     * @param params
     * @return
     */
    int count(List<Param> params);

}
