package com.bocloud.cos.operate;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.entity.SkuSpec;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.AlterationParam;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.SpecModel;
import com.bocloud.cos.repository.*;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * paas资源操作抽象类
 *
 * @author tyl
 * @since 2021/5/21 9:42
 */
@Service
public class IaasResourceOperate extends AbstractResourceOperate implements InitializingBean {


    protected IaasResourceOperate(CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
                                  OrderItemRepository itemRepository, ServiceOrderRepository orderRepository,
                                  ServiceCategoryRepository categoryRepository, LockFactory lockFactory,
                                  OrderSnService orderSnService) {
        super(serviceRepository, skuRepository, itemRepository, orderRepository, categoryRepository, lockFactory, orderSnService);
    }

    @Override
    public OrderItem.Catalog getOrderItemCatalog() {
        return null;
    }

    @Override
    public List<OrderItem> buildAlterationItem(String serviceCode, String instance, AlterationParam.Operation operation,
                                               List<ElementModel> elements, Date expiredTime) {
        Assert.notEmpty(elements, "变更内容不能为空");
        ElementModel elementModel = elements.get(0);
        Calendar expiredCalendar = Calendar.getInstance();
        Calendar effectiveCalendar = Calendar.getInstance();
        switch (operation) {
            case AddDisk:
                OrderItem serverRelationItem = itemRepository.queryLastSuccessBySn(instance);
                DurationModel addDuration =
                        JSONObject.parseObject(serverRelationItem.getDuration(), DurationModel.class);
                //预付费
                if (addDuration.getAmount() != null && addDuration.getAmount() > 0) {
                    expiredCalendar.setTime(expiredTime);
                    effectiveCalendar.setTime(new Date());
                    if (addDuration.getMode().equals(DurationModel.Mode.Month)) {
                        int expiredYear = expiredCalendar.get(Calendar.YEAR);
                        int effectiveYear = effectiveCalendar.get(Calendar.YEAR);
                        int expired = expiredCalendar.get(Calendar.MONTH);
                        int effective = effectiveCalendar.get(Calendar.MONTH);
                        addDuration.setAmount((expiredYear - effectiveYear) * 12 + (expired - effective));
                    } else if (addDuration.getMode().equals(DurationModel.Mode.Year)) {
                        int expired = expiredCalendar.get(Calendar.YEAR);
                        int effective = effectiveCalendar.get(Calendar.YEAR);
                        addDuration.setAmount(expired - effective);
                    }
                }
                return this.handlerAddDiskItems(addDuration, elements, expiredTime);
            case EditDisk:
                String uuid = elementModel.getUuid();
                OrderItem diskRelationItem = itemRepository.queryLastSuccessBySn(uuid);
                DurationModel editDuration =
                        JSONObject.parseObject(diskRelationItem.getDuration(), DurationModel.class);
                if (editDuration.getAmount() != null && editDuration.getAmount() > 0) {
                    expiredCalendar.setTime(expiredTime);
                    effectiveCalendar.setTime(new Date());
                    if (editDuration.getMode().equals(DurationModel.Mode.Month)) {
                        int expiredYear = expiredCalendar.get(Calendar.YEAR);
                        int effectiveYear = effectiveCalendar.get(Calendar.YEAR);
                        int expired = expiredCalendar.get(Calendar.MONTH);
                        int effective = effectiveCalendar.get(Calendar.MONTH);
                        editDuration.setAmount((expiredYear - effectiveYear) * 12 + (expired - effective));
                    } else if (editDuration.getMode().equals(DurationModel.Mode.Year)) {
                        int expired = expiredCalendar.get(Calendar.YEAR);
                        int effective = effectiveCalendar.get(Calendar.YEAR);
                        editDuration.setAmount(expired - effective);
                    }
                }
                OrderItem editItem = this.handlerEditItem(editDuration, uuid, elementModel, expiredTime);
                return Lists.newArrayList(editItem);
            case Upgrade:
                OrderItem relationItem = itemRepository.queryLastSuccessBySn(instance);
                DurationModel upDuration = JSONObject.parseObject(relationItem.getDuration(), DurationModel.class);
                if (upDuration.getAmount() != null && upDuration.getAmount() > 0) {
                    expiredCalendar.setTime(expiredTime);
                    effectiveCalendar.setTime(new Date());
                    if (upDuration.getMode().equals(DurationModel.Mode.Month)) {
                        int expiredYear = expiredCalendar.get(Calendar.YEAR);
                        int effectiveYear = effectiveCalendar.get(Calendar.YEAR);
                        int expired = expiredCalendar.get(Calendar.MONTH);
                        int effective = effectiveCalendar.get(Calendar.MONTH);
                        upDuration.setAmount((expiredYear - effectiveYear) * 12 + (expired - effective));
                    } else if (upDuration.getMode().equals(DurationModel.Mode.Year)) {
                        int expired = expiredCalendar.get(Calendar.YEAR);
                        int effective = effectiveCalendar.get(Calendar.YEAR);
                        upDuration.setAmount(expired - effective);
                    }
                }
                OrderItem upgradeItem = handlerUpgradeItem(upDuration, instance, elementModel, elements, expiredTime);
                return Lists.newArrayList(upgradeItem);
            default:
                break;
        }
        return null;
    }

    private List<OrderItem> handlerAddDiskItems(DurationModel duration, List<ElementModel> elements, Date expiredTime) {
        List<OrderItem> items = new ArrayList<>();
        for (ElementModel element : elements) {
            element.setCategory(element.getCategory());
            //变更的服务
            Long alterationSkuId = element.getSkuId();
            String serviceCode = element.getServiceCode();
            CloudService elementService = serviceRepository.queryByCode(serviceCode);
            ServiceSku sku = skuRepository.query(element.getSkuId());
            BigDecimal price = OrderPriceUtils.getUnitPrice(elementService, sku, element.getSpecs(), duration);
            element.setPrice(price);
            //构建新添加磁盘的订单项
            OrderItem addItem = new OrderItem();
            addItem.setSkuId(alterationSkuId);
            String addSn = this.getItemSn(alterationSkuId);
            Assert.notNull(addSn, "生成订购单号失败");
            addItem.setSn(addSn);
            element.setUuid(addSn);
            addItem.setServiceCode(element.getServiceCode());
            addItem.setStatus(OrderItemStatus.Changing);
            addItem.setDuration(JSONObject.toJSONString(duration));
            addItem.setDeleted(false);
            addItem.setResources("[]");
            addItem.setElements(JSONObject.toJSONString(Lists.newArrayList(element)));
            addItem.setBasicPrice(price);
            addItem.setEffectiveTime(new Date());
            addItem.setExpiredTime(expiredTime);
            items.add(addItem);
        }
        return items;
    }

    private OrderItem handlerEditItem(DurationModel duration, String uuid, ElementModel element, Date expiredTime) {
        element.setCategory(element.getCategory());
        //变更的服务
        Long alterationSkuId = element.getSkuId();
        //获取价格
        String serviceCode = element.getServiceCode();
        CloudService elementService = serviceRepository.queryByCode(serviceCode);
        ServiceSku sku = skuRepository.query(alterationSkuId);
        BigDecimal price = OrderPriceUtils.getUnitPrice(elementService, sku, element.getSpecs(), duration);
        element.setPrice(price);
        //构建新添加磁盘的订单项
        OrderItem orderItem = new OrderItem();
        orderItem.setSkuId(alterationSkuId);
        orderItem.setSn(uuid);
        orderItem.setServiceCode(element.getServiceCode());
        orderItem.setStatus(OrderItemStatus.Changing);
        orderItem.setDuration(JSONObject.toJSONString(duration));
        orderItem.setDeleted(false);
        orderItem.setResources("[]");
        orderItem.setElements(JSONObject.toJSONString(Lists.newArrayList(element)));
        orderItem.setBasicPrice(price);
        orderItem.setEffectiveTime(new Date());
        orderItem.setExpiredTime(expiredTime);
        return orderItem;
    }

    private OrderItem handlerUpgradeItem(DurationModel duration, String instance, ElementModel element, List<ElementModel> elements,
                                         Date expiredTime) {
        element.setCategory(element.getCategory());
        //变更的服务
        Long alterationSkuId = element.getSkuId();
        ServiceSku sku = skuRepository.query(alterationSkuId);
        // 处理规格计算
        List<SpecModel> specModels = element.getSpecs();
        Integer cpuTotal;
        int memTotal;
        if (specModels == null || specModels.isEmpty()) {
            List<SkuSpec> skuSpecs = JSONArray.parseArray(sku.getSpec(), SkuSpec.class);
            cpuTotal = skuSpecs.stream().filter(skuSpec -> "cpu".equals(skuSpec.getSpecName()))
                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
            memTotal = skuSpecs.stream().filter(skuSpec -> "memory".equals(skuSpec.getSpecName()))
                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
            List<SpecModel> elementSpec = new ArrayList<>();
            SpecModel cpuSpec = new SpecModel();
            cpuSpec.setCpu(cpuTotal);
            elementSpec.add(cpuSpec);
            SpecModel memSpec = new SpecModel();
            memSpec.setMemory(BigDecimal.valueOf(memTotal));
            elementSpec.add(memSpec);
            element.setSpecs(elementSpec);
        }
        String serviceCode = element.getServiceCode();
        CloudService elementService = serviceRepository.queryByCode(serviceCode);
        BigDecimal price = OrderPriceUtils.getUnitPrice(elementService, sku, element.getSpecs(), duration);
        element.setPrice(price);
        //构建变更规格的订单项
        OrderItem upgradeItem = new OrderItem();
        upgradeItem.setSkuId(alterationSkuId);
        upgradeItem.setSn(instance);
        upgradeItem.setServiceCode(element.getServiceCode());
        upgradeItem.setStatus(OrderItemStatus.Changing);
        upgradeItem.setDuration(JSONObject.toJSONString(duration));
        upgradeItem.setDeleted(false);
        upgradeItem.setResources("[]");
        upgradeItem.setElements(JSONObject.toJSONString(Lists.newArrayList(elements)));
        upgradeItem.setBasicPrice(price);
        upgradeItem.setEffectiveTime(new Date());
        upgradeItem.setExpiredTime(expiredTime);
        return upgradeItem;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // 云主机
        ResourceOperateFactory.register("vmware.standard.server", this);
        ResourceOperateFactory.register("sangfor.standard.server", this);
        ResourceOperateFactory.register("smartx.standard.server", this);
        ResourceOperateFactory.register("openstack.standard.server", this);
        ResourceOperateFactory.register("aliyun.standard.server", this);
        ResourceOperateFactory.register("huawei.standard.server", this);
        ResourceOperateFactory.register("fusioncloud.standard.server", this);
        ResourceOperateFactory.register("manageone.standard.server", this);
        ResourceOperateFactory.register("tce.standard.server", this);
        ResourceOperateFactory.register("tencent.standard.server", this);
        ResourceOperateFactory.register("azure.standard.server", this);
        ResourceOperateFactory.register("aws.standard.server", this);
        ResourceOperateFactory.register("fusionsphere.standard.server", this);
        //VPC
        ResourceOperateFactory.register("vmware.nsx.vpc", this);
        ResourceOperateFactory.register("openstack.standard.vpc", this);
        ResourceOperateFactory.register("tce.standard.vpc", this);
        ResourceOperateFactory.register("manageone.standard.vpc", this);
        ResourceOperateFactory.register("huawei.standard.vpc", this);
        ResourceOperateFactory.register("tencent.standard.vpc", this);
        ResourceOperateFactory.register("aliyun.standard.vpc", this);
        ResourceOperateFactory.register("azure.standard.vpc", this);
        ResourceOperateFactory.register("aws.standard.vpc", this);
        // EIP
        ResourceOperateFactory.register("manageone.standard.eip", this);
        ResourceOperateFactory.register("openstack.standard.eip", this);
        ResourceOperateFactory.register("fusioncloud.standard.eip", this);
        ResourceOperateFactory.register("network.aliyun.eip", this);
        ResourceOperateFactory.register("network.huawei.eip", this);
        ResourceOperateFactory.register("network.tencent.eip", this);
        ResourceOperateFactory.register("network.tce.eip", this);
        ResourceOperateFactory.register("network.qcloud.eip", this);
        ResourceOperateFactory.register("network.aws.eip", this);
        //SLB
        ResourceOperateFactory.register("network.aliyun.slb", this);
        ResourceOperateFactory.register("network.aws.slb", this);
        // K8S
        ResourceOperateFactory.register("vmware.k8s.cluster", this);
    }
}
