package com.bocloud.cos.operate;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.AlterationParam;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.SpecModel;
import com.bocloud.cos.repository.*;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * paas资源操作抽象类
 *
 * @author tyl
 * @since 2021/5/21 9:42
 */
@Service
public class PaasResourceOperate extends AbstractResourceOperate implements InitializingBean {

    protected PaasResourceOperate(CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
                                  OrderItemRepository itemRepository, ServiceOrderRepository orderRepository,
                                  ServiceCategoryRepository categoryRepository, LockFactory lockFactory,
                                  OrderSnService orderSnService) {
        super(serviceRepository, skuRepository, itemRepository, orderRepository, categoryRepository, lockFactory, orderSnService);
    }

    @Override
    public OrderItem.Catalog getOrderItemCatalog() {
        return OrderItem.Catalog.complex;
    }

    @Override
    public List<OrderItem> buildAlterationItem(String serviceCode, String instance, AlterationParam.Operation operation,
                                               List<ElementModel> elements, Date expiredTime) {
        Assert.notEmpty(elements, "变更内容不能为空");
        if (operation == AlterationParam.Operation.Upgrade) {
            ElementModel element = elements.get(0);
            OrderItem upOldItem = itemRepository.queryLastSuccessBySn(instance);
            DurationModel upDuration = JSONObject.parseObject(upOldItem.getDuration(), DurationModel.class);
            //预付费
            if (upDuration.getAmount() != null && upDuration.getAmount() > 0) {
                Calendar expiredCalendar = Calendar.getInstance();
                Calendar effectiveCalendar = Calendar.getInstance();
                expiredCalendar.setTime(expiredTime);
                effectiveCalendar.setTime(new Date());
                if (upDuration.getMode().equals(DurationModel.Mode.Month)) {
                    int expiredYear = expiredCalendar.get(Calendar.YEAR);
                    int effectiveYear = effectiveCalendar.get(Calendar.YEAR);
                    int expired = expiredCalendar.get(Calendar.MONTH);
                    int effective = effectiveCalendar.get(Calendar.MONTH);
                    upDuration.setAmount((expiredYear - effectiveYear) * 12 + (expired - effective));
                } else if (upDuration.getMode().equals(DurationModel.Mode.Year)) {
                    int expired = expiredCalendar.get(Calendar.YEAR);
                    int effective = effectiveCalendar.get(Calendar.YEAR);
                    upDuration.setAmount(expired - effective);
                }
            }
            return handlerUpgradeItem(instance, upDuration, element, elements, expiredTime);
        }
        return null;
    }

    private List<OrderItem> handlerUpgradeItem(String instance, DurationModel duration, ElementModel element,
                                               List<ElementModel> elements, Date expiredTime) {
        Long elementSkuId = element.getSkuId();
        ServiceSku sku = skuRepository.query(elementSkuId);
        CloudService skuService = serviceRepository.query(sku.getServiceId());
        List<SkuSpec> skuSpecs = JSONArray.parseArray(sku.getSpec(), SkuSpec.class);
        List<OrderItem> result = new ArrayList<>();
        ServiceCategory serviceCategory = categoryRepository.query(sku.getCategoryId());
        JSONObject object = JSONObject.parseObject(serviceCategory.getProps());
        Integer elementCount = 1;
        if (null != object) {
            elementCount = object.getInteger("nodes");
        }
        element.setCount(elementCount);
        element.setCategory(element.getCategory());
        String sn;
        if ("vmware.storage.disk".equals(skuService.getCode())) {
            // 数据盘升级
            Integer dataDisk = skuSpecs.stream().filter(skuSpec -> "disk".equals(skuSpec.getSpecName()))
                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
            List<SpecModel> specModels = new ArrayList<>();
            SpecModel diskSpec = new SpecModel();
            diskSpec.setDisk(dataDisk);
            specModels.add(diskSpec);
            element.setSpecs(specModels);
            OrderItem diskItem = itemRepository.queryLastDiskSn(instance);
            sn = diskItem.getSn();
        } else {
            Integer cpuTotal = skuSpecs.stream().filter(skuSpec -> "cpu".equals(skuSpec.getSpecName()))
                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
            Integer memTotal = skuSpecs.stream().filter(skuSpec -> "memory".equals(skuSpec.getSpecName()))
                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
//            Integer sysDiskTotal = skuSpecs.stream().filter(skuSpec -> "disk".equals(skuSpec.getSpecName()))
//                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
            List<SpecModel> specModels = new ArrayList<>();
            SpecModel cpuSpec = new SpecModel();
            cpuSpec.setCpu(cpuTotal);
            specModels.add(cpuSpec);
            SpecModel memSpec = new SpecModel();
            memSpec.setMemory(BigDecimal.valueOf(memTotal));
            specModels.add(memSpec);
//            SpecModel diskSpec = new SpecModel();
//            diskSpec.setDisk(sysDiskTotal);
//            specModels.add(diskSpec);
            element.setSpecs(specModels);
            sn = instance;
        }
        BigDecimal price = OrderPriceUtils.getUnitPrice(skuService, sku, element.getSpecs(), duration);
        element.setPrice(price);
        // 构建变更规格的订单项
        OrderItem upgradeItem = new OrderItem();
        Assert.notNull(sn, "旧订单号查询失败");
        upgradeItem.setSn(sn);
        upgradeItem.setSkuId(elementSkuId);
        upgradeItem.setServiceCode(element.getServiceCode());
        upgradeItem.setStatus(OrderItemStatus.Changing);
        upgradeItem.setDuration(JSONObject.toJSONString(duration));
        upgradeItem.setDeleted(false);
        upgradeItem.setResources("[]");
        upgradeItem.setElements(JSONObject.toJSONString(Lists.newArrayList(elements)));
        upgradeItem.setEffectiveTime(new Date());
        upgradeItem.setExpiredTime(expiredTime);
        upgradeItem.setBasicPrice(price.multiply(BigDecimal.valueOf(elementCount)));
        result.add(upgradeItem);
        return result;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        ResourceOperateFactory.register("rds.mysql", this);
        ResourceOperateFactory.register("rds.sqlserver", this);
        ResourceOperateFactory.register("rds.postgresql", this);
        ResourceOperateFactory.register("kvs.redis", this);
        ResourceOperateFactory.register("kvs.mongodb", this);
        ResourceOperateFactory.register("cms.kafka", this);
        ResourceOperateFactory.register("cms.rabbitmq", this);
    }

}
