package com.bocloud.cos.operate;

import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * 资源操作工厂
 * 区分不同的实现类的目的是为了在资源操作时调用不同的服务
 *
 * @author zhangyf
 * @since 2021-12-02
 */
public class ResourceOperateFactory {

    private ResourceOperateFactory() {

    }

    private static final Map<String, ResourceOperate> RESOURCE_OPERATE_REPO = new HashMap<>();

    /**
     * 服务编码和实例注册
     *
     * @param serviceCode
     * @param resourceOperate
     */
    public static void register(String serviceCode, ResourceOperate resourceOperate) {
        synchronized (RESOURCE_OPERATE_REPO) {
            if (RESOURCE_OPERATE_REPO.containsKey(serviceCode)) {
                throw new IllegalArgumentException("ResourceOperate already registered that provides service code: " + serviceCode);
            }
            RESOURCE_OPERATE_REPO.put(serviceCode, resourceOperate);
        }
    }

    /**
     * 根据服务编码获取实例
     *
     * @param serviceCode 服务编码
     * @return
     */
    public static ResourceOperate getInstance(String serviceCode) {
        assert StringUtils.isNoneBlank(serviceCode);
        if (serviceCode.startsWith("operations.")) {
            return ResourceOperateFactory.RESOURCE_OPERATE_REPO.get("operations.");
        }
        if (serviceCode.startsWith("fabric.")) {
            return ResourceOperateFactory.RESOURCE_OPERATE_REPO.get("fabric.");
        }
        return ResourceOperateFactory.RESOURCE_OPERATE_REPO.get(serviceCode);
    }

}
