package com.bocloud.cos.operate;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.AlterationParam;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.repository.*;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author tyl
 * @since 2021/5/21 10:40
 */
@Service
public class StorageOperate extends IaasResourceOperate {

    protected StorageOperate(CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository, OrderItemRepository itemRepository,
                             ServiceOrderRepository orderRepository, ServiceCategoryRepository categoryRepository, LockFactory lockFactory,
                             OrderSnService orderSnService) {
        super(serviceRepository, skuRepository, itemRepository, orderRepository, categoryRepository, lockFactory, orderSnService);
    }

    @Override
    public List<OrderItem> buildAlterationItem(String serviceCode, String instance, AlterationParam.Operation operation,
                                               List<ElementModel> elements, Date expiredTime) {
        //查询旧的订单
        OrderItem oldItem = itemRepository.queryLastSuccessBySn(instance);
        DurationModel duration = JSONObject.parseObject(oldItem.getDuration(), DurationModel.class);
        Assert.notEmpty(elements, "变更内容不能为空");
        //变更的服务
        switch (operation) {
            case AddDisk:
                return this.handlerAddDiskItems(oldItem, duration, elements);
            case EditDisk:
                OrderItem orderItem = this.handlerEditItem(oldItem, duration, elements);
                return Lists.newArrayList(orderItem);
            default:
                break;
        }
        return null;
    }

    private List<OrderItem> handlerAddDiskItems(OrderItem oldItem, DurationModel duration,
                                                List<ElementModel> elements) {
        List<OrderItem> items = new ArrayList<>();
        for (ElementModel element : elements) {
            Long elementSkuId = element.getSkuId();
            element.setCategory(element.getCategory());
            CloudService elementService = serviceRepository.queryByCode(element.getServiceCode());
            ServiceSku sku = skuRepository.query(elementSkuId);
            BigDecimal price = OrderPriceUtils.getUnitPrice(elementService, sku, element.getSpecs(), duration);
            element.setPrice(price);
            // 构建新添加磁盘的订单项
            OrderItem addDiskItem = new OrderItem();
            addDiskItem.setSkuId(elementSkuId);
            String addItemSn = this.getItemSn(elementSkuId);
            Assert.notNull(addItemSn, "生成订购单号失败");
            addDiskItem.setSn(addItemSn);
            element.setUuid(addItemSn);
            addDiskItem.setOrderId(oldItem.getOrderId());
            addDiskItem.setServiceCode(element.getServiceCode());
            addDiskItem.setStatus(OrderItemStatus.Changing);
            addDiskItem.setDuration(oldItem.getDuration());
            addDiskItem.setDeleted(false);
            addDiskItem.setResources("[]");
            addDiskItem.setElements(JSONObject.toJSONString(Lists.newArrayList(element)));
            addDiskItem.setEffectiveTime(new Date());
            addDiskItem.setExpiredTime(oldItem.getExpiredTime());
            addDiskItem.setBasicPrice(price);
            items.add(addDiskItem);
        }
        return items;
    }

    private OrderItem handlerEditItem(OrderItem oldItem, DurationModel duration, List<ElementModel> elements) {
        ElementModel element = elements.get(0);
        element.setCategory(element.getCategory());
        //变更的服务
        Long elementSkuId = element.getSkuId();
        CloudService alteraService = serviceRepository.queryByCode(element.getServiceCode());
        ServiceSku sku = skuRepository.query(elementSkuId);
        BigDecimal price = OrderPriceUtils.getUnitPrice(alteraService, sku, element.getSpecs(), duration);
        element.setPrice(price);
        // 不开启计费则子订单计费为0
        if (!alteraService.getBillable()) {
            price = BigDecimal.ZERO;
        }
        //构建新添加磁盘的订单项
        OrderItem orderItem = new OrderItem();
        orderItem.setSkuId(elementSkuId);
        orderItem.setSn(oldItem.getSn());
        orderItem.setServiceCode(element.getServiceCode());
        orderItem.setStatus(OrderItemStatus.Opening);
        orderItem.setDuration(oldItem.getDuration());
        orderItem.setOrderId(orderItem.getOrderId());
        orderItem.setDeleted(false);
        orderItem.setResources("[]");
        orderItem.setElements(JSONObject.toJSONString(Lists.newArrayList(elements)));
        orderItem.setBasicPrice(price);
        return orderItem;
    }

    @Override
    public void afterPropertiesSet() {
        // 存储
        ResourceOperateFactory.register("openstack.standard.volume", this);
        ResourceOperateFactory.register("smartx.standard.volume", this);
        ResourceOperateFactory.register("manageone.standard.volume", this);
        ResourceOperateFactory.register("fusioncloud.standard.volume", this);
        ResourceOperateFactory.register("aliyun.standard.volume", this);
        ResourceOperateFactory.register("tce.standard.cbs", this);
        ResourceOperateFactory.register("huawei.standard.volume", this);
        ResourceOperateFactory.register("tencent.standard.volume", this);
        ResourceOperateFactory.register("azure.standard.volume", this);
        ResourceOperateFactory.register("aws.standard.volume", this);
        ResourceOperateFactory.register("fusionsphere.standard.volume", this);
    }
}
