package com.bocloud.cos.operate;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.*;
import com.bocloud.cos.repository.*;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @since 2021/5/21 15:16
 */
@Service
public class TaskResourceOperate extends AbstractResourceOperate implements InitializingBean {

    final Logger logger = LoggerFactory.getLogger(PaasResourceOperate.class);

    protected TaskResourceOperate(CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
                                  OrderItemRepository itemRepository, ServiceOrderRepository orderRepository,
                                  ServiceCategoryRepository categoryRepository, LockFactory lockFactory,
                                  OrderSnService orderSnService) {
        super(serviceRepository, skuRepository, itemRepository, orderRepository, categoryRepository, lockFactory,
                orderSnService);
    }

    @Override
    public List<OrderItem> buildApplyItem(String serviceCode, EmptionModel emptionModel, List<ElementModel> elements) {
        List<OrderItem> items = new ArrayList<>();
        try {
            DurationModel duration = emptionModel.getDuration();
            // 订购数量
            Integer count = emptionModel.getCount();
            Assert.isTrue(null != count && count > 0, "订购数量不能小于0");
            //构建新的订单项
            OrderItem orderItem = new OrderItem();
            orderItem.setSkuId(null);
            String itemSn = this.getItemSn(null);
            Assert.notNull(itemSn, "生成订购单号失败");
            orderItem.setSn(itemSn);
            orderItem.setServiceCode(serviceCode);
            orderItem.setStatus(OrderItemStatus.Opening);
            orderItem.setDuration(JSONObject.toJSONString(duration));
            orderItem.setDeleted(false);
            orderItem.setResources("[]");
            orderItem.setElements("[]");
            CloudService cloudService = serviceRepository.queryByCode(serviceCode);
            BigDecimal basicPrice = getBasicPrice(duration, cloudService);
            orderItem.setBasicPrice(basicPrice);
            items.add(orderItem);
        } catch (Exception e) {
            logger.error("build  apply item error", e);
        }
        return items;
    }

    @Override
    public OrderItem.Catalog getOrderItemCatalog() {
        return null;
    }

    @Override
    public List<OrderItem> buildAlterationItem(String serviceCode, String instance, AlterationParam.Operation operation,
                                               List<ElementModel> elements, Date expiredTime) {
        return null;
    }

    private BigDecimal getBasicPrice(DurationModel duration, CloudService cloudService) {
        List<ServicePrice> servicePrices = JSONArray.parseArray(cloudService.getBasicPrice(), ServicePrice.class);
        if (!cloudService.getBillable() || servicePrices == null || servicePrices.isEmpty()) {
            return BigDecimal.ZERO;
        }
        Map<String, ServicePrice> specBasicMap = servicePrices.stream().collect(Collectors.toMap(ServicePrice::getSpecName,
                Function.identity()));
        ServicePrice servicePrice = null;
        for (Map.Entry<String, ServicePrice> entry : specBasicMap.entrySet()) {
            if (null != entry.getValue()){
                servicePrice = entry.getValue();
                break;
            }
        }
        if (servicePrice == null) {
            return BigDecimal.ZERO;
        }
        DurationModel.Mode mode = duration.getMode();
        BigDecimal basicPrice;
        switch (mode) {
            case Year:
                basicPrice = servicePrice.getYearPrice();
                break;
            case Month:
                basicPrice = servicePrice.getMonthPrice();
                break;
            default:
                basicPrice = BigDecimal.ZERO;
                break;
        }
        return basicPrice;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        ResourceOperateFactory.register("operations.", this);
        ResourceOperateFactory.register("fabric.", this);
    }

}
