package com.bocloud.cos.service;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.cos.entity.Icon;
import com.bocloud.cos.intf.IconService;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.IconRepository;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Service
public class IconServiceImpl implements IconService {
    private static final Logger logger = LoggerFactory.getLogger(IconServiceImpl.class);

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private IconRepository iconRepository;
    @Autowired
    private CloudServiceRepository cloudServiceRepository;

    @Override
    public BocloudResult list(int page, int rows, List<Param> params, Map<String, String> sorter, boolean simple) {
        GridBean gridBean = null;
        try {
            if (null == params) {
                params = new ArrayList<Param>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            int total = this.iconRepository.count(params);
            List<Icon> list = this.iconRepository.list(page, rows, params, sorter);
            gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query notice icon list fail:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(Icon icon) {
        try {
            this.iconRepository.save(icon);
            return new BocloudResult(true, icon, "添加成功");
        } catch (Exception e) {
            logger.error("Create notice icon error:", e);
            return new BocloudResult(false, "添加失败");
        }
    }

    @Override
    public BocloudResult remove(Long id) {
        String path = Icon.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if (cloudServiceRepository.isExists(id)) {
                return new BocloudResult(false, "图标使用中，不可删除");
            }
            Icon icon = iconRepository.query(id);
            if (icon.getReserved().equals(true)) {
                return new BocloudResult(false, "此图标不可删除");
            } else {
                this.iconRepository.delete(id);
                return new BocloudResult(true, "删除成功");
            }
        } catch (Exception e) {
            logger.error("Remove notice icon fail:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

}
