package com.bocloud.cos.service;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.cos.entity.OrderSn;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.repository.OrderItemRepository;
import com.bocloud.cos.repository.OrderSnRepository;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.concurrent.TimeUnit;

/**
 * 订单号业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年5月9日
 */
@Service
public class OrderSnServiceImpl implements OrderSnService {

    private static final Logger logger = LoggerFactory.getLogger(OrderSnServiceImpl.class);
    @Autowired
    private OrderSnRepository orderSnRepository;
    @Autowired
    private OrderItemRepository itemRepository;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public BocloudResult<String> generate(Long skuId) {
        String path = OrderSn.class.getSimpleName() + "_sku_" + skuId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(2, TimeUnit.MINUTES), "获取子订单号请求超时");
            OrderSn orderSn = orderSnRepository.query(skuId);
            Assert.notNull(orderSn, "订单号不存在");
            String catalogSn = orderSn.getCatalogSn();
            String serviceSn = orderSn.getServiceSn();
            String categorySn = orderSn.getCategorySn();
            String skuSn = orderSn.getSkuSn();
            Long nextNum = null == orderSn.getCurrentNum() ? 0 : orderSn.getCurrentNum();
            String sn = null;
            boolean flag = true;
            while (flag) {
                nextNum++;
                sn = catalogSn + serviceSn + categorySn + skuSn + String.format("%010d", nextNum);
                if (!itemRepository.existBySn(sn)) {
                    flag = false;
                }
            }
            orderSn.setCurrentNum(nextNum);
            orderSnRepository.modify(orderSn.getId(), nextNum);
            return new BocloudResult<>(true, sn, "生成成功");
        } catch (Exception e) {
            logger.error("Generate order sn fail:", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }
}
