package com.bocloud.cos.service;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.entity.PortalPanel;
import com.bocloud.cos.intf.PortalPanelService;
import com.bocloud.cos.repository.PortalPanelRepository;
import com.bocloud.cos.internal.SmsUserService;
import com.bocloud.cos.internel.entity.User;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author lyy
 */
@Service
public class PortalPanelServiceImpl implements PortalPanelService {
    private static final Logger logger = LoggerFactory.getLogger(PortalPanelServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private PortalPanelRepository portalPanelRepository;
    @Autowired
    private SmsUserService smsUserService;

    @Override
    public BocloudResult list(String serverName, RequestContext context) {
        try {
            List<PortalPanel> portalPanels;
            if (CMC.equals(serverName)) {
                portalPanels = this.portalPanelRepository.listPortalPanel(serverName);
            } else {
                BocloudResult<User> userResult = smsUserService.detail(context.getTarget());
                if (userResult.isFailed() || null == userResult.getData()) {
                    return new BocloudResult(false, "当前用户存在问题");
                }
                User loginUser = userResult.getData();
                portalPanels = this.portalPanelRepository.list(serverName, loginUser.getTenantId(), null);
            }
            return new BocloudResult(true, portalPanels, "查询成功");
        } catch (Exception e) {
            logger.error("Query Portal Panel error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    @Transactional
    public BocloudResult reset(String serverName, RequestContext context) {
        String path = PortalPanel.class.getSimpleName() + "_" + serverName;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            List<PortalPanel> portalPanels = portalPanelRepository.queryDefaultByServerName(serverName);
            if (portalPanels.isEmpty()) {
                return new BocloudResult(false, "无出厂数据，无法恢复");
            }
            for (PortalPanel portalPanel : portalPanels) {
                portalPanel.setId(null);
                portalPanel.setCreatorId(context.getTarget());
                portalPanel.setGmtCreate(new Date());
            }
            portalPanelRepository.removeByServerName(serverName, context.getTarget());
            for (PortalPanel entity : portalPanels) {
                portalPanelRepository.save(entity);
            }
            return new BocloudResult(true, "恢复成功");
        } catch (Exception e) {
            logger.error("reset portal panel fail:", e);
            throw new RuntimeException(e.getMessage(), e);
        }
    }

    @Override
    public BocloudResult save(List<PortalPanel> portalPanels, String serverName, Long userId, Long tenantId,
                              Long projectId) {
        try {
            Map<Long, PortalPanel> old = Maps.newHashMap();
            List<PortalPanel> panels;
            if (CMC.equals(serverName)) {
                panels = portalPanelRepository.queryByServerName(serverName);
            } else {
                panels = portalPanelRepository.queryByServerName(serverName, tenantId);
            }
            for (PortalPanel portalPanel : panels) {
                old.put(portalPanel.getId(), portalPanel);
            }
            for (PortalPanel portalPanel : portalPanels) {
                if (portalPanel.getId() == null) {
                    portalPanel.setCreatorId(userId);
                    portalPanel.setGmtCreate(new Date());
                    portalPanel.setDeleted(false);
                    portalPanel.setServerName(serverName);
                    portalPanel.setTenantId(tenantId);
                    portalPanel.setProjectId(projectId);
                    // 没有id 走新增
                    portalPanelRepository.save(portalPanel);
                    continue;
                }
                if (old.containsKey(portalPanel.getId())) {
                    PortalPanel oldPanel = old.get(portalPanel.getId());
                    oldPanel.setGmtModify(new Date());
                    oldPanel.setMenderId(userId);
                    oldPanel.setConfig(portalPanel.getConfig());
                    oldPanel.setLocation(portalPanel.getLocation());
                    portalPanelRepository.update(oldPanel);
                    old.remove(portalPanel.getId());
                }
            }
            for (PortalPanel portalPanel : old.values()) {
                portalPanelRepository.remove(portalPanel.getId());
            }
        } catch (Exception e) {
            return new BocloudResult(false, "保存失败");
        }

        return new BocloudResult(true, "保存成功");
    }

}
