package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ApplyCategory;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.enums.ResourceCategory;
import com.bocloud.cos.internel.entity.*;
import com.bocloud.cos.intf.RecycledService;
import com.bocloud.cos.intf.ResourceUnsubscribeService;
import com.bocloud.cos.model.CancelationParam;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.internal.*;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
public class RecycledServiceImpl implements RecycledService {
    private static Logger logger = LoggerFactory.getLogger(RecycledServiceImpl.class);
    @Autowired
    private RecycledRepository recycledRepository;
    @Autowired
    private OrderItemRepository itemRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ImsServerService imsServerService;
    @Autowired
    private PmsInternalService pmsInternalService;
    @Autowired
    private ImsInternalService imsInternalService;
    @Autowired
    private ResourceUnsubscribeService resourceUnsubscribeService;
    @Autowired
    private SmsProjectService smsProjectService;
    @Autowired
    private OrderItemRepository orderItemRepository;
    @Autowired
    private CloudServiceRepository cloudServiceRepository;
    @Autowired
    private ServiceFlowConfigRepository serviceFlowConfigRepository;
    @Autowired
    private ServiceOrderRepository serviceOrderRepository;
    @Autowired
    private FmsFlowService fmsFlowService;

    @Override
    public BocloudResult list(int page, int rows, List<Param> params, Map<String, String> sorter,
                              RequestContext context) {
        try {
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(new HashMap<>());
            int total = 0;
            List<Recycled> list;
            if (context.getCatalog().equals(Catalog.Tenant)) {
                // 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
                List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
                List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());

                total = this.recycledRepository.count(null, context.getTenant(), projectIds, params);
                list = this.recycledRepository.list(page, rows, null, context.getTenant(), projectIds, params, sorter);
            } else if (context.getCatalog().equals(Catalog.User)) {
                // 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
                List<Long> projectIds = new ArrayList<>();
                List<Long> userManagerProjectsIds =
                        userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                List<Long> visibleProjectsIds =
                        visibleProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!visibleProjectsIds.isEmpty()) {
                    projectIds.addAll(visibleProjectsIds);
                }
                total = this.recycledRepository.count(context.getTarget(), null, projectIds, params);
                list = this.recycledRepository.list(page, rows, context.getTarget(), null, projectIds, params, sorter);
            } else {
                total = this.recycledRepository.count(params, null);
                list = this.recycledRepository.list(page, rows, sorter, params, null);
            }
            for (Recycled recycled : list) {
                String sn = recycled.getSn();
                Long resourceId = recycled.getResourceId();
                String type = recycled.getType();
                if (type.contains("-")) {
                    type = StringUtils.substringBefore(type, "-");
                }
                switch (ResourceCategory.valueOf(type)) {
                    case MySQL:
                    case PostgreSQL:
                    case SqlServer:
                    case Oracle:
                        Rds rdsDetail = pmsInternalService.rdsDetail(resourceId);
                        if (rdsDetail != null) {
                            recycled.setAccessIp(rdsDetail.getAddress());
                        }
                        break;
                    case MongoDB:
                    case Redis:
                    case MemCache:
                        Kvs kvsDetail = pmsInternalService.kvsDetail(resourceId);
                        if (kvsDetail != null) {
                            recycled.setAccessIp(kvsDetail.getAddress());
                        }
                        break;
                    case Kafka:
                    case RabbitMQ:
                    case ActiveMQ:
                    case RocketMQ:
                        Cms cmsDetail = pmsInternalService.cmsDetail(resourceId);
                        if (cmsDetail != null) {
                            recycled.setAccessIp(cmsDetail.getAddress());
                        }
                        break;
                    case Computer:
                        CloudServer server = imsInternalService.queryBySn(sn);
                        if (server != null) {
                            recycled.setVendorType(server.getVendorType());
                            recycled.setPrivateIps(server.getPrivateIps());
                            recycled.setManagerIp(server.getManagerIp());
                        }
                        break;
                    default:
                        break;
                }
            }
            GridBean gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Icon.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Recycled recycled = recycledRepository.query(id);
            if (null == recycled) {
                return new BocloudResult<>(false, "当前记录不存在");
            }
            OrderItem orderItem = itemRepository.queryBySn(recycled.getSn());
            if (OrderItemStatus.Unsubscribing.equals(orderItem.getStatus())) {
                return new BocloudResult<>(true, "该资源正在退订中，无法操作");
            }
            CancelationParam cancelationParam = new CancelationParam();
            cancelationParam.setInstance(recycled.getSn());
            Result cancelation = resourceUnsubscribeService.process(cancelationParam, context);
            if (cancelation.isSuccess()) {
                recycledRepository.remove(id, context.getTarget());
            }
            return new BocloudResult(true, "删除任务已下发.....");
        } catch (Exception e) {
            logger.error("Remove data fail:", e);
            return new BocloudResult(false, "删除失败 : " + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String sn, Long userId) {
        String path = Icon.class.getSimpleName() + "_" + sn;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            recycledRepository.remove(sn, userId);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove data fail:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult clear(RequestContext context) {
        try {
            List<Param> params = Lists.newArrayList();
            List<Recycled> list = new ArrayList<>();
            if (context.getCatalog().equals(Catalog.Tenant)) {
                // 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
                List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
                List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
                list = this.recycledRepository
                        .list(1, Integer.MAX_VALUE, null, context.getTenant(), projectIds, params, new HashMap<>());
            } else if (context.getCatalog().equals(Catalog.User)) {
                // 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
                List<Long> projectIds = new ArrayList<>();
                List<Long> userManagerProjectsIds =
                        userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                List<Long> visibleProjectsIds =
                        visibleProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!visibleProjectsIds.isEmpty()) {
                    projectIds.addAll(visibleProjectsIds);
                }
                list = this.recycledRepository
                        .list(1, Integer.MAX_VALUE, context.getTarget(), null, projectIds, params, new HashMap<>());
            }
            logger.info("list size : " + list.size());
            for (Recycled recycled : list) {
                try {
                    CancelationParam cancelationParam = new CancelationParam();
                    cancelationParam.setInstance(recycled.getSn());
                    OrderItem item = orderItemRepository.queryBySn(cancelationParam.getInstance());
                    Assert.notNull(item, "订单未找到");
                    CloudService service = cloudServiceRepository.queryByCode(item.getServiceCode());
                    Assert.notNull(service, "该服务不存在或者未发布");
                    Long flowId = serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.UnsubscribeOperate);
                    if (null == flowId) {
                        Result result = resourceUnsubscribeService.process(cancelationParam, context);
                        if (result.isFailed()) {
                            logger.error(result.getMessage());
                        }
                        // recycledRepository.remove(recycled.getId(), context.getTarget());
                        continue;
                    }
                    Map<String, Object> content = getParams(ApplyCategory.UnsubscribeOperate.name(), JSONObject.toJSONString(cancelationParam), "申请审核服务实例【" + cancelationParam.getInstance() + "】退订", context);
                    ServiceOrder serviceOrder = serviceOrderRepository.query(item.getOrderId());

                    BocloudResult execution = fmsFlowService.startInstanceById(flowId, context, serviceOrder.getProjectId(), content);
                    if (execution.isFailed()) {
                        logger.info("服务实例退订审核流程启动失败");
                    } else {
                        logger.info("服务实例退订审核流程启动成功");
                        // recycledRepository.remove(recycled.getId(), context.getTarget());
                    }
                } catch (RuntimeException e) {
                    logger.error(e.getMessage(), e);
                }
            }
            return new BocloudResult(true, "回收站清空任务已全部下发！");
        } catch (Exception e) {
            logger.error("clear data fail:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult create(Recycled recycled, RequestContext context) {
        try {
            if (null == recycled.getCreatorId()) {
                recycled.setCreatorId(0L);
            }
            recycled.setDeleted(false);
            recycled.setGmtCreate(new Date());
            recycledRepository.save(recycled);
            return new BocloudResult(true, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult restore(Long resourceId, Long id, String type, RequestContext context) {
        BocloudResult result = null;
        if (-1 != type.indexOf("-")) {
            type = StringUtils.substringBefore(type, "-");
        }
        try {
            switch (ResourceCategory.valueOf(type)) {
                case MySQL:
                case PostgreSQL:
                case SqlServer:
                case Oracle:
                    result = pmsInternalService.rdsRestore(resourceId);
                    break;
                case MongoDB:
                case Redis:
                case MemCache:
                    result = pmsInternalService.kvsRestore(resourceId);
                    break;
                case Kafka:
                case RabbitMQ:
                case ActiveMQ:
                case RocketMQ:
                    result = pmsInternalService.cmsRestore(resourceId);
                    break;
                case Zookeeper:
                case Etcd:
                    break;
                case Nginx:
                    break;
                case Tomcat:
                case Apache:
                case Weblogic:
                case Websphere:
                    break;
                case Computer:
                    result = imsServerService.vmsRestore(resourceId, context);
                    break;
                default:
                    break;
            }
            // sonar : result可能为null
            if (null == result){
                return new BocloudResult(false, "不支持的操作");
            }
            if (result.isSuccess()) {
                recycledRepository.remove(id, context.getTarget());
            }
        } catch (Exception e) {
            logger.error("restore data fail:", e);
            return new BocloudResult(false, "还原失败");
        }
        return result;
    }

    private Map<String, Object> getParams(String category, String params, String theme, RequestContext context) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("ApplyCategory", category);
        map.put("ApplyParams", params);
        map.put("ApplyTheme", theme);
        map.put("ApplyContext", context);
        return map;
    }
}
