package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.model.Result;
import com.bocloud.cos.context.AlterationContext;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ApplyCategory;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.cos.enums.ChargeCommon.ChargeMode;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.enums.QuotaCategory;
import com.bocloud.cos.internal.FmsFlowService;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.intf.ResourceAlterationService;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.intf.ServiceOrderService;
import com.bocloud.cos.model.AlterationParam;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.QuotaBean;
import com.bocloud.cos.model.SpecModel;
import com.bocloud.cos.operate.PaasResourceOperate;
import com.bocloud.cos.operate.ResourceOperate;
import com.bocloud.cos.operate.ResourceOperateFactory;
import com.bocloud.cos.operate.ServiceResourceMapping;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;
import java.util.Map.Entry;
import java.util.concurrent.TimeUnit;

/**
 * 服务操作业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月15日
 */
@Service
public class ResourceAlterationServiceImpl implements ResourceAlterationService {

    private static final Logger logger = LoggerFactory.getLogger(ResourceAlterationServiceImpl.class);
    private final ServiceOrderRepository serviceOrderRepository;
    private final AmqpTemplate amqpTemplate;
    private final OrderItemRepository orderItemRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceCategoryRepository serviceCategoryRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final SmsTenantService tenantService;
    private final ResourceQuotaService resourceQuotaService;
    private final ServiceBillRepository billRepository;
    private final LockFactory lockFactory;
    private final ServiceFlowConfigRepository serviceFlowConfigRepository;
    private final FmsFlowService fmsFlowService;
    private final ServiceOrderService orderService;

    public ResourceAlterationServiceImpl(ServiceOrderRepository serviceOrderRepository, AmqpTemplate amqpTemplate,
                                         OrderItemRepository orderItemRepository, CloudServiceRepository cloudServiceRepository,
                                         ServiceCategoryRepository serviceCategoryRepository, ServiceSkuRepository serviceSkuRepository,
                                         ResourceQuotaRepository resourceQuotaRepository, SmsTenantService tenantService,
                                         ResourceQuotaService resourceQuotaService, ServiceBillRepository billRepository,
                                         LockFactory lockFactory, ServiceFlowConfigRepository serviceFlowConfigRepository,
                                         FmsFlowService fmsFlowService, ServiceOrderService orderService) {
        this.serviceOrderRepository = serviceOrderRepository;
        this.amqpTemplate = amqpTemplate;
        this.orderItemRepository = orderItemRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.serviceCategoryRepository = serviceCategoryRepository;
        this.serviceSkuRepository = serviceSkuRepository;
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.tenantService = tenantService;
        this.resourceQuotaService = resourceQuotaService;
        this.billRepository = billRepository;
        this.lockFactory = lockFactory;
        this.serviceFlowConfigRepository = serviceFlowConfigRepository;
        this.fmsFlowService = fmsFlowService;
        this.orderService = orderService;
    }

    private Map<String, Object> getParams(String category, String params, String theme, RequestContext context) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("ApplyCategory", category);
        map.put("ApplyParams", params);
        map.put("ApplyTheme", theme);
        map.put("ApplyContext", context);
        return map;
    }

    @Override
    @Transactional
    public Result process(AlterationParam alterationParam, RequestContext context) {
        /** 根据sn查询是否有处于中间状态的订单项（如变更中、延期中、退订中等），如有，则直接RETURN **/
        String result = orderService.underwayCheck(alterationParam.getInstance());
        if (null != result) {
            return new BocloudResult<>(false, result);
        }

        AlterationContext model = this.alterCheck(alterationParam, context);
        ServiceOrder order = model.getOrder();
        CloudService service = model.getService();
        OrderItem oldItem = model.getItem();
        String code = service.getCode();
        // 获取资源变更操作对象 生成变更订单项
        List<OrderItem> alterationItems = model.getAlterationItems();
        ResourceOperate alterationOperate = model.getAlterationOperate();
        ServiceOrder newOrder = new ServiceOrder();
        newOrder.setServiceCode(service.getCode());
        // 生成订单号，增加分布式锁
        String path = IdGenerator.class.getSimpleName() + "_ordersn";
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
            newOrder.setSn(IdGenerator.getInstance().nextId());
        } catch (Exception e) {
            throw new RuntimeException("生成订单号失败：" + e.getMessage(), e);
        }
        newOrder.setChargeMode(order.getChargeMode());
        newOrder.setSlaAgreed(order.getSlaAgreed());
        if (ChargeMode.PostCharge.equals(order.getChargeMode())) {
            newOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
        } else {
            newOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
        }
        newOrder.setType(ChargeCommon.Type.alterat);
        newOrder.setGmtCreate(new Date());
        newOrder.setAutoRenew(order.getAutoRenew());
        newOrder.setVendorId(order.getVendorId());
        newOrder.setBusinessId(order.getBusinessId());
        newOrder.setProjectId(order.getProjectId());
        newOrder.setUserId(order.getUserId());
        newOrder.setTenantId(order.getTenantId());
        newOrder.setResourceStatus(ChargeCommon.ResourceStatus.changing);
        serviceOrderRepository.save(newOrder);
        Tenant tenant = tenantService.detail(order.getTenantId());
        switch (alterationParam.getOperation()) {
            case Upgrade:
                if (alterationOperate instanceof PaasResourceOperate) {
                    for (OrderItem alterationItem : alterationItems) {
                        List<ElementModel> elementModels =
                                JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                        if ("vmware.storage.disk".equals(alterationItem.getServiceCode())) {
                            // 解决中间件配置升级计费不正确问题
                            // 旧订单
                            List<ElementModel> oldElementModels = JSON.parseArray(oldItem.getElements(), ElementModel.class);
                            ElementModel oldElementModel = oldElementModels.get(0);
                            ElementModel diskElement = elementModels.get(0);
                            // 设置集群内实例数
                            Integer count = Optional.ofNullable(oldElementModel.getCount()).orElse(1);
                            diskElement.setCount(count);
                            List<SpecModel> diskSpecModel = diskElement.getSpecs();
                            Integer disk =
                                    diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                            .reduce(Integer::sum).orElse(0);
                            alterationParam.setDataDisk(disk);
                            // 重新设置基础价格
                            alterationItem.setBasicPrice(alterationItem.getBasicPrice().multiply(new BigDecimal(count.toString())));
                        } else {
                            ElementModel upgradeElement = elementModels.get(0);
                            List<SpecModel> specs = upgradeElement.getSpecs();
                            Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                                    .reduce(Integer::sum).orElse(0);
                            BigDecimal memory = specs.stream()
                                    .map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                    .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
//                            Integer disk = specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
//                                    .reduce(Integer::sum).orElse(0);
                            alterationParam.setUpgradeSkuId(upgradeElement.getSkuId());
                            alterationParam.setCpu(cpu);
                            alterationParam.setMemory(memory);
//                            alterationParam.setSystemDisk(disk);
                        }
                        alterationItem.setElements(JSONObject.toJSONString(elementModels));
                        alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                        alterationItem.setOrderId(newOrder.getId());
                        orderItemRepository.save(alterationItem);
                    }
                } else {
                    // 获取升级规格 设置升级后的cpu内存大小到变更参数里面
                    OrderItem upgradeItem = alterationItems.get(0);
                    List<ElementModel> elementModels =
                            JSONArray.parseArray(upgradeItem.getElements(), ElementModel.class);
                    ElementModel element = elementModels.get(0);
                    List<SpecModel> specs = element.getSpecs();
                    Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
                    BigDecimal memory =
                            specs.stream().map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                    .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                    alterationParam.setUpgradeSkuId(element.getSkuId());
                    alterationParam.setCpu(cpu);
                    alterationParam.setMemory(memory);
                    Integer systemDisk = specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
                    if (!systemDisk.equals(0)) {
                        alterationParam.setSystemDisk(systemDisk);
                    }
                    // 订单项设置折扣
                    elementModels.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                    upgradeItem.setElements(JSONObject.toJSONString(elementModels));
                    upgradeItem.setFinalPrice(upgradeItem.getBasicPrice().multiply(tenant.getDiscount()));
                    upgradeItem.setOrderId(newOrder.getId());
                    orderItemRepository.save(upgradeItem);
                }
                break;
            case EditDisk:
                OrderItem editItem = alterationItems.get(0);
                List<ElementModel> editDiskElements =
                        JSONArray.parseArray(editItem.getElements(), ElementModel.class);
                editDiskElements.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                editItem.setElements(JSONObject.toJSONString(editDiskElements));
                editItem.setFinalPrice(editItem.getBasicPrice().multiply(tenant.getDiscount()));
                editItem.setOrderId(newOrder.getId());
                orderItemRepository.save(editItem);
                break;
            case AddDisk:
                int diskTotal = 0;
                for (OrderItem alterationItem : alterationItems) {
                    List<ElementModel> diskElements =
                            JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                    ElementModel diskElement = diskElements.get(0);
                    List<SpecModel> diskSpecModel = diskElement.getSpecs();
                    Integer disk = diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
                    diskTotal += disk;
                    // 订单项设置折扣
                    diskElements.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                    alterationItem.setElements(JSONObject.toJSONString(diskElements));
                    alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                    alterationItem.setOrderId(newOrder.getId());
                    // 保存新订单
                    orderItemRepository.save(alterationItem);
                }
                alterationParam.setDataDisk(diskTotal);
                break;
            default:
                break;
        }

        alterationParam.setContext(context);
        alterationParam.setCategory(ServiceResourceMapping.getResourceCategory(code).name());
        alterationParam.setService(code);
        alterationParam.setOldStatus(oldItem.getStatus());
        alterationParam.setNewInstance(alterationItems.get(0).getSn());
        alterationParam.setOrderSn(newOrder.getSn());
        newOrder.setApplyParam(JSONObject.toJSONString(alterationParam));
        Long flowId =
                serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.AlterationOperate);
        if (null != flowId) {
            for (OrderItem alterationItem : alterationItems) {
                alterationItem.setStatus(OrderItemStatus.Audit);
                orderItemRepository.update(alterationItem);
            }
            newOrder.setOrderStatus(ChargeCommon.OrderStatus.audit);
            serviceOrderRepository.update(newOrder);
            logger.info("【服务变更】申请服务实例参数: {}", JSONObject.toJSONString(alterationItems));
            Map<String, Object> content =
                    getParams(ApplyCategory.AlterationOperate.name(), JSONObject.toJSONString(alterationParam),
                            "申请审核服务实例【" + alterationParam.getInstanceName() + "】变更", context);
            content.put("order", JSONObject.toJSONString(newOrder));
            content.put("requestContext", JSONObject.toJSONString(context));
            BocloudResult execution = fmsFlowService.startInstanceById(flowId, context, newOrder.getProjectId(), content);
            if (execution.isFailed()) {
                newOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                serviceOrderRepository.update(newOrder);
                throw new RuntimeException("服务实例变更审核流程启动失败");
            }
            return new Result(true, "服务实例变更审核流程启动成功");
        } else {

            // 变更操作 配额差值
            List<QuotaBean> quotaBeans = new ArrayList<>();
            // 变更后订单项的配额
            Map<String, Integer> alterationQuota = OrderItemUtils.getItemQuota(alterationItems);
            Map<String, Integer> oldQuota;
            oldQuota = OrderItemUtils.getItemQuota(Lists.newArrayList(oldItem));
            logger.info("服务变更上下文: {}", JSON.toJSONString(model));
            logger.info("变更后订单项的配额: {}", JSON.toJSONString(alterationQuota));
            logger.info("变更前订单项得配额: {}", JSON.toJSONString(oldQuota));
            for (Entry<String, Integer> entry : alterationQuota.entrySet()) {
                String quotaCode = entry.getKey();
                // 旧配额
                Integer number = Optional.ofNullable(oldQuota.get(quotaCode)).orElse(0);
                // 新配额和旧配额得差值
                quotaBeans.add(new QuotaBean(quotaCode, alterationQuota.get(quotaCode) - number));
                alterationQuota.put(quotaCode, alterationQuota.get(quotaCode) - number);
            }
            alterationParam.setQuotas(quotaBeans);
            // 校验配额并更新配额
            logger.info("校验配额并且更新配额的量：" + JSON.toJSONString(alterationQuota));
            handleQuota(alterationQuota, context.getTenant(), false);

            serviceOrderRepository.update(newOrder);
            // 预付费生成未支付账单
            if (ChargeMode.PreCharge == newOrder.getChargeMode()) {
                return new BocloudResult<>(true, newOrder.getSn(), "账单已生成，请支付");
            }
            if (alterationOperate instanceof PaasResourceOperate) {
                amqpTemplate.convertAndSend("paas.resource.modification.queue", alterationParam);
                logger.info("发送预付费变更消息到 paas.resource.modification.queue ,Param: {}", JSONObject.toJSONString(alterationParam));
            } else {
                amqpTemplate.convertAndSend("iaas.resource.modification.queue", alterationParam);
                logger.info("发送预付费变更消息到 iaas.resource.modification.queue ,Param: {}", JSONObject.toJSONString(alterationParam));
            }
            return new BocloudResult<>(true, "申请已提交");
        }
    }

    /**
     * 服务变更校验
     *
     * @param params
     * @param context
     * @return
     */
    private AlterationContext alterCheck(AlterationParam params, RequestContext context) {
        Assert.notNull(params.getOperation(), "请设置服务变更类型");
        // 服务资源实例id
        OrderItem oldItem = null;
        switch (params.getOperation()) {
            case Upgrade:
            case AddDisk:
                oldItem = orderItemRepository.queryBySn(params.getInstance());
                break;
            case EditDisk:
                List<ElementModel> upgrade = params.getUpgrade();
                ElementModel elementModel = upgrade.get(0);
                oldItem = orderItemRepository.queryBySn(elementModel.getUuid());
                break;
            default:
                break;
        }
        Assert.notNull(oldItem, "原订单明细未找到！");
        CloudService service = cloudServiceRepository.queryByCode(params.getService());
        Assert.notNull(service, "订单明细关联服务未找到！");
        Assert.isTrue("PUBLISHED".equals(service.getStatus()), "服务未发布");
        logger.info("旧订单elements：{}", JSONObject.toJSON(oldItem.getElements()));
        ServiceOrder oldOrder = serviceOrderRepository.query(oldItem.getOrderId());
        Assert.notNull(oldOrder, "原订单未找到！");
        if (ChargeMode.PreCharge == oldOrder.getChargeMode()) {
            Long tenantId = null;
            Long userId = null;
            if (Catalog.User == context.getCatalog()) {
                userId = context.getTarget();
            } else if (Catalog.Tenant == context.getCatalog()) {
                tenantId = context.getTenant();
            }
            int count = billRepository
                    .count(ChargeCommon.BillStatus.unpay.name(), ChargeMode.PreCharge.name(), tenantId, userId,
                            oldOrder.getId());
            if (count > 0) {
                throw new RuntimeException("当前订单存在未支付的账单，请先支付");
            }
        }
        Tenant tenant = tenantService.detail(context.getTenant());
        Assert.notNull(tenant, "申请租户不存在");
        ServiceCategory category = null;
        if (null != oldItem.getSkuId() && 0L != oldItem.getSkuId()) {
            ServiceSku sku = serviceSkuRepository.query(oldItem.getSkuId());
            Assert.notNull(sku, "订单明细规格未找到！");
            category = serviceCategoryRepository.query(sku.getCategoryId());
            Assert.notNull(category, "订单明细关联服务类型未找到！");
        }
        ResourceOperate alterationOperate = ResourceOperateFactory.getInstance(params.getService());
        Assert.notNull(alterationOperate, service.getName() + "服务不支持变更操作");
        List<OrderItem> alterationItems = alterationOperate.buildAlterationItem(params.getService(), params.getInstance(),
                params.getOperation(), params.getUpgrade(), params.getExpiredTime());
        Map<String, Integer> alterationQuota = OrderItemUtils.getItemQuota(alterationItems);
        Map<String, Integer> oldQuota;
        if ((alterationOperate instanceof PaasResourceOperate)) {
            ElementModel elementModel = params.getUpgrade().get(0);
            String serviceCode = elementModel.getServiceCode();
            /*List<OrderItem> oldItems =
                    orderItemRepository.listByOrderId(oldOrder.getId(), elementModel.getServiceCode());
            oldQuota = OrderItemUtils.getItemQuota(oldItems);*/
            // 1. 根据资源sn找到资源《订购》的订单
            ServiceOrder applyOrder = serviceOrderRepository.queryOrderBySnAndType(params.getInstance(), ChargeCommon.Type.apply.name());
            if (null == applyOrder) {
                throw new IllegalArgumentException("旧订单未找到");
            }
            // 2。根据serviceCode找到变更资源的sn
            List<OrderItem> orderItems = orderItemRepository.listByOrderId(applyOrder.getId(), serviceCode);
            if (null == orderItems || orderItems.isEmpty()) {
                throw new IllegalArgumentException("旧订单项未找到");
            }

            OrderItem applyItem = orderItems.get(0);
            // 3. 根据变更资源的sn找到最新有效的变更资源的订单项
            // 3.1 最近订单项
            oldItem = orderItemRepository.queryLastSuccessBySn(applyItem.getSn());
            if (null == oldItem) {
                throw new IllegalArgumentException("最近订单项未找到");
            }
            // 3.2 最近订单
            oldOrder = serviceOrderRepository.query(oldItem.getOrderId());
            if (null == oldOrder) {
                throw new IllegalArgumentException("最近订单未找到");
            }
            // 4。生成旧配额
            oldQuota = OrderItemUtils.getItemQuota(Lists.newArrayList(oldItem));
        } else {
            oldQuota = OrderItemUtils.getItemQuota(Lists.newArrayList(oldItem));
        }
        for (Entry<String, Integer> entry : alterationQuota.entrySet()) {
            String code = entry.getKey();
            Integer number = Optional.ofNullable(oldQuota.get(code)).orElse(0);
            alterationQuota.put(code, alterationQuota.get(code) - number);
        }
        // 校验并扣除配额
        this.handleQuota(alterationQuota, context.getTenant(), true);
        AlterationContext model =
                new AlterationContext(oldOrder, oldItem, service, category, alterationOperate, alterationItems, tenant);
        return model;
    }


    /**
     * 扣除配额
     *
     * @param quotaMap
     * @param tenantId
     * @param isCheck
     * @return
     * @throws Exception
     */
    private void handleQuota(Map<String, Integer> quotaMap, Long tenantId, boolean isCheck) {
        Tenant tenant = tenantService.detail(tenantId);
        if (!tenant.getQuotaAble()) {
            return;
        }
        if (null == quotaMap || quotaMap.isEmpty()) {
            // 无配额信息需更新
            logger.debug("quotaMap is null or empty! 无配额信息需更新");
        }
        List<ResourceQuota> list = resourceQuotaRepository.listByTenantId(tenantId);
        Assert.notEmpty(list, "未查询到配额信息");
        Map<String, ResourceQuota> map = new HashMap<>(8);
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (Entry<String, Integer> entry : quotaMap.entrySet()) {
            String code = entry.getKey();
            if (!"vmware.storage.disk".equals(code)) {
                String quotaName;
                if (QuotaCategory.CPU.getCode().equals(code)) {
                    quotaName = QuotaCategory.CPU.getName();
                } else if (QuotaCategory.MEM.getCode().equals(code)) {
                    quotaName = QuotaCategory.MEM.getName();
                } else if (QuotaCategory.DISK.getCode().equals(code)) {
                    quotaName = QuotaCategory.DISK.getName();
                } else {
                    CloudService cloudService = cloudServiceRepository.queryByCode(code);
                    quotaName = cloudService.getName();
                }
                if (!map.containsKey(entry.getKey())) {
                    throw new RuntimeException("未查询到" + quotaName + "配额信息");
                }
                ResourceQuota quota = map.get(entry.getKey());
                if (entry.getValue() + quota.getUsed() > quota.getQuota()) {
                    throw new RuntimeException("剩余" + quotaName + "配额不足");
                }
                quota.setQuotaActionValue(entry.getValue().longValue());
                quotas.add(quota);
            }
        }
        if (isCheck) {
            return;
        }
        logger.info("要扣除的配额数量：" + JSON.toJSONString(quotas));
        Result result = resourceQuotaService.quotaAction(quotas, ResourceQuota.Action.Order, tenantId);
        if (result.isFailed()) {
            throw new RuntimeException(result.getMessage());
        }
    }

    @Override
    public Result k8sAlteration(AlterationParam params) {
        try {
            logger.info("接收到k8s缩容消息：{}", JSONObject.toJSONString(params));
            OrderItem item = orderItemRepository.queryBySn(params.getInstance());
            Assert.notNull(item, "子订单明细不存在");
            ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
            Assert.notNull(order, "订单不存在");
            Catalog catalog = Catalog.Tenant;
            Long target = order.getTenantId();
            if (null != order.getUserId()) {
                catalog = Catalog.User;
                target = order.getUserId();
            }
            RequestContext context = new RequestContext(catalog, target, order.getTenantId(), null, null);
            return this.process(params, context);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, e.getMessage());
        }
    }

}
