package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.encrypt.Encryptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.context.ApplicationContext;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.*;
import com.bocloud.cos.enums.ChargeCommon.ChargeMode;
import com.bocloud.cos.internal.FmsFlowService;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.internel.entity.*;
import com.bocloud.cos.intf.CosService;
import com.bocloud.cos.intf.ResourceApplicationService;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.intf.RosService;
import com.bocloud.cos.model.*;
import com.bocloud.cos.model.DurationModel.Mode;
import com.bocloud.cos.operate.*;
import com.bocloud.cos.params.CloneNetworkCardParams;
import com.bocloud.cos.params.CloneParams;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;
import java.util.Map.Entry;
import java.util.concurrent.TimeUnit;

/**
 * 服务订购
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月15日
 */
@Service
public class ResourceApplicationServiceImpl implements ResourceApplicationService {

    private static final Logger logger = LoggerFactory.getLogger(ResourceApplicationServiceImpl.class);
    private final ServiceOrderRepository serviceOrderRepository;
    private final AmqpTemplate amqpTemplate;
    private final OrderItemRepository orderItemRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceCategoryRepository serviceCategoryRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final ServiceSlaRepository serviceSlaRepository;
    private final SlaInstanceRepository slaInstanceRepository;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final SmsTenantService tenantService;
    private final ResourceQuotaService resourceQuotaService;
    private final ImsInternalService imsInternalService;
    private final CosService cosService;
    private final RosService rosService;
    private final LockFactory lockFactory;
    private final TenantAccountRepository accountRepository;
    private final ServiceFlowConfigRepository serviceFlowConfigRepository;
    private final FmsFlowService fmsFlowService;

    public ResourceApplicationServiceImpl(ServiceOrderRepository serviceOrderRepository, AmqpTemplate amqpTemplate,
                                          OrderItemRepository orderItemRepository, CloudServiceRepository cloudServiceRepository,
                                          ServiceCategoryRepository serviceCategoryRepository, ServiceSkuRepository serviceSkuRepository,
                                          ServiceSlaRepository serviceSlaRepository, SlaInstanceRepository slaInstanceRepository,
                                          ResourceQuotaRepository resourceQuotaRepository, SmsTenantService tenantService,
                                          ResourceQuotaService resourceQuotaService, ImsInternalService imsInternalService,
                                          CosService cosService, RosService rosService, LockFactory lockFactory,
                                          TenantAccountRepository accountRepository, ServiceFlowConfigRepository serviceFlowConfigRepository,
                                          FmsFlowService fmsFlowService) {
        this.serviceOrderRepository = serviceOrderRepository;
        this.amqpTemplate = amqpTemplate;
        this.orderItemRepository = orderItemRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.serviceCategoryRepository = serviceCategoryRepository;
        this.serviceSkuRepository = serviceSkuRepository;
        this.serviceSlaRepository = serviceSlaRepository;
        this.slaInstanceRepository = slaInstanceRepository;
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.tenantService = tenantService;
        this.resourceQuotaService = resourceQuotaService;
        this.imsInternalService = imsInternalService;
        this.cosService = cosService;
        this.rosService = rosService;
        this.lockFactory = lockFactory;
        this.accountRepository = accountRepository;
        this.serviceFlowConfigRepository = serviceFlowConfigRepository;
        this.fmsFlowService = fmsFlowService;
    }

    @Override
    @Transactional
    public BocloudResult process(ApplicationParam applicationParam, RequestContext context) {
        logger.info("接收到订购消息：{}", JSONObject.toJSONString(applicationParam));
        // 检验ip状态并预占
        List<String> ipList = new ArrayList<>();
        JSONObject configs = applicationParam.getConfigs();
        JSONArray networkCardConfigs = configs.getJSONArray("networkCardConfigs");
        if (networkCardConfigs.size() >0) {
            for (Object networkCardConfig : networkCardConfigs) {
                JSONObject ipConfig = JSONObject.parseObject(networkCardConfig.toString());
                if ("Manual".equals(ipConfig.getString("ipPolicy"))) {
                    String address = ipConfig.getString("address");
                    JSONArray objects = JSONArray.parseArray(address);
                    for (int i =0; i < objects.size(); i++) {
                        ipList.add(objects.getString(i));
                    }
                }
            }
        }
        if (CollectionUtils.isNotEmpty(ipList)) {
            JSONObject ipString = new JSONObject();
            ipString.put("preempt", JSONObject.toJSONString(ipList));
            BocloudResult bocloudResult = imsInternalService.preemptOrFreeIp(JSONObject.toJSONString(ipString));
            logger.info("资源申请预占用ip结果：" + JSONObject.toJSONString(bocloudResult));
            if (bocloudResult.isFailed()) {
                return new BocloudResult(false, bocloudResult.getMessage());
            }
        }
        // 申请校验规格 校验服务是否存在等
        ApplicationContext applicationContext = buildApplicationContext(applicationParam, context);
        // 生成订单
        ServiceOrder serviceOrder = this.createOrder(applicationContext, context);
        // 预生成生成订单项，计算配额并校验
        List<OrderItem> orderItems = applicationContext.getOperate().buildApplyItem(applicationContext.getService().getCode(),
                applicationParam.getEmption(), applicationParam.getElements());
        serviceOrder.setItems(orderItems);
        applicationContext.setServiceOrder(serviceOrder);

        // 校验配额信息
        this.validateQuota(applicationContext);

        // 保存订单项
        this.saveOrderItems(applicationContext);

        // 保存SLA信息
        if (null != applicationContext.getSla()) {
            SlaInstance slaInstance = new SlaInstance(applicationContext.getSla().getName(), applicationContext.getSla().getContent(),
                    new Date(), context.getTarget(), applicationContext.getSla().getServiceId(), applicationContext.getSlaId(),
                    applicationContext.getServiceOrder().getId());
            slaInstanceRepository.save(slaInstance);
        }
        // 封装资源申请的必要参数到 applicationParam.configs
        this.warpApplicationConfig(applicationContext);

        applicationParam.setOrder(applicationContext.getServiceOrder());
        applicationParam.setContext(context);
        Long flowId = serviceFlowConfigRepository.getServiceFlowId(applicationContext.getService().getId(), ApplyCategory.ApplicationOperate);

        Boolean noFlow = Optional.ofNullable(applicationContext.getParams().getConfigs().getBoolean("noFlow")).orElse(false);
        if (null != flowId && !noFlow) {
            // 订单、订单项状态变更为审批中
            orderItems.forEach(item -> {
                item.setStatus(OrderItemStatus.Audit);
                orderItemRepository.update(item);
            });
            applicationContext.getServiceOrder().setOrderStatus(ChargeCommon.OrderStatus.audit);
            serviceOrderRepository.update(applicationContext.getServiceOrder());
            Map<String, Object> content = getParams(ApplyCategory.ApplicationOperate.name(), JSONObject.toJSONString(applicationParam),
                    "申请审核服务【" + applicationContext.getService().getName() + "】使用", context);
            content.put("order", JSONObject.toJSONString(applicationContext.getServiceOrder()));
            content.put("requestContext", JSONObject.toJSONString(context));
            BocloudResult execution = fmsFlowService.startInstanceById(flowId, context, applicationParam.getLocation().getProjectId(), content);
            if (execution.isFailed()) {
                throw new RuntimeException("服务申请使用审核流程启动失败");
            }
            return new BocloudResult(true, "服务申请使用审核流程启动成功");
        } else {

            // 扣除配额
            if (null != applicationContext.getResourceQuotas() && !applicationContext.getResourceQuotas().isEmpty()) {
                resourceQuotaService.quotaAction(applicationContext.getResourceQuotas(), ResourceQuota.Action.Order,applicationContext.getTenant().getId());
            }

            // 下发到底层服务的时候按年计费时间做特殊处理
            if (Mode.Year == applicationContext.getParams().getEmption().getDuration().getMode()) {
                EmptionModel paramEmption = applicationParam.getEmption();
                DurationModel dura = paramEmption.getDuration();
                dura.setAmount(dura.getAmount() * 12);
                dura.setMode(Mode.Month);
                paramEmption.setDuration(dura);
                applicationParam.setEmption(paramEmption);
            }
            applicationContext.getServiceOrder().setApplyParam(JSONObject.toJSONString(applicationParam));
            serviceOrderRepository.update(applicationContext.getServiceOrder());
            if (ChargeMode.PreCharge == applicationContext.getServiceOrder().getChargeMode()) {
                applicationContext.getServiceOrder().setOrderStatus(ChargeCommon.OrderStatus.unpay);
                serviceOrderRepository.update(applicationContext.getServiceOrder());
                return new BocloudResult<>(true, applicationContext.getServiceOrder().getSn(), "申请已提交,支付订单后资源即可创建");
            }
            // 后付费
            applicationContext.getServiceOrder().setOrderStatus(ChargeCommon.OrderStatus.paid);
            serviceOrderRepository.update(applicationContext.getServiceOrder());
            if (applicationContext.getService().getCode().startsWith("operations.")) {
                cosService.create(applicationParam, applicationParam.getContext());
            } else if (applicationContext.getService().getCode().startsWith("fabric.")) {
                rosService.create(applicationParam, applicationParam.getContext());
            } else if ("vmware.nsx.vpc".equalsIgnoreCase(applicationContext.getService().getCode())) {
                logger.info("后付费NSX-T Vpc申请参数结构：" + JSONObject.toJSONString(applicationParam));
                amqpTemplate.convertAndSend("network.creation.queue", applicationParam);
            } else {
                List<ApplicationParam> messageParams = Lists.newArrayList(applicationParam);
                logger.info("后付费资源申请参数结构：" + JSONObject.toJSONString(messageParams));
                if (applicationContext.getOperate() instanceof PaasResourceOperate) {
                    amqpTemplate.convertAndSend("paas.resource.creation.queue", messageParams);
                } else if (applicationContext.getOperate() instanceof IaasResourceOperate) {
                    amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                } else {
                    amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                }
            }
            return new BocloudResult<>(true, "申请已提交,请等待资源创建。");
        }
    }

    /**
     * 生成订单
     *
     * @param applicationContext
     * @param context
     * @return
     */
    private ServiceOrder createOrder(ApplicationContext applicationContext, RequestContext context) {
        ServiceOrder order = new ServiceOrder();
        order.setServiceCode(applicationContext.getService().getCode());
        // 生成订单号
        String orderSn = this.generateSn();
        order.setSn(orderSn);
        ChargeMode chargeMode = ChargeCommon.ChargeMode.PostCharge;
        if (Mode.Hour != applicationContext.getParams().getEmption().getDuration().getMode()) {
            chargeMode = ChargeCommon.ChargeMode.PreCharge;
        }
        order.setChargeMode(chargeMode);
        order.setSlaAgreed(null != applicationContext.getSlaId() && 0L != applicationContext.getSlaId());
        if (ChargeMode.PostCharge == chargeMode) {
            order.setOrderStatus(ChargeCommon.OrderStatus.paid);
        } else {
            order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
        }
        order.setType(ChargeCommon.Type.apply);
        order.setGmtCreate(new Date());
        order.setAutoRenew(false);
        // 设置订单归属
        setOrderLocation(order, applicationContext.getParams().getLocation());
        order.setTenantId(context.getTenant());
        order.setResourceStatus(ChargeCommon.ResourceStatus.creating);
        serviceOrderRepository.save(order);
        return order;
    }

    /**
     * 持久化保存订单项
     *
     * @param applicationContext
     */
    private void saveOrderItems(ApplicationContext applicationContext) {
        if (ListTool.isEmpty(applicationContext.getServiceOrder().getItems())) {
            return;
        }
        for (OrderItem item : applicationContext.getServiceOrder().getItems()) {
            item.setOrderId(applicationContext.getServiceOrder().getId());
            List<ElementModel> elementModels = JSONArray.parseArray(item.getElements(), ElementModel.class);
            if (elementModels.isEmpty()) {
                ElementModel elementModel = new ElementModel();
                elementModel.setRatio(applicationContext.getTenant().getDiscount());
                elementModels = Lists.newArrayList(elementModel);
            } else {
                elementModels.forEach(elementModel -> elementModel.setRatio(applicationContext.getTenant().getDiscount()));
            }
            item.setElements(JSONObject.toJSONString(elementModels));
            item.setFinalPrice(item.getBasicPrice().multiply(applicationContext.getTenant().getDiscount()));
            orderItemRepository.save(item);
        }
    }

    /**
     * 生成订单号
     *
     * @return
     */
    private String generateSn() {
        String path = IdGenerator.class.getSimpleName() + "_ordersn";
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
            return IdGenerator.getInstance().nextId();
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage(), e);
        }
    }

    /**
     * 设置订单位置信息
     *
     * @param order
     * @param location
     */
    private void setOrderLocation(ServiceOrder order, LocationModel location) {
        order.setVendorId(location.getVendorId());
        order.setBusinessId(location.getBusinessId());
        order.setProjectId(location.getProjectId());
        order.setUserId(location.getOwnerId());
    }

    /**
     * 创建服务申请上下文
     *
     * @param applicationParam
     * @param context
     * @return
     */
    private ApplicationContext buildApplicationContext(ApplicationParam applicationParam, RequestContext context) {
        ApplicationContext applicationContext = new ApplicationContext();
        applicationContext.setParams(applicationParam);
        Tenant tenant = tenantService.detail(context.getTenant());
        applicationContext.setTenant(tenant);
        this.validateParams(applicationParam, applicationContext, context);
        logger.info("打印applicationContext：{}", JSON.toJSONString(applicationContext));
        // 资源操作实例
        ResourceOperate applyOperate = ResourceOperateFactory.getInstance(applicationContext.getService().getCode());
        Assert.notNull(applyOperate, "暂不支持该类服务订购操作");
        applicationContext.setOperate(applyOperate);
        // 资源类型，用于纳管创建资源时的类型标识
        ResourceCategory resourceCategory = ServiceResourceMapping.getResourceCategory(applicationContext.getService().getCode());
        applicationContext.setResourceCategory(resourceCategory);
        return applicationContext;
    }

    /**
     * 校验服务申请参数，构造服务订购上下文
     *
     * @param param
     * @param applicationContext
     * @return
     */
    private void validateParams(ApplicationParam param, ApplicationContext applicationContext, RequestContext context) {
        CloudService service = cloudServiceRepository.queryByCode(param.getService());
        Assert.notNull(service, "该服务不存在");
        Assert.isTrue("PUBLISHED".equals(service.getStatus()), "服务未发布");
        Tenant tenant = tenantService.detail(context.getTenant());
        Assert.notNull(tenant, "申请人所在租户不存在");
        // 商品sku
        List<ElementModel> elements = param.getElements();
        // 服务协议信息
        Long slaId = param.getSla();
        ServiceSla sla = null;
        if (null == slaId) {
            List<ServiceSla> slas = serviceSlaRepository.listByService(service.getId(), ServiceSla.Status.enable.name());
            if (!ListTool.isEmpty(slas)) {
                sla = slas.get(0);
                slaId = sla.getId();
            }
        } else {
            sla = serviceSlaRepository.query(slaId);
            Assert.notNull(sla, "该服务的SLA协议不存在，无法订购");
        }
        Long skuId = null;
        for (ElementModel element : elements) {
            if (service.getCode().equals(element.getServiceCode())) {
                skuId = element.getSkuId();
                break;
            }
        }
        ServiceCategory category = null;
        ServiceSku sku = null;
        if (null != skuId && 0L != skuId) {
            sku = serviceSkuRepository.query(skuId);
            Assert.notNull(sku, "该服务产品不存在");
            category = serviceCategoryRepository.query(sku.getCategoryId());
            Assert.notNull(category, "该服务类型不存在");
            // sku不为空，则说明需要走计费，需要校验租户授信额度
            TenantAccount account = accountRepository.getAccount(context.getTenant());
            Assert.notNull(account, "未查询到租户授信额度");
        }
        applicationContext.setService(service);
        applicationContext.setSlaId(slaId);
        applicationContext.setSla(sla);
        applicationContext.setSkuId(skuId);
        applicationContext.setSku(sku);
        applicationContext.setCategory(category);
    }

    /**
     * 校验配额信息
     *
     * @param applicationContext
     */
    private void validateQuota(ApplicationContext applicationContext) {
        // 计算订单项配额
        Map<String, Integer> quotaMap = OrderItemUtils.getItemQuota(applicationContext.getServiceOrder().getItems());

        if (!applicationContext.getTenant().getQuotaAble()) {
            // 租户未开启配额,不需要计算配额
            return;
        }
        if (null == quotaMap || quotaMap.isEmpty()) {
            return;
        }
        List<ResourceQuota> list = resourceQuotaRepository.listByTenantId(applicationContext.getTenant().getId());
        Assert.notEmpty(list, "未查询到配额信息");
        Map<String, ResourceQuota> map = new HashMap<>(8);
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (Entry<String, Integer> entry : quotaMap.entrySet()) {
            String code = entry.getKey();
            if ("vmware.storage.disk".equals(code)) {
                continue;
            }
            String quotaName;
            if (QuotaCategory.CPU.getCode().equals(code)) {
                quotaName = QuotaCategory.CPU.getName();
            } else if (QuotaCategory.MEM.getCode().equals(code)) {
                quotaName = QuotaCategory.MEM.getName();
            } else if (QuotaCategory.DISK.getCode().equals(code)) {
                quotaName = QuotaCategory.DISK.getName();
            } else {
                CloudService cloudService = cloudServiceRepository.queryByCode(code);
                quotaName = cloudService.getName();
            }
            if (!map.containsKey(entry.getKey())) {
                throw new RuntimeException("未查询到" + quotaName + "配额信息");
            }
            ResourceQuota quota = map.get(entry.getKey());
            if (entry.getValue() + quota.getUsed() > quota.getQuota()) {
                throw new RuntimeException("剩余" + quotaName + "配额不足");
            }
            quota.setQuotaActionValue(entry.getValue().longValue());
            quotas.add(quota);
        }
        applicationContext.setResourceQuotas(quotas);
    }

    /**
     * 封装资源申请的必要参数
     *
     * @param applicationContext 服务申请上下文
     */
    private void warpApplicationConfig(ApplicationContext applicationContext) {
        JSONObject configs = applicationContext.getParams().getConfigs();
        if (null == configs) {
            configs = new JSONObject();
        }
        configs.put("skuId", applicationContext.getSkuId());
        if (null != applicationContext.getSku()) {
            configs.put("skuName", applicationContext.getSku().getName());
        }
        if (null != applicationContext.getCategory()) {
            configs.put("categoryId", applicationContext.getCategory().getId());
            configs.put("categoryCode", applicationContext.getCategory().getCode());
            configs.put("categoryProps", applicationContext.getCategory().getProps());
        }
        // 资源类型，用于纳管创建资源时的类型标识
        configs.put("category", applicationContext.getResourceCategory().name());
        if (StringUtils.isNotEmpty(applicationContext.getService().getTemplateName())) {
            configs.put("templateName", applicationContext.getService().getTemplateName());
        }
        applicationContext.getParams().setConfigs(configs);
    }

    @Override
    public Result updateOrder(OrderMessage message) {
        try {
            if (null == message) {
                return new Result(false, "消息格式不正确");
            }
            message.setCategory(ResourceCategory.Kubernetes.name());
            logger.info("接收到k8s结果消息：{}", JSONObject.toJSON(message));
            OrderItemStatus status = message.getStatus();
            if (OrderItemStatus.OpenedSuccess == status || OrderItemStatus.OpenedFailure == status) {
                OrderItem item = orderItemRepository.queryBySn(message.getSn());
                if (null == item) {
                    return new Result(false, "订单明细不存在");
                }
                logger.info("k8s结果订单明细消息：{}", JSONObject.toJSON(item));
                ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
                if (null == order) {
                    return new Result(false, "订单不存在");
                }
                logger.info("k8s结果订单消息：{}", JSONObject.toJSON(order));
                Long tenantId = order.getTenantId();
                Long target = order.getUserId();
                Catalog catalog = Catalog.User;
                if (null != tenantId) {
                    catalog = Catalog.Tenant;
                    target = tenantId;
                }
                RequestContext context = new RequestContext(catalog, target, tenantId, null, null);
                String duration = item.getDuration();
                if (!StringUtils.isNotEmpty(duration)) {
                    return new Result(false, "订购时长信息不存在");
                }
                DurationModel durationModel = JSONObject.parseObject(duration, DurationModel.class);
                DurationModel.Mode mode = durationModel.getMode();
                Integer month = durationModel.getAmount();
                Date effectiveTime = new Date();
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(effectiveTime);
                if (DurationModel.Mode.Hour == mode) {
                    calendar.setTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
                } else {
                    calendar.add(Calendar.MONTH, month);
                }
                message.setEffectiveTime(effectiveTime);
                message.setExpiredTime(calendar.getTime());
                message.setContext(context);
                if (OrderItemStatus.OpenedFailure == message.getStatus()) {
                    List<ResourceModel> resourceModels = message.getResources();
                    List<String> instanceIds = new ArrayList<>();
                    if (!ListTool.isEmpty(resourceModels)) {
                        for (ResourceModel resourceModel : resourceModels) {
                            instanceIds.add(resourceModel.getUuid());
                        }
                        imsInternalService.delete(instanceIds);
                    }
                }
            } else if (OrderItemStatus.ChangeSuccess == status || OrderItemStatus.ChangeFailure == status) {
                OrderItem item = orderItemRepository.queryBySn(message.getNewSn());
                if (null == item) {
                    return new Result(false, "订单明细不存在");
                }
                ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
                if (null == order) {
                    return new Result(false, "订单不存在");
                }
                List<OrderItem> oldItems = orderItemRepository.listByOrderId(order.getId());
                OrderItem changingItem = null;
                for (OrderItem orderItem : oldItems) {
                    if (OrderItemStatus.Changing == orderItem.getStatus()) {
                        changingItem = orderItem;
                        break;
                    }
                }
                if (null == changingItem) {
                    return new Result(false, "变更前子订单未找到");
                }
                // 查询变更前账单
                Long tenantId = order.getTenantId();
                Long target = order.getUserId();
                Catalog catalog = Catalog.User;
                if (null != tenantId) {
                    catalog = Catalog.Tenant;
                    target = tenantId;
                }
                RequestContext context = new RequestContext(catalog, target, tenantId, null, null);
                message.setEffectiveTime(new Date());
                message.setExpiredTime(changingItem.getExpiredTime());
                message.setContext(context);
                message.setSn(changingItem.getSn());
                if (OrderItemStatus.ChangeFailure == message.getStatus()) {
                    List<ResourceModel> resourceModels = message.getResources();
                    List<String> instanceIds = new ArrayList<>();
                    if (!ListTool.isEmpty(resourceModels)) {
                        for (ResourceModel resourceModel : resourceModels) {
                            instanceIds.add(resourceModel.getUuid());
                        }
                        imsInternalService.delete(instanceIds);
                    }
                }
            } else {
                return new Result(false, "订单更新失败");
            }
            amqpTemplate.convertAndSend("resource.result.queue", message);
            return new Result(true, "订单更新成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new Result(false, "订单更新失败");
        }
    }

    @Override
    @Transactional
    public Result clone(CloneParams cloneParams, RequestContext context) {
        // 生成克隆参数
        logger.info("接收到克隆消息：{}", JSONObject.toJSON(cloneParams));
        Assert.notNull(cloneParams, "参数格式错误");
        Long templateId = cloneParams.getTemplateId();
        String instance = cloneParams.getInstance();
        JSONObject config = new JSONObject();
        config.put("templateId", templateId);
        config.put("instance", instance);
        Assert.notNull(instance, "未找到资源关联的子订单");
        OrderItem item = orderItemRepository.queryBySn(instance);
        Assert.notNull(item, "未找到资源关联的子订单");
        ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
        Assert.notNull(order, "未找到资源关联的订单");
        ApplicationParam applicationParam = JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class);
        BocloudResult serverResult = imsInternalService.serverDetail(context.getApiKey(), templateId);
        Assert.isTrue(serverResult.isSuccess(), serverResult.getMessage());
        CloudServer server = JSONObject.parseObject(JSONObject.toJSONString(serverResult.getData()), CloudServer.class);
        // 查询磁盘列表
        List<Volume> volumes;
        if (item.getServiceCode().contains("smartx.")) {
            volumes = imsInternalService.getVolumesForSmartX(server.getId(), server.getVendorId());
        } else {
            volumes = imsInternalService.listByVmId(server.getId());
        }
        // 系统盘
        Volume sysDisk = volumes.stream()
                .filter(volume -> "Hard disk 1".equalsIgnoreCase(volume.getName()) || "硬盘 1".equalsIgnoreCase(volume.getName()))
                .findFirst().orElse(null);
        Assert.notNull(sysDisk, "未查询到系统盘信息");
        Long skuId = sysDisk.getSkuId();
        if (null != skuId && 0L != skuId) {
            ServiceSku diskSku = serviceSkuRepository.query(skuId);
            Assert.notNull(diskSku, "云主机系统盘关联的产品SKU未找到");
            config.put("diskCategoryId", diskSku.getCategoryId());
        }
        List<NetworkCard> netWordCards = imsInternalService.listNetworkCards(server.getId());
        JSONArray networkCardConfigs = new JSONArray();
        Map<Long, CloneNetworkCardParams> cloneNetworkCardMap = new HashMap<>();
        if (cloneParams.getNetworkCards() != null) {
            List<CloneNetworkCardParams> cloneNetworkCards = cloneParams.getNetworkCards();
            for (CloneNetworkCardParams cloneNetworkCard : cloneNetworkCards) {
                cloneNetworkCardMap.put(cloneNetworkCard.getNetworkCardId(), cloneNetworkCard);
            }
        }
        // 原云主机配置
        for (NetworkCard card : netWordCards) {
            JSONObject networkCardConfig = new JSONObject();
            if (cloneNetworkCardMap.containsKey(card.getId())) {
                CloneNetworkCardParams cloneNetworkCardParams = cloneNetworkCardMap.get(card.getId());
                networkCardConfig.put("ipPoolId", cloneNetworkCardParams.getIpPoolId());
                networkCardConfig.put("ipPolicy", cloneNetworkCardParams.getIpPolicy());
                networkCardConfig.put("networkCardId", card.getId());
                networkCardConfig.put("portGroupId", cloneNetworkCardParams.getPortGroupId());
                networkCardConfig.put("address", cloneNetworkCardParams.getAddresses());
                networkCardConfigs.add(networkCardConfig);
                continue;
            }
            networkCardConfig.put("ipPoolId", cloneParams.getIpPoolId());
            networkCardConfig.put("ipPolicy", cloneParams.getIpPolicy());
            networkCardConfig.put("networkCardId", card.getId());
            networkCardConfig.put("portGroupId", cloneParams.getPortGroupId());
            networkCardConfig.put("address", cloneParams.getAddresses());
            networkCardConfigs.add(networkCardConfig);
        }
        ApplicationParam newApplication = new ApplicationParam();
        LocationModel locationModel = new LocationModel();
        locationModel.setVendorId(server.getVendorId());
        PoolGroup poolGroup = imsInternalService.getPoolGroup(server.getPoolGroupId());
        Assert.notNull(poolGroup, "没找相应的资源池");
        locationModel.setAz(poolGroup.getAvailablitiyZone());
        locationModel.setRegion(poolGroup.getRegion());
        locationModel.setPoolGroupId(poolGroup.getId());
        locationModel.setProjectId(server.getProjectId());
        locationModel.setBusinessId(server.getBusinessId());
        locationModel.setCreatorId(server.getCreatorId());
        locationModel.setMenderId(server.getMenderId());
        locationModel.setOwnerId(server.getOwnerId());
        locationModel.setRegion(server.getRegion());
        locationModel.setVendorId(server.getVendorId());
        locationModel.setTenantId(server.getTenantId());
        locationModel.setVendorType(server.getVendorType());
        newApplication.setLocation(locationModel);
        DurationModel duration = new DurationModel();
        duration.setMode(Mode.Hour);
        EmptionModel emption = new EmptionModel();
        emption.setCount(1);
        emption.setAutoRenew(false);
        emption.setDuration(duration);
        newApplication.setEmption(emption);
        newApplication.setService(item.getServiceCode());

        List<ElementModel> elementModels = new ArrayList<>();
        Integer cpu = server.getCpu();
        BigDecimal memory = server.getMemory();
        // cpu内存规格
        ElementModel element = new ElementModel();
        element.setServiceCode(newApplication.getService());
        List<SpecModel> specs = new ArrayList<>();
        specs.add(new SpecModel(cpu, memory));
        Long skuId1 = Optional.ofNullable(server.getSkuId()).orElse(0L);
        element.setSkuId(skuId1);
        element.setCategoryId(server.getCategoryId());
        element.setSpecs(specs);
        elementModels.add(element);
        // 磁盘规格
        for (Volume volume : volumes) {
            ElementModel volumeElement = new ElementModel();
            Long volumeSkuId = volume.getSkuId();
            if (null != volumeSkuId && 0L != volumeSkuId) {
                ServiceSku diskSku = serviceSkuRepository.query(volumeSkuId);
                volumeElement.setSkuId(volumeSkuId);
                volumeElement.setCategoryId(diskSku.getCategoryId());
            }
            volumeElement.setServiceCode("vmware.storage.disk");
            List<SpecModel> diskSpecs = new ArrayList<>();
            diskSpecs.add(new SpecModel(volume.getSize()));
            volumeElement.setSpecs(diskSpecs);
            elementModels.add(volumeElement);
        }
        newApplication.setElements(elementModels);
        config.put("name", cloneParams.getName());
        config.put("isAddShterm", cloneParams.getIsAddShterm());
        config.put("cpu", cpu);
        config.put("memory", memory);
        config.put("disk", sysDisk.getSize());
        config.put("skuId", item.getSkuId());
        config.put("category", ResourceCategory.Computer.name());
        config.put("networkCardConfigs", networkCardConfigs);
        if (StringUtils.isNotEmpty(server.getPswd())) {
            Encryptor encryptor = new AESEncrptor();
            config.put("password", encryptor.encrypt(server.getPswd(), null));
        }
        newApplication.setConfigs(config);
        newApplication.setContext(context);
        // 申请校验规格 校验服务是否存在等
        ApplicationContext applicationContext = buildApplicationContext(newApplication, context);

        // 生成订单
        ServiceOrder cloneOrder = this.createOrder(applicationContext,context);

        // 预生成生成订单项，计算配额并校验
        List<OrderItem> orderItems = applicationContext.getOperate().buildApplyItem(applicationContext.getService().getCode(),
                newApplication.getEmption(), newApplication.getElements());
        cloneOrder.setItems(orderItems);
        applicationContext.setServiceOrder(cloneOrder);
        // 校验配额信息
        this.validateQuota(applicationContext);

        // 扣除配额
        if (null != applicationContext.getResourceQuotas() && !applicationContext.getResourceQuotas().isEmpty()) {
            resourceQuotaService.quotaAction(applicationContext.getResourceQuotas(), ResourceQuota.Action.Order,applicationContext.getTenant().getId());
        }

        // 保存订单项
        this.saveOrderItems(applicationContext);

        cloneOrder.setApplyParam(JSONObject.toJSONString(newApplication));
        newApplication.setOrder(cloneOrder);
        logger.info("克隆消息{},发送成功", JSONObject.toJSONString(Lists.newArrayList(newApplication)));
        amqpTemplate.convertAndSend("iaas.resource.creation.queue", Lists.newArrayList(newApplication));
        return new Result(true, "克隆申请已提交");
    }

    private Map<String, Object> getParams(String category, String params, String theme, RequestContext context) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("ApplyCategory", category);
        map.put("ApplyParams", params);
        map.put("ApplyTheme", theme);
        map.put("ApplyContext", context);
        return map;
    }
}
