package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.internel.entity.*;
import com.bocloud.cos.utils.DateUtils;
import com.bocloud.cos.utils.MathUtils;
import com.bocloud.cos.enums.BocloudStatus;
import com.bocloud.cos.enums.CloudProviderType;
import com.bocloud.cos.enums.VendorType;
import com.bocloud.cos.intf.ResourceScreenService;
import com.bocloud.cos.model.ChargeResultTrend;
import com.bocloud.cos.model.ChargeTrendData;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.rpc.interfaces.ims.ImsRpcService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 平台资源大屏业务类
 *
 * @author tyl
 * @since 2020/11/25 21:49
 */
@Service
public class ResourceScreenServiceImpl implements ResourceScreenService {
    private final Logger logger = LoggerFactory.getLogger(ResourceScreenServiceImpl.class);
    private final ImsInternalService imsInternalService;
    @Autowired
    private ImsRpcService imsRpcService;

    public ResourceScreenServiceImpl(ImsInternalService imsInternalService) {
        this.imsInternalService = imsInternalService;
    }

    @Override
    public BocloudResult totalOverview(RequestContext requestContext) {
        JSONObject result = new JSONObject();
        try {
            List<CloudServer> cloudServers = imsInternalService.listVmsByVendor(null, false, requestContext);
            long running =
                    cloudServers.stream().filter(cloudServer -> "RUNNING".equals(cloudServer.getStatus())).count();
            List<Server> servers = imsInternalService.listHosts();
            result.put("hosts", servers.size());
            result.put("servers", cloudServers.size());
            result.put("running", running);
            return new BocloudResult<>(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("get total overview error,", e);
            return new BocloudResult<>(false, result, "获取数据中心总体情况失败");
        }
    }

    @Override
    public BocloudResult platformOverview(RequestContext requestContext) {
        JSONObject result = new JSONObject();
        try {
            List<CloudVendor> vendorList = imsInternalService.getVendorList();
            Map<Boolean, List<CloudVendor>> collect =
                    vendorList.stream().collect(Collectors.groupingBy(CloudVendor::getIsPublic));
            List<CloudVendor> priVendor = Optional.ofNullable(collect.get(false)).orElse(new ArrayList<>());
            List<CloudVendor> pubVendor = Optional.ofNullable(collect.get(true)).orElse(new ArrayList<>());
            result.put("priNum", priVendor.size());
            result.put("pubNum", pubVendor.size());
            return new BocloudResult<>(true, result, "获取平台情况成功");
        } catch (Exception e) {
            logger.error("get platform overview error,", e);
            return new BocloudResult<>(false, result, "获取平台情况失败");
        }
    }

    @Override
    public BocloudResult usedOverview(RequestContext requestContext) {
        JSONObject result = new JSONObject();
        try {
            List<CloudServer> cloudServers = imsInternalService.listVmsByVendor(null, false, requestContext);
            //私有云平台
            List<CloudVendor> cloudVendors = imsInternalService.listCloudVendor(null);
            Set<String> vendors =
                    cloudVendors.stream().filter(cloudVendor -> !cloudVendor.getIsPublic())
                            .map(CloudVendor::getType).collect(Collectors.toSet());
            Set<Long> vendorIds =
                    cloudVendors.stream().filter(cloudVendor -> !cloudVendor.getIsPublic())
                            .map(CloudVendor::getId).collect(Collectors.toSet());
            List<CloudServer> priServers = cloudServers.stream().filter(
                            cloudServer -> null != cloudServer.getVendorType() && vendors.contains(cloudServer.getVendorType()))
                    .collect(Collectors.toList());
            //计算使用量
            Integer cpuUsed =
                    priServers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
            BigDecimal memUsed = priServers.stream()
                    .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
            Integer diskUsed = 0;
            List<Volume> volumes = getvolums(priServers);
            diskUsed +=
                    volumes.stream().map(volume1 -> Optional.ofNullable(volume1.getSize()).orElse(0)).reduce(Integer::sum)
                            .orElse(0);
            //计算总量
            long cpuTotal = 0L;
            Long memTotal = 0L;
            int diskTotal = 0;
            List<com.bocloud.ims.entity.resource.Server> hosts = imsRpcService.getServerAll(vendorIds);
            hosts.removeIf(server -> null == server.getVendorId());
            Map<Long, List<com.bocloud.ims.entity.resource.Server>> hostMap = hosts.stream().filter(server -> null != server.getVendorId())
                    .collect(Collectors.groupingBy(com.bocloud.ims.entity.resource.Server::getVendorId));
            List<CloudVendor> vmVendors = imsInternalService.listCloudVendor(VendorType.VMWARE.name());
            for (CloudVendor vendor : vmVendors) {
                List<com.bocloud.ims.entity.resource.Server> servers = Optional.ofNullable(hostMap.get(vendor.getId())).orElse(new ArrayList<>());
                for (com.bocloud.ims.entity.resource.Server server : servers) {
                    com.bocloud.ims.entity.resource.ServerConfig config = server.getConfig();
                    //从平台获取超分比
                    List<CloudVendor> collect = cloudVendors.stream().filter(vendor1 -> vendor1.getId().equals(server.getVendorId())).collect(Collectors.toList());
                    Assert.notNull(collect, "云平台不存在");
                    JSONObject authentication = JSONObject.parseObject(collect.get(0).getAuthentication());
                    // 超分比
                    Long excessRatio = 0L;
                    if (null != authentication.get("excessRatio") && CloudProviderType.VMWARE.name().equals(collect.get(0).getType())) {
                        excessRatio = Long.valueOf(String.valueOf(authentication.get("excessRatio")));
                    }
                    if (null != excessRatio && 0L != excessRatio) {
                        cpuTotal += config.getCpuCore() * config.getCpuAmount() * excessRatio;
                        memTotal += config.getMemoryCapacity().longValue() * excessRatio;
                    } else {
                        cpuTotal += (long) config.getCpuCore() * config.getCpuAmount();
                        memTotal += config.getMemoryCapacity().longValue();
                    }
                    diskTotal += config.getDiskCapacity().intValue();
                }
            }
            List<com.bocloud.ims.entity.resource.Server> opServers =
                    hosts.stream().filter(server -> server.getVendorType().equals(VendorType.OPENSTACK.name()))
                            .collect(Collectors.toList());
            for (com.bocloud.ims.entity.resource.Server opServer : opServers) {
                com.bocloud.ims.entity.resource.ServerConfig config = opServer.getConfig();
                cpuTotal += (long) config.getCpuCore() * config.getCpuAmount();
                diskTotal += config.getDiskCapacity().intValue();
                memTotal += config.getMemoryCapacity().longValue();
            }
            result.put("cpuTotal", cpuTotal);
            result.put("memTotal", memTotal);
            result.put("diskTotal", MathUtils.round(diskTotal / 1024D, 2));
            result.put("cpuUnused", cpuTotal - Long.valueOf(cpuUsed + ""));
            result.put("menUnused", MathUtils.round(memTotal - memUsed.doubleValue(), 2));
            result.put("diskUnused", MathUtils.round((diskTotal - diskUsed) / 1024D, 2));
            return new BocloudResult<>(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("get used overview error,", e);
            return new BocloudResult<>(false, result, "获取资源使用情况失败");
        }
    }

    @Override
    public BocloudResult count(String type, RequestContext requestContext) {
        JSONObject result = new JSONObject();
        try {
            //云平台
            List<CloudVendor> cloudVendors = imsInternalService.listCloudVendor(null);
            Set<String> vendors =
                    cloudVendors.stream()
                            .map(CloudVendor::getType).collect(Collectors.toSet());
            List<CloudServer> cloudServerList = imsInternalService.listVmsByVendor(type, false, requestContext).stream().filter(
                            cloudServer -> null != cloudServer.getVendorType() && vendors.contains(cloudServer.getVendorType()))
                    .collect(Collectors.toList());
            Integer cpuNum =
                    cloudServerList.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
            BigDecimal memNum = cloudServerList.stream()
                    .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            Integer diskNum = 0;
            List<Volume> volumes = getvolums(cloudServerList);
            diskNum +=
                    volumes.stream().map(volume1 -> Optional.ofNullable(volume1.getSize()).orElse(0)).reduce(Integer::sum)
                            .orElse(0);
            result.put("serverNum", cloudServerList.size());
            result.put("cpu", cpuNum);
            result.put("mem", memNum.doubleValue());
            result.put("disk", MathUtils.round(diskNum / 1024D, 2));
            return new BocloudResult(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("get vendor resource count error ", e);
            return new BocloudResult(false, result, "查询平台云资源统计失败:" + e.getMessage());
        }
    }

    private List<Volume> getvolums(List<CloudServer> cloudServerList) {
        List<Volume> volumes = new ArrayList<>();
        //单独处理云硬盘，可能出现数据盘的情况
        if (!ListTool.isEmpty(cloudServerList)) {
            //取出CloudServer下所有的vendorId
            List<Long> cloudVendorIds =
                    cloudServerList.stream().map(CloudServer::getVendorId).distinct().collect(Collectors.toList());
            //筛选指定云主机下的磁盘--由于存在数据盘，所以改用取volum信息，没有用server对应的disk信息
            List<Long> serverIds = cloudServerList.stream().map(CloudServer::getId).collect(Collectors.toList());
            for (Long vendorId : cloudVendorIds) {
                BocloudResult serviceVolume = imsInternalService.getVolume(vendorId);
                if (serviceVolume.getData() != null) {
                    List<Volume> volumeList =
                            JSONArray.parseArray(JSONObject.toJSONString(serviceVolume.getData()), Volume.class);
                    volumeList = volumeList.stream().filter(volume1 -> serverIds.contains(volume1.getServerId()))
                            .collect(Collectors.toList());
                    volumes.addAll(volumeList);
                }
            }
        }
        return volumes;
    }

    @Override
    public BocloudResult status(String type, RequestContext requestContext) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<CloudServer> cloudServerList = imsInternalService.listVmsByVendor(type, false, requestContext);
            Map<String, List<CloudServer>> serversMap =
                    cloudServerList.stream().collect(Collectors.groupingBy(CloudServer::getStatus));
            List<Object> data = new ArrayList<>();
            for (Map.Entry<String, List<CloudServer>> entry : serversMap.entrySet()) {
                String key;
                if ("RUNNING".equals(entry.getKey()) || "GENERAL".equals(entry.getKey())) {
                    key = "开机";
                } else if ("STOPPED".equals(entry.getKey())) {
                    key = "关机";
                } else if ("ERROR".equals(entry.getKey()) || "UNKNOWN".equals(entry.getKey())) {
                    key = "断开";
                } else if ("EXCEPTION".equals(entry.getKey())) {
                    key = "异常";
                } else if ("SUSPENDED".equals(entry.getKey())) {
                    key = "挂起";
                } else {
                    continue;
                }
                int count = entry.getValue().size();
                keys.add(key);
                data.add(count);
            }
            values.add(new ChargeTrendData("云主机数量（个）", data));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询云主机状态统计成功");
        } catch (Exception e) {
            logger.error("get vendor resource count error ", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询云主机状态统计失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult vendorTypes(RequestContext requestContext) {
        JSONArray result = new JSONArray();
        try {
            List<CloudVendor> vendors = imsInternalService.listCloudVendor(null, requestContext);
            Set<String> collect = vendors.stream().map(CloudVendor::getType).collect(Collectors.toSet());
            for (VendorType type : VendorType.values()) {
                for (String vendorType : collect) {
                    if (type.name().equals(vendorType)) {
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put("name", type.getName());
                        jsonObject.put("value", vendorType);
                        result.add(jsonObject);
                        break;
                    }
                }
            }
            return new BocloudResult(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("get vendor types error ", e);
            return new BocloudResult(false, result, "查询平台类型失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult resourceApply(String type, RequestContext requestContext) {
        List<String> timeList = DateUtil.timeToList(new JSONObject());
        List<String> keys = new ArrayList<>(timeList);
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<CloudServer> cloudServerList = imsInternalService.listVmsByVendor(type, false, requestContext);
            Set<String> vendorTypes =
                    imsInternalService.listCloudVendor(type, requestContext).stream().map(CloudVendor::getType)
                            .filter(s -> !s.isEmpty()).collect(Collectors.toSet());
            HashMap<String, List<Object>> dataMap = new HashMap<>(8);
            for (String vendorType : vendorTypes) {
                dataMap.put(vendorType, new ArrayList<>());
            }
            for (String time : timeList) {
                //筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
                List<CloudServer> targetVms = cloudServerList.stream().filter(
                                cloudServer -> !BocloudStatus.Server.BUILDING.name().equals(cloudServer.getStatus()) && cloudServer
                                        .getGmtCreate()
                                        .before(DateUtils.stringToDate(time.substring(0, 10) + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                                        && cloudServer.getGmtCreate()
                                        .after(DateUtils.stringToDate(time.substring(0, 10) + " 00:00:00", "yyyy-MM-dd HH:mm:ss")))
                        .collect(Collectors.toList());
                Map<String, List<CloudServer>> vmsMap =
                        targetVms.stream().filter(cloudServer -> cloudServer.getVendorType() != null)
                                .collect(Collectors.groupingBy(CloudServer::getVendorType));
                for (Map.Entry<String, List<Object>> entry : dataMap.entrySet()) {
                    String vendorType = entry.getKey();
                    List<Object> data = entry.getValue();
                    List<CloudServer> servers = vmsMap.get(vendorType);
                    Integer serverNumber = servers != null ? servers.size() : 0;
                    data.add(serverNumber);
                }
            }
            for (Map.Entry<String, List<Object>> entry : dataMap.entrySet()) {
                String key = entry.getKey();
                List<Object> data = entry.getValue();
                VendorType vendorType = VendorType.valueOf(key);
                values.add(new ChargeTrendData(vendorType.getName(), data));
            }
            return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询云主机申请趋势成功");
        } catch (Exception e) {
            logger.error("get current vendor resource apply info error,", e);
            return new BocloudResult<>(false, new ChargeResultTrend(keys, values), "查询云主机申请趋势失败");
        }
    }

    @Override
    public BocloudResult cloudResourceTrend(String type, RequestContext requestContext) {
        List<String> timeList = DateUtil.timeToList(new JSONObject());
        List<String> keys = new ArrayList<>(timeList);
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<CloudServer> cloudServerList = imsInternalService.listVmsByVendor(type, false, requestContext);
            List<Volume> volumes = getvolums(cloudServerList);
            List<Object> cpuData = new ArrayList<>();
            List<Object> memData = new ArrayList<>();
            List<Object> diskData = new ArrayList<>();
            for (String time : timeList) {
                //筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
                List<CloudServer> targetVms = cloudServerList.stream().filter(
                                cloudServer -> !BocloudStatus.Server.BUILDING.name().equals(cloudServer.getStatus()) && cloudServer
                                        .getGmtCreate()
                                        .before(DateUtils.stringToDate(time.substring(0, 10) + " 23:59:59", "yyyy-MM-dd HH:mm:ss")))
                        .collect(Collectors.toList());
                Integer cpuNum =
                        targetVms.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                                .reduce(Integer::sum).orElse(0);
                cpuData.add(cpuNum);
                BigDecimal memNum = targetVms.stream()
                        .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                memData.add(memNum.doubleValue());
                //筛选指定云主机下的磁盘--由于存在数据盘，所以改用取volum信息，没有用server对应的disk信息
                List<Long> serverIds = targetVms.stream().map(CloudServer::getId).collect(Collectors.toList());
                List<Volume> volumeList = volumes.stream().filter(volume -> serverIds.contains(volume.getServerId()))
                        .collect(Collectors.toList());
                Integer diskNum = volumeList.stream().map(volume -> Optional.ofNullable(volume.getSize()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                diskData.add(diskNum);
            }
            values.add(new ChargeTrendData("CPU(C)", cpuData));
            values.add(new ChargeTrendData("内存(GB)", memData));
            values.add(new ChargeTrendData("磁盘(GB)", diskData));
            return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询公有云平台资源使用趋势成功");
        } catch (Exception e) {
            logger.error("get current vendor resource used info error,", e);
            return new BocloudResult<>(false, new ChargeResultTrend(keys, values), "查询公有云平台资源使用趋势失败");
        }
    }
}
