package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.*;
import com.bocloud.cos.internal.*;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.intf.ServiceBillService;
import com.bocloud.cos.intf.TenantAccountService;
import com.bocloud.cos.model.*;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.utils.ExportUtil;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.utils.MathUtils;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.cos.internel.entity.Project;
import com.bocloud.cos.internel.entity.User;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.groupingBy;


/**
 * 账单业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年6月12日
 */
@Service
public class ServiceBillServiceImpl implements ServiceBillService {

    private static final Logger logger = LoggerFactory.getLogger(ServiceBillServiceImpl.class);
    private final ServiceBillRepository billRepository;
    private final BillItemRepository itemRepository;
    private final ServiceOrderRepository orderRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final OrderItemRepository orderItemRepository;
    private final ResourceQuotaService resourceQuotaService;
    private final AmqpTemplate amqpTemplate;
    private final SmsSystemConfigService configService;
    private final SmsInternalService socInternalService;
    private final LockFactory lockFactory;
    private final TenantAccountService accountService;
    private final SmsUserService smsUserService;
    private final ServiceSkuRepository skuRepository;
    private final ServiceFlowConfigRepository serviceFlowConfigRepository;
    private final SmsProjectService smsProjectService;
    private final FmsFlowService fmsFlowService;


    public ServiceBillServiceImpl(ServiceOrderRepository orderRepository, ServiceBillRepository billRepository,
                                  LockFactory lockFactory, SmsProjectService smsProjectService, BillItemRepository itemRepository,
                                  CloudServiceRepository cloudServiceRepository, ResourceQuotaRepository resourceQuotaRepository,
                                  ServiceSkuRepository skuRepository, OrderItemRepository orderItemRepository,
                                  ResourceQuotaService resourceQuotaService, ServiceFlowConfigRepository serviceFlowConfigRepository,
                                  AmqpTemplate amqpTemplate, SmsUserService smsUserService, TenantAccountService accountService,
                                  SmsSystemConfigService configService, SmsInternalService socInternalService,
                                  FmsFlowService fmsFlowService) {
        this.orderRepository = orderRepository;
        this.billRepository = billRepository;
        this.lockFactory = lockFactory;
        this.smsProjectService = smsProjectService;
        this.itemRepository = itemRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.skuRepository = skuRepository;
        this.orderItemRepository = orderItemRepository;
        this.resourceQuotaService = resourceQuotaService;
        this.serviceFlowConfigRepository = serviceFlowConfigRepository;
        this.amqpTemplate = amqpTemplate;
        this.smsUserService = smsUserService;
        this.accountService = accountService;
        this.configService = configService;
        this.socInternalService = socInternalService;
        this.fmsFlowService = fmsFlowService;
    }

    @Override
    public BocloudResult<GridBean<ServiceBill>> list(Pager pager, RequestContext context) {
        try {
            List<ServiceBill> list;
            int total;
            GridBean<ServiceBill> gridBean;
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            sorter.put("gmtCreate", Common.ONE);
            String serviceCode = null;
            String orderSn = null;
            for (Param param : params) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("serviceCode")) {
                    serviceCode = paramMap.get("serviceCode").toString();
                    paramMap.remove("serviceCode");
                }
                if (paramMap.containsKey("orderSn")) {
                    orderSn = paramMap.get("orderSn").toString();
                    paramMap.remove("orderSn");
                }
            }
            List<Long> projectIds = new ArrayList<>();
            if (Catalog.Tenant == context.getCatalog()) {
                // 租户能看到 1.属于当前租户的订单
                params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
                total = billRepository.count(params, serviceCode, orderSn, projectIds);
                list = billRepository
                        .list(pager.getPage(), pager.getRows(), sorter, params, serviceCode, orderSn, projectIds);
            } else if (Catalog.User == context.getCatalog()) {
                // 用户能看到 1.属于当前用户的订单 2.当前用户管理的项目的订单
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Long> userManagerProjectsIds =
                        userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                total = billRepository.count(params, serviceCode, orderSn, context.getTarget(), projectIds);
                list = billRepository
                        .list(pager.getPage(), pager.getRows(), sorter, params, serviceCode, orderSn, context.getTarget(),
                                projectIds);
            } else {
                total = billRepository.count(params, serviceCode, orderSn, projectIds);
                list = billRepository
                        .list(pager.getPage(), pager.getRows(), sorter, params, serviceCode, orderSn, projectIds);
            }
            for (ServiceBill bill : list) {
                ServiceOrder order = orderRepository.query(bill.getOrderId());
                if (null != order) {
                    CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
                    if (null != service) {
                        bill.setServiceName(service.getName());
                    }
                }
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public XSSFWorkbook export(List<Param> params, RequestContext context) {
        try {
            List<ServiceBill> list;
            Map<String, String> sorter = Maps.newHashMap();
            sorter.put("gmtCreate", Common.ONE);
            String serviceCode = null;
            String orderSn = null;
            for (Param param : params) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("serviceCode")) {
                    serviceCode = paramMap.get("serviceCode").toString();
                    paramMap.remove("serviceCode");
                }
                if (paramMap.containsKey("orderSn")) {
                    orderSn = paramMap.get("orderSn").toString();
                    paramMap.remove("orderSn");
                }
            }
            List<Long> projectIds = new ArrayList<>();
            if (Catalog.Tenant == context.getCatalog()) {
                // 租户能看到 1.属于当前租户的订单
                params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
                list = billRepository.list(1, Integer.MAX_VALUE, sorter, params, serviceCode, orderSn, projectIds);
            } else if (Catalog.User == context.getCatalog()) {
                // 用户能看到 1.属于当前用户的订单 2.当前用户管理的项目的订单
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Long> userManagerProjectsIds =
                        userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                list = billRepository.list(1, Integer.MAX_VALUE, sorter, params, serviceCode, orderSn, context.getTarget(),
                        projectIds);
            } else {
                list = billRepository.list(1, Integer.MAX_VALUE, sorter, params, serviceCode, orderSn, projectIds);
            }
            for (ServiceBill bill : list) {
                ServiceOrder order = orderRepository.query(bill.getOrderId());
                if (null != order) {
                    CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
                    if (null != service) {
                        bill.setServiceName(service.getName());
                    }
                }
            }
            // 创建HSSFWorkbook
            String sheetName = "账单统计数据";
            String[] cellTitle = new String[]{"账期", "资源类型", "流水号", "计费类型", "账单状态", "订单号", "账单金额", "创建时间", "租户名称"};
            String[][] rows;
            if (!ListTool.isEmpty(list)) {
                rows = new String[list.size()][cellTitle.length];
                for (int i = 0; i < list.size(); i++) {
                    ServiceBill bill = list.get(i);
                    rows[i][0] = bill.getMonth();
                    rows[i][1] = bill.getServiceName();
                    rows[i][2] = bill.getSn();
                    rows[i][3] = bill.getChargeMode().equals(ChargeCommon.ChargeMode.PostCharge.name()) ? "后付费" : "预付费";
                    switch (bill.getStatus()) {
                        case unpay:
                            rows[i][4] = "未支付";
                            break;
                        case paid:
                            rows[i][4] = "已支付";
                            break;
                        case canceld:
                            rows[i][4] = "已取消";
                            break;
                        default:
                            break;
                    }
                    rows[i][5] = bill.getOrderSn();
                    rows[i][6] = Optional.ofNullable(bill.getMoney()).orElse(BigDecimal.ZERO).toString();
                    rows[i][7] = bill.getGmtCreate().toString().substring(0, 19);
                    rows[i][8] = bill.getTenantName();
                }
            } else {
                rows = new String[0][cellTitle.length];
            }
            return ExportUtil.getXSSFWorkbook(sheetName, cellTitle, rows);
        } catch (Exception e) {
            logger.error("create XSSFWorkBook error,", e);
            return null;
        }
    }

    @Override
    public BocloudResult listByCondition(String condition, RequestContext context) {
        try {
            JSONObject jsonObject = JSONObject.parseObject(condition);
            String conditionString = jsonObject.getString("condition");
            Long tenantId = 0L != context.getTenant() ? context.getTenant() : jsonObject.getLong("tenantId");
            switch (conditionString) {
                case "totalBills":
                    return this.totalBills(context);
                case "billStatistics":
                    return this.billStatistics(jsonObject.getString("start"), jsonObject.getString("end"),
                            jsonObject.getLong("tenantId"), context.getProject());
                case "incomeTrend":
                    Cycle cycle = jsonObject.getString("cycle") != null ? Cycle.valueOf(jsonObject.getString("cycle")) : null;
                    return this.incomeTrend(jsonObject.getString("start"), jsonObject.getString("end"), cycle,
                            tenantId, context.getProject());
                case "productTrend":
                    return this.productTrend(jsonObject.getString("start"), jsonObject.getString("end"),
                            jsonObject.getLong("tenantId"), context.getProject());
                case "tenantTop5":
                    return this.tenantTop5(jsonObject.getString("start"), jsonObject.getString("end"));
                case "userTop5":
                    return this.userTop5(jsonObject.getString("start"), jsonObject.getString("end"));
                case "productIncome":
                    return this.productIncome(jsonObject.getString("start"), jsonObject.getString("end"));
                case "queryBySn":
                    ServiceBill bill = billRepository.queryBySn(jsonObject.getString("sn"));
                    if (null != bill) {
                        bill.setItems(itemRepository.listByBillId(bill.getId()));
                        ServiceOrder order = orderRepository.query(bill.getOrderId());
                        if (null != order) {
                            CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
                            if (null != service) {
                                bill.setServiceName(service.getName());
                            }
                        }
                    }
                    return new BocloudResult<>(true, bill, "查询成功");
                default:
                    return new BocloudResult<>(false, "不支持这种查询");
            }
        } catch (Exception e) {
            logger.error("list bill by condition error,", e);
            return BocloudResult.FAILED("查询失败:" + e.getMessage());
        }
    }

    /**
     * 租户消费总计
     *
     * @param context
     * @return
     * @throws Exception
     */
    private BocloudResult totalBills(RequestContext context) {
        Long tenantId = null;
        if (null != context && RequestContext.Catalog.Tenant == context.getCatalog()) {
            tenantId = context.getTenant();
        }
        Double billAmount = billRepository.summary(null, tenantId, null, null);
        return new BocloudResult<>(true, BigDecimal.valueOf(billAmount).setScale(2, RoundingMode.HALF_UP), "查询成功");
    }

    /**
     * 租户消费趋势统计
     *
     * @param start
     * @param end
     * @param cycle
     * @param tenantId
     * @param projectId
     * @return
     * @throws Exception
     */
    private BocloudResult incomeTrend(String start, String end, Cycle cycle, Long tenantId, Long projectId) throws Exception {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat MONTH_FORMAT = new SimpleDateFormat("yyyy-MM");
        cycle = Optional.ofNullable(cycle).orElse(Cycle.DAY);
        // 统计范围
        ChargeResultTrend resultTrend;
        List<String> keys = new ArrayList<>();
        Date endDate = new Date();
        endDate = DateUtils.addDays(endDate, 1);
        Date beginDate = DateUtils.addMonths(endDate, -6);
        if (StringUtils.hasText(start) && StringUtils.hasText(end)) {
            beginDate = DateUtils.parseDate(start, "yyyy-MM-dd HH:mm:ss");
            endDate = DateUtils.parseDate(end, "yyyy-MM-dd HH:mm:ss");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<Map<String, Object>> items = itemRepository.summary(sdf.format(beginDate), sdf.format(endDate), cycle, tenantId, projectId);
        List<ChargeTrendData> values = new ArrayList<>();
        List<Object> moneyCount = new ArrayList<>();
        while (beginDate.before(endDate)) {
            String key;
            switch (cycle) {
                case MONTH:
                    key = MONTH_FORMAT.format(beginDate);
                    break;
                case DAY:
                default:
                    key = format.format(beginDate);
                    break;
            }
            keys.add(key);
            BigDecimal money = BigDecimal.valueOf(0);
            for (Map<String, Object> item : items) {
                if (null == item.get("gmt_create") || null == item.get("money")) {
                    continue;
                }
                String gmtCreate = (String) item.get("gmt_create");
                BigDecimal totalMoney = (BigDecimal) item.get("money");
                if (key.equalsIgnoreCase(gmtCreate)) {
                    money = money.add(totalMoney);
                }
            }
            moneyCount.add(money);
            switch (cycle) {
                case MONTH:
                    beginDate = DateUtils.addMonths(beginDate, 1);
                    break;
                case DAY:
                default:
                    beginDate = DateUtils.addDays(beginDate, 1);
                    break;
            }
        }
        values.add(new ChargeTrendData("消费金额(元)", moneyCount));
        resultTrend = new ChargeResultTrend(keys, values);
        return new BocloudResult<>(true, resultTrend, "查询成功");
    }

    /**
     * 租户产品收入趋势统计
     *
     * @param start
     * @param end
     * @param tenantId
     * @return
     * @throws Exception
     */
    private BocloudResult productTrend(String start, String end, Long tenantId, Long projectId) throws Exception {
        // 统计范围
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat MONTH_FORMAT = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat FORMAT = new SimpleDateFormat("yyyyMM");
        ChargeResultTrend resultTrend;
        List<String> keys = new ArrayList<>();
        Date endDate = new Date();
        endDate = DateUtils.addDays(endDate, 1);
        Date beginDate = DateUtils.addMonths(endDate, -6);
        if (StringUtils.hasText(start) && StringUtils.hasText(end)) {
            beginDate = DateUtils.parseDate(start, "yyyy-MM-dd HH:mm:ss");
            endDate = DateUtils.parseDate(end, "yyyy-MM-dd HH:mm:ss");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<Map<String, Object>> items =
                itemRepository.summary(sdf.format(beginDate), sdf.format(endDate), null, tenantId, projectId);
        List<ChargeTrendData> values = new ArrayList<>();
        Map<String, List<Map<String, Object>>> serviceMap = new HashMap<>();
        Map<String, List<Object>> resultMap = new HashMap<>();
        for (Map<String, Object> item : items) {
            if (null == item.get("service_name")) {
                continue;
            }
            String serviceName = (String) item.get("service_name");
            List<Map<String, Object>> billItems = serviceMap.get(serviceName);
            if (null == billItems) {
                billItems = new ArrayList<>();
            }
            billItems.add(item);
            serviceMap.put(serviceName, billItems);
        }
        while (beginDate.before(endDate)) {
            String key = format.format(beginDate);
            keys.add(key);
            for (String serviceName : serviceMap.keySet()) {
                List<Map<String, Object>> itemList = serviceMap.get(serviceName);
                BigDecimal count = BigDecimal.valueOf(0);
                for (Map<String, Object> item : itemList) {
                    if (null == item.get("gmt_create") || null == item.get("money")) {
                        continue;
                    }
                    String gmtCreate = (String) item.get("gmt_create");
                    BigDecimal totalMoney = (BigDecimal) item.get("money");
                    if (key.equalsIgnoreCase(gmtCreate)) {
                        count = count.add(totalMoney);
                    }
                }
                List<Object> objects = resultMap.get(serviceName);
                if (null == objects) {
                    objects = new ArrayList<>();
                }
                objects.add(count);
                resultMap.put(serviceName, objects);
            }
            beginDate = DateUtils.addDays(beginDate, 1);
        }
        for (String key : resultMap.keySet()) {
            values.add(new ChargeTrendData(key, resultMap.get(key)));
        }
        resultTrend = new ChargeResultTrend(keys, values);
        return new BocloudResult<>(true, resultTrend, "查询成功");
    }

    /**
     * 租户账单top5
     *
     * @param start
     * @param end
     * @return
     * @throws Exception
     */
    private BocloudResult tenantTop5(String start, String end) throws Exception {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        HashMap<String, BigDecimal> sortedMap = new HashMap<>(8);
        List<ServiceBill> serviceBills = billRepository.listByTenant(start, end, null, null).stream()
                .filter(serviceBill -> serviceBill.getTenantName() != null).collect(Collectors.toList());
        Map<String, List<ServiceBill>> tenantBillMap =
                serviceBills.stream().collect(groupingBy(ServiceBill::getTenantName));
        for (Map.Entry<String, List<ServiceBill>> tenantEntry : tenantBillMap.entrySet()) {
            String tenantName = tenantEntry.getKey();
            List<ServiceBill> tenantServiceBills = tenantEntry.getValue();
            BigDecimal income = tenantServiceBills.stream()
                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            sortedMap.put(tenantName, income);
        }
        List<Map.Entry<String, BigDecimal>> sortedResult =
                sortedMap.entrySet().stream().sorted(((o1, o2) -> o2.getValue().compareTo(o1.getValue()))).limit(5)
                        .collect(Collectors.toList());
        ArrayList<Object> data = new ArrayList<>();
        for (Map.Entry<String, BigDecimal> entry : sortedResult) {
            keys.add(entry.getKey());
            data.add(entry.getValue());
        }
        values.add(new ChargeTrendData("消费金额(元)", data));
        return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询成功");
    }

    /**
     * 租户账单top5
     *
     * @param start
     * @param end
     * @return
     * @throws Exception
     */
    private BocloudResult userTop5(String start, String end) throws Exception {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        HashMap<String, BigDecimal> sortedMap = new HashMap<>(8);
        List<ServiceBill> serviceBills = billRepository.listByUser(start, end, null, null).stream()
                .filter(serviceBill -> serviceBill.getUserName() != null).collect(Collectors.toList());
        Map<String, List<ServiceBill>> userBillsMap =
                serviceBills.stream().collect(groupingBy(ServiceBill::getUserName));
        for (Map.Entry<String, List<ServiceBill>> userEntry : userBillsMap.entrySet()) {
            String userName = userEntry.getKey();
            List<ServiceBill> tenantServiceBills = userEntry.getValue();
            BigDecimal income = tenantServiceBills.stream()
                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            sortedMap.put(userName, income);
        }
        List<Map.Entry<String, BigDecimal>> sortedResult =
                sortedMap.entrySet().stream().sorted(((o1, o2) -> o2.getValue().compareTo(o1.getValue()))).limit(5)
                        .collect(Collectors.toList());
        ArrayList<Object> data = new ArrayList<>();
        for (Map.Entry<String, BigDecimal> entry : sortedResult) {
            keys.add(entry.getKey());
            data.add(entry.getValue());
        }
        values.add(new ChargeTrendData("消费金额(元)", data));
        return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询成功");
    }

    /**
     * 产品收入分布
     *
     * @param start
     * @param end
     * @return
     * @throws Exception
     */
    private BocloudResult productIncome(String start, String end) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        List<Map<String, Object>> serviceBills = itemRepository.productSumary(start, end);

        Map<String, BigDecimal> resultMap = new HashMap<>();
        for (Map<String, Object> item : serviceBills) {
            if (null == item.get("service_name") || null == item.get("money")) {
                continue;
            }
            String serviceName = (String) item.get("service_name");
            BigDecimal money = resultMap.get(serviceName);
            if (null == money) {
                money = BigDecimal.valueOf(0L);
            }
            money = money.add((BigDecimal) item.get("money"));
            resultMap.put(serviceName, money);
        }
        ArrayList<Object> data = new ArrayList<>();
        for (Map.Entry<String, BigDecimal> entry : resultMap.entrySet()) {
            String serviceName = entry.getKey();
            BigDecimal income = entry.getValue();
            if (income.compareTo(BigDecimal.ZERO) > 0) {
                keys.add(serviceName);
                data.add(income);
            }
        }
        values.add(new ChargeTrendData("消费金额(元)", data));
        return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询成功");
    }

    /**
     * 租户产品消费统计
     *
     * @param start
     * @param end
     * @param tenantId
     * @return
     * @throws Exception
     */
    private BocloudResult billStatistics(String start, String end, Long tenantId, Long projectId) throws Exception {
        List<Map<String, Object>> items = itemRepository.summary(start, end, null, tenantId, projectId);
        Map<String, BigDecimal> resultMap = new HashMap<>();
        for (Map<String, Object> item : items) {
            if (null == item.get("service_name") || null == item.get("money")) {
                continue;
            }
            String serviceName = (String) item.get("service_name");
            BigDecimal money = resultMap.get(serviceName);
            if (null == money) {
                money = BigDecimal.valueOf(0L);
            }
            money = money.add((BigDecimal) item.get("money"));
            resultMap.put(serviceName, money);
        }
        List<String> keys = new ArrayList<>();
        List<Object> data = new ArrayList<>();
        LineChartResult chartResult;
        LineChartResult.Values[] values;
        for (String key : resultMap.keySet()) {
            keys.add(key);
            data.add(resultMap.get(key));
        }
        values = new LineChartResult.Values[]{new LineChartResult.Values("消费金额(元)", data)};
        chartResult = new LineChartResult(keys, values);
        return new BocloudResult<>(true, chartResult, "查询成功");
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result create(ServiceBill bill, RequestContext context) {
        try {
            bill.setSn(IdGenerator.getInstance().nextId());
            billRepository.save(bill);
            return new Result(true, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "创建失败");
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result modify(ServiceBill bill, RequestContext context) {
        String path = ServiceBill.class.getSimpleName() + "_" + bill.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceBill database = billRepository.query(bill.getId());
            Assert.notNull(database, "数据不存在！");
            BeanUtils.copyProperties(bill, database);
            boolean flag = billRepository.update(database);
            if (!flag) {
                return new Result(false, "修改失败");
            }
            return new Result(true, "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public BocloudResult<ServiceBill> detail(Long id) {
        try {
            ServiceBill bill = billRepository.query(id);
            if (null == bill) {
                return new BocloudResult<>(false, "账单不存在");
            }
            ServiceOrder order = orderRepository.query(bill.getOrderId());
            if (null != order) {
                CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
                if (null != service) {
                    bill.setServiceName(service.getName());
                }
            }
            return new BocloudResult<>(true, bill, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result remove(Long id, RequestContext context) {
        String path = ServiceBill.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceBill bill = billRepository.query(id);
            Assert.notNull(bill, "数据不存在！");
            Boolean flag = billRepository.remove(id);
            if (flag) {
                itemRepository.removeByBillId(id);
            }
            return flag ? new Result(true, "删除成功") : new Result(false, "删除失败");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public BocloudResult<GridBean<BillItem>> items(Pager pager, RequestContext context) {
        List<BillItem> list;
        int total;
        GridBean<BillItem> gridBean;
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Long tenantId = null;
            for (Param param : params) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("tenantId")) {
                    tenantId = Long.valueOf(paramMap.get("tenantId").toString());
                    paramMap.remove("tenantId");
                }
            }
            total = itemRepository.count(params, tenantId);
            list = itemRepository.list(pager.getPage(), pager.getRows(), sorter, params, tenantId);
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result createItem(BillItem item, RequestContext context) {
        try {
            itemRepository.save(item);
            return new Result(true, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "创建失败");
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result modifyItem(BillItem item, RequestContext context) {
        String path = BillItem.class.getSimpleName() + "_" + item.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            BillItem database = itemRepository.query(item.getId());
            Assert.notNull(database, "数据不存在！");
            BeanUtils.copyProperties(item, database);
            boolean flag = itemRepository.update(database);
            if (!flag) {
                return new Result(false, "修改失败");
            }
            return new Result(true, "修改成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public BocloudResult<BillItem> detailItem(Long id) {
        try {
            BillItem item = itemRepository.query(id);
            if (null == item) {
                return new BocloudResult<>(false, "子账单不存在");
            }
            return new BocloudResult<>(true, item, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result removeItem(Long id, RequestContext context) {
        String path = BillItem.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            BillItem item = itemRepository.query(id);
            Assert.notNull(item, "数据不存在！");
            Boolean flag = itemRepository.remove(id);
            return flag ? new Result(true, "删除成功") : new Result(false, "删除失败");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result batchSave(List<BillItem> billItems) {
        try {
            boolean flag = itemRepository.batchSave(billItems, 1000);
            if (!flag) {
                return new Result(false, "批量保存失败");
            }
            return new Result(true, "批量保存成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new Result(false, "批量保存失败");
        }
    }

    @Override
    public Result pay(String sn, Long couponId, RequestContext context) {
        try {
            ServiceBill bill = billRepository.queryBySn(sn);
            Assert.notNull(bill, "账单不存在");
            Assert.isTrue(ChargeCommon.BillStatus.paid != bill.getStatus(), "账单已支付，无需再次支付");
            ServiceOrder order = orderRepository.query(bill.getOrderId());
            Assert.notNull(order, "订单不存在");
            BigDecimal money = null == bill.getMoney() ? BigDecimal.valueOf(0) : bill.getMoney();
            BigDecimal discountMoney = BigDecimal.valueOf(0L);
            RequestContext requestContext =
                    new RequestContext(Catalog.Tenant, order.getTenantId(), order.getTenantId(), order.getProjectId(),
                            null);
            Long tenantId = order.getTenantId();
            if (null != order.getUserId() && 0L != order.getUserId()) {
                BocloudResult<User> userResult = smsUserService.detail(order.getUserId());
                if (userResult.isSuccess() && null != userResult.getData()) {
                    User user = userResult.getData();
                    tenantId = user.getTenantId();
                    requestContext.setTarget(user.getId());
                    requestContext.setTenant(user.getTenantId());
                    requestContext.setCatalog(Catalog.User);
                }
            }
            if (null != tenantId && 0L != tenantId) {
                // 扣款
                Result result = accountService
                        .accountAction(tenantId, bill.getId(), TenantAccount.Action.Consume, bill.getMoney(),
                                requestContext);
                Assert.isTrue(result.isSuccess(), result.getMessage());
                bill.setStatus(ChargeCommon.BillStatus.paid);
                billRepository.update(bill);
            }
            String params = bill.getParams();
            if (StringUtils.hasText(params)) {
                logger.info("账单相关资源参数：{}", params);
                JSONObject object = JSONObject.parseObject(params);
                String paramType = object.getString("paramType");
                if (AlterationParam.class.getSimpleName().equals(paramType)) {
                    AlterationParam param = JSONObject.parseObject(object.getString("params"), AlterationParam.class);
                    param.setChargeMode(ChargeCommon.ChargeMode.PreCharge);
                    CloudService service = cloudServiceRepository.queryByCode(param.getService());
                    Assert.notNull(service, "该服务不存在或者未发布");
                    Long flowId = serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.AlterationOperate);
                    if (null == flowId) {
                        amqpTemplate.convertAndSend("resource.modification.queue", param);
                    } else {
                        Map<String, Object> content =
                                getParams("申请服务实例变更", param.getInstanceName(), ApplyCategory.AlterationOperate.name(),
                                        JSONObject.toJSONString(param), "申请审核服务实例【" + param.getInstanceName() + "】变更",
                                        "审批通过，服务实例变更方可进行", context);
                        BocloudResult execution = fmsFlowService.startInstanceById(flowId, context, null, content);
                        if (execution.isFailed()) {
                            amqpTemplate.convertAndSend("resource.modification.queue", param);
                        }
                    }
                } else if (ExtensionParam.class.getSimpleName().equals(paramType)) {
                    ExtensionParam param = JSONObject.parseObject(object.getString("params"), ExtensionParam.class);
                    OrderItem item = orderItemRepository.queryBySn(param.getInstance());
                    Assert.notNull(item, "订单明细不存在");

                    CloudService service = cloudServiceRepository.queryByCode(param.getService());
                    Assert.notNull(service, "该服务不存在或者未发布");
                    Long flowId = serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.ExtensionOperate);
                    if (null == flowId) {
                        handleExtension(param, order, item, context);
                    } else {
                        JSONObject extension = new JSONObject();
                        extension.put("extension", param);
                        extension.put("order", order);
                        extension.put("item", item);
                        Map<String, Object> content =
                                getParams("申请服务实例延期", param.getInstanceName(), ApplyCategory.ExtensionOperate.name(),
                                        extension.toJSONString(), "申请审核服务实例【" + param.getInstanceName() + "】延期",
                                        "审批通过，服务实例延期方可进行", context);
                        BocloudResult execution = fmsFlowService.startInstanceById(flowId, context, null, content);
                        if (execution.isFailed()) {
                            handleExtension(param, order, item, context);
                        }
                    }
                }
            }
            return new Result(true, "账单支付成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, e.getMessage());
        }
    }

    private Map<String, Object> getParams(String title, String name, String category, String params, String theme,
                                          String remark, RequestContext context) {
        List<SimpleBean> beans = Lists.newArrayList();
        beans.add(SimpleBean.getBean("申请主题", title));
        beans.add(SimpleBean.getBean("服务名称", name));
        Map<String, Object> map = MapTools.simpleMap(Common.PARAMS, beans);
        map.put("ApplyCategory", category);
        map.put("ApplyParams", params);
        map.put("ApplyTheme", theme);
        map.put("ApplyRemark", remark);
        map.put("ApplyContext", context);
        return map;
    }

    private void handleExtension(ExtensionParam param, ServiceOrder order, OrderItem item, RequestContext context) {
        amqpTemplate.convertAndSend("resource.extension.queue", param);
        if ("vmware.k8s.cluster".equals(item.getServiceCode())) {
            JSONObject jsonObject = new JSONObject();
            String duration = item.getDuration();
            if (StringUtils.hasText(duration)) {
                DurationModel durationModel = JSONObject.parseObject(duration, DurationModel.class);
                Integer oldMonth = durationModel.getAmount();
                if (null == oldMonth) {
                    oldMonth = 0;
                }
                oldMonth += param.getExtension();
                jsonObject.put("orderTime", oldMonth);
            }
            this.callBoc(order.getSn(), jsonObject, "/boclouddeployerapi/cmpCluster/v2.4/extensionOrderCallback",
                    "success", "延期支付成功", context);
        }
    }

    @Override
    public Result cancel(String sn, RequestContext context) {
        try {
            ServiceBill bill = billRepository.queryBySn(sn);
            Assert.notNull(bill, "账单不存在");
            if (ChargeCommon.BillStatus.canceld == bill.getStatus()) {
                return new Result(false, "账单已取消，无需再次取消");
            }
            bill.setStatus(ChargeCommon.BillStatus.canceld);
            billRepository.update(bill);
            // 删除账单明细
            itemRepository.removeByBillId(bill.getId());
            String params = bill.getParams();
            if (StringUtils.hasText(params)) {
                JSONObject object = JSONObject.parseObject(params);
                String paramType = object.getString("paramType");
                if (AlterationParam.class.getSimpleName().equals(paramType)) {
                    AlterationParam alteration =
                            JSONObject.parseObject(object.getString("params"), AlterationParam.class);
                    // 取消支付后恢复原子订单明细状态
                    OrderItem item = orderItemRepository.queryBySn(alteration.getInstance());
                    if (null != item && null != alteration.getOldStatus()) {
                        item.setStatus(alteration.getOldStatus());
                        orderItemRepository.update(item);
                    }
                    // 新订单标记为变更失败
                    OrderItem newItem = orderItemRepository.queryBySn(alteration.getNewInstance());
                    if (null != newItem) {
                        newItem.setStatus(OrderItemStatus.ChangeFailure);
                        orderItemRepository.update(newItem);
                    }
                    ServiceOrder order = orderRepository.query(bill.getOrderId());
                    List<QuotaBean> quotaBeans = alteration.getQuotas();
                    if (null != order) {
                        if (!ListTool.isEmpty(quotaBeans)) {
                            // 回滚配额
                            this.handleQuota(alteration.getQuotas(), order.getTenantId(), order.getProjectId());
                        }
                        if ("vmware.k8s.cluster".equals(order.getServiceCode())) {
                            this.callBoc(order.getSn(), new JSONObject(),
                                    "/boclouddeployerapi/cmpCluster/v2.4/updateCluster", "cancel", "扩容取消支付", context);
                        }
                    }
                } else if (ExtensionParam.class.getSimpleName().equals(paramType)) {
                    ExtensionParam extension = JSONObject.parseObject(object.getString("params"), ExtensionParam.class);
                    // 取消支付后恢复原子订单明细状态
                    OrderItem item = orderItemRepository.queryBySn(extension.getInstance());
                    if (null != item && null != extension.getOldStatus()) {
                        item.setStatus(extension.getOldStatus());
                        orderItemRepository.update(item);
                    }
                    ServiceOrder order = orderRepository.query(bill.getOrderId());
                    if (null != order && "vmware.k8s.cluster".equals(order.getServiceCode())) {
                        this.callBoc(order.getSn(), new JSONObject(),
                                "/boclouddeployerapi/cmpCluster/v2.4/extensionOrderCallback", "cancel", "延期取消支付", context);
                    }
                }
            }
            return new Result(true, "订单取消成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, "订单取消失败");
        }
    }

    /**
     * 调用boc
     * http://10.10.5.23:30022/boclouddeployerapi/cmpCluster/v2.4/createCluster
     *
     * @param callback
     * @param params
     * @param bocUrl
     * @param result   success|failed
     * @param message
     * @return
     */
    private Result callBoc(String callback, JSONObject params, String bocUrl, String result, String message,
                           RequestContext context) {
        HttpClient httpClient = new HttpClient(20 * 1000 * 1000, PostDataFormat.RAW);
        String token = socInternalService.getToken(context.getTarget(), context.getCatalog().name());
        logger.info("获取token：{}", token);
        try {
            params.put("callback", callback);
            params.put("result", result);
            params.put("message", message);
            Map<String, Object> header = new HashMap<>();
            header.put("accept", "application/json");
            header.put("content-type", "application/json");
            if (StringUtils.hasText(token)) {
                header.put("token", token);
            }
            String url = configService.getByCode("k8sAddress") + bocUrl;
            logger.info("调用boc地址：{}", url);
            logger.info("调用boc参数：{}", params.toJSONString());
            return httpClient.post(header, params, url);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return Result.FAILED("回调容器平台失败");
        } finally {
            httpClient.close();
        }
    }

    /**
     * 回滚配额
     *
     * @param beans
     * @param tenantId
     * @param projectId
     * @throws Exception
     */
    private void handleQuota(List<QuotaBean> beans, Long tenantId, Long projectId) throws Exception {
        List<ResourceQuota> list = resourceQuotaRepository.listByTenantId(projectId);
        if (ListTool.isEmpty(list)) {
            return;
        }
        Map<String, ResourceQuota> map = new HashMap<>();
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (QuotaBean bean : beans) {
            if (!map.containsKey(bean.getCode())) {
                continue;
            }
            ResourceQuota quota = map.get(bean.getCode());
            // 记录回滚配额操作量
            quota.setQuotaActionValue(bean.getValue().longValue());
            quotas.add(quota);
        }
        resourceQuotaService.quotaAction(quotas, ResourceQuota.Action.Callback, tenantId);
    }

    /**
     * @param code    统计类型
     * @param range   统计范围
     * @param context
     * @return
     */
    @Override
    public BocloudResult<LineChartResult> stats(String code, String range, RequestContext context) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat MONTH_FORMAT = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat FORMAT = new SimpleDateFormat("yyyyMM");
        try {
            List<String> keys = new ArrayList<>();
            List<Object> data = new ArrayList<>();
            LineChartResult chartResult;
            LineChartResult.Values[] values;
            BocloudResult<LineChartResult> result;
            List<Param> params = new ArrayList<>();
            // 统计范围
            List<Map<String, Object>> bills;
            switch (code) {
                // 租户收入金额统计
                case "tenantStats":
                    params.add(new Param(MapTools.simpleMap("month", MONTH_FORMAT.format(new Date())), Sign.UEQ));
                    BigDecimal summary = billRepository.summary(params);
                    result = new BocloudResult(true, new StatsResult("总金额(元)", summary), "查询成功");
                    break;
                // 租户收入金额TOP5
                case "tenantStatsTop5":
                    bills = billRepository.tenantTop5();
                    for (Map<String, Object> bill : bills) {
                        keys.add((String) bill.get("tenant_name"));
                        data.add(null == bill.get("total") ? 0 : bill.get("total"));
                    }
                    values = new LineChartResult.Values[]{new LineChartResult.Values("金额(元)", data)};
                    chartResult = new LineChartResult(keys, values);
                    result = new BocloudResult<>(true, chartResult, "查询成功");
                    break;
                // 租户收入金额TOP5
                case "projectStatsTop5":
                    List<Project> projects = smsProjectService.listTenantProject(context.getTenant());
                    List<Long> projectIds = projects.stream().map(Project::getId).collect(Collectors.toList());
                    if (ListTool.isEmpty(projectIds)) {
                        projectIds = new ArrayList<>();
                        projectIds.add(-1L);
                    }
                    bills = billRepository.projectTop5(projectIds);
                    for (Map<String, Object> bill : bills) {
                        keys.add((String) bill.get("project_name"));
                        data.add(null == bill.get("total") ? 0 : bill.get("total"));
                    }
                    values = new LineChartResult.Values[]{new LineChartResult.Values("金额(元)", data)};
                    chartResult = new LineChartResult(keys, values);
                    result = new BocloudResult<>(true, chartResult, "查询成功");
                    break;
                // 租户收入统计趋势
                case "tenantTrend":
                    if (!StringUtils.hasText(range)) {
                        range = StatsRange.WEEK.name();
                    }
                    int months = -11;
                    while (months <= 0) {
                        Calendar calendar = Calendar.getInstance();
                        calendar.add(Calendar.MONTH, months);
                        params.add(
                                new Param(MapTools.simpleMap("month", MONTH_FORMAT.format(calendar.getTime())), Sign.UEQ));
                        keys.add(MONTH_FORMAT.format(calendar.getTime()));
                        BigDecimal tenantSummary = billRepository.summary(params);
                        data.add(null == tenantSummary ? 0.0 : tenantSummary);
                        months++;
                    }
                    values = new LineChartResult.Values[]{new LineChartResult.Values("金额(元)", data)};
                    chartResult = new LineChartResult(keys, values);
                    result = new BocloudResult<>(true, chartResult, "查询成功");
                    break;
                case "TenantConsume":
                    int field;
                    int allInterval;
                    if (StringUtils.isEmpty(range)) {
                        range = StatsRange.WEEK.name();
                        field = Calendar.DAY_OF_WEEK;
                        allInterval = -6;
                    }
                    switch (StatsRange.valueOf(range)) {
                        case WEEK:
                            field = Calendar.DATE;
                            allInterval = -6;
                            break;
                        case MONTH:
                            field = Calendar.DATE;
                            allInterval = -29;
                            break;
                        case QUARTER:
                            field = Calendar.MONTH;
                            allInterval = -2;
                            break;
                        case HALFYEAR:
                            field = Calendar.MONTH;
                            allInterval = -5;
                            break;
                        case YEAR:
                            field = Calendar.MONTH;
                            allInterval = -11;
                            break;
                        default:
                            field = Calendar.DATE;
                            allInterval = -6;
                            break;
                    }
                    while (allInterval <= 0) {
                        Calendar calendar = Calendar.getInstance();
                        calendar.add(field, allInterval);
                        params.add(
                                new Param(MapTools.simpleMap("month", MONTH_FORMAT.format(calendar.getTime())), Sign.UEQ));
                        if (field == Calendar.DATE) {
                            keys.add(format.format(calendar.getTime()));
                        } else {
                            keys.add(MONTH_FORMAT.format(calendar.getTime()));
                        }
                        BigDecimal tenantSummary = billRepository.summary(params);
                        data.add(null == tenantSummary ? 0.0 : tenantSummary);
                        allInterval++;
                    }
                    values = new LineChartResult.Values[]{new LineChartResult.Values("金额(元)", data)};
                    chartResult = new LineChartResult(keys, values);
                    result = new BocloudResult<>(true, chartResult, "查询成功");
                    break;
                default:
                    result = new BocloudResult<>(false, "暂不支持");
                    break;
            }
            return result;
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    public enum StatsRange {
        WEEK, // 一周
        MONTH, // 一月
        QUARTER, // 季度
        HALFYEAR, // 半年
        YEAR// 一年
    }

    public enum BillCode {
        IncomeTotal, // 收入总计
        IncomeTop5, // 收入TOP5
        RegionTop5, // 地域收入TOP5
        ProductStats, // 产品统计
        MonthIncomeTrend, // 月收入趋势统计
        TenantTop5, // 订购租户数TOP5
        TenantIncomeTop5, // 租户收入TOP5
        TenantMonthIncomeTrend, // 租户月收入趋势统计
        TenantIncomeTotal// 租户收入总计
    }

    @Override
    public BocloudResult dashboard(String code, String start, String end, String city, Long managerId, Long tenantId) {
        SimpleDateFormat FORMAT = new SimpleDateFormat("yyyyMM");
        try {
            List<Map<String, Object>> resultList;
            ChargeResultTrend trend = null;
            List<String> keys = new ArrayList<>();
            List<ChargeTrendData> values = new ArrayList<>();
            List<Object> data = new ArrayList<>();
            switch (BillCode.valueOf(code)) {
                case IncomeTotal:// 收入总计
                case TenantIncomeTotal:// 租户消费总计
                    Double summary = billRepository.summary(null, tenantId, managerId, city);
                    return new BocloudResult(true, null == summary ? 0.0 : summary, "查询成功");
                // 收入TOP5
                case IncomeTop5:
                    resultList = billRepository.managerTop5(start, end, city, managerId);
                    if (!ListTool.isEmpty(resultList)) {
                        for (Map<String, Object> result : resultList) {
                            keys.add(String.valueOf(result.get("manager_name")));
                            data.add(null == result.get("total") ? 0 : result.get("total"));
                        }
                    }
                    values.add(new ChargeTrendData("金额(元)", data));
                    trend = new ChargeResultTrend(keys, values);
                    break;
                // 地域收入TOP5
                case RegionTop5:
                    resultList = billRepository.cityTop5(start, end, city, managerId);
                    if (!ListTool.isEmpty(resultList)) {
                        for (Map<String, Object> result : resultList) {
                            keys.add(buildCityName(String.valueOf(result.get("city"))));
                            data.add(null == result.get("total") ? 0 : result.get("total"));
                        }
                    }
                    values.add(new ChargeTrendData("金额(元)", data));
                    trend = new ChargeResultTrend(keys, values);
                    break;
                // 租户收入TOP5
                case TenantIncomeTop5:
                    resultList = billRepository.tenantTop5(start, end, city, tenantId);
                    if (!ListTool.isEmpty(resultList)) {
                        for (Map<String, Object> result : resultList) {
                            keys.add(String.valueOf(result.get("tenant_name")));
                            data.add(null == result.get("total") ? 0 : result.get("total"));
                        }
                    }
                    values.add(new ChargeTrendData("金额(元)", data));
                    trend = new ChargeResultTrend(keys, values);
                    break;
                case MonthIncomeTrend:// 月收入趋势统计
                case TenantMonthIncomeTrend:// 租户月收入趋势统计
                    Date endDate = new Date();
                    if (null != end) {
                        endDate = FORMAT.parse(end);
                    }
                    DateUtils.addMonths(endDate, 1);
                    Date beginDate = DateUtils.addYears(endDate, -1);
                    if (null != start) {
                        beginDate = FORMAT.parse(start);
                    }
                    while (beginDate.before(endDate)) {
                        String key = FORMAT.format(beginDate);
                        keys.add(key);
                        Double count = billRepository.summary(key, tenantId, managerId, city);
                        data.add(count);
                        beginDate = DateUtils.addMonths(beginDate, 1);
                    }
                    values.add(new ChargeTrendData("金额(元)", data));
                    trend = new ChargeResultTrend(keys, values);
                    break;
                default:
                    break;
            }
            return new BocloudResult<>(true, trend, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 获取城市名称
     *
     * @param city
     * @return
     */
    private String buildCityName(String city) {
        String name = "未知";
        if ("350100".equals(city)) {
            name = "福州市";
        } else if ("350200".equals(city)) {
            name = "厦门市";
        } else if ("350300".equals(city)) {
            name = "莆田市";
        } else if ("350400".equals(city)) {
            name = "三明市";
        } else if ("350500".equals(city)) {
            name = "泉州市";
        } else if ("350600".equals(city)) {
            name = "漳州市";
        } else if ("350700".equals(city)) {
            name = "南平市";
        } else if ("350800".equals(city)) {
            name = "龙岩市";
        } else if ("350900".equals(city)) {
            name = "宁德市";
        }
        return name;
    }

    @Override
    public Result recycleBill(String orderItemSn) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat MONTH_FORMAT = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat FORMAT = new SimpleDateFormat("yyyyMM");
        try {
            OrderItem item = orderItemRepository.queryBySn(orderItemSn);
            Assert.notNull(item, "订单明细不存在");
            ServiceOrder order = orderRepository.query(item.getOrderId());
            Assert.notNull(order, "订单不存在");
            if (ChargeCommon.ChargeMode.PreCharge == order.getChargeMode()) {
                return new Result(true, "预付费无需生产账单");
            }
            ServiceBill bill =
                    new ServiceBill(IdGenerator.getInstance().nextId(), order.getId(), ChargeCommon.BillStatus.paid, null,
                            order.getTenantId(), MONTH_FORMAT.format(new Date()), new Date());
            bill.setUserId(order.getUserId());
            bill.setProjectId(order.getProjectId());
            bill.setBusinessId(order.getBusinessId());
            billRepository.save(bill);
            BigDecimal finalPrice = item.getFinalPrice();
            if (null == finalPrice) {
                finalPrice = BigDecimal.valueOf(0);
            }
            // 当前跑批账单开始时间
            Date currBillTime = getBillStartDate();
            if (null != item.getMoveOutTime()) {
                logger.info("回收站上一次移出时间：{}", item.getMoveOutTime());
                currBillTime = item.getMoveOutTime();
            }
            // 生成账单时间
            Date discardDate = new Date();
            Long mintues = 0L;
            BillItem billItem = new BillItem();
            mintues = (discardDate.getTime() - currBillTime.getTime()) / 60000;
            billItem.setBillStart(currBillTime);
            billItem.setBillEnd(discardDate);
            BigDecimal billMoney =
                    finalPrice.divide(new BigDecimal(60), 5, RoundingMode.HALF_UP).multiply(new BigDecimal(mintues));
            billItem.setItemId(item.getId());
            billItem.setSkuId(item.getSkuId());
            billItem.setChargeMode(order.getChargeMode());
            billItem.setPrice(billMoney);
            billItem.setMoney(billMoney);
            billItem.setGmtCreate(new Date());
            billItem.setCatalog(BillItem.Catalog.standard);
            OrderItemUtils.elementNames(item);
            billItem.setResource(JSONObject.toJSONString(item.getElementList()));
            billItem.setBillId(bill.getId());
            billItem.setUserId(order.getUserId());
            billItem.setTenantId(order.getTenantId());
            billItem.setProjectId(order.getProjectId());
            billItem.setBusinessId(order.getBusinessId());
            billItem.setServiceCode(item.getServiceCode());
            billItem.setInstance(item.getSn());
            itemRepository.save(billItem);
            bill.setMoney(billMoney);
            billRepository.update(bill);
            RequestContext requestContext =
                    new RequestContext(Catalog.Tenant, order.getTenantId(), order.getTenantId(), order.getProjectId(),
                            null);
            Long tenantId = order.getTenantId();
            if (null != order.getUserId() && 0L != order.getUserId()) {
                BocloudResult<User> userResult = smsUserService.detail(order.getUserId());
                if (userResult.isSuccess() && null != userResult.getData()) {
                    User user = userResult.getData();
                    tenantId = user.getTenantId();
                    requestContext.setTarget(user.getId());
                    requestContext.setTenant(user.getTenantId());
                    requestContext.setCatalog(Catalog.User);
                }
            }
            if (null != tenantId && 0L != tenantId) {
                // 扣款
                Result result = accountService
                        .accountAction(tenantId, bill.getId(), TenantAccount.Action.Consume, bill.getMoney(),
                                requestContext);
                Assert.isTrue(result.isSuccess(), result.getMessage());
                bill.setStatus(ChargeCommon.BillStatus.paid);
                billRepository.update(bill);
            }
            return new Result(true, "账单生成成功");
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new Result(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult report(String code, String vendorType, String range, Long tenantId, RequestContext context) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat MONTH_FORMAT = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat FORMAT = new SimpleDateFormat("yyyyMM");
        try {
            if (StringUtils.isEmpty(range)) {
                range = StatsRange.WEEK.name();
            }
            BocloudResult result = new BocloudResult<>(true, "查询成功");
            Date startDate;
            Date endDate = new Date();
            endDate = DateUtils.addDays(endDate, 1);
            switch (StatsRange.valueOf(range)) {
                case WEEK:
                    startDate = DateUtils.addWeeks(endDate, -1);
                    break;
                case MONTH:
                    startDate = DateUtils.addMonths(endDate, -1);
                    break;
                case QUARTER:
                    startDate = DateUtils.addMonths(endDate, -3);
                    break;
                case HALFYEAR:
                    startDate = DateUtils.addMonths(endDate, -6);
                    break;
                case YEAR:
                    startDate = DateUtils.addYears(endDate, -1);
                    break;
                default:
                    startDate = DateUtils.addWeeks(endDate, -1);
                    break;
            }
            ChargeResultTrend trend;
            List<ChargeTrendData> data = new ArrayList<>();
            List<String> keys = new ArrayList<>();
            List<StatsResult> statsResults = new ArrayList<>();
            switch (code) {
                // 消费总数
                case "TotalConsumption":
                    List<Map<String, Object>> totalMap = billRepository
                            .totalConsumption(format.format(startDate) + " 00:00:00", format.format(endDate) + " 23:59:59",
                                    vendorType, tenantId);
                    Map<String, Object> totalConsumptionResult = new HashMap<>();
                    if (!totalMap.isEmpty()) {
                        for (Map<String, Object> total : totalMap) {
                            String vendor =
                                    null == total.get("vendor_type") ? "VMWARE" : (String) total.get("vendor_type");
                            keys.add(this.getVendorName(vendor));
                            List<Object> totalResult = new ArrayList<>();
                            Object totalMoney = null == total.get("total_money") ? 0 : total.get("total_money");
                            totalResult.add(totalMoney);
                            data.add(new ChargeTrendData(getVendorName(vendor), totalResult));
                            // 饼图数据
                            statsResults.add(new StatsResult(getVendorName(vendor), totalMoney));
                        }
                        trend = new ChargeResultTrend(keys, data);
                        totalConsumptionResult.put("table", trend);
                        totalConsumptionResult.put("pie", statsResults);
                    }
                    result.setData(totalConsumptionResult);
                    // 处理成饼图
                    break;
                // 消费top5
                case "Top5Consumption":
                    Map<String, Object> top5ConsumptionResult = new HashMap<>();
                    List<Map<String, Object>> tenantTop5Map = billRepository
                            .top5Consumption(format.format(startDate) + " 00:00:00", format.format(endDate) + " 23:59:59",
                                    vendorType, true, null);
                    if (!tenantTop5Map.isEmpty()) {
                        List<ChargeTrendData> tenantData = new ArrayList<>();
                        List<String> tenantKeys = new ArrayList<>();
                        List<Object> totalResult = new ArrayList<>();
                        for (Map<String, Object> total : tenantTop5Map) {
                            String tenantName =
                                    null == total.get("tenant_name") ? "" : (String) total.get("tenant_name");
                            tenantKeys.add(tenantName);
                            Object totalMoney = null == total.get("total_money") ? 0 : total.get("total_money");
                            totalResult.add(totalMoney);
                        }
                        tenantData.add(new ChargeTrendData("金额(元)", totalResult));
                        top5ConsumptionResult.put("tenant", new ChargeResultTrend(tenantKeys, tenantData));
                    }
                    List<Map<String, Object>> projectTop5Map = billRepository
                            .top5Consumption(format.format(startDate) + " 00:00:00", format.format(endDate) + " 23:59:59",
                                    vendorType, false, tenantId);
                    if (!projectTop5Map.isEmpty()) {
                        List<ChargeTrendData> projectData = new ArrayList<>();
                        List<String> projectKeys = new ArrayList<>();
                        List<Object> totalResult = new ArrayList<>();
                        for (Map<String, Object> total : projectTop5Map) {
                            String projectName =
                                    null == total.get("project_name") ? "" : (String) total.get("project_name");
                            projectKeys.add(projectName);
                            Object totalMoney = null == total.get("total_money") ? 0 : total.get("total_money");
                            totalResult.add(totalMoney);
                        }
                        projectData.add(new ChargeTrendData("金额(元)", totalResult));
                        top5ConsumptionResult.put("project", new ChargeResultTrend(projectKeys, projectData));
                    }
                    result.setData(top5ConsumptionResult);
                    break;
                // 云资源消费趋势
                case "ResourceConsumptionTrend":
                    List<Map<String, Object>> bills = billRepository
                            .resourceConsumptionTrend(format.format(startDate) + " 00:00:00",
                                    format.format(endDate) + " 23:59:59", vendorType, tenantId);
                    List<Object> vmwares = new ArrayList<>();
                    List<Object> tianyis = new ArrayList<>();
                    List<Object> h3cs = new ArrayList<>();
                    List<Object> huaweis = new ArrayList<>();
                    while (startDate.before(endDate)) {
                        String key = format.format(startDate);
                        keys.add(key);
                        double vmware = 0.0D;
                        double tianyi = 0.0D;
                        double h3c = 0.0D;
                        double huawei = 0.0D;
                        for (Map<String, Object> bill : bills) {
                            if (null == bill.get("vendor_type") || null == bill.get("money")) {
                                continue;
                            }
                            String type = (String) bill.get("vendor_type");
                            String gmtCreate = (String) bill.get("gmt_create");
                            BigDecimal money = (BigDecimal) bill.get("money");
                            if (key.equalsIgnoreCase(gmtCreate)) {
                                switch (type) {
                                    case "VMWARE":
                                        vmware += money.doubleValue();
                                        break;
                                    case "TIANYI":
                                        tianyi += money.doubleValue();
                                        break;
                                    case "H3C":
                                        h3c += money.doubleValue();
                                        break;
                                    case "HUAWEI":
                                        huawei += money.doubleValue();
                                        break;
                                    default:
                                        break;
                                }
                            }
                        }
                        vmware = MathUtils.round(vmware, 2);
                        vmwares.add(vmware);
                        tianyi = MathUtils.round(tianyi, 2);
                        tianyis.add(tianyi);
                        h3c = MathUtils.round(h3c, 2);
                        h3cs.add(h3c);
                        huawei = MathUtils.round(huawei, 2);
                        huaweis.add(huawei);
                        startDate = DateUtils.addDays(startDate, 1);
                    }
                    if (!StringUtils.hasText(vendorType)) {
                        data.add(new ChargeTrendData("政务云", vmwares));
                        data.add(new ChargeTrendData("天翼云", tianyis));
                        data.add(new ChargeTrendData("华三云", h3cs));
                        data.add(new ChargeTrendData("华为云", huaweis));
                    } else {
                        if (CloudProviderType.VMWARE.name().equalsIgnoreCase(vendorType)) {
                            data.add(new ChargeTrendData("政务云", vmwares));
                        } else if (CloudProviderType.TIANYI.name().equalsIgnoreCase(vendorType)) {
                            data.add(new ChargeTrendData("天翼云", tianyis));
                        } else if (CloudProviderType.H3C.name().equalsIgnoreCase(vendorType)) {
                            data.add(new ChargeTrendData("华三云", h3cs));
                        } else if (CloudProviderType.HUAWEI.name().equalsIgnoreCase(vendorType)) {
                            data.add(new ChargeTrendData("华为云", huaweis));
                        }
                    }
                    result.setData(new ChargeResultTrend(keys, data));
                    break;
                //单位消费趋势
                case "TenantConsumptionTrend":
                    List<Map<String, Object>> tenantBills = billRepository
                            .resourceConsumptionTrend(format.format(startDate) + " 00:00:00",
                                    format.format(endDate) + " 23:59:59", vendorType, tenantId);
                    Map<String, List<Object>> tenantMap = new HashMap<>(8);
                    for (Map<String, Object> bill : tenantBills) {
                        if (null == bill.get("tenant_name")) {
                            continue;
                        }
                        tenantMap.put((String) bill.get("tenant_name"), new ArrayList<>());
                    }
                    //过滤所有租户
                    while (startDate.before(endDate)) {
                        String key = format.format(startDate);
                        keys.add(key);
                        Map<String, Double> tenantMoneyMap = new HashMap<>(8);
                        for (Map<String, Object> bill : tenantBills) {
                            if (null == bill.get("money") || null == bill.get("tenant_name")) {
                                continue;
                            }
                            String gmtCreate = (String) bill.get("gmt_create");
                            BigDecimal money = (BigDecimal) bill.get("money");
                            if (key.equalsIgnoreCase(gmtCreate)) {
                                String tenantName = (String) bill.get("tenant_name");
                                if (!tenantMoneyMap.containsKey(tenantName)) {
                                    tenantMoneyMap.put(tenantName, 0.0);
                                }
                                Double tenantMoney = tenantMoneyMap.get(tenantName);
                                tenantMoney += money.doubleValue();
                                tenantMoneyMap.put(tenantName, tenantMoney);
                            }
                        }
                        for (String tenantName : tenantMap.keySet()) {
                            Double money = tenantMoneyMap.get(tenantName);
                            if (null == money) {
                                money = 0.0;
                            }
                            List<Object> moneyObjs = tenantMap.get(tenantName);
                            if (ListTool.isEmpty(moneyObjs)) {
                                moneyObjs = new ArrayList<>();
                            }
                            money = MathUtils.round(money, 2);
                            moneyObjs.add(money);
                            tenantMap.put(tenantName, moneyObjs);
                        }
                        startDate = DateUtils.addDays(startDate, 1);
                    }
                    for (String tenantName : tenantMap.keySet()) {
                        data.add(new ChargeTrendData(tenantName, tenantMap.get(tenantName)));
                    }
                    result.setData(new ChargeResultTrend(keys, data));
                    break;
                //重点项目消费趋势
                case "ProjectConsumptionTrend":
                    List<Map<String, Object>> projectBills = billRepository
                            .projectConsumptionTrend(format.format(startDate) + " 00:00:00",
                                    format.format(endDate) + " 23:59:59", vendorType, tenantId);
                    Map<String, List<Object>> projectMap = new HashMap<>();
                    for (Map<String, Object> bill : projectBills) {
                        if (null == bill.get("project_name")) {
                            continue;
                        }
                        projectMap.put((String) bill.get("project_name"), new ArrayList<>());
                    }
                    //过滤所有租户
                    while (startDate.before(endDate)) {
                        String key = format.format(startDate);
                        keys.add(key);
                        Map<String, Double> projectMoneyMap = new HashMap<>();
                        for (Map<String, Object> bill : projectBills) {
                            if (null == bill.get("money") || null == bill.get("project_name")) {
                                continue;
                            }
                            String gmtCreate = (String) bill.get("gmt_create");
                            BigDecimal money = (BigDecimal) bill.get("money");
                            if (key.equalsIgnoreCase(gmtCreate)) {
                                String projectName = (String) bill.get("project_name");
                                if (!projectMoneyMap.containsKey(projectName)) {
                                    projectMoneyMap.put(projectName, 0.0);
                                }
                                Double projectMoney = projectMoneyMap.get(projectName);
                                projectMoney += money.doubleValue();
                                projectMoneyMap.put(projectName, projectMoney);
                            }
                        }
                        for (String projectName : projectMap.keySet()) {
                            Double money = projectMoneyMap.get(projectName);
                            if (null == money) {
                                money = 0.0;
                            }
                            List<Object> moneyObjs = projectMap.get(projectName);
                            if (ListTool.isEmpty(moneyObjs)) {
                                moneyObjs = new ArrayList<>();
                            }
                            money = MathUtils.round(money, 2);
                            moneyObjs.add(money);
                            projectMap.put(projectName, moneyObjs);
                        }
                        startDate = DateUtils.addDays(startDate, 1);
                    }
                    for (String projectName : projectMap.keySet()) {
                        data.add(new ChargeTrendData(projectName, projectMap.get(projectName)));
                    }
                    result.setData(new ChargeResultTrend(keys, data));
                    break;
                default:
                    result.setSuccess(false);
                    result.setMessage("暂不支持");
                    break;
            }
            return result;
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    /**
     * 获取云平台名称
     *
     * @param vendorType
     * @return
     */
    private String getVendorName(String vendorType) {
        String vendorName = "";
        if (CloudProviderType.VMWARE.name().equalsIgnoreCase(vendorType)) {
            vendorName = "政务云";
        } else if (CloudProviderType.TIANYI.name().equalsIgnoreCase(vendorType)) {
            vendorName = "天翼云";
        } else if (CloudProviderType.H3C.name().equalsIgnoreCase(vendorType)) {
            vendorName = "华三云";
        } else if (CloudProviderType.HUAWEI.name().equalsIgnoreCase(vendorType)) {
            vendorName = "华为云";
        }
        return vendorName;
    }

    /**
     * 当前小时开始时间
     *
     * @return
     */
    private Date getBillStartDate() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR));
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }
}
