package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.enums.ResourceCategory;
import com.bocloud.cos.internal.*;
import com.bocloud.cos.intf.*;
import com.bocloud.cos.model.*;
import com.bocloud.cos.operate.*;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.utils.ExportUtil;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.cos.internel.entity.Project;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.cos.internel.entity.User;
import com.bocloud.cos.internel.entity.Volume;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.bocloud.cos.operate.OrderPriceUtils.getPrice;
import static java.util.stream.Collectors.groupingBy;

/**
 * 订单业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Service
public class ServiceOrderServiceImpl implements ServiceOrderService {

    private static final Logger logger = LoggerFactory.getLogger(ServiceOrderServiceImpl.class);
    private final ServiceOrderRepository orderRepository;
    private final OrderItemRepository itemRepository;
    private final ServiceSkuRepository skuRepository;
    private final ServiceBillRepository billRepository;
    private final BillItemRepository billItemRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final SmsInternalService socService;
    private final SmsUserService smsUserService;
    private final LockFactory lockFactory;
    private final AmqpTemplate amqpTemplate;
    private final SmsSystemConfigService configService;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final ResourceQuotaService resourceQuotaService;
    private final ImsInternalService cmpInternalService;
    private final CosService cosService;
    private final RosService rosService;
    private final OrderSnService orderSnService;
    private final SmsTenantService smsTenantService;
    private final SmsProjectService smsProjectService;
    private final TenantAccountService accountService;
    private final ChargeItemRepository chargeItemRepository;
    //    private static final SimpleDateFormat FORMAT = new SimpleDateFormat("yyyy-MM");
    private final ServiceFactory serviceFactory;

    public ServiceOrderServiceImpl(OrderItemRepository itemRepository, ServiceOrderRepository orderRepository,
                                   ImsInternalService cmpInternalService, ResourceQuotaService resourceQuotaService, OrderSnService orderSnService,
                                   ServiceSkuRepository skuRepository, ServiceBillRepository billRepository, BillItemRepository billItemRepository,
                                   SmsSystemConfigService configService, CloudServiceRepository cloudServiceRepository,
                                   SmsInternalService socService, SmsUserService smsUserService, TenantAccountService accountService,
                                   LockFactory lockFactory, RosService rosService, ServiceCategoryRepository categoryRepository,
                                   SmsTenantService smsTenantService, ResourceQuotaRepository resourceQuotaRepository, AmqpTemplate amqpTemplate,
                                   CosService cosService, SmsProjectService smsProjectService, ChargeItemRepository chargeItemRepository,
                                   ServiceFactory serviceFactory) {
        this.itemRepository = itemRepository;
        this.orderRepository = orderRepository;
        this.cmpInternalService = cmpInternalService;
        this.resourceQuotaService = resourceQuotaService;
        this.orderSnService = orderSnService;
        this.skuRepository = skuRepository;
        this.billRepository = billRepository;
        this.billItemRepository = billItemRepository;
        this.configService = configService;
        this.cloudServiceRepository = cloudServiceRepository;
        this.socService = socService;
        this.smsUserService = smsUserService;
        this.accountService = accountService;
        this.lockFactory = lockFactory;
        this.rosService = rosService;
        this.smsTenantService = smsTenantService;
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.amqpTemplate = amqpTemplate;
        this.cosService = cosService;
        this.smsProjectService = smsProjectService;
        this.chargeItemRepository = chargeItemRepository;
        this.serviceFactory = serviceFactory;
    }

    @Override
    public BocloudResult<List<ServiceOrderInfo>> orderCount(RequestContext requestContext, String token) {
        try {
            Map<String, String> servicePathMap = new LinkedHashMap<>();
            servicePathMap.put("云主机", "/cloud_resource/ecs");
//            servicePathMap.put("云硬盘", "/storage/disk");
            servicePathMap.put("VPC", "/network/vpc");
            servicePathMap.put("弹性IP", "/network/ip");
            servicePathMap.put("负载均衡", "/network/Balancer");
            servicePathMap.put("MySQL", "/database/mysql");
            servicePathMap.put("SQLServer", "/database/sqlserver");
            servicePathMap.put("PostgreSQL", "/database/pg");
            servicePathMap.put("MongoDB", "/database/mongodb");
            servicePathMap.put("Redis", "/database/redis");
            servicePathMap.put("RabbitMQ", "/middleware/rabbitmq");
            servicePathMap.put("Kafka", "/middleware/kafka");
            HashMap<String, ServiceOrderInfo> permission = new HashMap<>();
            List<CloudService> tenantServices = cloudServiceRepository.listTenantService(requestContext.getTenant());
            for (CloudService service : tenantServices) {
                String code = service.getCode();
                if ("vmware.storage.disk".equals(code)) {
                    continue;
                }
                String suffix = code.substring(code.lastIndexOf(".") + 1);
                switch (suffix) {
                    case "server":
                        if (!permission.containsKey("云主机")) {
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.IMS),
                                            token, "/v1/vms/countByUser",
                                            Lists.newArrayList(new Param(MapTools.simpleMap("isRecycle", 0), Sign.EQ)));
                            permission.put("云主机",
                                    new ServiceOrderInfo("云主机", service.getIcon(), count, servicePathMap.get("云主机")));
                        }
                        break;
//                    case "volume":
//                    case "cbs":
//                    case "disk":
//                        if (!permission.containsKey("云硬盘")) {
//                            Long count = this.listVolumeNumber(token);
//                            permission.put("云硬盘",
//                                    new ServiceOrderInfo("云硬盘", service.getIcon(), count, servicePathMap.get("云硬盘")));
//                        }
//                        break;
                    case "vpc":
                        if (!permission.containsKey("VPC")) {
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.IMS),
                                            token, "/v1/vpcs", null);
                            permission.put("VPC",
                                    new ServiceOrderInfo("VPC", service.getIcon(), count, servicePathMap.get("VPC")));
                        }
                        break;
                    case "eip":
                        if (!permission.containsKey("弹性IP")) {
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.IMS),
                                            token, "/v1/floatips", null);
                            Long elasticipCount =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.IMS),
                                            token, "/v1/elasticips", null);
                            permission.put("弹性IP",
                                    new ServiceOrderInfo("弹性IP", service.getIcon(), count + elasticipCount,
                                            servicePathMap.get("弹性IP")));
                        }
                        break;
                    case "slb":
                        if (!permission.containsKey("负载均衡")) {
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.IMS),
                                            token, "/v1/lbs", null);
                            permission.put("负载均衡",
                                    new ServiceOrderInfo("负载均衡", service.getIcon(), count, servicePathMap.get("负载均衡")));
                        }
                        break;
                    case "mysql":
                        if (!permission.containsKey("MySQL")) {
                            Map<String, Object> map = MapTools.simpleMap("isRecycle", 0);
                            map.put("type", "mysql");
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.PMS),
                                            token, "/v1/rds", Lists.newArrayList(new Param(map, Sign.EQ)));
                            permission.put("MySQL",
                                    new ServiceOrderInfo("MySQL", service.getIcon(), count, servicePathMap.get("MySQL")));
                        }
                        break;
                    case "sqlserver":
                        if (!permission.containsKey("SQLServer")) {
                            Map<String, Object> map = MapTools.simpleMap("isRecycle", 0);
                            map.put("type", "sqlserver");
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.PMS),
                                            token, "/v1/rds", Lists.newArrayList(new Param(map, Sign.EQ)));
                            permission.put("SQLServer", new ServiceOrderInfo("SQLServer", service.getIcon(), count,
                                    servicePathMap.get("SQLServer")));
                        }
                        break;
                    case "postgresql":
                        if (!permission.containsKey("PostgreSQL")) {
                            Map<String, Object> map = MapTools.simpleMap("isRecycle", 0);
                            map.put("type", "postgresql");
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.PMS),
                                            token, "/v1/rds", Lists.newArrayList(new Param(map, Sign.EQ)));
                            permission.put("PostgreSQL", new ServiceOrderInfo("PostgreSQL", service.getIcon(), count,
                                    servicePathMap.get("PostgreSQL")));
                        }
                        break;
                    case "mongodb":
                        if (!permission.containsKey("MongoDB")) {
                            Map<String, Object> map = MapTools.simpleMap("isRecycle", 0);
                            map.put("type", "mongodb");
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.PMS),
                                            token, "/v1/kvs", Lists.newArrayList(new Param(map, Sign.EQ)));
                            permission.put("MongoDB", new ServiceOrderInfo("MongoDB", service.getIcon(), count,
                                    servicePathMap.get("MongoDB")));
                        }
                        break;
                    case "redis":
                        if (!permission.containsKey("Redis")) {
                            Map<String, Object> map = MapTools.simpleMap("isRecycle", 0);
                            map.put("type", "redis");
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.PMS),
                                            token, "/v1/kvs", Lists.newArrayList(new Param(map, Sign.EQ)));
                            permission.put("Redis",
                                    new ServiceOrderInfo("Redis", service.getIcon(), count, servicePathMap.get("Redis")));
                        }
                        break;
                    case "rabbitmq":
                        if (!permission.containsKey("RabbitMQ")) {
                            Map<String, Object> map = MapTools.simpleMap("isRecycle", 0);
                            map.put("type", "rabbitmq");
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.PMS),
                                            token, "/v1/cms", Lists.newArrayList(new Param(map, Sign.EQ)));
                            permission.put("RabbitMQ", new ServiceOrderInfo("RabbitMQ", service.getIcon(), count,
                                    servicePathMap.get("RabbitMQ")));
                        }
                        break;
                    case "kafka":
                        if (!permission.containsKey("Kafka")) {
                            Map<String, Object> map = MapTools.simpleMap("isRecycle", 0);
                            map.put("type", "kafka");
                            Long count =
                                    this.listResourceNumber(com.bocloud.service.core.Service.create(BoCloudService.PMS),
                                            token, "/v1/cms", Lists.newArrayList(new Param(map, Sign.EQ)));
                            permission.put("Kafka",
                                    new ServiceOrderInfo("Kafka", service.getIcon(), count, servicePathMap.get("Kafka")));
                        }
                        break;
                    default:
                        break;
                }
            }
            List<ServiceOrderInfo> result = new ArrayList<>();
            for (String service : servicePathMap.keySet()) {
                ServiceOrderInfo orderInfo = permission.get(service);
                if (orderInfo != null) {
                    result.add(orderInfo);
                }
            }
            return new BocloudResult<>(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message,", e);
            return new BocloudResult<>(false, "查询已订购资源信息失败");
        }
    }

    @Override
    public int count(List<Param> params, RequestContext context) {
        int total;
        ListIterator<Param> listIterator = params.listIterator();
        String uuid = null;
        while (listIterator.hasNext()) {
            Param param = listIterator.next();
            Map<String, Object> paramMap = param.getParam();
            if (paramMap.containsKey("uuid")) {
                uuid = (String) paramMap.get("uuid");
                paramMap.remove("uuid");
            }
        }
        if (Catalog.Tenant == context.getCatalog()) {
            // 租户能看到 1.属于当前租户的订单
            params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            total = orderRepository.count(params, uuid);
        } else if (Catalog.User == context.getCatalog()) {
            // 用户能看到 1.属于当前用户的订单 2.当前用户管理的项目的订单
            List<Long> projectIds = new ArrayList<>();
            List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
            List<Long> userManagerProjectsIds =
                    userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
            if (!userManagerProjectsIds.isEmpty()) {
                projectIds.addAll(userManagerProjectsIds);
            }
            total = orderRepository.count(params, context.getTarget(), projectIds, uuid);
        } else {
            total = orderRepository.count(params, uuid);
        }
        return total;
    }

    private long listVolumeNumber(String token) {
        Pager pager = new Pager(1, 9999, new ArrayList<>(), null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory
                .build(BoCloudService.IMS, "/v1/volumes", BoCloudMethod.GET, MapTools.simpleMap(Common.TOKEN, token),
                        requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return 0L;
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.hasText(content)) {
            return 0L;
        }
        JSONObject gridBean = JSONObject.parseObject(content);
        List<Volume> volumes = JSONArray.parseArray(gridBean.getJSONArray("rows").toJSONString(), Volume.class);
        volumes.removeIf(volume -> "VMWARE".equals(volume.getVendorType()));
        return volumes.size();
    }

    private Long listResourceNumber(com.bocloud.service.core.Service service, String token, String url,
                                    List<Param> params) {
        Pager pager = new Pager(1, 9999, Optional.ofNullable(params).orElse(new ArrayList<>()), null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(service, url, BoCloudMethod.GET, MapTools.simpleMap(Common.TOKEN, token), requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return 0L;
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.hasText(content)) {
            return 0L;
        }
        JSONObject gridBean = JSONObject.parseObject(content);
        return gridBean.getLong("total");
    }

    @Override
    public BocloudResult<GridBean<ServiceOrder>> list(Pager pager, RequestContext context) {
        try {
            List<ServiceOrder> list;
            int total;
            GridBean<ServiceOrder> gridBean;
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            ListIterator<Param> listIterator = params.listIterator();
            String uuid = null;
            while (listIterator.hasNext()) {
                Param param = listIterator.next();
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("uuid")) {
                    uuid = (String) paramMap.get("uuid");
                    paramMap.remove("uuid");
                }
            }
            sorter.put("gmtCreate", Common.ONE);
            if (Catalog.Tenant == context.getCatalog()) {
                // 租户能看到 1.属于当前租户的订单
                params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
                total = orderRepository.count(params, uuid);
                list = orderRepository.list(pager.getPage(), pager.getRows(), sorter, params, uuid);
            } else if (Catalog.User == context.getCatalog()) {
                // 用户能看到 1.属于当前用户的订单 2.当前用户管理的项目的订单
                List<Long> projectIds = new ArrayList<>();
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Long> userManagerProjectsIds =
                        userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                total = orderRepository.count(params, context.getTarget(), projectIds, uuid);
                list = orderRepository
                        .list(pager.getPage(), pager.getRows(), sorter, params, context.getTarget(), projectIds, uuid);
            } else {
                total = orderRepository.count(params, uuid);
                list = orderRepository.list(pager.getPage(), pager.getRows(), sorter, params, uuid);
            }
            for (ServiceOrder order : list) {
                List<OrderItem> items = itemRepository.listByOrderId(order.getId());
                BigDecimal money = BigDecimal.valueOf(0L);
                for (OrderItem item : items) {
                    OrderItemUtils.elementNames(item);
                    BigDecimal itemMoney = null == item.getFinalPrice() ? BigDecimal.valueOf(0L) : item.getFinalPrice();
                    if (StringUtils.hasText(item.getDuration())) {
                        DurationModel durationModel = JSONObject.parseObject(item.getDuration(), DurationModel.class);
                        if (null != durationModel && null != durationModel.getAmount() && 0 != durationModel
                                .getAmount()) {
                            itemMoney = itemMoney.multiply(BigDecimal.valueOf(durationModel.getAmount()));
                        }
                    }
                    money = money.add(itemMoney);
                }
                CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
                if (null != service) {
                    order.setServiceName(service.getName());
                }
                order.setMoney(money);
                order.setItems(items);
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public XSSFWorkbook export(List<Param> params, RequestContext context) {
        try {
            List<ServiceOrder> list;
            Map<String, String> sorter = Maps.newHashMap();
            sorter.put("gmtCreate", Common.ONE);
            ListIterator<Param> listIterator = params.listIterator();
            String uuid = null;
            while (listIterator.hasNext()) {
                Param param = listIterator.next();
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("uuid")) {
                    uuid = (String) paramMap.get("uuid");
                    paramMap.remove("roleId");
                }
            }
            if (Catalog.Tenant == context.getCatalog()) {
                // 租户能看到 1.属于当前租户的订单
                params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
                list = orderRepository.list(1, Integer.MAX_VALUE, sorter, params);
            } else if (Catalog.User == context.getCatalog()) {
                // 用户能看到 1.属于当前用户的订单 2.当前用户管理的项目的订单
                List<Long> projectIds = new ArrayList<>();
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Long> userManagerProjectsIds =
                        userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                list =
                        orderRepository.list(1, Integer.MAX_VALUE, sorter, params, context.getTarget(), projectIds, uuid);
            } else {
                list = orderRepository.list(1, Integer.MAX_VALUE, sorter, params, uuid);
            }
            for (ServiceOrder order : list) {
                CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
                if (null != service) {
                    order.setServiceName(service.getName());
                }
            }
            // 创建HSSFWorkbook
            String sheetName = "订单统计数据";
            String[] cellTitle = new String[]{"订单编号", "资源类型", "收费模式", "订单状态", "订单类型", "所属租户", "所属用户", "创建时间"};
            String[][] rows;
            if (!ListTool.isEmpty(list)) {
                rows = new String[list.size()][cellTitle.length];
                for (int i = 0; i < list.size(); i++) {
                    ServiceOrder order = list.get(i);
                    rows[i][0] = order.getSn();
                    rows[i][1] = order.getServiceName();
                    rows[i][2] = order.getChargeMode().equals(ChargeCommon.ChargeMode.PostCharge) ? "后付费" : "预付费";
                    switch (order.getOrderStatus()) {
                        case unpay:
                            rows[i][3] = "未支付";
                            break;
                        case paid:
                            rows[i][3] = "已支付";
                            break;
                        case canceld:
                            rows[i][3] = "已取消";
                            break;
                        case audit:
                            rows[i][3] = "审批中";
                            break;
                        default:
                            break;
                    }
                    if (null != order.getType()) {
                        switch (order.getType()) {
                            case apply:
                                rows[i][4] = "新购";
                                break;
                            case alterat:
                                rows[i][4] = "变更";
                                break;
                            case extension:
                                rows[i][4] = "延期";
                                break;
                            case unsubscribe:
                                rows[i][4] = "退订";
                                break;
                            default:
                                break;
                        }
                    }
                    rows[i][5] = order.getTenantName();
                    rows[i][6] = order.getUserName();
                    rows[i][7] = order.getGmtCreate().toString().substring(0, 19);
                }
            } else {
                rows = new String[0][cellTitle.length];
            }
            return ExportUtil.getXSSFWorkbook(sheetName, cellTitle, rows);
        } catch (Exception e) {
            logger.error("create XSSFWorkBook error,", e);
            return null;
        }
    }

    private List<ServiceOrder> getServiceOrder(RequestContext context) throws Exception {
        if (Catalog.Tenant == context.getCatalog()) {
            // 租户能看到 1.属于当前租户的订单
            List<Param> params =
                    Lists.newArrayList(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            return orderRepository.list(1, Integer.MAX_VALUE, null, params);
        } else if (Catalog.User == context.getCatalog()) {
            // 用户能看到 1.属于当前用户的订单 2.当前用户管理的项目的订单
            List<Long> projectIds = new ArrayList<>();
            List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
            List<Long> userManagerProjectsIds =
                    userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
            if (!userManagerProjectsIds.isEmpty()) {
                projectIds.addAll(userManagerProjectsIds);
            }
            return orderRepository.list(1, Integer.MAX_VALUE, null, null, context.getTarget(), projectIds, null);
        } else {
            return orderRepository.list(1, Integer.MAX_VALUE, null, null);
        }
    }

    @Override
    public BocloudResult<ServiceOrder> queryBySn(String condition, String sn) {
        try {
            switch (condition) {
                case "queryBySn":
                    return this.queryBySn(sn);
                case "getServiceOrder":
                    return this.getServiceOrder(sn);
                case "queryByItemSn":
                    OrderItem item = itemRepository.queryBySn(sn);
                    return new BocloudResult(true, item, "查询成功");
                default:
                    return new BocloudResult<>(false, "查询失败:不支持这种查询。");
            }
        } catch (Exception e) {
            logger.error(",", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }

    }

    @Override
    public BocloudResult listByCondition(String condition, RequestContext context) {
        try {
            JSONObject jsonObject = JSON.parseObject(condition);
            switch (jsonObject.getString("condition")) {
                case "resourceStatusOverview":
                    //自服务首页订单资源状态概览
                    return this.resourceStatusOverview(context);
                case "orderPaidOverview":
                    //订单概览 支付统计
                    return this.orderPaidOverview(jsonObject.getString("start"), jsonObject.getString("end"), context);
                case "orderPaidTrend":
                    //订单概览 支付趋势统计
                    return this.orderPaidTrend(jsonObject.getString("start"), jsonObject.getString("end"), context);
                case "orderCategoryOverview":
                    //订单概览 订单类型统计
                    return this.orderCategoryOverview(jsonObject.getString("start"), jsonObject.getString("end"), context);
                case "orderStatusOverview":
                    //订单概览 订单状态统计
                    return this.orderStatusOverview(jsonObject.getString("start"), jsonObject.getString("end"), context);
                case "orderChargeModeOverview":
                    //订单概览 订单付费类型统计
                    return this.orderChargeModeOverview(jsonObject.getString("start"), jsonObject.getString("end"), context);
                case "tenantOrdersTop5":
                    //订单概览 租户订单top5
                    return this.tenantOrdersTop5(jsonObject.getString("start"), jsonObject.getString("end"));
                case "userOrdersTop5":
                    //订单概览 用户订单top5
                    return this.userOrdersTop5(jsonObject.getString("start"), jsonObject.getString("end"));
                case "getServiceOrder":
                    return this.getServiceOrder(jsonObject.getString("sn"));
                case "queryByItemSn":
                    OrderItem item = itemRepository.queryBySn(jsonObject.getString("sn"));
                    return new BocloudResult<>(true, item, "查询成功");
                default:
                    return new BocloudResult<>(false, "不支持该查询");
            }
        } catch (Exception e) {
            logger.error("list service order by condition error,", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private BocloudResult<ChargeResultTrend> resourceStatusOverview(RequestContext context) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<ServiceOrder> serviceOrder = this.getServiceOrder(context);
            Map<ChargeCommon.ResourceStatus, List<ServiceOrder>> statusListMap =
                    serviceOrder.stream().filter(order -> null != order.getResourceStatus())
                            .collect(groupingBy(ServiceOrder::getResourceStatus));
            List<Object> data = new ArrayList<>();
            for (ChargeCommon.ResourceStatus status : ChargeCommon.ResourceStatus.values()) {
                List<ServiceOrder> orders = Optional.ofNullable(statusListMap.get(status)).orElse(new ArrayList<>());
                switch (status) {
                    case finish:
                        keys.add("已结束");
                        data.add(orders.size());
                        break;
                    case ready:
                        keys.add("创建完成");
                        data.add(orders.size());
                        break;
                    case failed:
                        keys.add("创建失败");
                        data.add(orders.size());
                        break;
                    case canceld:
                        keys.add("已取消");
                        data.add(orders.size());
                        break;
                    default:
                        break;
                }
            }
            ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
            values.add(chargeTrendData);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get order resource status overview error", e);
            return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    /**
     * 云主机服务申请趋势
     *
     * @return
     * @throws Exception
     */
    @Override
    public BocloudResult<ChargeResultTrend> vmTrend() {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date endDate = new Date();
        Date beginDate = DateUtils.addMonths(endDate, -1);
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("gmtCreate", format.format(beginDate)), Sign.GET));
        params.add(new Param(MapTools.simpleMap("gmtCreate", format.format(endDate)), Sign.LET));
        try {
            List<ServiceOrder> orders = orderRepository.list(1, Integer.MAX_VALUE, new HashMap<>(), params);
            ChargeResultTrend trend;
            Map<String, List<ServiceOrder>> serviceMap = new HashMap<>();
            Map<String, List<Object>> resultMap = new HashMap<>();
            for (ServiceOrder order : orders) {
                if (hasVmService(order.getServiceCode())) {
                    List<ServiceOrder> serviceOrders = serviceMap.get(order.getServiceName());
                    if (null == serviceOrders) {
                        serviceOrders = new ArrayList<>();
                    }
                    serviceOrders.add(order);
                    serviceMap.put(order.getServiceName(), serviceOrders);
                }
            }
            List<String> keys = new ArrayList<>();
            while (beginDate.before(endDate)) {
                String key = format.format(beginDate);
                keys.add(key);
                Date start = getStartDate(DateUtils.parseDate(key, "yyyy-MM-dd HH:mm:ss"));
                Date end = getEndDate(DateUtils.parseDate(key, "yyyy-MM-dd HH:mm:ss"));
                for (String serviceName : serviceMap.keySet()) {
                    List<ServiceOrder> orderList = serviceMap.get(serviceName);
                    Integer count = 0;
                    for (ServiceOrder order : orderList) {
                        if (null != order.getGmtCreate() && order.getGmtCreate().after(start) && order.getGmtCreate()
                                .before(end)) {
                            count++;
                        }
                    }
                    List<Object> objects = resultMap.get(serviceName);
                    if (null == objects) {
                        objects = new ArrayList<>();
                    }
                    objects.add(count);
                    resultMap.put(serviceName, objects);
                }
                beginDate = DateUtils.addDays(beginDate, 1);
            }
            List<ChargeTrendData> values = new ArrayList<>();
            for (String key : resultMap.keySet()) {
                values.add(new ChargeTrendData(key, resultMap.get(key)));
            }
            trend = new ChargeResultTrend(keys, values);
            return new BocloudResult<>(true, trend, "查询成功");
        } catch (Exception e) {
            logger.error("get vm trend error,", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private boolean hasVmService(String serviceCode) {
        if (!StringUtils.hasText(serviceCode)) {
            return false;
        }
        return ServiceResourceMapping.getMapping().get(ResourceCategory.Computer).contains(serviceCode);
    }

    private static Date getStartDate(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR));
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    private static Date getEndDate(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR));
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        return calendar.getTime();
    }

    private BocloudResult<ServiceOrder> getServiceOrder(String itemSn) {
        OrderItem item = itemRepository.queryBySn(itemSn);
        if (null == item) {
            return new BocloudResult<>(false, "订单项不存在");
        }
        ServiceOrder order = orderRepository.query(item.getOrderId());
        if (null != order) {
            return new BocloudResult<>(true, order, "查询成功");
        }
        return new BocloudResult<>(false, "查询失败");
    }

    /**
     * 根据sn查询
     *
     * @param sn
     * @return
     */
    private BocloudResult<ServiceOrder> queryBySn(String sn) {
        try {
            ServiceOrder order = orderRepository.queryBySn(sn);
            if (null == order) {
                return new BocloudResult<>(false, "订单不存在");
            }
            List<OrderItem> items = itemRepository.listByOrderId(order.getId());
            BigDecimal money = BigDecimal.valueOf(0L);
            String serviceCode = null;
            for (OrderItem item : items) {
                BigDecimal itemMoney = null == item.getFinalPrice() ? BigDecimal.valueOf(0L) : item.getFinalPrice();
                if (StringUtils.hasText(item.getDuration())) {
                    DurationModel durationModel = JSONObject.parseObject(item.getDuration(), DurationModel.class);
                    if (null != durationModel && null != durationModel.getAmount() && 0 != durationModel.getAmount()) {
                        itemMoney = itemMoney.multiply(BigDecimal.valueOf(durationModel.getAmount()));
                    }
                }
                money = money.add(itemMoney);
                OrderItemUtils.elementNames(item);
                if (!StringUtils.hasText(serviceCode)) {
                    ServiceSku sku = skuRepository.query(item.getSkuId());
                    if (null != sku) {
                        serviceCode = sku.getServiceCode();
                    }
                }
            }
            order.setServiceCode(serviceCode);
            order.setMoney(money);
            order.setItems(items);
            return new BocloudResult<>(true, order, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 状态分布
     *
     * @param start
     * @param end
     * @param context
     * @return
     */
    private BocloudResult<List<StatsResult>> orderStatusOverview(String start, String end, RequestContext context) {
        List<ServiceOrder> serviceOrders = orderRepository.timeBetweenOrders(start, end, context);
        Map<ChargeCommon.OrderStatus, List<ServiceOrder>> serviceOrderMap =
                serviceOrders.stream().filter(serviceOrder -> serviceOrder.getResourceStatus() != null)
                        .collect(groupingBy(ServiceOrder::getOrderStatus));
        List<StatsResult> result = new ArrayList<>();
        for (ChargeCommon.OrderStatus status : ChargeCommon.OrderStatus.values()) {
            List<ServiceOrder> orders = Optional.ofNullable(serviceOrderMap.get(status)).orElse(new ArrayList<>());
            int number = orders.size();
            switch (status) {
                case paid:
                    result.add(new StatsResult("已支付", number));
                    break;
                case unpay:
                    result.add(new StatsResult("未支付", number));
                    break;
                case canceld:
                    result.add(new StatsResult("已取消", number));
                    break;
                case audit:
                    result.add(new StatsResult("审批中", number));
                    break;
                default:
                    break;
            }
        }
        return new BocloudResult<>(true, result, "查询成功");
    }

    /**
     * 消费趋势统计
     *
     * @param start
     * @param end
     * @param context
     * @return
     */
    private BocloudResult<List<StatsResult>> orderChargeModeOverview(String start, String end, RequestContext context) {
        List<ServiceOrder> serviceOrders = orderRepository.timeBetweenOrders(start, end, context);
        Map<ChargeCommon.ChargeMode, List<ServiceOrder>> serviceOrderMap =
                serviceOrders.stream().filter(serviceOrder -> serviceOrder.getChargeMode() != null)
                        .collect(groupingBy(ServiceOrder::getChargeMode));
        List<StatsResult> result = new ArrayList<>();
        for (Map.Entry<ChargeCommon.ChargeMode, List<ServiceOrder>> resourceStatusListEntry : serviceOrderMap
                .entrySet()) {
            ChargeCommon.ChargeMode chargeMode = resourceStatusListEntry.getKey();
            int number = resourceStatusListEntry.getValue().size();
            switch (chargeMode) {
                case PreCharge:
                    result.add(new StatsResult("预付费", number));
                    break;
                case PostCharge:
                    result.add(new StatsResult("后付费", number));
                    break;
                default:
                    break;
            }
        }
        return new BocloudResult<>(true, result, "查询成功");
    }

    private BocloudResult<ChargeResultTrend> tenantOrdersTop5(String start, String end) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        List<ServiceOrder> serviceOrders = orderRepository.timeBetweenOrders(start, end, null);
        Map<String, List<ServiceOrder>> serviceOrderMap =
                serviceOrders.stream().filter(serviceOrder -> serviceOrder.getTenantName() != null)
                        .collect(groupingBy(ServiceOrder::getTenantName));
        List<Map.Entry<String, List<ServiceOrder>>> sortedEntry =
                serviceOrderMap.entrySet().stream().sorted(((o1, o2) -> o2.getValue().size() - o1.getValue().size()))
                        .limit(5).collect(Collectors.toList());
        ArrayList<Object> data = new ArrayList<>();
        for (Map.Entry<String, List<ServiceOrder>> entry : sortedEntry) {
            String tenant = entry.getKey();
            int number = entry.getValue().size();
            keys.add(tenant);
            data.add(number);
        }
        ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
        values.add(chargeTrendData);
        return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询成功");
    }

    private BocloudResult<ChargeResultTrend> userOrdersTop5(String start, String end) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        List<ServiceOrder> serviceOrders = orderRepository.timeBetweenOrders(start, end, null);
        Map<String, List<ServiceOrder>> serviceOrderMap =
                serviceOrders.stream().filter(serviceOrder -> serviceOrder.getUserName() != null)
                        .collect(groupingBy(ServiceOrder::getUserName));
        List<Map.Entry<String, List<ServiceOrder>>> sortedEntry =
                serviceOrderMap.entrySet().stream().sorted(((o1, o2) -> o2.getValue().size() - o1.getValue().size()))
                        .limit(5).collect(Collectors.toList());
        ArrayList<Object> data = new ArrayList<>();
        for (Map.Entry<String, List<ServiceOrder>> entry : sortedEntry) {
            String tenant = entry.getKey();
            int number = entry.getValue().size();
            keys.add(tenant);
            data.add(number);
        }
        ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
        values.add(chargeTrendData);
        return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询成功");
    }

    /**
     * 订单类型统计
     *
     * @param start
     * @param end
     * @param context
     * @return
     * @throws Exception
     */
    private BocloudResult<ChargeResultTrend> orderCategoryOverview(String start, String end, RequestContext context) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        List<ServiceOrder> serviceOrders = orderRepository.timeBetweenOrders(start, end, context);
        Map<ChargeCommon.Type, List<ServiceOrder>> serviceOrderMap =
                serviceOrders.stream().filter(serviceOrder -> serviceOrder.getType() != null)
                        .collect(groupingBy(ServiceOrder::getType));

        ArrayList<Object> data = new ArrayList<>();
        for (ChargeCommon.Type type : ChargeCommon.Type.values()) {
            List<ServiceOrder> orders = Optional.ofNullable(serviceOrderMap.get(type)).orElse(new ArrayList<>());
            switch (type) {
                case apply:
                    keys.add("新购");
                    data.add(orders.size());
                    break;
                case alterat:
                    keys.add("变更");
                    data.add(orders.size());
                    break;
                case extension:
                    keys.add("延期");
                    data.add(orders.size());
                    break;
                case unsubscribe:
                    keys.add("退订");
                    data.add(orders.size());
                    break;
                default:
                    break;
            }
        }

        ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
        values.add(chargeTrendData);
        return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询成功");
    }

    /**
     * 订单增长趋势统计
     *
     * @param start
     * @param end
     * @param context
     * @return
     * @throws Exception
     */
    private BocloudResult<ChargeResultTrend> orderPaidTrend(String start, String end, RequestContext context)
            throws Exception {
        // 统计范围
        ChargeResultTrend resultTrend;
        List<String> keys = new ArrayList<>();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("startTime", start);
        jsonObject.put("endTime", end);
        List<String> timeToList = DateUtil.timeToList(jsonObject);
        List<ServiceOrder> orders = orderRepository.timeBetweenOrders(null, end, context);
        List<ChargeTrendData> values = new ArrayList<>();
        List<Object> unpayCount = new ArrayList<>();
        List<Object> paidCount = new ArrayList<>();
        List<Object> canceldCount = new ArrayList<>();
        List<Object> auditCount = new ArrayList<>();
        for (String time : timeToList) {
            keys.add(time);
            Map<ChargeCommon.OrderStatus, List<ServiceOrder>> orderMap = orders.stream().filter(
                    order -> null != order.getOrderStatus() && order.getGmtCreate().before(DateUtil.timeDateProcess(time)))
                    .collect(groupingBy(ServiceOrder::getOrderStatus));
            for (ChargeCommon.OrderStatus orderStatus : ChargeCommon.OrderStatus.values()) {
                List<ServiceOrder> statusOrders =
                        Optional.ofNullable(orderMap.get(orderStatus)).orElse(new ArrayList<>());
                switch (orderStatus) {
                    case canceld:
                        canceldCount.add(statusOrders.size());
                        break;
                    case paid:
                        paidCount.add(statusOrders.size());
                        break;
                    case unpay:
                        unpayCount.add(statusOrders.size());
                        break;
                    case audit:
                        auditCount.add(statusOrders.size());
                        break;
                    default:
                        break;
                }
            }
        }
        values.add(new ChargeTrendData("未支付(个)", unpayCount));
        values.add(new ChargeTrendData("已支付(个)", paidCount));
        values.add(new ChargeTrendData("已取消(个)", canceldCount));
        values.add(new ChargeTrendData("审批中(个)", auditCount));
        resultTrend = new ChargeResultTrend(keys, values);
        return new BocloudResult<>(true, resultTrend, "查询成功");
    }

    /**
     * 订单总数
     *
     * @param start
     * @param end
     * @param context
     * @return
     * @throws Exception
     */
    public BocloudResult<ChargeResultTrend> orderPaidOverview(String start, String end, RequestContext context) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        Map<ChargeCommon.OrderStatus, List<ServiceOrder>> orderStatusListMap =
                orderRepository.timeBetweenOrders(start, end, context).stream()
                        .filter(serviceOrder -> serviceOrder.getOrderStatus() != null)
                        .collect(groupingBy(ServiceOrder::getOrderStatus));
        ArrayList<Object> data = new ArrayList<>();
        for (ChargeCommon.OrderStatus value : ChargeCommon.OrderStatus.values()) {
            switch (value) {
                case canceld:
                    keys.add("已取消");
                    break;
                case paid:
                    keys.add("已支付");
                    break;
                case unpay:
                    keys.add("未支付");
                    break;
                case audit:
                    keys.add("审批中");
                    break;
                default:
                    break;
            }
            List<ServiceOrder> orders = Optional.ofNullable(orderStatusListMap.get(value)).orElse(new ArrayList<>());
            data.add(orders.size());
        }
        ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
        values.add(chargeTrendData);
        return new BocloudResult<>(true, new ChargeResultTrend(keys, values), "查询成功");
    }

    /**
     * 订单top5
     *
     * @param start
     * @param end
     * @param tenant
     * @param zone
     * @return
     * @throws Exception
     */
    @Override
    public BocloudResult<LineChartResult> top5(String start, String end, Boolean tenant, Boolean zone) {
        try {

            List<Map<String, Object>> resultList = orderRepository.top5(start, end, tenant, zone);
            List<String> keys = new ArrayList<>();
            List<Object> data = new ArrayList<>();
            Map<String, String> provinceMap = cmpInternalService.tenantProvince();
            LineChartResult chartResult;
            LineChartResult.Values[] values;
            for (Map<String, Object> order : resultList) {
                String name = (null != tenant && tenant) ? String.valueOf(order.get("tenant_name")) :
                        provinceMap.get(String.valueOf(order.get("city")));
                keys.add(name);
                data.add(order.get("total"));
            }
            values = new LineChartResult.Values[]{new LineChartResult.Values("订单个数(个)", data)};
            chartResult = new LineChartResult(keys, values);
            return new BocloudResult<>(true, chartResult, "查询成功");
        } catch (Exception e) {
            logger.error("get service order top5 error,", e);
            return new BocloudResult<>(false, null, "查询失败");
        }
    }

    @Override
    public BocloudResult<ServiceOrder> create(ServiceOrder order, RequestContext context) {
        // 生成订单号，增加分布式锁
        String path = IdGenerator.class.getSimpleName() + "_ordersn";
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
            Map<String, CloudService> serviceMap;
            if (null != order.getTenantId() && 0L != order.getTenantId()) {
                serviceMap = cloudServiceRepository.listTenantService(order.getTenantId()).stream()
                        .collect(Collectors.toMap(CloudService::getCode, Function.identity(), (m1, m2) -> m1));
                if (!serviceMap.containsKey(order.getServiceCode())) {
                    CloudService cloudService = cloudServiceRepository.queryByCode(order.getServiceCode());
                    return new BocloudResult<>(false, "租户未授权" + cloudService.getName() + "服务");
                }
            } else if (null != order.getUserId() && 0L != order.getUserId()) {
                BocloudResult<User> userInfo = smsUserService.detail(order.getUserId());
                User user = userInfo.getData();
                Assert.notNull(user, "用户不存在");
                serviceMap = cloudServiceRepository.listTenantService(user.getTenantId()).stream()
                        .collect(Collectors.toMap(CloudService::getCode, Function.identity(), (m1, m2) -> m1));

                if (!serviceMap.containsKey(order.getServiceCode())) {
                    CloudService cloudService = cloudServiceRepository.queryByCode(order.getServiceCode());
                    return new BocloudResult<>(false, "用户所属租户未授权" + cloudService.getName() + "服务");
                }
            }
            order.setSn(IdGenerator.getInstance().nextId());
            order.setResourceStatus(ChargeCommon.ResourceStatus.ready);
            order.setGmtCreate(new Date());
            orderRepository.save(order);
            return new BocloudResult<>(true, order, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message", e);
            return new BocloudResult<>(false, "创建失败");
        }
    }

    @Override
    public Result modify(ServiceOrder order, RequestContext context) {
        String path = ServiceOrder.class.getSimpleName() + "_" + order.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceOrder database = orderRepository.query(order.getId());
            Assert.notNull(database, "数据不存在！");
            BeanUtils.copyProperties(order, database);
            database.setGmtModify(new Date());
            boolean flag = orderRepository.update(database);
            if (!flag) {
                return new Result(false, "修改失败");
            }
            return new Result(true, "修改成功");
        } catch (Exception e) {
            logger.error("Get error message,", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public BocloudResult<ServiceOrder> detail(Long id) {
        try {
            ServiceOrder order = orderRepository.query(id);
            if (null == order) {
                return new BocloudResult<>(false, "订单不存在");
            }
            List<OrderItem> items = itemRepository.listByOrderId(order.getId());
            for (OrderItem item : items) {
                OrderItemUtils.elementNames(item);
            }
            List<ServiceBill> serviceBills = billRepository.listByOrderId(id);
            BigDecimal orderMoney =
                    serviceBills.stream().filter(bill -> bill.getStatus().equals(ChargeCommon.BillStatus.paid))
                            .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                            .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
            order.setMoney(orderMoney);
            order.setItems(items);
            CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
            if (null != service) {
                order.setServiceName(service.getName());
            }
            return new BocloudResult<>(true, order, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result remove(Long id, RequestContext context) {
        String path = ServiceOrder.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceOrder order = orderRepository.query(id);
            Assert.notNull(order, "数据不存在！");
            Boolean flag = orderRepository.remove(id);
            if (flag) {
                itemRepository.removeByOrderId(id);
            }
            return flag ? new Result(true, "删除成功") : new Result(false, "删除失败");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public BocloudResult<GridBean<OrderItem>> items(Pager pager, RequestContext context) {
        List<OrderItem> list;
        int total;
        GridBean<OrderItem> gridBean;
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            total = itemRepository.count(params);
            list = itemRepository.list(pager.getPage(), pager.getRows(), sorter, params);
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<OrderItem> createItem(OrderItem item, RequestContext context) {
        try {
            CloudService cloudService = cloudServiceRepository.queryByCode(item.getServiceCode());
            String duration = item.getDuration();
            if (StringUtils.hasText(duration)) {
                DurationModel durationModel = JSONObject.parseObject(duration, DurationModel.class);
                Integer month = durationModel.getAmount();
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(new Date());
                if (null != month) {
                    calendar.add(Calendar.MONTH, month);
                } else {
                    calendar.setTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
                }
                item.setExpiredTime(calendar.getTime());
            }
            ServiceOrder order = orderRepository.query(item.getOrderId());
            Assert.notNull(order, "订单不存在");
            if (null == item.getSkuId()) {
                // 生成订单号，增加分布式锁
                String itemPath = IdGenerator.class.getSimpleName() + "_order_item_sn";
                try (BocloudACLock lock = lockFactory.getACLock(itemPath)) {
                    Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
                    item.setSn(IdGenerator.getInstance().nextId());
                } catch (Exception e) {
                    logger.error("Get error message：{}", e.getMessage());
                    return new BocloudResult<>(false, "资源申请失败 : " + e.getMessage());
                }
            } else {
                BocloudResult<String> snResult = orderSnService.generate(item.getSkuId());
                if (snResult.isFailed()) {
                    return new BocloudResult<>(false, snResult.getMessage());
                }
                item.setSn(snResult.getData());
                logger.info("生成的订单号：{}", snResult.getData());
            }
            item.setDeleted(false);
            Long tenantId = order.getTenantId();
            Tenant tenant = smsTenantService.detail(tenantId);
            buildOrderItem(item, tenant);
            itemRepository.save(item);
            //添加到计费中 公有云和 无租户订单（管理项目分配）不添加到跑批中
            if (!cloudService.getIsPublic() && tenant != null) {
                ChargeItem chargeItem = new ChargeItem();
                chargeItem.setServiceCode(item.getServiceCode());
                chargeItem.setPrice(item.getFinalPrice());
                chargeItem.setGmtCreate(new Date());
                chargeItem.setItemSn(item.getSn());
                chargeItem.setItemId(item.getId());
                chargeItem.setDeleted(false);
                chargeItem.setType(ChargeItem.Type.loop);
                chargeItemRepository.baseSave(chargeItem);
            }
            return new BocloudResult<>(true, item, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult<>(false, "创建失败:" + e.getMessage());
        }
    }

    /**
     * 构造订单明细数据
     *
     * @param item
     * @param tenant
     * @return
     */
    public void buildOrderItem(OrderItem item, Tenant tenant) {
        BigDecimal tenantDiscount = BigDecimal.valueOf(1L);
        if (null != tenant && null != tenant.getDiscount()) {
            tenantDiscount = tenant.getDiscount();
        }
        try {
            CloudService cloudService = cloudServiceRepository.queryByCode(item.getServiceCode());
            List<ServicePrice> elementPrice =
                    Optional.ofNullable(JSONArray.parseArray(cloudService.getBasicPrice(), ServicePrice.class))
                            .orElse(new ArrayList<>());
            List<ElementModel> elements = JSONArray.parseArray(item.getElements(), ElementModel.class);
            ElementModel element = elements.get(0);
            List<SpecModel> specModels = element.getSpecs();
            DurationModel durationModel = JSONObject.parseObject(item.getDuration(), DurationModel.class);
            BigDecimal price = BigDecimal.ZERO;
            Map<String, BigDecimal> priceMap = getPrice(durationModel, elementPrice);
            logger.info("priceMap -> {}", JSONObject.toJSONString(priceMap));
            if (specModels != null && !specModels.isEmpty()) {
                Integer cpuTotal =
                        specModels.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0)).reduce(Integer::sum)
                                .orElse(0);
                Double memTotal =
                        specModels.stream().map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal::add).orElse(BigDecimal.ZERO).doubleValue();
                int diskTotal =
                        specModels.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0)).reduce(Integer::sum)
                                .orElse(0);
                Integer bandWidth =
                        specModels.stream().map(spec -> Optional.ofNullable(spec.getBandWidth()).orElse(0)).reduce(Integer::sum)
                                .orElse(0);

                BigDecimal cpuPrice = Optional.ofNullable(priceMap.get("cpu")).orElse(BigDecimal.ZERO)
                        .multiply(BigDecimal.valueOf(cpuTotal));
                BigDecimal memPrice = Optional.ofNullable(priceMap.get("memory")).orElse(BigDecimal.ZERO)
                        .multiply(BigDecimal.valueOf(memTotal));
                BigDecimal diskPrice = Optional.ofNullable(priceMap.get("disk")).orElse(BigDecimal.ZERO)
                        .multiply(BigDecimal.valueOf(diskTotal));
                BigDecimal bandWidthPrice = Optional.ofNullable(priceMap.get("bandWidth")).orElse(BigDecimal.ZERO)
                        .multiply(BigDecimal.valueOf(bandWidth));
                price = cpuPrice.add(memPrice).add(diskPrice).add(bandWidthPrice);
                element.setPrice(price);
                element.setRatio(tenantDiscount);
            }

            // FIXME: openStack eip 单独设计计费, 拿sku规格, 同租户端保持一致
            if ("openstack.standard.eip".equals(item.getServiceCode())) {
                ServiceSku eipSku = skuRepository.getBySkuCode("openstack.standard.eip");
                JSONObject spec = (JSONObject) JSONObject.parseArray(eipSku.getSpec()).get(0);
                Integer specValue = spec.getInteger("specValue");
                price = Optional.ofNullable(priceMap.get("bandWidth")).orElse(BigDecimal.ZERO)
                        .multiply(BigDecimal.valueOf(specValue));
                element.setPrice(price);
            }

            item.setElements(JSONObject.toJSONString(elements));
            item.setBasicPrice(price);
            item.setFinalPrice(price.multiply(tenantDiscount));
        } catch (Exception e) {
            logger.error("Get error message：", e);
        }
    }

    public void processElement(OrderItem item, BigDecimal tenantDiscount) {
        String elements = item.getElements();
        List<ElementModel> elementList = JSONArray.parseArray(elements, ElementModel.class);
        for (ElementModel element : elementList) {
            element.setRatio(tenantDiscount);
        }
        item.setElements(JSONObject.toJSONString(elementList));
    }

    @Override
    public Result modifyItem(OrderItem item, RequestContext context) {
        String path = OrderItem.class.getSimpleName() + "_" + item.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            OrderItem database = itemRepository.query(item.getId());
            Assert.notNull(database, "数据不存在！");
            BeanUtils.copyProperties(item, database);
            boolean flag = itemRepository.update(database);
            if (!flag) {
                return new Result(false, "修改失败");
            }
            return new Result(true, "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public BocloudResult<OrderItem> detailItem(Long id) {
        try {
            OrderItem item = itemRepository.query(id);
            if (null == item) {
                return new BocloudResult<>(false, "订单项不存在");
            }
            return new BocloudResult<>(true, item, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result removeItem(Long id, RequestContext context) {
        String path = OrderItem.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            OrderItem item = itemRepository.query(id);
            Assert.notNull(item, "数据不存在！");
            Boolean flag = itemRepository.remove(id);
            return flag ? new Result(true, "删除成功") : new Result(false, "删除失败");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public BocloudResult<List<ServiceOrder>> orders(String chargeMode) {
        try {
            List<ServiceOrder> orders = orderRepository.orders(chargeMode);
            //            for (ServiceOrder order : orders) {
            //                order.setItems(itemRepository.listByOrderId(order.getId()));
            //            }
            return new BocloudResult<>(true, orders, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, null, "查询失败");
        }
    }

    @Override
    public Result updateOrder(ServiceOrder order, RequestContext context) {
        String path = ServiceOrder.class.getSimpleName() + "_" + order.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceOrder database = orderRepository.query(order.getId());
            Assert.notNull(database, "订单不存在");
            database.setResourceStatus(order.getResourceStatus());
            database.setGmtModify(new Date());
            // 修改订单的 单位和项目
            if (null != order.getTenantId() && null != order.getProjectId()) {
                database.setTenantId(order.getTenantId());
                database.setProjectId(order.getProjectId());
            }
            orderRepository.update(database);
            return new Result(true, "订单更新成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new Result(false, "订单更新失败");
        }
    }

    @Override
    public Result pay(String sn, Long couponId, RequestContext context) {
        try {
            ServiceOrder order = orderRepository.queryBySn(sn);
            if (null == order) {
                return new Result(false, "订单不存在");
            }
            if (ChargeCommon.OrderStatus.paid == order.getOrderStatus()) {
                return new Result(false, "订单已支付，无需再次支付");
            }
            // 生成预付费账单
            switch (order.getType()) {
                case apply:
                    ServiceBill applyBill = this.buildApplyBill(order);
                    Assert.notNull(applyBill, "生成账单时发生错误");
                    Result result = handleBill(order, applyBill);
                    Assert.isTrue(result.isSuccess(), result.getMessage());
                    order.setOrderStatus(ChargeCommon.OrderStatus.paid);
                    orderRepository.update(order);
                    handleApply(order, context);
                    break;
                case alterat:
                    ServiceBill alterationBill = this.buildAlterationBill(order);
                    Assert.notNull(alterationBill, "生成账单时发生错误");
                    Result handleResult = handleBill(order, alterationBill);
                    Assert.isTrue(handleResult.isSuccess(), handleResult.getMessage());
                    order.setOrderStatus(ChargeCommon.OrderStatus.paid);
                    orderRepository.update(order);
                    handleAltera(order);
                    break;
                case extension:
                    ServiceBill extensionBill = this.buildExtensionBill(order);
                    Assert.notNull(extensionBill, "生成账单时发生错误");
                    Result extensionResult = handleBill(order, extensionBill);
                    Assert.isTrue(extensionResult.isSuccess(), extensionResult.getMessage());
                    order.setOrderStatus(ChargeCommon.OrderStatus.paid);
                    orderRepository.update(order);
                    handleExtension(order);
                    break;
                default:
                    break;
            }
            return new BocloudResult<>(true, "订单支付成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult<>(false, "订单支付失败:" + e.getMessage());
        }
    }

    /**
     * 扣款方法
     *
     * @param order
     * @param bill
     */
    private Result handleBill(ServiceOrder order, ServiceBill bill) {
        RequestContext requestContext =
                new RequestContext(Catalog.Tenant, order.getTenantId(), order.getTenantId(), order.getProjectId(), null);
        Long tenantId = order.getTenantId();
        if (null != order.getUserId() && 0L != order.getUserId()) {
            BocloudResult<User> userResult = smsUserService.detail(order.getUserId());
            if (userResult.isSuccess() && null != userResult.getData()) {
                User user = userResult.getData();
                tenantId = user.getTenantId();
                requestContext.setTarget(user.getId());
                requestContext.setTenant(user.getTenantId());
                requestContext.setCatalog(Catalog.User);
            }
        }
        if (null != tenantId && 0L != tenantId) {
            // 扣款
            Result result = accountService
                    .accountAction(tenantId, bill.getId(), TenantAccount.Action.Consume, bill.getMoney(), requestContext);
            if (result.isSuccess()) {
                bill.setStatus(ChargeCommon.BillStatus.paid);
                billRepository.update(bill);
                return new Result(true, "扣款支付成功");
            } else {
                bill.setStatus(ChargeCommon.BillStatus.canceld);
                billRepository.update(bill);
                return new Result(false, result.getMessage());
            }
        }
        return new Result(false, "订单没有所属租户");
    }

    private void handleApply(ServiceOrder order, RequestContext context) {
        if ("vmware.k8s.cluster".equals(order.getServiceCode())) {
            // 回调boc
            callBoc(order.getSn(), JSONObject.parseObject(order.getApplyParam()), "success", "支付成功", context);
        } else if (order.getServiceCode().startsWith("operations.")) {
            cosService.create(JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class), context);
        } else if (order.getServiceCode().startsWith("fabric.")) {
            rosService.create(JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class), context);
        } else if ("vmware.nsx.vpc".equalsIgnoreCase(order.getServiceCode())) {
            logger.info("预付费NSX-T Vpc申请参数结构：" + order.getApplyParam());
            amqpTemplate.convertAndSend("network.creation.queue",
                    JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class));
        } else {
            ResourceOperate applyOperate = ResourceOperateFactory.getInstance(order.getServiceCode());
            ApplicationParam params = JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class);
            ArrayList<ApplicationParam> messageParams = Lists.newArrayList(params);
            logger.info("预付费资源申请参数结构：" + JSONObject.toJSONString(messageParams));
            if (applyOperate instanceof PaasResourceOperate) {
                amqpTemplate.convertAndSend("paas.resource.creation.queue", messageParams);
            } else if (applyOperate instanceof IaasResourceOperate) {
                amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
            } else {
                amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
            }
        }
    }

    private void handleAltera(ServiceOrder order) {
        ResourceOperate alterationOperate = ResourceOperateFactory.getInstance(order.getServiceCode());
        logger.info("预付费资源变更参数结构：" + order.getApplyParam());
        AlterationParam params = JSONObject.parseObject(order.getApplyParam(), AlterationParam.class);
        if (alterationOperate instanceof PaasResourceOperate) {
            amqpTemplate.convertAndSend("paas.resource.modification.queue", params);
        } else {
            amqpTemplate.convertAndSend("iaas.resource.modification.queue", params);
        }
    }

    private void handleExtension(ServiceOrder order) {
        ResourceOperate extensionOperate = ResourceOperateFactory.getInstance(order.getServiceCode());
        logger.info("预付费资源延期参数结构：" + order.getApplyParam());
        ExtensionParam extensionParam = JSONObject.parseObject(order.getApplyParam(), ExtensionParam.class);
        if (extensionOperate instanceof PaasResourceOperate) {
            amqpTemplate.convertAndSend("paas.resource.extension.queue", extensionParam);
        } else {
            if ("vmware.nsx.vpc".equals(order.getServiceCode())) {
                amqpTemplate.convertAndSend("network.resource.extension.queue", extensionParam);
            } else {
                amqpTemplate.convertAndSend("iaas.resource.extension.queue", extensionParam);
            }
        }
    }

    /**
     * 保存账单
     *
     * @param order
     */
    private ServiceBill buildApplyBill(ServiceOrder order) {
        SimpleDateFormat FORMAT = new SimpleDateFormat("yyyy-MM");
        List<OrderItem> items = itemRepository.listByOrderId(order.getId());
        ServiceBill bill =
                new ServiceBill(IdGenerator.getInstance().nextId(), order.getId(), ChargeCommon.BillStatus.unpay, null,
                        order.getTenantId(), FORMAT.format(new Date()), new Date());
        bill.setUserId(order.getUserId());
        bill.setProjectId(order.getProjectId());
        bill.setBusinessId(order.getBusinessId());
        bill.setMoney(BigDecimal.ZERO);
        billRepository.save(bill);
        BigDecimal money = BigDecimal.valueOf(0);
        for (OrderItem item : items) {
            BigDecimal finalPrice = item.getFinalPrice();
            if (null == finalPrice) {
                finalPrice = BigDecimal.valueOf(0);
            }
            DurationModel duration = JSONObject.parseObject(item.getDuration(), DurationModel.class);
            Integer amount = duration.getAmount();
            BigDecimal billMoney = finalPrice.multiply(BigDecimal.valueOf(amount));
            BillItem billItem = new BillItem();
            billItem.setBillId(bill.getId());
            billItem.setItemId(item.getId());
            billItem.setSkuId(item.getSkuId());
            billItem.setChargeMode(order.getChargeMode());
            billItem.setPrice(billMoney);
            Calendar calendar = Calendar.getInstance();
            billItem.setBillStart(calendar.getTime());
            calendar.setTime(new Date());
            calendar.add(Calendar.MONTH, amount);
            billItem.setBillEnd(calendar.getTime());
            billItem.setMoney(billMoney);
            billItem.setGmtCreate(new Date());
            billItem.setCatalog(BillItem.Catalog.standard);
            billItem.setUserId(order.getUserId());
            billItem.setTenantId(order.getTenantId());
            billItem.setProjectId(order.getProjectId());
            billItem.setBusinessId(order.getBusinessId());
            billItem.setServiceCode(item.getServiceCode());
            billItem.setInstance(item.getSn());
            OrderItemUtils.elementNames(item);
            billItem.setResource(JSONObject.toJSONString(item.getElementList()));
            billItemRepository.save(billItem);
            money = money.add(billMoney);
        }
        bill.setMoney(money);
        billRepository.update(bill);
        return bill;
    }

    /**
     * 生成变更账单(预付费补差价)
     *
     * @param order
     * @return
     * @throws Exception
     */
    private ServiceBill buildAlterationBill(ServiceOrder order) {
        //查找当月账单
        String month = new SimpleDateFormat("yyyy-MM").format(new Date());
        ServiceBill bill = billRepository.getMonthBill(order.getId(), month);
        if (null == bill) {
            bill =
                    new ServiceBill(IdGenerator.getInstance().nextId(), order.getId(), ChargeCommon.BillStatus.paid, null,
                            order.getTenantId(), month, new Date());
            bill.setSn(IdGenerator.getInstance().nextId());
            bill.setOrderId(order.getId());
            bill.setStatus(ChargeCommon.BillStatus.unpay);
            bill.setMonth(DateUtil.localYearMonthFormat(LocalDateTime.now()));
            bill.setGmtCreate(new Date());
            bill.setTenantId(order.getTenantId());
            bill.setProjectId(order.getProjectId());
            bill.setBusinessId(order.getBusinessId());
            bill.setUserId(order.getUserId());
            bill.setMoney(BigDecimal.ZERO);
            billRepository.save(bill);
        }
        //新变更的订单项
        List<OrderItem> orderItems = itemRepository.listByOrderId(order.getId());
        BigDecimal billMoney = bill.getMoney();
        for (OrderItem alterationItem : orderItems) {
            // 账单开始时间
            Date billStart = new Date();
            //查询上一个正常的订购单 （计算差价 计算时间）
            OrderItem oldItem = itemRepository.queryLastSuccessBySn(alterationItem.getSn());
            // 到期时间
            Date billEnd = null == alterationItem.getExpiredTime() ? new Date() : alterationItem.getExpiredTime();
            //算差价
            BigDecimal intervalPrice = alterationItem.getFinalPrice();
            BigDecimal money = BigDecimal.ZERO;
            if (oldItem != null) {
                intervalPrice = alterationItem.getFinalPrice().subtract(oldItem.getFinalPrice());
                if (intervalPrice.compareTo(BigDecimal.ZERO) < 0) {
                    intervalPrice = BigDecimal.ZERO;
                }
                //算差的时间(分钟)
                long min = (billEnd.getTime() - billStart.getTime()) / 60000;
                DurationModel durationModel = JSONObject.parseObject(alterationItem.getDuration(), DurationModel.class);
                switch (durationModel.getMode()) {
                    case Month:
                        money = intervalPrice.divide(BigDecimal.valueOf(31L * 24L * 60L), 5, RoundingMode.HALF_UP)
                                .multiply(BigDecimal.valueOf(min));
                        break;
                    case Year:
                        money = intervalPrice.divide(BigDecimal.valueOf(366L * 24L * 60L), 5, RoundingMode.HALF_UP)
                                .multiply(BigDecimal.valueOf(min));
                        break;
                    default:
                        break;
                }
            } else {
                Calendar expiredCalendar = Calendar.getInstance();
                expiredCalendar.setTime(alterationItem.getExpiredTime());
                Calendar effectiveCalendar = Calendar.getInstance();
                effectiveCalendar.setTime(alterationItem.getEffectiveTime());
                int expired = 0;
                int effective = 0;
                DurationModel durationModel = JSONObject.parseObject(alterationItem.getDuration(), DurationModel.class);
                switch (durationModel.getMode()) {
                    case Month:
                        int expiredYear = expiredCalendar.get(Calendar.YEAR);
                        int effectiveYear = effectiveCalendar.get(Calendar.YEAR);
                        expired = expiredCalendar.get(Calendar.MONTH);
                        effective = effectiveCalendar.get(Calendar.MONTH);
                        money = intervalPrice
                                .multiply(BigDecimal.valueOf((expiredYear - effectiveYear) * 12L + expired - effective));
                        break;
                    case Year:
                        expired = expiredCalendar.get(Calendar.YEAR);
                        effective = effectiveCalendar.get(Calendar.YEAR);
                        money = intervalPrice.multiply(BigDecimal.valueOf(compute(expired) - compute(effective)));
                        break;
                    default:
                        break;
                }
            }
            BillItem billItem = new BillItem();
            billItem.setBillId(bill.getId());
            billItem.setItemId(alterationItem.getId());
            billItem.setSkuId(alterationItem.getSkuId());
            billItem.setChargeMode(order.getChargeMode());
            billItem.setPrice(money);
            billItem.setBillStart(billStart);
            billItem.setBillEnd(billEnd);
            billItem.setMoney(money);
            billItem.setGmtCreate(new Date());
            billItem.setCatalog(BillItem.Catalog.alteration);
            billItem.setResource(JSONObject.toJSONString(alterationItem.getElementList()));
            billItem.setUserId(order.getUserId());
            billItem.setTenantId(order.getTenantId());
            billItem.setProjectId(order.getProjectId());
            billItem.setBusinessId(order.getBusinessId());
            billItem.setServiceCode(alterationItem.getServiceCode());
            OrderItemUtils.elementNames(alterationItem);
            billItem.setResource(JSONObject.toJSONString(alterationItem.getElementList()));
            billItem.setInstance(alterationItem.getSn());
            billItemRepository.save(billItem);
            billMoney = billMoney.add(billItem.getMoney());
        }
        bill.setMoney(billMoney);
        billRepository.update(bill);
        return bill;
    }

    public long compute(Integer factor) {
        Long aLong = Long.valueOf(factor);
        return aLong;
    }

    /**
     * 处理延期订单
     *
     * @param order
     * @return
     */
    private ServiceBill buildExtensionBill(ServiceOrder order) {
        //查找当月账单
        String month = new SimpleDateFormat("yyyy-MM").format(new Date());
        ServiceBill bill = billRepository.getMonthBill(order.getId(), month);
        if (null == bill) {
            bill =
                    new ServiceBill(IdGenerator.getInstance().nextId(), order.getId(), ChargeCommon.BillStatus.paid, null,
                            order.getTenantId(), month, new Date());
            bill.setSn(IdGenerator.getInstance().nextId());
            bill.setOrderId(order.getId());
            bill.setStatus(ChargeCommon.BillStatus.unpay);
            bill.setMonth(DateUtil.localYearMonthFormat(LocalDateTime.now()));
            bill.setGmtCreate(new Date());
            bill.setTenantId(order.getTenantId());
            bill.setProjectId(order.getProjectId());
            bill.setBusinessId(order.getBusinessId());
            bill.setUserId(order.getUserId());
            bill.setMoney(BigDecimal.ZERO);
            billRepository.save(bill);
        }
        List<OrderItem> orderItems = itemRepository.listByOrderId(order.getId());
        BigDecimal totalBillMoney = bill.getMoney();
        for (OrderItem item : orderItems) {
            BigDecimal finalPrice = item.getFinalPrice();
            DurationModel model = JSONObject.parseObject(item.getDuration(), DurationModel.class);
            //计算价格
            BigDecimal billPrice = finalPrice.multiply(BigDecimal.valueOf(model.getAmount()));
            BillItem billItem = new BillItem();
            billItem.setBillId(bill.getId());
            billItem.setItemId(item.getId());
            billItem.setSkuId(item.getSkuId());
            billItem.setChargeMode(order.getChargeMode());
            billItem.setPrice(billPrice);
            billItem.setBillStart(item.getEffectiveTime());
            billItem.setBillEnd(item.getExpiredTime());
            billItem.setMoney(billPrice);
            billItem.setGmtCreate(new Date());
            billItem.setCatalog(BillItem.Catalog.extension);
            billItem.setUserId(order.getUserId());
            billItem.setTenantId(order.getTenantId());
            billItem.setProjectId(order.getProjectId());
            billItem.setBusinessId(order.getBusinessId());
            OrderItemUtils.elementNames(item);
            billItem.setResource(JSONObject.toJSONString(item.getElementList()));
            billItem.setServiceCode(item.getServiceCode());
            billItemRepository.save(billItem);
            totalBillMoney = totalBillMoney.add(billPrice);
        }
        bill.setMoney(totalBillMoney);
        billRepository.update(bill);
        return bill;
    }

    /**
     * 调用boc
     * http://10.10.5.23:30022/boclouddeployerapi/cmpCluster/v2.4/createCluster
     *
     * @param callback
     * @param result
     * @return
     */
    private Result callBoc(String callback, JSONObject params, String result, String message, RequestContext context) {
        String token = socService.getToken(context.getTarget(), context.getCatalog().name());
        logger.info("获取token：{}", token);
        HttpClient httpClient = new HttpClient(20 * 1000 * 1000, PostDataFormat.RAW);
        try {
            params.put("result", result);
            params.put("callback", callback);
            params.put("message", message);
            Map<String, Object> header = new HashMap<>();
            header.put("accept", "application/json");
            header.put("content-type", "application/json");
            if (StringUtils.hasText(token)) {
                header.put("token", token);
            }
            String url = configService.getByCode("k8sAddress") + "/boclouddeployerapi/cmpCluster/v2.4/createVM";
            logger.info("调用boc地址：{}", url);
            logger.info("调用boc参数：{}", params.toJSONString());
            return httpClient.post(header, MapTools.simpleMap("vmInfo", params), url);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return Result.FAILED("回调容器平台失败");
        } finally {
            httpClient.close();
        }
    }

    @Override
    public Result cancel(String sn, RequestContext context) {
        try {
            ServiceOrder order = orderRepository.queryBySn(sn);
            if (null == order) {
                return new Result(false, "订单不存在");
            }

            if (ChargeCommon.OrderStatus.canceld == order.getOrderStatus()) {
                return new Result(false, "订单已取消，无需再次取消");
            }
            List<OrderItem> items = itemRepository.listByOrderId(order.getId());

            // 待扣除的配额
            Map<String, Integer> quotaMap = OrderItemUtils.getItemQuota(items);

            // 资源订购的单子不考虑之前的配额
            if (!ChargeCommon.Type.apply.equals(order.getType())) {
                /**
                 * 1. 根据未支付的订单项生成配额
                 * 2. 找到最后一次成功的订单生成旧配额
                 * 3. 新-旧配额
                 * 4. 恢复配额
                 */

                Map<String, Integer> oldQuota = new HashMap<>();
                // 旧配额
                // 1。根据serviceCode找到变更资源的sn
                List<OrderItem> orderItems = itemRepository.listByOrderId(order.getId());
                if (null == orderItems || orderItems.isEmpty()) {
                    throw new IllegalArgumentException("旧订单项未找到");
                }

                OrderItem applyItem = orderItems.get(0);
                // 3. 根据变更资源的sn找到最新有效的变更资源的订单项
                // 3.1 最近订单项
                OrderItem orderItem = itemRepository.queryLastSuccessBySn(applyItem.getSn());
                if (null == orderItem) {
                    throw new IllegalArgumentException("最近订单项未找到");
                }
                // 3.2 最近订单
                ServiceOrder oldOrder = orderRepository.query(orderItem.getOrderId());
                if (null == oldOrder) {
                    throw new IllegalArgumentException("最近订单未找到");
                }
                // 4。生成旧配额
                oldQuota = OrderItemUtils.getItemQuota(Lists.newArrayList(orderItem));

                for (Map.Entry<String, Integer> entry : quotaMap.entrySet()) {
                    String code = entry.getKey();
                    Integer number = Optional.ofNullable(oldQuota.get(code)).orElse(0);
                    quotaMap.put(code, quotaMap.get(code) - number);
                }
            }

            List<QuotaBean> beans = new ArrayList<>();
            for (String key : quotaMap.keySet()) {
                beans.add(new QuotaBean(key, quotaMap.get(key)));
            }
            // 回滚配额
            this.handleQuota(beans, order.getTenantId());
            if ("vmware.k8s.cluster".equals(order.getServiceCode())) {
                // 回调boc
                callBoc(order.getSn(), JSONObject.parseObject(order.getApplyParam()), "cancel", "支付取消", context);
            }

            // 恢复配额后再取消订单

            order.setOrderStatus(ChargeCommon.OrderStatus.canceld);
            order.setResourceStatus(ChargeCommon.ResourceStatus.canceld);
            orderRepository.update(order);

            if (items != null) {
                items.forEach(item -> item.setStatus(OrderItemStatus.Canceld));
                itemRepository.batchUpdate(items, items.size());
            }
            return new Result(true, "订单取消成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, "订单取消失败");
        }
    }

    /**
     * 回滚配额
     *
     * @param beans
     * @param tenantId
     */
    private void handleQuota(List<QuotaBean> beans, Long tenantId) {
        List<ResourceQuota> list = resourceQuotaRepository.listByTenantId(tenantId);
        Map<String, ResourceQuota> map = new HashMap<>();
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (QuotaBean bean : beans) {
            if (!map.containsKey(bean.getCode())) {
                continue;
            }
            ResourceQuota quota = map.get(bean.getCode());
            // 记录回滚配额操作量
            quota.setQuotaActionValue(bean.getValue().longValue());
            quotas.add(quota);
        }
        resourceQuotaService.quotaAction(quotas, ResourceQuota.Action.Callback, tenantId);
    }

    @Override
    public BocloudResult<LineChartResult> stats(String code) {
        try {
            List<String> keys = new ArrayList<>();
            List<Object> data = new ArrayList<>();
            LineChartResult chartResult;
            LineChartResult.Values[] values;
            BocloudResult<LineChartResult> result;
            List<Param> params = new ArrayList<>();
            List<Map<String, Object>> items;
            switch (code) {
                // 租户订购产品统计TOP5
                case "tenantTop5":
                    items = itemRepository.tenantTop5();
                    for (Map<String, Object> item : items) {
                        keys.add(this.getServiceName((String) item.get("service_code")));
                        data.add(item.get("total"));
                    }
                    values = new LineChartResult.Values[]{new LineChartResult.Values("总数(个)", data)};
                    chartResult = new LineChartResult(keys, values);
                    result = new BocloudResult<>(true, chartResult, "查询成功");
                    break;
                // 上架产品总数
                case "normalSku":
                    params.add(new Param(MapTools.simpleMap("processStatus", "ReleaseApproved"), Sign.EQ));
                    result = new BocloudResult(true, new StatsResult("个数", skuRepository.count(params)), "查询成功");
                    break;
                // 订购租户数TOP5
                case "tenantOrderTop5":
                    items = itemRepository.tenantOrderTop5();
                    for (Map<String, Object> item : items) {
                        keys.add((String) item.get("tenant_name"));
                        data.add(item.get("total"));
                    }
                    values = new LineChartResult.Values[]{new LineChartResult.Values("总数(个)", data)};
                    chartResult = new LineChartResult(keys, values);
                    result = new BocloudResult<>(true, chartResult, "查询成功");
                    break;
                default:
                    result = new BocloudResult<>(false, "暂不支持");
                    break;
            }
            return result;
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 获取服务名称
     *
     * @param serviceCode
     * @return
     */
    private String getServiceName(String serviceCode) {
        CloudService service = cloudServiceRepository.queryByCode(serviceCode);
        if (null != service) {
            return service.getName();
        }
        return null;
    }

    @Override
    public Result updateItem(String sn, String status, RequestContext context) {
        OrderItem item;
        try {
            item = itemRepository.queryBySn(sn);
            Assert.notNull(item, "订单明细不存在");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, "订单明细更新失败:" + e.getMessage());
        }
        OrderItemStatus oldStatus = item.getStatus();
        String path = OrderItem.class.getSimpleName() + "_" + item.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            item.setStatus(OrderItemStatus.valueOf(status));
            if (OrderItemStatus.OpenedSuccess == item.getStatus()) {
                item.setEffectiveTime(new Date());
                if (StringUtils.hasText(item.getDuration())) {
                    DurationModel durationModel = JSONObject.parseObject(item.getDuration(), DurationModel.class);
                    if (null != durationModel) {
                        Integer month = durationModel.getAmount();
                        if (null != month && DurationModel.Mode.Hour == durationModel.getMode()) {
                            Calendar calendar = Calendar.getInstance();
                            calendar.add(Calendar.MONTH, month);
                            item.setExpiredTime(calendar.getTime());
                        }
                    }
                } else {
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
                    item.setExpiredTime(calendar.getTime());
                }
            }
            // 原来订单是已终止，新订单是开通成功，说明是回收站移出
            if (OrderItemStatus.Termination == oldStatus && OrderItemStatus.OpenedSuccess == item.getStatus()) {
                item.setMoveOutTime(new Date());
            }
            itemRepository.update(item);
            return new Result(true, "订单明细更新成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, "订单明细更新失败");
        }
    }

    @Override
    public Result removeTenantOrder(Long tenantId, RequestContext context) {
        try {
            List<ServiceOrder> orderList = orderRepository.listByTenantId(tenantId);
            List<Long> orderIds = orderList.stream().map(ServiceOrder::getId).collect(Collectors.toList());
            if (!orderIds.isEmpty()) {
                Boolean ordersFlag = orderRepository.batchRemove(orderIds);
                Boolean itemFlag = itemRepository.removeByOrderIds(orderIds);
                Boolean billsFlag = billRepository.removeByOrderIds(orderIds);
                return ordersFlag && itemFlag && billsFlag ? new Result(true, "删除成功") : new Result(false, "删除失败");
            } else {
                return new Result(true, "租户不存在订单");
            }
        } catch (Exception e) {
            logger.error("delete tenant orders error,", e);
            return BocloudResult.FAILED("删除失败");
        }
    }

    @Override
    public Result updateElements(String sn, String elements, RequestContext context) {
        OrderItem item;
        try {
            item = itemRepository.queryBySn(sn);
            Assert.notNull(item, "订单明细不存在");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, "订单明细更新失败:" + e.getMessage());
        }
        String path = OrderItem.class.getSimpleName() + "_" + item.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            item.setElements(elements);
            itemRepository.update(item);
            return new Result(true, "订单明细更新成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, "订单明细更新失败");
        }
    }

    @Override
    public Result createAlterationItem(String sn, Long skuId, RequestContext context) {
        logger.info("接收到资源变更订单消息：order sn is {},sku id is {}", sn, skuId);
        OrderItem oldItem;
        try {
            oldItem = itemRepository.queryBySn(sn);
            Assert.notNull(oldItem, "订单号为【" + sn + "】的子订单明细未找到");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new Result(false, e.getMessage());
        }
        String path = OrderItem.class.getSimpleName() + "_" + oldItem.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceOrder order = orderRepository.query(oldItem.getOrderId());
            Assert.notNull(order, "订单不存在");
            OrderItem newItem = JSONObject.parseObject(JSONObject.toJSONString(oldItem), OrderItem.class);
            // 生成新订单号
            BocloudResult<String> snResult = orderSnService.generate(skuId);
            Assert.isTrue(snResult.isSuccess(), snResult.getMessage());
            newItem.setSn(snResult.getData());
            newItem.setSkuId(skuId);
            newItem.setStatus(OrderItemStatus.ChangeSuccess);
            newItem.setEffectiveTime(new Date());
            BigDecimal basicPrice = BigDecimal.valueOf(0);
            BigDecimal finalPrice = BigDecimal.valueOf(0);
            // 此处用来获取系统盘大小，按照数据盘价格进行收费
            int systemDiskSize = 0;
            BigDecimal systemDiskPrice = null;
            BigDecimal systemDiskRatio = null;
            CloudService cloudService = cloudServiceRepository.queryByCode(oldItem.getServiceCode());
            List<ServicePrice> servicePrices = JSONArray.parseArray(cloudService.getBasicPrice(), ServicePrice.class);
            Map<String, ServicePrice> specBasicMap =
                    servicePrices.stream().collect(Collectors.toMap(ServicePrice::getSpecName, Function.identity()));
            //旧订单计费信息
            DurationModel durationModel = JSONObject.parseObject(oldItem.getDuration(), DurationModel.class);
            // 旧订单配置信息
            String elements = oldItem.getElements();
            ServiceSku sku = skuRepository.query(skuId);
            Assert.notNull(sku, "用户所选的产品规格SKU不存在");
            if (StringUtils.hasText(elements)) {
                List<ElementModel> elementModels = JSONArray.parseArray(elements, ElementModel.class);
                for (ElementModel elementModel : elementModels) {
                    List<SpecModel> specs = elementModel.getSpecs();
                    if (!ListTool.isEmpty(specs) && specs.size() == 3) {
                        elementModel.setSkuId(skuId);
                        String spec = sku.getSpec();
                        Assert.notNull(spec, "产品SKU的规格数据为空");
                        List<JSONObject> specList = JSONArray.parseArray(spec, JSONObject.class);
                        Integer cpu = 0;
                        Integer memory = 0;
                        for (JSONObject specObj : specList) {
                            if ("cpu".equalsIgnoreCase(specObj.getString("specName"))) {
                                cpu = specObj.getInteger("specValue");
                            }
                            if ("memory".equalsIgnoreCase(specObj.getString("specName"))) {
                                memory = specObj.getInteger("specValue");
                            }
                        }
                        for (SpecModel specModel : specs) {
                            Integer specCpu = specModel.getCpu();
                            if (null != specCpu) {
                                specModel.setCpu(cpu);
                            }
                            BigDecimal memCpu = specModel.getMemory();
                            if (null != memCpu) {
                                specModel.setMemory(BigDecimal.valueOf(memory));
                            }
                            // 寻找系统盘大小
                            Integer disk = specModel.getDisk();
                            if (null != disk) {
                                systemDiskSize = disk;
                            }
                        }
                        logger.info("修改后的specs:{}", JSONObject.toJSONString(specs));
                        elementModel.setSpecs(specs);
                        if (cloudService.getBillPolicy().equals(CloudService.BillPolicy.spec)) {
                            BigDecimal priceDecimal;
                            switch (durationModel.getMode()) {
                                case Month:
                                    priceDecimal = sku.getMonthPrice();
                                    break;
                                case Year:
                                    priceDecimal = sku.getYearPrice();
                                    break;
                                default:
                                    priceDecimal = sku.getHourPrice();
                                    break;
                            }
                            elementModel.setPrice(priceDecimal);
                            basicPrice = basicPrice.add(priceDecimal);
                            finalPrice = finalPrice.add(priceDecimal.multiply(elementModel.getRatio()));
                        } else {
                            for (SpecModel specModel : specs) {
                                Integer specCpu = specModel.getCpu();
                                if (null != specCpu) {
                                    ServicePrice servicePrice = specBasicMap.get("cpu");
                                    BigDecimal cpuPrice = ServicePrice.getPrice(durationModel.getMode(), servicePrice);
                                    basicPrice = basicPrice.add(cpuPrice.multiply(BigDecimal.valueOf(specCpu)));
                                    finalPrice = finalPrice.add(cpuPrice.multiply(BigDecimal.valueOf(specCpu))
                                            .multiply(elementModel.getRatio()));
                                }
                                BigDecimal memCpu = specModel.getMemory();
                                if (null != memCpu) {
                                    ServicePrice servicePrice = specBasicMap.get("memory");
                                    BigDecimal memoryPrice =
                                            ServicePrice.getPrice(durationModel.getMode(), servicePrice);
                                    basicPrice = basicPrice.add(memoryPrice.multiply(memCpu));
                                    finalPrice =
                                            finalPrice.add(memoryPrice.multiply(memCpu).multiply(elementModel.getRatio()));
                                }
                                // 寻找系统盘大小
                                Integer disk = specModel.getDisk();
                                if (null != disk) {
                                    ServicePrice servicePrice = specBasicMap.get("disk");
                                    BigDecimal diskPrice = ServicePrice.getPrice(durationModel.getMode(), servicePrice);
                                    basicPrice = basicPrice.add(diskPrice.multiply(BigDecimal.valueOf(disk)));
                                    finalPrice = finalPrice.add(
                                            diskPrice.multiply(BigDecimal.valueOf(disk)).multiply(elementModel.getRatio()));
                                }
                            }
                            elementModel.setPrice(basicPrice);
                        }
                    } else {
                        if (cloudService.getBillPolicy().equals(CloudService.BillPolicy.spec)) {
                            BigDecimal priceDecimal;
                            switch (durationModel.getMode()) {
                                case Month:
                                    priceDecimal = sku.getMonthPrice();
                                    break;
                                case Year:
                                    priceDecimal = sku.getYearPrice();
                                    break;
                                default:
                                    priceDecimal = sku.getHourPrice();
                                    break;
                            }
                            elementModel.setPrice(priceDecimal);
                            basicPrice = basicPrice.add(priceDecimal);
                            finalPrice = finalPrice.add(priceDecimal.multiply(elementModel.getRatio()));
                        } else {
                            ServicePrice servicePrice = servicePrices.get(0);
                            BigDecimal price;
                            switch (durationModel.getMode()) {
                                case Month:
                                    price = servicePrice.getMonthPrice();
                                    break;
                                case Year:
                                    price = servicePrice.getYearPrice();
                                    break;
                                default:
                                    price = servicePrice.getHourPrice();
                                    break;
                            }
                            SpecModel spec = specs.get(0);
                            if (null != spec.getDisk() && 0L != spec.getDisk()) {
                                basicPrice = basicPrice.add(price.multiply(BigDecimal.valueOf(spec.getDisk())));
                                finalPrice = finalPrice.add(price.multiply(BigDecimal.valueOf(spec.getDisk()))
                                        .multiply(elementModel.getRatio()));
                                systemDiskPrice = price;
                                systemDiskRatio = elementModel.getRatio();
                            } else {
                                basicPrice = basicPrice.add(price);
                                finalPrice = finalPrice.add(price.multiply(elementModel.getRatio()));
                            }
                            elementModel.setPrice(price);
                        }
                    }
                }
                newItem.setElements(JSONObject.toJSONString(elementModels));
            }
            // 价格追加系统盘大小
            if (null == systemDiskPrice) {
                String serviceCode = oldItem.getServiceCode();
                String volumeCode = null;
                if (serviceCode.contains("tianyi")) {
                    volumeCode = "tianyi.standard.volume";
                } else if (serviceCode.contains("vmware")) {
                    volumeCode = "vmware.storage.disk";
                } else if (serviceCode.contains("huawei")) {
                    volumeCode = "huawei.standard.volume";
                }
                List<ServiceSku> serviceSkus = skuRepository.listByServiceCode(volumeCode);
                Assert.notEmpty(serviceSkus, "未找到合适的磁盘规格");
                ServiceSku volumeSku = serviceSkus.get(0);
                systemDiskPrice = Optional.ofNullable(getPrice(durationModel, volumeSku)).orElse(BigDecimal.ZERO);
                systemDiskRatio = BigDecimal.valueOf(1);
            }
            basicPrice = basicPrice.add(systemDiskPrice.multiply(BigDecimal.valueOf(systemDiskSize)));
            finalPrice =
                    finalPrice.add(systemDiskPrice.multiply(BigDecimal.valueOf(systemDiskSize)).multiply(systemDiskRatio));
            Tenant tenant = smsTenantService.detail(order.getTenantId());
            Assert.notNull(tenant, "租户不存在");
            BigDecimal tenantDiscount = tenant.getDiscount();
            if (null == tenantDiscount) {
                tenantDiscount = BigDecimal.valueOf(1);
            }
            // 比较租户折扣之后的价格，如果租户的折扣后价格更便宜，则使用租户的折扣
            BigDecimal tenantPrice = basicPrice.multiply(tenantDiscount);
            if (tenantPrice.compareTo(finalPrice) < 0) {
                finalPrice = tenantPrice;
                processElement(newItem, tenantDiscount);
            }
            newItem.setBasicPrice(basicPrice);
            newItem.setFinalPrice(finalPrice);
            newItem.setId(null);
            // 保存新订单
            itemRepository.save(newItem);
            // 废弃旧订单
            oldItem.setChangerId(context.getTarget());
            oldItem.setStatus(OrderItemStatus.Abandoned);
            oldItem.setExpiredTime(new Date());
            itemRepository.update(oldItem);
            return new Result(true, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult<ServiceOrder> redistribute(Long id, Long tenantId, Long projectId, Long userId,
                                                    RequestContext context) {
        try {
            ServiceOrder order = orderRepository.query(id);
            Assert.notNull(order, "订单不存在");
            order.setUserId(userId);
            order.setProjectId(projectId);
            order.setTenantId(tenantId);
            orderRepository.update(order);
            List<ServiceBill> bills = billRepository.listByOrderId(id);
            for (ServiceBill bill : bills) {
                bill.setUserId(userId);
                bill.setProjectId(projectId);
                bill.setTenantId(tenantId);
                billRepository.update(bill);
            }
            return new BocloudResult<>(true, "重新分配成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    @Override
    public String underwayCheck(String sn) {
        List<OrderItem> underwayOrders = itemRepository.queryUnderwayOrders(sn);
        if (CollectionUtils.isNotEmpty(underwayOrders)) {
            return String.format("资源存在%s的订单，稍后请重试", underwayOrders.get(0).getStatus().getMsg());
        }
        return null;
    }
}
