package com.bocloud.cos.service.task;

import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.entity.Cos;
import com.bocloud.cos.model.NotifyMessage;
import com.bocloud.cos.repository.CosRepository;
import com.bocloud.cos.internal.SmsSystemConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;

public class CosExpiredTask extends AbstractExpiredTask {
    private static Logger logger = LoggerFactory.getLogger(CosExpiredTask.class);
    private CosRepository cosRepository;
    private Cos cos;

    public CosExpiredTask(Cos cos, SmsSystemConfigService smsSystemConfigService, AmqpTemplate amqpTemplate,
                          CosRepository cosRepository) {
        super(smsSystemConfigService, amqpTemplate);
        this.cos = cos;
        this.cosRepository = cosRepository;
    }

    @Override
    public NotifyMessage process() {
        try {
            Long residueTime = cos.getExpiredTime().getTime() - System.currentTimeMillis();// 资源剩余时间
            String residue = "";
            if (residueTime < 0L) {
                residue = "资源已经到期!";
                cosRepository.updateStatus(cos.getId(), "unavailable");
            } else if (residueTime % ONE_DAY == 0L) {
                residue = residueTime / ONE_DAY + "天";
            } else if (residueTime % ONE_DAY > 0L) {
                residue = residueTime / ONE_DAY + "天" + residueTime % ONE_DAY / (60 * 60 * 1000L) + "小时";
            }
            String subject = "运维服务:{" + cos.getName() + "}资源到期预警";
            String content = null;
            RequestContext context = null;
            context = new RequestContext(
                    cos.getTenantId() > 0 ? RequestContext.Catalog.Tenant : RequestContext.Catalog.User,
                    cos.getCreatorId(), cos.getTenantId(), null, null);
            content = "运维服务:(" + cos.getName() + ")即将到期!剩余时间为：" + residue;
            return new NotifyMessage(context, subject, content, true, false);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return null;
        }
    }

    @Override
    public boolean preCheck() {
        // 资源到期时间和当前时间差已达到资源到期提前提醒时间
        if (cos.getExpiredTime().getTime() - System.currentTimeMillis() <= resourceExpireRemindMillis) {
            return true;
        }
        return false;
    }

}
