package com.bocloud.cos.service.task;

import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.entity.Ros;
import com.bocloud.cos.model.NotifyMessage;
import com.bocloud.cos.repository.RosRepository;
import com.bocloud.cos.internal.SmsSystemConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;

public class RosExpiredTask extends AbstractExpiredTask {
    private static Logger logger = LoggerFactory.getLogger(RosExpiredTask.class);
    private RosRepository rosRepository;
    private Ros ros;

    public RosExpiredTask(Ros ros, SmsSystemConfigService smsSystemConfigService, AmqpTemplate amqpTemplate,
                          RosRepository rosRepository) {
        super(smsSystemConfigService, amqpTemplate);
        this.ros = ros;
        this.rosRepository = rosRepository;
    }

    @Override
    public NotifyMessage process() {
        try {
            Long residueTime = ros.getExpiredTime().getTime() - System.currentTimeMillis();// 资源剩余时间
            String residue = "";
            if (residueTime < 0L) {
                residue = "资源已经到期!";
                rosRepository.updateStatus(ros.getId(), "unavailable");
            } else if (residueTime % ONE_DAY == 0L) {
                residue = residueTime / ONE_DAY + "天";
            } else if (residueTime % ONE_DAY > 0L) {
                residue = residueTime / ONE_DAY + "天" + residueTime % ONE_DAY / (60 * 60 * 1000L) + "小时";
            }
            String subject = "资源编排服务:{" + ros.getName() + "}资源到期预警";
            String content = null;
            RequestContext context = null;
            context = new RequestContext(
                    ros.getTenantId() > 0 ? RequestContext.Catalog.Tenant : RequestContext.Catalog.User,
                    ros.getCreatorId(), ros.getTenantId(), null, null);
            content = "资源编排服务:(" + ros.getName() + ")即将到期!剩余时间为：" + residue;
            return new NotifyMessage(context, subject, content, true, false);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return null;
        }
    }

    @Override
    public boolean preCheck() {
        // 资源到期时间和当前时间差已达到资源到期提前提醒时间
        if (ros.getExpiredTime().getTime() - System.currentTimeMillis() <= resourceExpireRemindMillis) {
            return true;
        }
        return false;
    }

}
