package com.bocloud.cos.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 日期工具类
 *
 * @author sxp
 * @since V1.0 Aug 21, 2016
 */
public class DateUtils {

    public static final String STANDARD_DATE_FORMAT = "yyyy-MM-dd";

    public static final String STANDARD_DATETIME_FORMAT = "yyyy-MM-dd HH:mm:ss";

    public static final String WHOLE_DATETIME_FORMAT = "yyyy-MM-dd HH:mm:ss.SSS";


    /**
     * 将字符串转成时间
     *
     * @param dateString 时间的字符串，例如：2016-07-22
     * @param format     字符串匹配模式，例如：yyyy-MM-dd
     * @return 返回dateString对应的Date对象
     * @author sxp
     * @since V1.0 Jul 22, 2016
     */
    public static Date stringToDate(String dateString, String format) {
        if (dateString == null || "".equals(dateString)) {
            throw new RuntimeException("传入参数中的[时间串]不能为空！");
        }
        if (format == null || "".equals(format)) {
            throw new RuntimeException("传入参数中的[时间格式]不能为空！");
        }

        SimpleDateFormat df = new SimpleDateFormat(format);
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串

        Date date;

        try {
            date = df.parse(dateString);
        } catch (ParseException e) {
            throw new RuntimeException(e.getMessage());
        }

        return date;
    }

}
