package com.bocloud.cos.utils;

import com.bocloud.cos.utils.arima.ARIMA;
import com.google.common.collect.Lists;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collections;
import java.util.List;

/**
 * 预测算法
 *
 * @author lkx
 */
public class PredictUtils {

    /**
     * 加权平均法预测
     * <p>
     * 每个input的权值依次翻倍
     *
     * @param input
     * @return
     */
    public static Double weighted(List<Object> input) {
        try {
            int size = input.size();
            List<Double> weights = Lists.newArrayList();
            // 计算基础起始值
            double first = BigDecimal.valueOf(Math.pow(2, size - 1d))
                    .divide(BigDecimal.valueOf(Math.pow(2, size) - 1d), 10, RoundingMode.HALF_UP).doubleValue();
            double sum = 0;
            for (int index = 0; index < size - 1; index++) {
                // 每个权值是前一个的两倍
                double weight = BigDecimal.valueOf(first).multiply(BigDecimal.valueOf(Math.pow(1d / 2, index)))
                        .setScale(10, BigDecimal.ROUND_HALF_UP).doubleValue();
                weights.add(weight);
                sum += weight;
            }
            // 最后一个权值用1减来获取
            weights.add(1 - sum);
            // 进行权值排序
            Collections.sort(weights);
            double result = 0.0;
            for (int index = 0; index < size; index++) {
                // 加权平均获取预测值
                result += BigDecimal.valueOf(Double.parseDouble(input.get(index).toString())).multiply(BigDecimal.valueOf(weights.get(index)))
                        .setScale(10, BigDecimal.ROUND_HALF_UP).doubleValue();
            }
            return result;
        } catch (Exception e) {
            return 0.0;
        }
    }

    /**
     * arima法预测
     * <p>
     * data的size至少为15
     *
     * @param data
     * @return
     */
    public static Integer arima(List<Object> data) {
        try {
            double[] dataArray = new double[data.size() - 1];
            for (int i = 0; i < data.size() - 1; i++) {
                dataArray[i] = Double.parseDouble(data.get(i).toString());
            }
            ARIMA arima = new ARIMA(dataArray);
            int[] model = arima.getARIMAmodel();
            return arima.aftDeal(arima.predictValue(model[0], model[1]));
        } catch (Exception e) {
            return -1;
        }
    }
}
